﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using Newtonsoft.Json;
using System.Net.Http;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking Billing API endpoints.
    /// </summary>
    public class BillingClient : CelloHttpClient
    {
        public BillingClient(string baseAddress = "https://localhost/api/billing/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }
        /// <summary>
        /// This method is used to invoke <c>'search_invoice'</c> endpoint.
        /// </summary>
        /// <param name="condition">search condition</param>
        /// <returns>search result</returns>
        public InvoiceSearchResultDTO SearchInvoices(CelloSaaS.RestApiSDK.InvoiceSearchConditionDTO condition)
        {
            var uri = "search_invoice";

            var response = this.Client.PostAsJsonAsync(uri, condition).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<InvoiceSearchResultDTO>().Result;
                return entity;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_invoice_byId'</c> endpoint.
        /// </summary>
        /// <param name="id">invoice primary key</param>
        /// <param name="tenantId">tenant identifier</param>
        /// <returns>returns the invoice entity if found else null</returns>
        public InvoiceDTO GetInvoiceById(Guid id, Guid tenantId)
        {
            var uri = "get_invoice_byId?invoice_id=" + id;
            uri += "&tenantId=" + tenantId;

            var response = this.Client.GetAsync(uri).Result;
            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<InvoiceDTO>().Result;
                return entity;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_bill_statistics'</c> endpoint.
        /// </summary>
        /// <param name="condition">condition</param>
        /// <returns>returns the bill statistics</returns>
        public Dictionary<Guid, BillStatisticsDTO> GetBillStatistics(BillStatisticsSearchConditionDTO condition)
        {
            var uri = "get_bill_statistics";
            var response = this.Client.PostAsJsonAsync(uri, condition).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<Dictionary<Guid, BillStatisticsDTO>>().Result;
                return entity;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_revenues'</c> endpoint.
        /// </summary>
        /// <param name="fromDate">from date</param>
        /// <param name="toDate">to date</param>
        /// <param name="childTenantId">child tenant identifier</param>
        /// <returns>date, (total bills, amount, paid)</returns>
        public Dictionary<DateTime, Tuple<int, double, double, double>> GetRevenues(DateTime? fromDate, DateTime? toDate, string childTenantId)
        {
            var uri = "get_revenues?fromDate=" + fromDate;
            uri += "&toDate=" + toDate;
            uri += "&childTenantId=" + childTenantId;

            var response = this.Client.GetAsync(uri).Result;
            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<Dictionary<DateTime, Tuple<int, double, double, double>>>().Result;
                return entity;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);

        }
    }
}
