﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;
using System.Web;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking DataPartition API endpoints.
    /// </summary>
    public class DataPartitionClient : CelloHttpClient
    {
        public DataPartitionClient(string baseAddress = "https://localhost/api/datapartition/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }
        /// <summary>
        /// This method is used to invoke <c>'get_default_partition_id'</c> endpoint.
        /// </summary>
        /// <returns>default partition identifier(GUID)</returns>
        public Guid GetDefaultPartitionId()
        {
            var uri = "get_default_partition_id";
            
            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<Guid>().Result;
                return result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'set_default_partition_id'</c> endpoint.
        /// </summary>
        /// <param name="partition_id">partition identifier(mandatory)</param>
        public void SetDefaultPartitionId(Guid partition_id)
        {
            var uri = "set_default_partition_id?partition_id=" + partition_id.ToString();

            var response = this.Client.PutAsync(uri, null).Result;

            if (response.IsSuccessStatusCode)
            {
                return;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_connection_string'</c> endpoint.
        /// </summary>
        /// <param name="module_code">module code</param>
        /// <param name="partition_id">Partition identifier</param>
        /// <returns>Specified connection string based on the parameters</returns>
        public string GetConnectionString(string module_code, Guid? partition_id = null)
        {
            var uri = "get_connection_string?module_code=" + HttpUtility.UrlEncode(module_code);
            if (partition_id.HasValue) uri += "&partition_id=" + partition_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var data = response.Content.ReadAsAsync<string>().Result;
                return data;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_all_connection_strings'</c> endpoint.
        /// </summary>
        /// <param name="partition_id">partition identifier</param>
        /// <returns>collection of all connection string details</returns>
        public IEnumerable<CelloSaaS.RestApiSDK.DataPartitionConnDTO> GetAllConnectionStrings(Guid? partition_id = null)
        {
            var uri = "get_all_connection_strings?a=1";
            if (partition_id.HasValue) uri += "&partition_id=" + partition_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var data = response.Content.ReadAsAsync<DataPartitionConnDTO[]>().Result;
                return data;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
