﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Net.Http;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking DataViews API endpoints.
    /// </summary>
    public class DataViewsClient : CelloHttpClient
    {
        public DataViewsClient(string baseAddress = "https://localhost/api/dataviews/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }

        /// <summary>
        /// This method is used to invoke <c>'get_all_dataviews'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>collection of all data views</returns>
        public IEnumerable<CelloSaaS.RestApiSDK.DataViewDTO> GetAllDataViews(Guid? tenant_id = null)
        {
            var uri = "get_all_dataviews";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<CelloSaaS.RestApiSDK.DataViewDTO[]>().Result;
                return result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_dataview'</c> endpoint.
        /// </summary>
        /// <param name="dataview_id">data view identifier</param>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>data view details</returns>
        public CelloSaaS.RestApiSDK.DataViewDTO GetDataView(string dataview_id, Guid? tenant_id = null)
        {
            var uri = "get_dataview?dataview_id=" + HttpUtility.UrlEncode(dataview_id);
            if (tenant_id.HasValue) uri += "&tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<DataViewDTO>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
