﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;
using System.Web;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking EntityMetadatas API endpoints.
    /// </summary>
    public class EntityMetadatasClient : CelloHttpClient
    {
        public EntityMetadatasClient(string baseAddress = "https://localhost/api/entitymetadatas/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }

        /// <summary>
        /// This method is used to invoke <c>'get_all_entity_metadatas'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>collection of all entity metadatas</returns>
        public IEnumerable<CelloSaaS.RestApiSDK.EntityMetadataDTO> GetAllEntityMetadatas(Guid? tenant_id = null)
        {
            var uri = "get_all_entity_metadatas";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<CelloSaaS.RestApiSDK.EntityMetadataDTO[]>().Result;
                return result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_entity_metadata'</c> endpoint.
        /// </summary>
        /// <param name="entity_id">entity identifier</param>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>entity metatdata details</returns>
        public CelloSaaS.RestApiSDK.EntityMetadataDTO GetEntityMetadata(string entity_id, Guid? tenant_id = null)
        {
            var uri = "get_entity_metadata?entity_id=" + HttpUtility.UrlEncode(entity_id);
            if (tenant_id.HasValue) uri += "&tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<EntityMetadataDTO>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_custom_data'</c> endpoint.
        /// </summary>
        /// <param name="ref_id">reference identifier</param>
        /// <param name="entity_id">entity identifier</param>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>custom data details</returns>
        public CelloSaaS.RestApiSDK.CustomDataDTO GetCustomData(string ref_id, string entity_id, Guid? tenant_id = null)
        {
            var uri = "get_custom_data?entity_id=" + HttpUtility.UrlEncode(entity_id);
            uri += "&ref_id=" + HttpUtility.UrlEncode(ref_id.ToString());
            if (tenant_id.HasValue) uri += "&tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<CustomDataDTO>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        ///  This method is used to invoke <c>'put_custom_data'</c> endpoint.
        /// </summary>
        /// <param name="data">CustomDataDTO</param>
        /// <returns>custom data</returns>
        public CustomDataDTO AddCustomData(CelloSaaS.RestApiSDK.CustomDataDTO data)
        {
            var uri = "put_custom_data";

            var response = this.Client.PutAsJsonAsync(uri, data).Result;

            if (response.StatusCode == System.Net.HttpStatusCode.Created)
            {
                var entity = response.Content.ReadAsAsync<CustomDataDTO>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'delete_custom_data'</c> endpoint.
        /// </summary>
        /// <param name="ref_id">reference identifier</param>
        /// <param name="entity_id">entity identifier</param>
        /// <param name="tenant_id">tenant identifier</param>
        public void DeleteCustomData(string ref_id, string entity_id, Guid? tenant_id = null)
        {
            var uri = "delete_custom_data?entity_id=" + HttpUtility.UrlEncode(entity_id);
            uri += "&ref_id=" + HttpUtility.UrlEncode(ref_id.ToString());
            if (tenant_id.HasValue) uri += "&tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.DeleteAsync(uri).Result;

            if (response.StatusCode == System.Net.HttpStatusCode.NoContent)
            {
                return;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
