﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    ///This client is used for invoking Events API endpoints.
    /// </summary>
    public class EventsClient : CelloHttpClient
    {
        public EventsClient(string baseAddress = "https://localhost/api/events/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }
        /// <summary>
        /// This method is used to invoke <c>'get_all_events'</c> endpoint.
        /// </summary>
        /// <param name="condition">Condition</param>
        /// <returns>collection of event details</returns>
        public IEnumerable<CelloSaaS.RestApiSDK.EventDTO> GetAllEvents(CelloSaaS.RestApiSDK.EventAduitSearchConditionDTO condition)
        {
            var uri = "get_all_events";

            var response = this.Client.PostAsJsonAsync(uri, condition).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<EventDTO[]>().Result;
                return entity;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_event'</c> endpoint.
        /// </summary>
        /// <param name="id">event identifier</param>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>specified event details</returns>
        public EventDTO GetEvent(Guid event_audit_id, Guid? tenant_id = null)
        {
            var uri = "get_event?event_id=" + event_audit_id.ToString();
            if (tenant_id.HasValue) uri += "&tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<EventDTO>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'register_event'</c> endpoint.
        /// </summary>
        /// <param name="data">required data for post an event</param>
        public void RegisterEvent(EventDTO data)
        {
            var uri = "register_event";
            var response = this.Client.PostAsJsonAsync(uri, data).Result;

            if (response.StatusCode == System.Net.HttpStatusCode.Created)
            {
                if (response.IsSuccessStatusCode)
                {
                    return;
                }
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
