﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;
using System.Web;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking Metering API endpoints.
    /// </summary>
    public class MeteringClient : CelloHttpClient
    {
        public MeteringClient(string baseAddress = "https://localhost/api/metering/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }
        /// <summary>
        ///  This method is used to invoke <c>'get_meter_amount'</c> endpoint.
        /// </summary>
        /// <param name="usage_code">usage code</param>
        /// <returns>used amount for the given usage</returns>
        public double GetMeterAmount(string usage_code)
        {
            var uri = "get_meter_amount?usage_code=" + HttpUtility.UrlEncode(usage_code);

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<double>().Result;
                return result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        ///  This method is used to invoke <c>'increment'</c> endpoint.
        /// </summary>
        /// <param name="usage_code">usage code</param>
        /// <param name="amount">amount</param>
        /// <returns><c>True</c>, if the metering count is incremented successfully, <c>false</c> otherwise</returns>
        public bool Increment(string usage_code, double amount)
        {
            var uri = "increment?usage_code=" + HttpUtility.UrlEncode(usage_code);
            uri += "&amount=" + amount;

            var response = this.Client.PostAsJsonAsync(uri, new { usage_code, amount }).Result;

            if (response.IsSuccessStatusCode)
            {
                return response.Content.ReadAsAsync<bool>().Result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'decrement'</c> endpoint.
        /// </summary>
        /// <param name="usage_code">usage code</param>
        /// <param name="amount">amount</param>
        public void Decrement(string usage_code, double amount)
        {
            var uri = "decrement?usage_code=" + HttpUtility.UrlEncode(usage_code);
            uri += "&amount=" + amount;

            var response = this.Client.DeleteAsync(uri).Result;

            if (response.StatusCode == System.Net.HttpStatusCode.NoContent)
            {
                return;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'search'</c> endpoint.
        /// </summary>
        /// <param name="condition">condition</param>
        /// <returns></returns>
        public MeteringSearchResultDTO Search(MeteringSearchConditionDTO condition)
        {
            var uri = "search";

            var response = this.Client.PostAsJsonAsync(uri, condition).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<MeteringSearchResultDTO>().Result;
                return result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'search_usage'</c> endpoint.
        /// </summary>
        /// <param name="condition">condition</param>
        /// <returns>search result</returns>
        public UsageSearchResultDTO SearchUsage(UsageSearchConditionDTO condition)
        {
            var uri = "search_usage";

            var response = this.Client.PostAsJsonAsync(uri, condition).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<UsageSearchResultDTO>().Result;
                return result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
