﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using Newtonsoft.Json;
using System.Net.Http;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking Packages API endpoints.
    /// </summary>
    public class PackagesClient : CelloHttpClient
    {
        public PackagesClient(string baseAddress = "https://localhost/api/packages/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }

        /// <summary>
        /// This method is used to invoke <c>'get_all_packages'</c> endpoint. 
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>collection of all packages</returns>
        public IEnumerable<PackageDTO> GetAllPackages(Guid? tenant_id = null)
        {
            var uri = "get_all_packages";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<PackageDTO[]>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_package'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        /// <param name="package_Id">package identifier.</param>
        /// <returns>specified package details</returns>
        public PackageDTO GetPackage(Guid package_Id)
        {
            var uri = "get_package?package_id=" + package_Id.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<PackageDTO>().Result;
                return entity;
            }

            if (response.StatusCode == System.Net.HttpStatusCode.NotFound)
            {
                return null;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

    }
}
