﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Net.Http;
using System.Net;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking Pickup lists API endpoints.
    /// </summary>
    public class PickuplistsClient : CelloHttpClient
    {
        public PickuplistsClient(string baseAddress = "https://localhost/api/pickuplists/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }
        /// <summary>
        /// This method is used to invoke <c>'get_pickup_lists'</c> endpoint.
        /// </summary>
        /// <returns>collection of all pickup lists</returns>
        public IEnumerable<CelloSaaS.RestApiSDK.PickuplistDTO> GetPickuplists()
        {
            var uri = "get_pickup_lists";

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<CelloSaaS.RestApiSDK.PickuplistDTO[]>().Result;
                return result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_pickup_list'</c> endpoint.
        /// </summary>
        /// <param name="pickuplist_id">pickup list identifier</param>
        /// <returns>specified pickup list details</returns>
        public CelloSaaS.RestApiSDK.PickuplistDTO GetPickuplist(Guid pickuplist_id)
        {
            var uri = "get_pickup_list?pickuplist_id=" + pickuplist_id;

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<PickuplistDTO>().Result;
                return entity;
            }
            if (response.StatusCode == HttpStatusCode.NotFound)
            {
                return null;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
