﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking ProductAnalytics API endpoints.
    /// </summary>
    public class ProductAnalyticsClient : CelloHttpClient
    {
        public ProductAnalyticsClient(string baseAddress = "https://localhost/api/productanalytics/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }
        /// <summary>
        /// This method is used to invoke <c>'get_all_product_analytics'</c> endpoint.
        /// </summary>
        /// <param name="condition">condition</param>
        /// <returns>collection of all product analytics</returns>
        public IEnumerable<CelloSaaS.RestApiSDK.ProductAnalyticsDTO> GetAllProductAnalytics(ProductAnalyticsSearchConditionDTO condition)
        {
            var uri = "get_all_product_analytics?a=1";
            var response = this.Client.PostAsJsonAsync(uri, condition).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<CelloSaaS.RestApiSDK.ProductAnalyticsDTO[]>().Result;
                return result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'post_product_analytics'</c> endpoint.
        /// </summary>
        /// <param name="data">data required for add a product analytics</param>
        public void PostProductAnalytics(ProductAnalyticsDTO data)
        {
            var uri = "post_product_analytics";

            var response = this.Client.PostAsJsonAsync(uri, data).Result;

            if (response.StatusCode == System.Net.HttpStatusCode.Created)
            {
                if (response.IsSuccessStatusCode)
                {
                    return;
                }
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
