﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking TenantLicenses API endpoints.
    /// </summary>
    public class TenantLicensesClient : CelloHttpClient
    {
        public TenantLicensesClient(string baseAddress = "https://localhost/api/tenantlicenses/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }
        /// <summary>
        /// This method is used to invoke <c>'get_all_tenant_licenses'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>collection all tenant licenses</returns>
        public IEnumerable<TenantLicenseDTO> GetAllTenantLicenses(Guid? tenant_id = null)
        {
            var uri = "get_all_tenant_licenses";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var licenses = response.Content.ReadAsAsync<IEnumerable<TenantLicenseDTO>>().Result;
                return licenses;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenant_license'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>specified tenant details</returns>
        public TenantLicenseDTO GetTenantLicense(Guid? tenant_id = null)
        {
            var uri = "get_tenant_license";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var license = response.Content.ReadAsAsync<TenantLicenseDTO>().Result;
                return license;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'put_tenant_license'</c> endpoint.
        /// </summary>
        /// <param name="entity">required data for update</param>
        public void PutTenantLicense(TenantLicenseDTO entity)
        {
            var uri = "put_tenant_license";

            var response = this.Client.PutAsJsonAsync(uri, entity).Result;

            if (response.IsSuccessStatusCode)
            {
                return;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'delete_tenant_license'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        public void DeleteTenantLicense(Guid? tenant_id = null)
        {
            var uri = "delete_tenant_license";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.DeleteAsync(uri).Result;

            if (response.StatusCode == System.Net.HttpStatusCode.NoContent)
            {
                return;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'activate_tenant_license'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        public void ActivateTenantLicense(Guid? tenant_id = null)
        {
            var uri = "activate_tenant_license";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.StatusCode == System.Net.HttpStatusCode.NoContent)
            {
                return;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'validate_tenant_license'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns><c>True</c>, if the tenant license is valid, <c>false</c> otherwise</returns>
        public bool ValidateTenantLicense(Guid? tenant_id = null)
        {
            var uri = "validate_tenant_license";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;
            if (response != null)
            {
                return response.Content.ReadAsAsync<bool>().Result;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_feature_by_moduleIds'</c> endpoint.
        /// </summary>
        /// <param name="modules_ids">module identifiers</param>
        /// <returns>collection of feature details</returns>
        public Dictionary<string, Dictionary<string, FeatureDTO>> GetFeatureByModuleIds(string[] modules_ids)
        {
            var uri = "get_feature_by_moduleIds";
            var response = this.Client.PostAsJsonAsync(uri, modules_ids).Result;
            if (response != null)
            {
                return response.Content.ReadAsAsync<Dictionary<string, Dictionary<string, FeatureDTO>>>().Result;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_usage_by_moduleIds'</c> endpoint.
        /// </summary>
        /// <param name="modules_ids">module identifiers</param>
        /// <returns>collection of usage details</returns>
        public Dictionary<string, Dictionary<string, UsageDTO>> GetUsageByModuleIds(string[] modules_ids)
        {
            var uri = "get_usage_by_moduleIds";
            var response = this.Client.PostAsJsonAsync(uri, modules_ids).Result;
            if (response != null)
            {
                return response.Content.ReadAsAsync<Dictionary<string, Dictionary<string, UsageDTO>>>().Result;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
