﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;
using System.Web;
namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking TenantRegistration API endpoints.
    /// </summary>
    public class TenantRegistrationClient : CelloHttpClient
    {
        public TenantRegistrationClient(string baseAddress = "https://localhost/api/tenantregistration/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }
        /// <summary>
        /// This method is used to invoke <c>'put_tenant_self_registration'</c> endpoint.
        /// </summary>
        /// <param name="data">required data for self registration</param>
        /// <returns>tenant identifier (GUID)</returns>
        public Guid PostSelfRegistratedTenant(TenantRegistrationDTO data)
        {
            var uri = "put_tenant_self_registration";
            var response = this.Client.PutAsJsonAsync(uri, data).Result;
            if (response.StatusCode == System.Net.HttpStatusCode.Created)
            {
                if (response.IsSuccessStatusCode)
                {
                    var entity = response.Content.ReadAsAsync<Guid>().Result;
                    return entity;
                }
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'is_priceplan_mapcode_Valid'</c> endpoint.
        /// </summary>
        /// <param name="planMapCode">plan map code</param>
        /// <param name="planMapPickupListId">plan map pickup list identifier</param>
        /// <param name="packageId">package identifier</param>
        /// <param name="parentTenantId">parent tenant identifier</param>
        /// <returns><c>True</c>, if the plan map code is valid, <c>false</c> otherwise</returns>
        public bool IsPricePlanMapCodeValid(Guid planMapCode, Guid planMapPickupListId, Guid packageId, Guid parentTenantId)
        {
            var uri = "is_priceplan_mapcode_Valid?planmap_code=" + planMapCode.ToString();
            uri += "&planmap_pickuplist_id=" + planMapPickupListId.ToString();
            uri += "&package_id=" + packageId.ToString();
            uri += "&parenttenant_id=" + parentTenantId.ToString();
            var response = this.Client.GetAsync(uri).Result;
            if (response.IsSuccessStatusCode)
            {
                return response.Content.ReadAsAsync<bool>().Result;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'packages_for_self_registration'</c> endpoint.
        /// </summary>
        /// <param name="defaultPricePlanPickupListId">default price plan pickup list identifier</param>
        /// <param name="frequency">billing frequency</param>
        /// <returns>collection of packages</returns>
        public IEnumerable<PackageDTO> GetPackagesForSelfRegistration(Guid defaultPricePlanPickupListId, BillingFrequency frequency)
        {
            var uri = "packages_for_self_registration?default_priceplan_pickuplist_id=" + defaultPricePlanPickupListId;
            uri += "&frequency=" + frequency;
            var response = Client.GetAsync(uri).Result;
            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<PackageDTO[]>().Result;
                return entity;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
