﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking TenantSettings API endpoints.
    /// </summary>
    public class TenantSettingsClient : CelloHttpClient
    {
        public TenantSettingsClient(string baseAddress = "https://localhost/api/tenantsettings/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenant_setting'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>
        /// <returns>collection of tenant settings</returns>
        public Dictionary<string, string> GetTenantSetting(Guid? tenant_id = null)
        {
            var uri = "get_tenant_setting";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var users = response.Content.ReadAsAsync<Dictionary<string, string>>().Result;
                return users;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenant_setting'</c> endpoint.
        /// </summary>
        /// <param name="tenant_Id">tenant identifier</param>
        /// <param name="attributeSet_Id">attribute set identifier</param>
        /// <returns></returns>
        public Dictionary<string, string> GetTenantSettings(string tenant_id, string attributeSet_id)
        {
            var uri = "get_tenant_setting?tenantId=" + tenant_id;
            uri += "&attributeSetId=" + attributeSet_id.ToString();

            var response = this.Client.GetAsync(uri).Result;
            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<Dictionary<string, string>>().Result;
                return result;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'put_tenant_setting'</c> endpoint.
        /// </summary>
        /// <param name="attributes">collection of attributes</param>
        /// <param name="tenant_id">tenant identifier</param>
        public void PutTenantSetting(Dictionary<string, string> attributes, Guid? tenant_id = null)
        {
            var uri = "put_tenant_setting";
            if (tenant_id.HasValue) uri += "?tenant_id=" + tenant_id.Value.ToString();

            var response = this.Client.PutAsJsonAsync(uri, attributes).Result;

            if (response.IsSuccessStatusCode)
            {
                return;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'delete_tenant_setting'</c> endpoint.
        /// </summary>
        /// <param name="attribute_id">attribute identifier</param>
        /// <param name="tenant_id">tenant identifier</param>
        public void DeleteTenantSetting(string attribute_id = null, Guid? tenant_id = null)
        {
            var uri = "delete_tenant_setting?a=1";
            if (tenant_id.HasValue) uri += "&tenant_id=" + tenant_id.Value.ToString();
            if (!string.IsNullOrEmpty(attribute_id)) uri += "&attribute_id=" + attribute_id;

            var response = this.Client.DeleteAsync(uri).Result;

            if (response.StatusCode == System.Net.HttpStatusCode.NoContent)
            {
                return;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }
    }
}
