﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// DataView DTO.
    /// </summary>
    public class DataViewDTO
    {
        /// <summary>
        /// Unique id of the Data View
        /// </summary>
        public int id { get; set; }

        /// <summary>
        /// Data View Identifier
        /// </summary>
        public string dataview_id { get; set; }

        /// <summary>
        /// Name of the data view
        /// </summary>
        public string name { get; set; }

        /// <summary>
        /// Description of the dataview
        /// </summary>
        public string description { get; set; }

        /// <summary>
        /// Unique Identifier of the entity.
        /// </summary>
        public string entity_id { get; set; }

        /// <summary>
        /// Unique identifier of the tenant.
        /// </summary>
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// Data view field details.
        /// </summary>
        public DataViewFieldDTO[] fields { get; set; }
    }

    /// <summary>
    /// DataViewField DTO
    /// </summary>
    public class DataViewFieldDTO
    {
        /// <summary>
        /// Unique identifier of the tenant.
        /// </summary>
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// Unique id of the data view field
        /// </summary>
        public int id { get; set; }

        /// <summary>
        /// Unique Field Identifier
        /// </summary>
        public string field_id { get; set; }

        /// <summary>
        /// Display name of the field.
        /// </summary>
        public string display_name { get; set; }

        /// <summary>
        /// Description of the field.
        /// </summary>
        public string description { get; set; }

        /// <summary>
        /// Unique identifier of the entity.
        /// </summary>
        public string entity_id { get; set; }

        /// <summary>
        /// Entity field metadata details
        /// </summary>
        public EntityFieldMetadataDTO field_metadata { get; set; }

        /// <summary>
        /// Field Type (ex:textbox)
        /// </summary>
        public string field_type { get; set; }

        /// <summary>
        /// Pickup list identifier of the field
        /// </summary>
        public string pickup_list_id { get; set; }

        /// <summary>
        /// A bool indicating field is visible or not.
        /// </summary>
        public bool is_visible { get; set; }

        /// <summary>
        ///  A bool indicating field is editable or not.
        /// </summary>
        public bool is_editable { get; set; }

        /// <summary>
        ///  A bool indicating field is hidden or not.
        /// </summary>
        public bool is_hidden { get; set; }

        /// <summary>
        ///  A bool indicating field is mandatory or not.
        /// </summary>
        public bool is_mandatory { get; set; }

        /// <summary>
        /// A bool indicating field is fixed or not.
        /// </summary>
        public bool is_fixed { get; set; }

        /// <summary>
        /// A bool indicating field is extended field or base field.
        /// </summary>
        ///<value>
        ///true:extended field
        ///false :base field
        ///</value>
        public bool is_extended_field { get; set; }

        /// <summary>
        /// Ordinal of the field
        /// </summary>
        public int ordinal { get; set; }

        /// <summary>
        /// Max length of the field.
        /// </summary>
        public int max_length { get; set; }

        /// <summary>
        /// Name of the dataview.
        /// </summary>
        public string viewdata_name { get; set; }

        /// <summary>
        /// Name of the property
        /// </summary>
        public string property_name { get; set; }

        /// <summary>
        /// Regular expression of the field
        /// </summary>
        public string regex { get; set; }

        /// <summary>
        ///  Regular expression error message of the field.It throw when field does not satisfy the regex.
        /// </summary>
        public string regex_error_message { get; set; }

        /// <summary>
        /// Required error message of the field.It is throw when field is empty or null.
        /// </summary>
        public string required_field_error_message { get; set; }

        /// <summary>
        /// Default value for this field
        /// </summary>
        public string default_value { get; set; }
    }
}
