﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This Data Transfer Object is used for email notification send.
    /// </summary>
    public class EmailDTO
    {
        /// <summary>
        /// notification identifier
        /// </summary>
        public Guid? notification_id { get; set; }
        /// <summary>
        /// notification name
        /// </summary>
        public string notification_name { get; set; }
        /// <summary>
        /// collection of placeholders
        /// </summary>
        public Dictionary<string, string> placeholders { get; set; }
        /// <summary>
        /// tenant identifier
        /// </summary>
        public Guid? tenant_id { get; set; }
        /// <summary>
        /// tracking identifier
        /// </summary>
        public string track_id { get; set; }
        /// <summary>
        /// to address
        /// </summary>
        [Required]
        public string to { get; set; }
        /// <summary>
        /// carbon copy
        /// </summary>
        public string cc { get; set; }
        /// <summary>
        /// blind carbon copy
        /// </summary>
        public string bcc { get; set; }
        //from address
        [EmailAddress]
        public string from { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object is used for the Email Audit.
    /// </summary>
    public class EmailAuditDTO : BaseDTO
    {   
        /// <summary>
        /// unique identifier
        /// </summary>
        public Guid id { get; set; }
        /// <summary>
        /// notification identifier
        /// </summary>
        public Guid notification_id { get; set; }
        /// <summary>
        /// notification name
        /// </summary>
        public string notification_name { get; set; }
        /// <summary>
        /// notification description
        /// </summary>
        public string notification_description { get; set; }
        /// <summary>
        /// notification category
        /// </summary>
        public string notification_category { get; set; }
        /// <summary>
        /// tenant identifier
        /// </summary>
        public Guid tenant_id { get; set; }
        /// <summary>
        /// subject
        /// </summary>
        public string subject { get; set; }
        /// <summary>
        /// content
        /// </summary>
        public string content { get; set; }
        /// <summary>
        /// from address
        /// </summary>
        public string from { get; set; }
        /// <summary>
        /// to address
        /// </summary>
        public string to { get; set; }
        /// <summary>
        /// carbon copy
        /// </summary>
        public string cc { get; set; }
        /// <summary>
        /// blind carbon copy
        /// </summary>
        public string bcc { get; set; }
        /// <summary>
        /// track identifier
        /// </summary>
        public string track_id { get; set; }
    }
    /// <summary>
    /// This Data transfer object is used for EmailAudit Search.
    /// </summary>
    public class EmailAuditSearchConditionDTO
    {
        /// <summary>
        /// unique identifier
        /// </summary>
        public Guid? tenant_id { get; set; }
        /// <summary>
        /// start date
        /// </summary>
        public DateTime? start_date { get; set; }
        /// <summary>
        /// end date
        /// </summary>
        public DateTime? end_date { get; set; }
        /// <summary>
        /// notification identifier
        /// </summary>
        public string notification_id { get; set; }
        /// <summary>
        /// notification name
        /// </summary>
        public string notification_name { get; set; }
        /// <summary>
        /// user identifier
        /// </summary>
        public Guid? user_id { get; set; }
        /// <summary>
        /// track identifier
        /// </summary>
        public string track_id { get; set; }
        /// <summary>
        /// page size
        /// </summary>
        public int page_size { get; set; }
        /// <summary>
        /// page number
        /// </summary>
        public int page_no { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object is used for the purpose of EmailAudit Search result.
    /// </summary>
    public class EmailAuditSearchResultDTO
    {
        /// <summary>
        /// collection of email audit details
        /// </summary>
        public IEnumerable<EmailAuditDTO> result { get; set; }
        /// <summary>
        /// total count
        /// </summary>
        public int total_count { get; set; }
        /// <summary>
        /// search condition
        /// </summary>
        public EmailAuditSearchConditionDTO search_condition { get; set; }
    }
}
