﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// EntityMetadata Data Transfer Object
    /// </summary>
    public class EntityMetadataDTO
    {
        /// <summary>
        /// Unique Identifier of the Entity
        /// </summary>
        public string entity_id { get; set; }

        /// <summary>
        /// Name of the Entity
        /// </summary>
        public string entity_name { get; set; }

        /// <summary>
        /// Unique Identifier of the tenant.
        /// </summary>
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// entity_schema
        /// </summary>
        public string entity_schema { get; set; }

        /// <summary>
        /// referenced_schema
        /// </summary>
        public string referenced_schema { get; set; }

        /// <summary>
        /// Name of the table to store the entity value.
        /// </summary>
        public string schema_table_name { get; set; }

        /// <summary>
        /// Name of the table to store the entity's extended field.
        /// </summary>
        public string extended_schema_table_name { get; set; }

        /// <summary>
        /// collection of privileges which this entity belongs to.
        /// </summary>
        public string[] privileges { get; set; }

        /// <summary>
        /// collection of features which this entity belongs to.
        /// </summary>
        public string[] features { get; set; }

        /// <summary>
        /// collection entity field meta data.
        /// </summary>
        public EntityFieldMetadataDTO[] fields { get; set; }

        /// <summary>
        /// Name of primary key column of the entity
        /// </summary>
        public string primary_key { get; set; }

        /// <summary>
        /// Connection string name of the entity extended table
        /// </summary>
        public string extn_schema_table_connection_name { get; set; }

        /// <summary>
        /// A bool indicating this entity is tenant based extension or not.
        /// </summary>
        public bool is_extensible { get; set; }

        /// <summary>
        /// Connection string name of the entity table
        /// </summary>
        public string entity_schema_table_connection_string_name { get; set; }

        /// <summary>
        /// Name of the display column.
        /// </summary>
        public string display_column_name { get; set; }

        /// <summary>
        /// Name of the tenant id column.
        /// </summary>
        public string tenant_id_column_name { get; set; }
    }

    /// <summary>
    /// EntityFieldMetadata Data Transfer Objecct
    /// </summary>
    public class EntityFieldMetadataDTO
    {
        /// <summary>
        /// Unique id of the field.
        /// </summary>
        public string field_id { get; set; }

        /// <summary>
        /// Unique id of the entity
        /// </summary>
        public string entity_id { get; set; }

        /// <summary>
        /// Unique identitifer of the tenant id
        /// </summary>
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// Name of the Field.
        /// </summary>
        public string field_name { get; set; }

        /// <summary>
        /// entity field type identifier
        /// </summary>
        public string type_id { get; set; }

        /// <summary>
        /// regular expression string for field
        /// </summary>
        public string validation_regex { get; set; }

        /// <summary>
        /// A bool indicating this field is unique or not.
        /// </summary>
        public bool is_unique { get; set; }

        /// <summary>
        /// Max length of the field
        /// </summary>
        public int length { get; set; }

        /// <summary>
        /// Identifier of the pickup list.
        /// </summary>
        public string pickup_list_id { get; set; }

        /// <summary>
        /// Name of the column which is mapped to this entity field
        /// </summary>
        public string mapped_column_name { get; set; }

        /// <summary>
        /// Reference table name
        /// </summary>
        public string reference_table_name { get; set; }

        /// <summary>
        /// Reference entity id
        /// </summary>
        public string reference_entity_id { get; set; }

        /// <summary>
        /// Name of the reference  column 
        /// </summary>
        public string reference_column_name { get; set; }

        /// <summary>
        /// Entity extn Column Details Id 
        /// </summary>
        public string extn_columndetails_id { get; set; }

        /// <summary>
        /// Entity Extended Column Details.
        /// </summary>
        public EntityExtendedColumnDetailsDTO column_details { get; set; }

        /// <summary>
        /// activated date time
        /// </summary>
        public DateTime activated_date { get; set; }

        /// <summary>
        /// deactivate time
        /// </summary>
        public DateTime deactivated_date { get; set; }
    }

    /// <summary>
    /// EntityExtendedColumnDetails DTO
    /// </summary>
    public class EntityExtendedColumnDetailsDTO
    {
        /// <summary>
        /// Unique Id of the extended field.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// Name of the column
        /// </summary>
        public string column_name { get; set; }
    }

    /// <summary>
    /// CustomData DTO
    /// </summary>
    public class CustomDataDTO
    {
        /// <summary>
        /// Unique id of the entity
        /// </summary>
        [Required]
        public string entity_id { get; set; }

        /// <summary>
        /// Reference identifier
        /// </summary>
        [Required]
        public string ref_id { get; set; }

        /// <summary>
        /// Unique identifier of the tenant
        /// </summary>
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// Field details
        /// </summary>
        [Required]
        public FieldDataDTO[] data { get; set; }
    }

    /// <summary>
    /// FieldData DTO
    /// </summary>
    public class FieldDataDTO
    {
        /// <summary>
        /// Unique id of the field
        /// </summary>
        [Required]
        public string field_id { get; set; }

        /// <summary>
        /// value of the Field
        /// </summary>
        [Required]
        public string value { get; set; }
    }
}
