﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// Represents the Event details that is
    /// passed when registering event with EventRegister
    /// </summary>
    public class EventDTO
    {
        /// <summary>
        /// Indicates Unique identifier
        /// </summary>
        public Guid id { get; set; }
        /// <summary>
        /// Indicates Event Identifier
        /// </summary>
        public Guid event_id { get; set; }
        /// <summary>
        /// Indicates Name of the event
        /// </summary>
        public string event_name { get; set; }
        /// <summary>
        /// Indicates description about the event
        /// </summary>
        public string event_description { get; set; }
        /// <summary>
        /// Indicates the user identifier
        /// </summary>
        public Guid user_id { get; set; }
        /// <summary>
        /// Indicates the template identifier
        /// </summary>
        public string template_id { get; set; }
        /// <summary>
        /// Indicates the transformed description
        /// </summary>
        public string transformed_description { get; set; }
        /// <summary>
        /// Indicates the XML parameter
        /// </summary>
        public string parameter_xml { get; set; }
        /// <summary>
        /// Indicates the tenant identifier
        /// </summary>
        public Guid? tenant_id { get; set; }
        /// <summary>
        /// Indicates the subject identifier
        /// </summary>
        public string subject_id { get; set; }
        /// <summary>
        /// Indicates the subject type
        /// </summary>
        public string subject_type { get; set; }
        /// <summary>
        /// Indicates the subject type
        /// </summary>
        public string subject_xml_value { get; set; }
        /// <summary>
        /// Indicates the subject xml value
        /// </summary>
        public string target_id { get; set; }
        /// <summary>
        /// Indicates the target identifier
        /// </summary>
        public string target_type { get; set; }
        /// <summary>
        /// Indicates the target type
        /// </summary>
        public string target_xml_value { get; set; }
        /// <summary>
        /// Indicate the context identifier
        /// </summary>
        public string context_id { get; set; }
        /// <summary>
        /// Indicates the type of context
        /// </summary>
        public string context_type { get; set; }
        /// <summary>
        /// Indicates the context XML value
        /// </summary>
        public string context_xml_value { get; set; }
        /// <summary>
        /// Indicates the event created date
        /// </summary>
        public DateTime created_on { get; set; }
        /// <summary>
        /// Indicates the Identifier who created the event
        /// </summary>
        public Guid created_by { get; set; }
        /// <summary>
        /// Indicates True of False
        /// </summary>
        public bool status { get; set; }
        /// <summary>
        /// Indicates the event execution priority
        /// </summary>
        public EventPriorityDTO priority { get; set; }

        public EventStatusDTO event_job_execution_status { get; set; }
    }

    /// <summary>
    /// Describes Event execution priority
    /// </summary>
    public enum EventPriorityDTO
    {
        /// <summary>
        /// Execution priority is low
        /// </summary>
        Low,
        /// <summary>
        /// Execution priority is above low and below high
        /// </summary>
        Medium,
        /// <summary>
        /// Highest execution priority
        /// </summary>
        High
    }

    /// <summary>
    /// Describes Event execution status
    /// </summary>
    public enum EventStatusDTO
    {
        /// <summary>
        /// New denote newly entered entry not yet processed.
        /// </summary>       
        New,

        /// <summary>
        /// Queued denote this is added to some queue and waiting to be processed.
        /// </summary>        
        Queued,

        /// <summary>
        /// InProgress denotes this is send to the dispatcher (In-Execution).
        /// </summary>        
        InProgress,

        /// <summary>
        /// Success denotes this is processed successfully.
        /// </summary>        
        Success,

        /// <summary>
        /// Failure denotes failure to execute.
        /// </summary>        
        Failure
    }
    /// <summary>
    /// This Data Transfer Object is used for the purpose of passing values for getting the specified event details
    /// </summary>
    public class EventAduitSearchConditionDTO
    {
        /// <summary>
        /// Indicates the array of event identifiers
        /// </summary>
        public string[] eventIds { get; set; }
        /// <summary>
        /// Indicates the user identifier
        /// </summary>
        public string userId { get; set; }
        /// <summary>
        /// Indicates the activityLog Identifier
        /// </summary>
        public string activityLogId { get; set; }
        /// <summary>
        /// Indicates the log time
        /// </summary>
        public DateTime? logTime { get; set; }
        /// <summary>
        /// Indicates the to log time
        /// </summary>
        public DateTime? toLogTime { get; set; }
        /// <summary>
        /// Indicates the from log time
        /// </summary>
        public DateTime? fromLogTime { get; set; }
        /// <summary>
        /// Indicates the tenant identifier
        /// </summary>
        public string tenantId { get; set; }
        /// <summary>
        /// Indicates the event identifier
        /// </summary>
        public string eventId { get; set; }
        /// <summary>
        /// Indicates the activity event log identifier
        /// </summary>
        public string activityEventLogId { get; set; }
        /// <summary>
        /// Indicates the status of the event
        /// </summary>
        public string eventStatus { get; set; }
        /// <summary>
        /// Indicates the subject identifier
        /// </summary>
        public string subjectId { get; set; }
        /// <summary>
        /// Indicates the type of the subject
        /// </summary>
        public string subjectType { get; set; }
        /// <summary>
        /// Indicates the type of the target
        /// </summary>
        public string targetType { get; set; }
        /// <summary>
        /// Indicates the target identifier
        /// </summary>
        public string targetId { get; set; }
        /// <summary>
        /// Indicates the context identifier
        /// </summary>
        public string contextId { get; set; }
        /// <summary>
        /// Indicates the type of the context
        /// </summary>
        public string contextType { get; set; }
        /// <summary>
        /// Indicates the locale identifier
        /// </summary>
        public string localeId { get; set; }
        /// <summary>
        /// Indicates the condition
        /// </summary>
        public string condition {  get  {return string.Empty; } set{} }
        /// <summary>
        /// Indicates the sort string
        /// </summary>
        public string sortString { get; set; }
        /// <summary>
        /// Indicates the sort expression
        /// </summary>
        public string sortExpression { get; set; }
        /// <summary>
        /// Indicates the settotalcount
        /// </summary>
        public bool setTotalCount { get; set; }
        /// <summary>
        /// Indicates the record start
        /// </summary>
        public int recordStart { get; set; }
        /// <summary>
        /// Indicates the record end
        /// </summary>
        public int recordEnd { get; set; }
        /// <summary>
        /// Indicates the page number
        /// </summary>
        public int pageNo { get; set; }
        /// <summary>
        /// Indicates the page size
        /// </summary>
        public int pageSize { get; set; }
    }    
}
