﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{
    public class InvoiceDTO
    {
        public Guid Id { get; set; }
        public Guid TenantId { get; set; }
        public Guid ParentTenantId { get; set; }
        public Guid PackageId { get; set; }
        public Guid PricePlanId { get; set; }
        public InvoiceTypeDTO InvoiceType { get { return (InvoiceTypeDTO)InvoiceTypeValue; } set { InvoiceTypeValue = (int)value; } }
        public int InvoiceTypeValue { get; set; }
        public string InvoiceNo { get; set; }
        public DateTime InvoiceDate { get; set; }
        public DateTime StartDate { get; set; }
        public DateTime EndDate { get; set; }
        public DateTime NextInvoiceDate { get; set; }
        public DateTime DueDate { get; set; }
        public Int32 InvoiceStatusValue { get; set; }
        public InvoiceStatusDTO InvoiceStatus { get { return (InvoiceStatusDTO)InvoiceStatusValue; } set { InvoiceStatusValue = (int)value; } }
        public int BillFrequencyValue { get; set; }
        public BillingFrequency BillFrequency { get { return (BillingFrequency)BillFrequencyValue; } set { BillFrequencyValue = (int)value; } }
        public double PendingBalance { get; set; }
        public string Memo { get; set; }
        public ICollection<InvoiceLineItemDTO> LineItems { get; set; }
        public ICollection<InvoicePaymentDTO> Payments { get; set; }
        public DateTime CreatedOn { get; set; }
        public string CreatedBy { get; set; }
        public DateTime? UpdatedOn { get; set; }

    }
    public class InvoiceLineItemDTO
    {
        public Guid Id { get; set; }
        public Guid InvoiceId { get; set; }
        public Guid TenantId { get; set; }
        public Guid ParentTenantId { get; set; }
        public string Name { get; set; }
        public string Description { get; set; }
        public string Type { get; set; }
        public string Usage { get; set; }
        public double Amount { get; set; }
        public bool IsUserAdded { get; set; }
    }
    public class InvoicePaymentDTO
    {
        public Guid Id { get; set; }
        public Guid InvoiceId { get; set; }
        public Guid TenantId { get; set; }
        public Guid ParentTenantId { get; set; }
        public int PaymentModeValue { get; set; }
        public PaymentModeDTO PaymentMode { get { return (PaymentModeDTO)PaymentModeValue; } set { PaymentModeValue = (int)value; } }
        public double PaidAmount { get; set; }
        public DateTime PaymentDate { get; set; }
        public string CreatedBy { get; set; }
        public string Notes { get; set; }
    }
    public enum InvoiceTypeDTO { regular, adjustment }
    public enum InvoiceStatusDTO { current, paid, unpaid, other }
    public enum PaymentModeDTO { card, cash, manual, credit }

    public class InvoiceSearchConditionDTO
    {
        public Guid[] TenantIds { get; set; }
        public Guid[] InvoiceIds { get; set; }
        public InvoiceStatusDTO? InvoiceStatus { get; set; }
        public DateTime? FromInvoiceDate { get; set; }
        public DateTime? ToInvoiceDate { get; set; }
        public DateTime? StartDate { get; set; }
        public DateTime? EndDate { get; set; }
        public DateTime? DueDate { get; set; }
        public int PageNo { get; set; }
        public int PageSize { get; set; }
        public string SortField { get; set; }
        public string SortDirection { get; set; }
        public BillingFrequency? BillFrequency { get; set; }
    }
    public class InvoiceSearchResultDTO
    {
        public Int32 TotalCount { get; set; }
        public IEnumerable<InvoiceDTO> Items { get; set; }
    }

}
