﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    ///  Metering Log model
    /// </summary>
    public class MeteringLogDTO
    {
        /// <summary>
        ///   Gets or sets the Metering Log Identifeir
        /// </summary>
        public Guid id { get; set; }

        /// <summary>
        ///   Gets or sets the Tenant Identifeir
        /// </summary>
        public Guid tenant_id { get; set; }

        /// <summary>
        ///   Gets or sets usag code
        /// </summary>
        public string usage_code { get; set; }

        /// <summary>
        ///   Gets or sets the usage amount
        /// </summary>
        public double amount { get; set; }

        /// <summary>
        ///   Gets or sets the identifier of user who created log
        /// </summary>
        public Guid user_id { get; set; }

        /// <summary>
        ///   Gets or sets the date time of log created
        /// </summary>
        public DateTime logged_date { get; set; }
    }

    /// <summary>
    /// Metering search condition class
    /// </summary>
    public class MeteringSearchConditionDTO
    {
        /// <summary>
        ///   Gets or sets the Tenant Identifier
        /// </summary>
        public Guid? tenant_id { get; set; }

        /// <summary>
        ///   Gets or sets usag code
        /// </summary>
        public string usage_code { get; set; }

        /// <summary>
        ///   Gets or sets the identifier of user who created log
        /// </summary>
        public Guid? user_id { get; set; }

        /// <summary>
        ///   Gets or sets the Start Date
        /// </summary>
        public DateTime? start_date { get; set; }

        /// <summary>
        ///   Gets or sets the Start Date
        /// </summary>
        public DateTime? end_date { get; set; }

        /// <summary>
        /// Page number to fetch
        /// </summary>
        public int page { get; set; }

        /// <summary>
        /// Page Size to fetch
        /// </summary>
        public int page_size { get; set; }
    }

    /// <summary>
    /// Metering search result dto
    /// </summary>
    public class MeteringSearchResultDTO
    {
        /// <summary>
        /// Contains the metering log details
        /// </summary>
        public IEnumerable<MeteringLogDTO> result { get; set; }

        /// <summary>
        /// Contains the total count
        /// </summary>
        public int total_count { get; set; }

        /// <summary>
        /// Contains the search condition
        /// </summary>
        public MeteringSearchConditionDTO condition { get; set; }
    }

    /// <summary>
    /// License usage search condition
    /// </summary>
    public class UsageSearchConditionDTO
    {
        /// <summary>
        /// Gets or sets the tenant id.
        /// </summary>
        /// <value>
        /// The tenant id.
        /// </value>
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// Gets or sets the usage codes.
        /// </summary>
        /// <value>
        /// The usage codes.
        /// </value>
        public string[] usage_codes { get; set; }

        /// <summary>
        /// Gets or sets the module codes.
        /// </summary>
        /// <value>
        /// The module codes.
        /// </value>
        public string[] module_codes { get; set; }
    }

    /// <summary>
    /// License usage search result
    /// </summary>
    public class UsageSearchResultDTO
    {
        /// <summary>
        /// Gets or sets the result.
        /// </summary>
        /// <value>
        /// The result.
        /// </value>
        public IEnumerable<UsageDetailDTO> result { get; set; }

        /// <summary>
        /// Gets or sets the total count.
        /// </summary>
        /// <value>
        /// The total count.
        /// </value>
        public int total_count { get; set; }

        /// <summary>
        /// Usage search condition details
        /// </summary>
        public UsageSearchConditionDTO condition { get; set; }
    }

    /// <summary>
    /// Usage Details
    /// </summary>
    public class UsageDetailDTO
    {
        /// <summary>
        /// Gets or sets the tenant id.
        /// </summary>
        /// <value>
        /// The tenant id.
        /// </value>
        public Guid tenant_id { get; set; }

        /// <summary>
        /// Gets or sets the usage code.
        /// </summary>
        /// <value>
        /// The usage code.
        /// </value>
        public string usage_code { get; set; }

        /// <summary>
        /// Gets or sets the name of the usage.
        /// </summary>
        /// <value>
        /// The name of the usage.
        /// </value>
        public string usage_name { get; set; }

        /// <summary>
        /// Gets or sets the usage type
        /// </summary>
        public string usage_type { get; set; }

        /// <summary>
        /// Gets or sets usage alert threshold in percentage
        /// </summary>
        public double threshold { get; set; }

        /// <summary>
        /// Gets or sets the module code.
        /// </summary>
        /// <value>
        /// The module code.
        /// </value>
        public string module_code { get; set; }

        /// <summary>
        /// Gets or sets the amount.
        /// </summary>
        /// <value>
        /// The amount.
        /// </value>
        public double amount { get; set; }

        /// <summary>
        /// If True generates adjustment invoice and inc/dec. on prepaid billing mode.
        /// </summary>
        public bool can_generate_invoice { get; set; }
    }
}
