﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{

    public class PackageDTO
    {
        /// <summary>
        /// Package Unique Identifier.
        /// </summary>
        public Guid id { get; set; }

        /// <summary>
        /// Tenant unique identifier (mandatory).
        /// </summary>
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// Unique Package Name (mandatory).
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string name { get; set; }

        /// <summary>
        /// Package Description (mandatory)
        /// </summary>
        [Required]
        [MaxLength(1000)]
        public string description { get; set; }

        /// <summary>
        /// Trail days for package. If trail days is set 0 , it is non trail package.
        /// </summary>
        [Required]
        [Range(0, int.MaxValue, ErrorMessage = "trial days must be a positive integers")]
        public int trial_days { get; set; }

        /// <summary>
        /// Price plans
        /// </summary>
        public PricePlanDTO[] plans { get; set; }

        /// <summary>
        /// Gets or Sets Modules (mandatory)
        /// </summary>
        [Required(ErrorMessage = "Package have at least one module.")]
        public ModuleDTO[] modules { get; set; }

        /// <summary>
        /// Gets or Sets Features (mandatory).
        /// </summary>
        [Required(ErrorMessage = "Package have at least one features.")]
        public FeatureDTO[] features { get; set; }

        /// <summary>
        /// Gets or Sets usages.
        /// </summary>
        public UsageDTO[] usages { get; set; }
    }

    /// <summary>
    /// This Data Transfer Object is used for Price Plan.
    /// </summary>
    public class PricePlanDTO
    {
        /// <summary>
        /// price plan unique identifier
        /// </summary>
        public Guid id { get; set; }
        /// <summary>
        /// tennat unique identifier
        /// </summary>
        public Guid? tenant_id { get; set; }
        /// <summary>
        /// name of the price plan
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string name { get; set; }
        /// <summary>
        /// description about the price plan
        /// </summary>
        [Required]
        [MaxLength(1000)]
        public string description { get; set; }
        /// <summary>
        /// billing frequency
        /// </summary>
        public BillingFrequency frequency { get; set; }
        /// <summary>
        /// bas eprice
        /// </summary>
        public double base_price { get; set; }
        /// <summary>
        /// array of items
        /// </summary>
        public PriceTableDTO[] items { get; set; }
    }

    /// <summary>
    /// This Data Transfer Object is used for the price table.
    /// </summary>
    public class PriceTableDTO
    {
        /// <summary>
        /// price table unique identifier
        /// </summary>
        public Guid id { get; set; }
        /// <summary>
        /// name of the unique identifier
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string name { get; set; }
        /// <summary>
        /// description about the price table
        /// </summary>
        [Required]
        [MaxLength(1000)]
        public string description { get; set; }
        /// <summary>
        /// slab variable
        /// </summary>
        [Required]
        [MaxLength(100)]
        public string slab_variable { get; set; }
        /// <summary>
        /// actedon variable
        /// </summary>
        [Required]
        [MaxLength(100)]
        public string actedon_variable { get; set; }
        /// <summary>
        /// calculate type details
        /// </summary>
        public CalculationType calculation_type { get; set; }
        /// <summary>
        /// factore type details
        /// </summary>
        public FactorType factor_type { get; set; }
        /// <summary>
        /// array of slabs
        /// </summary>
        [Required]
        public SlabDTO[] slabs { get; set; }
    }
    /// <summary>
    /// This enumeration is used for billing frequency
    /// </summary>
    public enum BillingFrequency { Monthly, Quarterly, Semiannual, Annual }
    /// <summary>
    /// This enumeration is used for calculation type
    /// </summary>
    public enum CalculationType { thresholdpricing, steppricing }
    /// <summary>
    /// This enum
    /// </summary>
    public enum FactorType { flat, each }
    /// <summary>
    /// This Data Transfer Object is used for slab details
    /// </summary>
    public class SlabDTO
    {
        /// <summary>
        /// slab unique identifier
        /// </summary>
        public Guid id { get; set; }
        /// <summary>
        /// start
        /// </summary>
        public double start { get; set; }
        /// <summary>
        /// end
        /// </summary>
        public double? end { get; set; }
        /// <summary>
        /// Factorial
        /// </summary>
        public double factor { get; set; }
    }
}
