﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{   
    /// <summary>
    /// This Data Transfer Object is used for the Product Analytics
    /// </summary>
    public class ProductAnalyticsSearchConditionDTO
    {
        /// <summary>
        /// Gets or sets the tenant id.
        /// </summary>
        /// <value>The tenant id.</value>
        public Guid tenant_id { get; set; }
        /// <summary>
        /// Gets or sets the user id.
        /// </summary>
        /// <value>The user id.</value>
        public Guid user_id { get; set; }
        /// <summary>
        /// Gets or sets the user name.
        /// </summary>
        /// <value>The user name.</value>
        public string user_name { get; set; }
        /// <summary>
        /// Gets or sets the name of the controller.
        /// </summary>
        /// <value>The name of the controller.</value>
        public string controller_name { get; set; }
        /// <summary>
        /// Gets or sets the name of the action.
        /// </summary>
        /// <value>The name of the action.</value>
        public string action_name { get; set; }
        /// <summary>
        /// Gets or sets from date.
        /// </summary>
        /// <value>From date.</value>
        public DateTime? from_date { get; set; }
        /// <summary>
        /// Gets or sets to date.
        /// </summary>
        /// <value>To date.</value>
        public DateTime? to_date { get; set; }
        /// <summary>
        /// Gets or sets the offset.
        /// </summary>
        /// <value>The offset.</value>
        public int offset { get; set; }
        /// <summary>
        /// Gets or sets the response time of the page.
        /// </summary>
        /// <value>The response time of the page.</value>
        public int response_time { get; set; }
        /// <summary>
        /// Gets or sets the response length of the page.
        /// </summary>
        /// <value>The response length of the page.</value>
        public int response_length { get; set; }
        /// <summary>
        /// Gets or sets the response time of the page.
        /// </summary>
        /// <value>The response time of the page.</value>
        public bool has_error { get; set; }
        /// <summary>
        /// Gets or sets the size of the page.
        /// </summary>
        /// <value>The size of the page.</value>
        public int page_size { get; set; }
        /// <summary>
        /// Gets or sets the page number.
        /// </summary>
        /// <value>The size of the page.</value>
        public int page_no { get; set; }
        /// <summary>
        /// Gets or sets the sort string.
        /// </summary>
        /// <value>The sort string.</value>
        public string sort_string { get; set; }
        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        /// <value>The sort direction.</value>
        public string sort_direction { get; set; }
        /// <summary>
        /// Gets or sets the user IP.
        /// </summary>
        /// <value>The user IP.</value>
        public string user_ip { get; set; }
        /// <summary>
        /// Gets or sets the total count.
        /// </summary>
        /// <value>The total count.</value>
        public int total_count { get; set; }
        /// <summary>
        /// Gets or sets the average response time.
        /// </summary>
        /// <value>The total count.</value>
        public decimal avg_response_time { get; set; }
        /// <summary>
        /// Gets or sets the average page size.
        /// </summary>
        /// <value>The total count.</value>
        public decimal avg_page_size { get; set; }
        /// <summary>
        /// Gets or sets the session id.
        /// </summary>
        /// <value>
        /// The session id.
        /// </value>
        public string session_id { get; set; }
    }
    public class ProductAnalyticsDTO
    {
        /// <summary>
        /// unique identifer
        /// </summary>
        public Guid id { get; set; }
        /// <summary>
        /// user identifier
        /// </summary>
        public Guid user_id { get; set; }
        /// <summary>
        /// user name
        /// </summary>
        public string user_name { get; set; }
        /// <summary>
        /// tenant identifier
        /// </summary>
        public Guid? tenant_id { get; set; }
        /// <summary>
        /// tenant name
        /// </summary>
        public string tenant_name { get; set; }
        /// <summary>
        /// controller name
        /// </summary>
        public string controller_name { get; set; }
        /// <summary>
        /// action name
        /// </summary>
        public string action_name { get; set; }
        /// <summary>
        /// time stamp
        /// </summary>
        public DateTime time_stamp { get; set; }
        /// <summary>
        /// browser details
        /// </summary>
        public string browser_details { get; set; }
        /// <summary>
        /// machine name
        /// </summary>
        public string machine_name { get; set; }
        /// <summary>
        /// request method
        /// </summary>
        public string request_method { get; set; }
        /// <summary>
        /// request mode
        /// </summary>
        public string request_mode { get; set; }
        /// <summary>
        /// exception details
        /// </summary>
        public string exception_details { get; set; }
        /// <summary>
        /// current url
        /// </summary>
        [DataType(DataType.Url)]
        [Required]
        public Uri current_url { get; set; }
        /// <summary>
        /// response status code
        /// </summary>
        public int response_status_code { get; set; }
        /// <summary>
        /// response status description
        /// </summary>
        public string response_status_description { get; set; }
        /// <summary>
        /// created on
        /// </summary>
        public DateTime created_on { get; set; }
        /// <summary>
        /// created by
        /// </summary>
        public Guid created_by { get; set; }
        /// <summary>
        /// status
        /// </summary>
        public bool status { get; set; }
        /// <summary>
        /// ip address
        /// </summary>
        public string ip_address { get; set; }
        /// <summary>
        /// session id
        /// </summary>
        public string session_id { get; set; }
        /// <summary>
        /// response lenght
        /// </summary>
        public long response_length { get; set; }
    }
}
