﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This Data Transfer Object contains the License details of tenant.
    /// </summary>
    public class TenantLicenseDTO
    {
        /// <summary>
        /// Unique identifier of the tenant license.
        /// </summary>
        public Guid id { get; set; }

        /// <summary>
        /// Unique identifier of the tenant.
        /// </summary>
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// Name of Package
        /// </summary>
        public string package_name { get; set; }

        /// <summary>
        /// Unique identifier of the package 
        /// </summary>
        public Guid package_id { get; set; }

        /// <summary>
        /// Unique identifier of the price plan 
        /// </summary>
        public Guid? priceplan_id { get; set; }

        /// <summary>
        /// Start date of the tenant license.
        /// </summary>
        public DateTime validity_start { get; set; }

        /// <summary>
        /// End date of the tenant license.
        /// </summary>
        public DateTime? validity_end { get; set; }

        /// <summary>
        /// End date of the trail.
        /// </summary>
        public DateTime? trial_end_date { get; set; }

        /// <summary>
        /// array of serices
        /// </summary>
        public ServiceDTO[] services { get; set; }

        /// <summary>
        /// array of modules
        /// </summary>
        public ModuleDTO[] modules { get; set; }

        /// <summary>
        /// array of features
        /// </summary>
        public FeatureDTO[] features { get; set; }

        /// <summary>
        /// array of usages
        /// </summary>
        public UsageDTO[] usages { get; set; }

        /// <summary>
        /// array of privileges
        /// </summary>
        public PrivilegeDTO[] privileges { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object contains Privilege details
    /// </summary>
    public class PrivilegeDTO
    {
        /// <summary>
        /// code of the privilege
        /// </summary>
        public string code { get; set; }
        /// <summary>
        /// name of the privilege
        /// </summary>
        public string name { get; set; }
        /// <summary>
        /// description about the privilege
        /// </summary>
        public string description { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object contains the Service details
    /// </summary>
    public class ServiceDTO
    {
        /// <summary>
        /// service code
        /// </summary>
        public string code { get; set; }
        /// <summary>
        /// service name
        /// </summary>
        public string name { get; set; }
        /// <summary>
        /// service description
        /// </summary>
        public string description { get; set; }
        /// <summary>
        /// array of modules
        /// </summary>
        public ModuleDTO[] modules { get; set; }
        /// <summary>
        /// array of features
        /// </summary>
        public FeatureDTO[] features { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object contains the Module details
    /// </summary>
    public class ModuleDTO
    {
        /// <summary>
        /// module code
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string code { get; set; }
        /// <summary>
        /// module name
        /// </summary>
        public string name { get; set; }
        /// <summary>
        /// service code
        /// </summary>
        public string service_code { get; set; }
        /// <summary>
        /// array of features
        /// </summary>
        public FeatureDTO[] features { get; set; }
        /// <summary>
        /// array of usage
        /// </summary>
        public UsageDTO[] usages { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object contains Feature details
    /// </summary>
    public class FeatureDTO
    {
        /// <summary>
        /// feature code
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string code { get; set; }
        /// <summary>
        /// feature name
        /// </summary>
        public string name { get; set; }
        /// <summary>
        /// module code
        /// </summary>
        public string module_code { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object contains Usage details
    /// </summary>
    public class UsageDTO
    {   
        /// <summary>
        /// Usage code
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string code { get; set; }
        /// <summary>
        /// usage name
        /// </summary>
        public string name { get; set; }
        /// <summary>
        /// type code
        /// </summary>
        public string type_code { get; set; }
        /// <summary>
        /// type name
        /// </summary>
        public string type_name { get; set; }
        /// <summary>
        /// module code
        /// </summary>
        public string module_code { get; set; }
        /// <summary>
        /// threshold
        /// </summary>
        public double threshold { get; set; }
        /// <summary>
        /// indicate true or false for generate invoice
        /// </summary>
        public bool can_generate_invoice { get; set; }
        /// <summary>
        /// allow over usage
        /// </summary>
        public bool allow_over_usage { get; set; }
        /// <summary>
        /// maximum capacity
        /// </summary>
        public double max_capacity { get; set; }
        /// <summary>
        /// minimum capacity
        /// </summary>
        public double min_capacity { get; set; }
    }
}
