﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This Data Transfer Object is used for tenant registration
    /// </summary>
    public class TenantRegistrationDTO
    {
        /// <summary>
        /// name of the tenant
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string friendly_name { get; set; }

        /// <summary>
        /// tenant code
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string friendly_code { get; set; }

        /// <summary>
        /// tenant username
        /// </summary>
        [MaxLength(200)]
        public string user_name { get; set; }

        /// <summary>
        /// tennat password
        /// </summary>
        [MaxLength(200)]
        [DataType(DataType.Password)]
        public string password { get; set; }

        /// <summary>
        /// tenant administrator first name
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string first_name { get; set; }

        /// <summary>
        /// tenant administrator last name
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string last_name { get; set; }

        /// <summary>
        /// contact number of the tenant
        /// </summary>
        [Required]
        [Phone]
        public string contact_number { get; set; }

        /// <summary>
        /// email id of the tenant administrator
        /// </summary>
        [MaxLength(200)]
        [EmailAddress]
        public string email_id { get; set; }

        /// <summary>
        /// URL of the tenant
        /// </summary>
        [DataType(DataType.Url)]
        public string url { get; set; }

        /// <summary>
        /// website of the tenant
        /// </summary>
        [DataType(DataType.Url)]
        public string website { get; set; }

        /// <summary>
        /// package identifier
        /// </summary>
        [Required]
        public Guid package_id { get; set; }

        /// <summary>
        /// billing frequency
        /// </summary>
        [Required]
        public BillingFrequency billing_frequency { get; set; }

        /// <summary>
        /// additional fields
        /// </summary>
        public Dictionary<string, string> additional_fields { get; set; }

        /// <summary>
        /// planmap code
        /// </summary>
        public string plan_mapcode { get; set; }

        /// <summary>
        /// price plan identifier
        /// </summary>
        [Required]
        public Guid priceplan_id { get; set; }

        /// <summary>
        /// date of registration
        /// </summary>
        public DateTime registration_date { get; set; }

        /// <summary>
        /// approval status
        /// </summary>
        public ApprovalStatus? approval_status { get; set; }

        /// <summary>
        /// plan map pickuplist identifier
        /// </summary>
        public string planmap_pickuplist_id { get; set; }

        public Guid data_partition_id { get; set; }

    }

    /// <summary>
    /// Tenant Approval status constants
    /// </summary>
    public enum ApprovalStatus
    {
        /// <summary>
        /// Waiting for approval. Newly created tenant
        /// </summary>
        WAITINGFORAPPROVAL,

        /// <summary>
        /// Approved by the admin
        /// </summary>
        APPROVED,

        /// <summary>
        /// Rejected by the admin
        /// </summary>
        REJECTED
    }
}
