﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    /// This class is resposible for unit testing of Billing API.
    /// </summary>
    [TestClass]
    public class BillingClientTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "billing/";
        public static string BillingClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid InvoiceID = Guid.Parse("81A0720A-BCB5-E411-BF4E-A41F727D4E53");
        public static Guid TenantID = Guid.Parse("6AB54AA8-BBB5-E411-BF4E-A41F727D4E53");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");
        
        ExceptionHelper exceptionHelper = null;
       
        public BillingClientTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        /// <summary>
        /// This method is used to test search invoices.
        /// </summary>
        [TestMethod]
        public void SearchInvoices()
        {
            using (var client = GetClient())
            {
                DateTime from = new DateTime(2014, 09, 23);
                DateTime duedate = new DateTime(2016, 09, 23);
                DateTime to = new DateTime(2015, 10, 23);
                var result = client.SearchInvoices(new InvoiceSearchConditionDTO
                {
                    PageNo = 1,
                    PageSize = 25,
                    BillFrequency = 0,
                    TenantIds = new Guid[] { TenantID },
                });
                Assert.IsNotNull(result);
            }
        }
        /// <summary>
        /// This method is used to test get invoice by invoice identifier.
        /// </summary>
        [TestMethod]
        public void GetInvoiceById()
        {
            using (var client = GetClient())
            {
                var result = client.GetInvoiceById(InvoiceID, TenantID);
                Assert.IsNotNull(result);
            }
        }
        /// <summary>
        /// This method is used to  test get invocie by invalid invoice identifier.
        /// </summary>
        [TestMethod]
        public void GetInvoiceById_InvalidTenantId()
        {
            using (var client = GetClient())
            {
                try
                {
                    var result = client.GetInvoiceById(InvoiceID, WrongTenantId);
                    Assert.IsNull(result);
                }
                catch (Exception Ex)
                {
                    Assert.IsTrue(true);
                }
            }
        }
        /// <summary>
        /// This method is used to test bill statistics.
        /// </summary>
        [TestMethod]
        public void GetBillStatistics()
        {
            DateTime from = new DateTime(2014, 09, 23);
            DateTime to = new DateTime(2016, 10, 23);
            using (var client = GetClient())
            {
                var result = client.GetBillStatistics(new BillStatisticsSearchConditionDTO
                {
                    TenantIds = new Guid[] { TenantID },
                });
                Assert.IsNotNull(result);
            }
        }
        /// <summary>
        /// This method is used to test revenues.
        /// </summary>
        [TestMethod]
        public void GetRevenues()
        {
            using (var client = GetClient())
            {
                DateTime from = new DateTime(2014, 01, 01);
                DateTime to = new DateTime(2016, 11, 11);

                var result = client.GetRevenues(from, to, TenantID.ToString());
                Assert.IsNotNull(result);
            }
        }
        /// <summary>
        /// This method is used to test  the revenues with invalid date.
        /// </summary>
        [TestMethod]
        public void GetRevenues_InvalidDate()
        {
            using (var client = GetClient())
            {
                    DateTime from = new DateTime(2066, 01, 01);
                    DateTime to = new DateTime(5112, 11, 11);
                    var result = client.GetRevenues(from, to,TenantID.ToString());
                    Assert.AreEqual(result.Count, 0);             
            }
        }
        /// <summary>
        /// This method is used to test the revenues with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void GetRevenues_InvalidTenantID()
        {
            using (var client = GetClient())
            {
                try
                {
                    DateTime from = new DateTime(2014, 01, 01);
                    DateTime to = new DateTime(2014, 11, 11);
                    var result = client.GetRevenues(from, to, WrongTenantId.ToString());
                    Assert.IsNotNull(result);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.NullReferenceErrorCode);
                }
            }
        }

        /// <summary>
        /// Client
        /// </summary>
        /// <returns></returns>
        private static BillingClient GetClient()
        {
            var client = new BillingClient(
                baseAddress: BillingClientBaseAddress,
                username: ClientUserName,
                api_key: APIKey);
            return client;
        }
    }
}
