﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    /// This class is resposible for unit testing of DataPartition API.
    /// </summary>
    [TestClass]
    public class DataPartitionClientUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "datapartition/";
        public static string DataPartitionClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static Guid DataPartitionID = Guid.Parse("50CA12FD-2030-400A-AC8D-7F40DBBCA866");        

        public static string ModuleCode = "ApplicationConnectionString";

        ExceptionHelper exceptionHelper = null;

        public DataPartitionClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        /// <summary>
        /// This method is used to test get default partition.
        /// </summary>
        [TestMethod]
        public void TestGetDefaultPartition()
        {
            using (var client = GetClient())
            {
                var id = client.GetDefaultPartitionId();
                Assert.IsNotNull(id);
            }
        }
        /// <summary>
        /// This method is used to test the get connection string.
        /// </summary>
        [TestMethod]
        public void TestGet_ConnectionString()
        {
            using (var client = GetClient())
            {
                var data = client.GetConnectionString(ModuleCode, DataPartitionID);
                Assert.IsNotNull(data);
            }
        }

        /// <summary>
        /// This method is used to test get all connection string.
        /// </summary>
        [TestMethod]
        public void TestGet_All_ConnectionStrings()
        {
            using (var client = GetClient())
            {
                var data = client.GetAllConnectionStrings(DataPartitionID);
                Assert.IsNotNull(data);
            }
        }
        /// <summary>
        /// This method is used to test the get all connection string withour partition identifier.
        /// </summary>
        [TestMethod]
        public void TestGetConnectionString_WithoutPartitionID()
        {
            using (var client = GetClient())
            {
                var data = client.GetConnectionString(ModuleCode);
                Assert.IsNull(data);
            }
        }
        /// <summary>
        /// This method is used to test the get connection string with invalid module code.
        /// </summary>
        [TestMethod]
        public void TestGetConnectionString_InvalidModuleCode()
        {
            using (var client = GetClient())
            {
                var data = client.GetConnectionString(ModuleCode);
                Assert.IsNull(data);
            }
        }
        /// <summary>
        /// This method is used to test the get connection string with empty module code.
        /// </summary>
        [TestMethod]
        public void TestGetConnectionString_EmptyModuleCode()
        {
            try
            {
                using (var client = GetClient())
                {
                    var data = client.GetConnectionString(string.Empty, DataPartitionID);
                    Assert.IsNull(data);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the get the connection string with empty parameters.
        /// </summary>
        [TestMethod]
        public void TestGetConnectionString_EmptyParameters()
        {
            try
            {
                using (var client = GetClient())
                {
                    var data = client.GetConnectionString(string.Empty, Guid.Empty);
                    Assert.IsNull(data);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the set default partition identfier.
        /// </summary>
        [TestMethod]
        public void TestSet_Default_PartitionID()
        {
            using (var client = GetClient())
            {
                client.SetDefaultPartitionId(DataPartitionID);
                var id = client.GetDefaultPartitionId();
                Assert.AreEqual(id, DataPartitionID);
            }
        }
        /// <summary>
        /// This method is used to test the set default partition identifiwe which is aldreay default.
        /// </summary>
        [TestMethod]
        public void TestSet_Default_PartitionID_AlreadyDefault()
        {
            using (var client = GetClient())
            {
                client.SetDefaultPartitionId(DataPartitionID);
                var id = client.GetDefaultPartitionId();
                Assert.AreEqual(id, DataPartitionID);
            }
        }
        /// <summary>
        /// Client
        /// </summary>
        /// <returns></returns>
        private static DataPartitionClient GetClient()
        {
            var client = new DataPartitionClient(
                baseAddress: DataPartitionClientBaseAddress,
                username: ClientUserName,
                api_key: APIKey);
            return client;
        }
    }
}
