﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    ///  This class is resposible for unit testing of DataViews API.
    /// </summary>
    [TestClass]
    public class DataViewsClientUnitTest
    {

        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "dataviews/";
        public static string DataViewsClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();


        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        public static string TenantAddressDetailsFormDataViewId = "TenantAddressDetailsForm";
        public static string InvalidDataViewID = "Dashborad";

        ExceptionHelper exceptionHelper = null;

        public DataViewsClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        /// <summary>
        /// This method is used to test the get all data views.
        /// </summary>
        [TestMethod]
        public void TestGetAllDataViews()
        {
            using (var client = GetClient())
            {
                var data = client.GetAllDataViews();
                Assert.IsNotNull(data);
            }
        }

        /// <summary>
        /// This method is used to test the get data view.
        /// </summary>
        [TestMethod]
        public void TestGetDataView()
        {
            GetDataViewDetails(TenantAddressDetailsFormDataViewId);
        }

        public static DataViewDTO GetDataViewDetails(string dataViewId)
        {
            DataViewDTO dataViewDTO = null;
            using (var client = GetClient())
            {
                dataViewDTO = client.GetDataView(dataViewId);
                Assert.IsNotNull(dataViewDTO);
            }
            return dataViewDTO;
        }
        /// <summary>
        /// This method is used to test the get data view with tanant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetDataView_WithTenantID()
        {
            using (var client = GetClient())
            {
                Guid tenant_id = new Guid();
                tenant_id = TenantId;
                var data = client.GetDataView(TenantAddressDetailsFormDataViewId, tenant_id);
                Assert.IsNotNull(data);
            }
        }
        
        /// <summary>
        /// This method is used to test the get data view empty string and tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetDataView_EmptyStringandTenant()
        {
            try
            {
                using (var client = GetClient())
                {
                    var data = client.GetDataView("",Guid.Empty);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the get data view with invalid data view.
        /// </summary>
        [TestMethod]
        public void TestGetDataView_InvalidDataView()
        {
            using (var client = GetClient())
            {
                try
                {
                    var data = client.GetDataView(InvalidDataViewID);
                    Assert.IsNull(data);
                }
                catch (Exception Ex)
                {
                    Assert.IsNotNull(Ex.Message);
                    Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidDataViewIDErrorMsg));
                }
            }
        }

        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static DataViewsClient GetClient()
        {
            var client = new DataViewsClient(
              baseAddress: DataViewsClientBaseAddress,
              username: ClientUserName,
              api_key: APIKey);
            return client;
        }
    }
}
