﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    /// This class is resposible for unit testing of EventsClient API.
    /// </summary>
    [TestClass]
    public class EventsClientUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "Events/";
        public static string EventsClientBaseAddress = RootURI + APIEndPoint;
        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();

        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        public static Guid EventAuditId = Guid.Parse("FB766B7B-CB19-4CEB-A05C-86291DCB57EC");
        public static Guid AddUserEventId = Guid.Parse("078EFD5E-D019-E211-89C2-000000000000");
        public static Guid LogOnEventId = Guid.Parse("12CDA0B9-4D87-E011-A964-6C626D6BBC12");
        public static Guid InvalidEventID = Guid.Parse("11111111-D019-E211-89C2-000000000000");

        public static string EventSubjectId = "146bcbbb-a6b5-e411-957a-b8ac6f3fc48b";
        public static string EventSubjectType = "UserDetails";

        ExceptionHelper exceptionHelper = null;

        public EventsClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        /// <summary>
        /// This method is used to test the get all events.
        /// </summary>
        [TestMethod]
        public void TestGetAllEvents()
        {
            using (var client = GetClient())
            {
                var lstEvents = client.GetAllEvents(null);
                Assert.IsNotNull(lstEvents);
            }
        }
        /// <summary>
        /// This method is used to test the get event by event audit and tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetEvent_ByEventAuditIDandTenantID()
        {
            using (var client = GetClient())
            {
                var result = client.GetEvent(EventAuditId, TenantId);
                Assert.IsNotNull(result);
            }
        }
        /// <summary>
        /// This method is used to test the get event by event audit identifier.
        /// </summary>
        [TestMethod]
        public void TestGetEvent_ByEventAuditID()
        {
            using (var client = GetClient())
            {
                var result = client.GetEvent(EventAuditId);
                Assert.IsNotNull(result);
            }
        }
        /// <summary>
        /// This method is used to test the get event by wrong event identifier.
        /// </summary>
        [TestMethod]
        public void TestGetEvent_ByWrongEventID()
        {
            using (var client = GetClient())
            {
                var result = client.GetEvent(InvalidEventID);
                Assert.IsNull(result);
            }
        }
        /// <summary>
        /// This method is used to test the get the event by empty parameters.
        /// </summary>
        [TestMethod]
        public void TestGetEvent_EmptyParameter()
        {
            using (var client = GetClient())
            {
                try
                {
                    var result = client.GetEvent(Guid.Empty);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                }
            }
        }
        /// <summary>
        /// This method is used to test the add event.
        /// </summary>
        [TestMethod]
        public void TestPostEvents()
        {
            using (var client = GetClient())
            {
                    client.RegisterEvent(new EventDTO
                    {
                        event_id = LogOnEventId,
                        user_id = UserId,
                        subject_id = EventSubjectId,
                        subject_type = EventSubjectType,
                        subject_xml_value = null
                    });
                    Assert.IsTrue(true);
            }
        }
        /// <summary>
        /// This method is used to test the post events with null event identifier.
        /// </summary>
        [TestMethod]
        public void TestPostEvents_EventIDasNull()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.RegisterEvent(new EventDTO
                    {
                        event_id = Guid.Empty,
                        user_id = Guid.Empty,
                        subject_id =EventSubjectId,
                        subject_type = EventSubjectType,
                        subject_xml_value = null
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
            }
        }
        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static EventsClient GetClient()
        {
            var client = new EventsClient(
               baseAddress: EventsClientBaseAddress,
               username: ClientUserName,
               api_key: APIKey);
            return client;
        }
    }
}
