﻿using System;
using System.Collections.Generic;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    /// This class is resposible for unit testing of Notifications API.
    /// </summary>
    [TestClass]
    public class NotificationsClientUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "Notifications/";
        public static string NotificationsClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        public static Guid NotificationId = Guid.Parse("F04BAE41-EAA7-46D8-81D0-9C3A1919E774");

        ExceptionHelper exceptionHelper = null;

        public NotificationsClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        #region positive
        /// <summary>
        /// This method is used to test the search email audits.
        /// </summary>
        [TestMethod]
        public void TestSearchEmailAudits()
        {
            using (var client = GetClient())
            {
                var lstAudits = client.SearchEmailAudits(new EmailAuditSearchConditionDTO
                {
                });
                Assert.IsNotNull(lstAudits);
            }
        }
        /// <summary>
        /// This method is used to test the search email audit bu notification identifier.
        /// </summary>
        [TestMethod]
        public void TestSearchEmailAudits_ByNoficationID()
        {
            using (var client = GetClient())
            {
                var lstAudits = client.SearchEmailAudits(new EmailAuditSearchConditionDTO
                {
                    notification_id = NotificationId.ToString(),
                    tenant_id = TenantId
                });
                Assert.IsNotNull(lstAudits);
            }
        }
        /// <summary>
        /// This method is used to test the send email.
        /// </summary>
        [TestMethod]
        public void TestSendEmail()
        {
            using (var client = GetClient())
            {
                client.SendEmail(new EmailDTO
                {
                    to = "ethan.hunt@mif.com",
                    cc = "john.clif@mif.com",
                    bcc = "david.luke@mif.com",
                    notification_name = "User Password Mailer",
                    placeholders = new Dictionary<string, string>() 
                    { 
                        { "username", "EthanHunt" },
                        { "UserDetails.User.CreatedOn", DateTime.Now.ToString() },
                        { "activationLink", RootURI+ Guid.NewGuid().ToString() }                     
                    },
                    track_id = Guid.NewGuid().ToString()
                });
            }
        }
        #endregion positive

        #region negative
        /// <summary>
        /// This method is used to test the search email audit with invalid parameters.
        /// </summary>
        [TestMethod]
        public void TestSearchEmailAudits_InvalidParameters()
        {
            using (var client = GetClient())
            {
                     var lstAudits = client.SearchEmailAudits(new EmailAuditSearchConditionDTO
                    {
                        notification_id = string.Empty,
                        tenant_id = WrongTenantId
                    });
                    Assert.IsNull(lstAudits.result);
                    Assert.AreEqual(lstAudits.total_count, 0);
            }
        }
        /// <summary>
        /// This method is used to test the search email audits with empty identifier.
        /// </summary>
        [TestMethod]
        public void TestSearchEmailAudits_EmptyID()
        {
                using (var client = GetClient())
                {
                    var lstAudits = client.SearchEmailAudits(new EmailAuditSearchConditionDTO
                    {
                        notification_id = string.Empty,
                        tenant_id = Guid.Empty

                    });                    
                    Assert.IsNull(lstAudits.result);
                    Assert.AreEqual(lstAudits.total_count, 0);
                }
        }
        /// <summary>
        /// This method is used to test the send email with identifiers as null.
        /// </summary>
        [TestMethod]
        public void TestSendEmail_IDasNull()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.SendEmail(new EmailDTO
                    {
                        to = "",
                        cc = "",
                        bcc = "",
                        notification_name = "User Password Mailer",
                        placeholders = new Dictionary<string, string>() 
                        { 
                            { "username", "EthanHunt" },
                            { "UserDetails.User.CreatedOn", DateTime.Now.ToString() },
                            { "activationLink", RootURI+ Guid.NewGuid().ToString() }
                        },
                        track_id = Guid.NewGuid().ToString()
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the send email with invalid expression.
        /// </summary>
        [TestMethod]
        public void TestSendEmail_InvalidExpression()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.SendEmail(new EmailDTO
                    {
                        to = "123asdf",
                        cc = "asdf1234",
                        bcc = "#.com",
                        notification_name = "User Password Mailer",
                        placeholders = new Dictionary<string, string>() 
                        { 
                            { "username", "EthanHunt" },
                            { "UserDetails.User.CreatedOn", DateTime.Now.ToString() },
                            { "activationLink", RootURI+ Guid.NewGuid().ToString() 
                        }                      
                    },
                        track_id = Guid.NewGuid().ToString()
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the send email activation link as null.
        /// </summary>
        [TestMethod]
        public void TestSendEmail_ActivationLinkasNull()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.SendEmail(new EmailDTO
                    {
                        to = "demouser@mif.com",
                        cc = "demouser1@mif.com",
                        bcc = "demouser3@mif.com",
                        notification_name = "User Password Mailer without Activation Link",
                        placeholders = new Dictionary<string, string>() { 
                        { "username", "DemoUser" },
                        { "UserDetails.User.CreatedOn", DateTime.Now.ToString() },
                        { "activationLink", null}                      
                    },
                        track_id = Guid.NewGuid().ToString()
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.NullReferenceErrorCode);
            }
        }
        #endregion

        #region Client
        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static NotificationsClient GetClient()
        {
            var client = new NotificationsClient(
                baseAddress: NotificationsClientBaseAddress,
                username: ClientUserName,
                api_key: APIKey);
            return client;
        }
        #endregion
    }
}
