﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
namespace WebApiSDKUnitTest
{
    /// <summary>
    ///  This class is resposible for unit testing of Packages API.
    /// </summary>
    [TestClass]
    public class PackagesClientTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "Packages/";
        public static string PackagesClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid BasicPackageId = Guid.Parse("B3125B4D-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid UltimatePackageId = Guid.Parse("20419343-BBB5-E411-BF4E-A41F727D4E53");

        public static Guid InvalidPackageId = Guid.Parse("B3125B4D-BBB8-E411-BF4E-A41F853D4E53");

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        ExceptionHelper exceptionHelper = null;

        public PackagesClientTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        /// <summary>
        /// This method is used to test the get all packages.
        /// </summary>
        [TestMethod]
        public void TestGetAllPackages()
        {
            using (var client = GetClient())
            {
                //Test with tenant Id
                var resultwithtenantId = client.GetAllPackages(TenantId);
                Assert.IsNotNull(resultwithtenantId);
            }
        }
        /// <summary>
        /// This method is used to test the get all packages without tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetAllPackagesWithOutTenantId()
        {
            using (var client = GetClient())
            {
                //test with out tenant Id
                var resultwithouttenantId = client.GetAllPackages();
                Assert.IsNotNull(resultwithouttenantId);
            }
        }
        /// <summary>
        /// This method is used to test the get all packages with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetAllPackagesWithWrongTenantId()
        {
            using (var client = GetClient())
            {
                try
                {
                    // test method with wrong tenantId
                    var resultwithWrongtenantId = client.GetAllPackages(WrongTenantId);
                    Assert.IsNull(resultwithWrongtenantId);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.UnauthorizedAccessExceptionErrorCode);
                }
            }
        }
        /// <summary>
        /// This method is used to test the get package by package identifier.
        /// </summary>
        [TestMethod]
        public void TestGetPackageByID()
        {
            using (var client = GetClient())
            {
                var entity = client.GetPackage(BasicPackageId);
                Assert.IsNotNull(entity);
                Assert.AreEqual(entity.id,BasicPackageId);

            }
        }
        /// <summary>
        /// This method is used to test the get package by invalid identifier.
        /// </summary>
        [TestMethod]
        public void TestGetPackageByInvalidID()
        {
            using (var client = GetClient())
            {
                //InValid package ID
                var entity = client.GetPackage(InvalidPackageId);
                Assert.IsNull(entity);
            }
        }
        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static PackagesClient GetClient()
        {
            var client = new PackagesClient(
                 baseAddress: PackagesClientBaseAddress,
                 username: ClientUserName,
                 api_key: APIKey);
            return client;
        }

    }
}
