﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    ///  This class is resposible for unit testing of Pickuplists API.
    /// </summary>
    [TestClass]
    public class PickuplistsClientUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "Pickuplists/";
        public static string PickuplistsClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        public static Guid SelfRegistrationPickupListId = Guid.Parse("450F9B19-C3B0-E211-87C3-000C29F3404B");
        public static string SelfRegistrationPickupListName = "Self Registration Packages";

        public static Guid InvalidPickupListId = Guid.Parse("450F9B19-C3B8-E211-87D3-000C29F3404B");

        /// <summary>
        /// This method is used to test the get pickup lists.
        /// </summary>
        [TestMethod]
        public void TestGetPickuplists()
        {
            using (var client = GetClient())
            {
                var data = client.GetPickuplists();
                Assert.IsNotNull(data);
            }
        }
        /// <summary>
        /// This method is used to test the get pickup list by pickup list identifier.
        /// </summary>
        [TestMethod]
        public void TestGetPickuplist_ByID()
        {
            using (var client = GetClient())
            {
                var data = client.GetPickuplist(SelfRegistrationPickupListId);
                Assert.IsNotNull(data);
                Assert.AreEqual(data.name,SelfRegistrationPickupListName);
            }
        }
        /// <summary>
        /// This method is used to test the get pcikup list with invalid pickup list identifier.
        /// </summary>
        [TestMethod]
        public void TestPickuplist_InvalidID()
        {
            using (var client = GetClient())
            {
                var data = client.GetPickuplist(InvalidPickupListId);
                Assert.IsNull(data);
            }
        }
        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static PickuplistsClient GetClient()
        {
            var client = new PickuplistsClient(
                  baseAddress: PickuplistsClientBaseAddress,
                  username: ClientUserName,
                  api_key: APIKey);
            return client;
        }
    }
}
