﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    ///  This class is resposible for unit testing of ProductAnalytics API.
    /// </summary>
    [TestClass]
    public class ProductAnalyticsClientUnitTest
    {

        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "ProductAnalytics/";
        public static string ProductAnalyticsClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        ExceptionHelper exceptionHelper = null;

        public ProductAnalyticsClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        /// <summary>
        /// This method is used to testthe get all product analytics.
        /// </summary>
        [TestMethod]
        public void TestGetAllProductAnalytics()
        {
            using (var client = GetClient())
            {
                    var lstEvents = client.GetAllProductAnalytics(new ProductAnalyticsSearchConditionDTO
                    {
                        page_no = 1,
                        page_size = 100,
                        tenant_id = TenantId
                    });
                    Assert.IsNotNull(lstEvents);
            }
        }
        /// <summary>
        /// This method is used to test the post product analytics.
        /// </summary>
        [TestMethod]
        public void TestPostProductAnalytics()
        {
            using (var client = GetClient())
            {
                client.PostProductAnalytics(new ProductAnalyticsDTO
                {
                    action_name = "TestAction",
                    controller_name = "TestController",
                    machine_name = Environment.MachineName,
                    time_stamp = DateTime.Now,
                    current_url = new Uri(RootURI+"api/productanalytics/"),
                    ip_address = "192.168.0.111",
                    request_method = "POST",
                    request_mode = "WebApi",
                    response_length = 100,
                    response_status_code = 200,
                    response_status_description = "OK"
                });
            }
        }
        /// <summary>
        /// This method is used to test the post product analytics with invalid URL.
        /// </summary>
        [TestMethod]
        public void TestPostProductAnalytics_InvalidURL()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.PostProductAnalytics(new ProductAnalyticsDTO
                    {
                        action_name = "",
                        controller_name = "",
                        machine_name = null,
                        time_stamp = DateTime.Now,
                        current_url = new Uri("SaaS"),
                        ip_address = "192.168.0.1",
                        request_method = "",
                        request_mode = "",
                        response_length = 0,
                        response_status_code = 0,
                        response_status_description = ""
                    });
                }
            }
            catch (Exception Ex)
            {
                Assert.IsNotNull(Ex.Message);
                Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidURIErrorMsg));
            }
        }
        
        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static ProductAnalyticsClient GetClient()
        {
            var client = new ProductAnalyticsClient(
                 baseAddress: ProductAnalyticsClientBaseAddress,
                 username: ClientUserName,
                 api_key: APIKey);
            return client;
        }
    }
}
