﻿using System;
using System.Collections.Generic;
using System.Linq;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    ///  This class is resposible for unit testing of TenantLicenses API.
    /// </summary>
    [TestClass]
    public class TenantLicensesClientUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "TenantLicenses/";
        public static string TenantLicensesClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();


        public static Guid InvoiceID = Guid.Parse("81A0720A-BCB5-E411-BF4E-A41F727D4E53");
        public static Guid TenantID = Guid.Parse("6AB54AA8-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid CoreFusionTenantID = Guid.Parse("6AB54AA8-BBB5-E411-BF4E-A41F727D4E53");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        public static Guid BasicPackageId = Guid.Parse("B3125B4D-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid UltimatePackageId = Guid.Parse("20419343-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid InvalidPackageId = Guid.Parse("B3125B4D-BBB8-E421-BF4E-A41F727D4E53");

        public static Guid BasicBasePricePlan = Guid.Parse("9797D859-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid Ultimate10PdiscountPricePlan = Guid.Parse("F3235066-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid InvalidPricePlan = Guid.Parse("F3235066-BDD5-E411-BF4E-A41F727D4E53");

        ExceptionHelper exceptionHelper = null;

        public TenantLicensesClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        /// <summary>
        /// This method is used to test the get tenant license.
        /// </summary>
        [TestMethod]
        public void TestGetTenantLicense()
        {
            using (var client = GetClient())
            {
                var license = client.GetTenantLicense();
                Assert.IsNotNull(license);
            }
        }
        /// <summary>
        /// This method is used to test the get all tenant license.
        /// </summary>
        [TestMethod]
        public void TestGetAllTenantLicenses()
        {
            using (var client = GetClient())
            {
                var license = client.GetAllTenantLicenses();
                Assert.IsNotNull(license);
            }
        }
        /// <summary>
        /// This method is used to test the get all tenant license with tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetAllTenantLicenses_WithTenantID()
        {
            using (var client = GetClient())
            {
                var license = client.GetAllTenantLicenses(TenantID);
                Assert.IsNotNull(license);
            }
        }
        /// <summary>
        /// This method is used to test the get all tenant license with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetAllTenantLicenses_InvalidTenantID()
        {
            using (var client = GetClient())
            {
                    var license = client.GetAllTenantLicenses(WrongTenantId);
                    Assert.IsNotNull(license);
                    Assert.AreEqual(license.Count(), 0);           
            }
        }
        /// <summary>
        /// This method is used to test the put tenant license.
        /// </summary>
        [TestMethod]
        public void TestPutTenantLicenses()
        {
            using (var client = GetClient())
            {
                    var license = client.GetTenantLicense(CoreFusionTenantID);
                    license.package_id = BasicPackageId;
                    license.priceplan_id = BasicBasePricePlan;
                    license.validity_start = DateTime.Now;
                    license.validity_end = license.validity_start.AddMonths(1);
                    client.PutTenantLicense(license);
                    var licence_afterupdate = client.GetTenantLicense(CoreFusionTenantID);
                    Assert.AreEqual(licence_afterupdate.package_name, "Basic");
                    Assert.AreEqual(licence_afterupdate.package_id, BasicPackageId);
                    Assert.AreEqual(licence_afterupdate.priceplan_id, BasicBasePricePlan);
                    Assert.AreEqual(licence_afterupdate.validity_end, licence_afterupdate.validity_start.AddMonths(1));

                    var RollBacklicense = client.GetTenantLicense(CoreFusionTenantID);
                    RollBacklicense.package_id = UltimatePackageId;
                    RollBacklicense.priceplan_id = Ultimate10PdiscountPricePlan;
                    RollBacklicense.validity_start = DateTime.Now;
                    RollBacklicense.validity_end = license.validity_start.AddMonths(1);
                    client.PutTenantLicense(RollBacklicense);
            }
        }
        /// <summary>
        /// This method is used to test the put tenant license with invalid validity start date.
        /// </summary>
        [TestMethod]
        public void TestPutTenantLicenses_InvalidValidityStartDate()
        {
            try
            {
                using (var client = GetClient())
                {
                    var license = client.GetTenantLicense();
                    license.package_id = BasicPackageId;
                    license.priceplan_id =BasicBasePricePlan;
                    license.validity_start = DateTime.Now.AddMonths(-12);
                    license.validity_end = license.validity_start.AddMonths(-1);
                    client.PutTenantLicense(license);
                    var licence_afterupdate = client.GetTenantLicense(CoreFusionTenantID);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the put tenant license with invalid validity end date.
        /// </summary>
        [TestMethod]
        public void TestPutTenantLicenses_InvalidValidityEndDate()
        {
            try
            {
                using (var client = GetClient())
                {
                    var license = client.GetTenantLicense();
                    license.package_id = BasicPackageId;
                    license.priceplan_id = BasicBasePricePlan;
                    license.validity_start = DateTime.Now;
                    license.validity_end = license.validity_start.AddMonths(-1);
                    client.PutTenantLicense(license);
                    var licence_afterupdate = client.GetTenantLicense(CoreFusionTenantID);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the put tenant license with invalid package and price plan identifier.
        /// </summary>
        [TestMethod]
        public void TestPutTenantLicenses_InvalidPackageandPricePlanID()
        {
            try
            {
                using (var client = GetClient())
                {
                    var license = client.GetTenantLicense();
                    license.package_id = InvalidPackageId;
                    license.priceplan_id = InvalidPricePlan;
                    license.validity_start = DateTime.Now;
                    license.validity_end = license.validity_start.AddMonths(1);
                    client.PutTenantLicense(license);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the delete tenant license.
        /// </summary>
        [TestMethod]
        public void TestDeleteTenant_License()
        {
                using (var client = GetClient())
                {
                    client.DeleteTenantLicense(CoreFusionTenantID);
                }
                    
        }
        /// <summary>
        /// This method is used to test the delete tenant license with invalid tennat identifier.
        /// </summary>
        [TestMethod]
        public void TestDeleteTenant_LicenseInvalidTenantId()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.DeleteTenantLicense(WrongTenantId);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.LicenseExceptionErrorCode);
                }
            }
        }
        /// <summary>
        /// This method is used to test the delete tenant license which is already deactivated.
        /// </summary>
        [TestMethod]
        public void TestDeleteTenant_LicenseAlreadyDeactivated()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.DeleteTenantLicense(CoreFusionTenantID);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.LicenseExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the activate tenant license.
        /// </summary>
        [TestMethod]
        public void TestActivateTenant_License()
        {
                using (var client = GetClient())
                {
                    client.ActivateTenantLicense(CoreFusionTenantID);
                }
        }
        /// <summary>
        /// This method is used to test the activate tenant license with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestActivateTenant_LicenseInvalidTenantId()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.ActivateTenantLicense(WrongTenantId);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.LicenseExceptionErrorCode);
                }
            }
        }
        /// <summary>
        /// This method is used to test the activate tenant license which is already activated.
        /// </summary>
        [TestMethod]
        public void TestActivateTenant_LicenseAlreadyActivated()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.ActivateTenantLicense(CoreFusionTenantID);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.LicenseExceptionErrorCode);
            }
        }
        
        /// <summary>
        /// This method is used to test the validate tenant license.
        /// </summary>
        [TestMethod]
        public void ValidateTenantLicense()
        {
                using (var client = GetClient())
                {
                    var result = client.ValidateTenantLicense(CoreFusionTenantID);
                    Assert.AreEqual(result, true);
                }          
        }
        /// <summary>
        /// This method is used to test the validate tenant license with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void ValidateTenantLicense_InvalidTenantId()
        {
                using (var client = GetClient())
                {
                    var result = client.ValidateTenantLicense(WrongTenantId);
                    Assert.AreEqual(result, false);
                }
        }
        /// <summary>
        /// This method is used to test the get feature by module identfiers.
        /// </summary>
        [TestMethod]
        public void GetFeatureByModuleIds()
        {
                using (var client = GetClient())
                {
                    var result = client.GetFeatureByModuleIds(new string[] { "User", "Setting" });
                    Assert.IsNotNull(result);
                }
           
        }
        /// <summary>
        /// This method is used to test the get feature by invalid module identfiers.
        /// </summary>
        [TestMethod]
        public void GetFeatureByModuleIds_InvalidModuleIds()
        {
                using (var client = GetClient())
                {
                    var result = client.GetFeatureByModuleIds(new string[] { "User1", "Setting1" });
                    Assert.AreEqual(result.Count, 0);
                }
           
        }
        /// <summary>
        /// This method is used to test the get usage by module identifiers.
        /// </summary>
        [TestMethod]
        public void GetUsageByModuleIds()
        {

                using (var client = GetClient())
                {
                    var result = client.GetUsageByModuleIds(new string[] { "User", "Setting", "Configuration" });
                    Assert.IsNotNull(result);
                }
                    
        }
        /// <summary>
        /// This method is used to test the get usage by invalid module identifiers.
        /// </summary>
        [TestMethod]
        public void GetUsageByModuleIds_InvalidModuleIds()
        {
                using (var client = GetClient())
                {
                    var result = client.GetUsageByModuleIds(new string[] { "User1", "Setting1" });
                    Assert.AreEqual(result.Count, 0);
                }
        }
        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static TenantLicensesClient GetClient()
        {
            var client = new TenantLicensesClient(
                 baseAddress: TenantLicensesClientBaseAddress,
                 username: ClientUserName,
                 api_key: APIKey);
            return client;
        }
    }
}
