﻿using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using CelloSaaS.RestApiSDK;

namespace WebApiSDKUnitTest
{
    /// <summary>
    ///   This class is resposible for unit testing of TenantRegistration API.
    /// </summary>
    [TestClass]
    public class TenantRegistrationUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "TenantRegistration/";
        public static string TenantRegistrationBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid planMapCode = Guid.Parse("f676a0f0-a874-e411-ae8a-b8ac6f3fc48b");
        public static Guid planMapPickupListId = Guid.Parse("59e6b31a-0e67-4cd2-87dc-1d9103b7c692");
        public static Guid packageId = Guid.Parse("f676a0f0-a874-e411-ae8a-b8ac6f3fc48b");
        public static Guid parentTenantId = Guid.Parse("B590CD25-3093-DF11-8DEB-001EC9DAB123");

        public static Guid DefaultPricePlanPickupListId = Guid.Parse("450F9B19-C3B0-E211-87C3-000C29F3404B");
        public static Guid InvalidDefaultPricePlanPickupListId = Guid.Parse("A25985C6-1234-1111-5678-004C29C8E241");

        public static Guid InvoiceID = Guid.Parse("81A0720A-BCB5-E411-BF4E-A41F727D4E53");
        public static Guid TenantID = Guid.Parse("6AB54AA8-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid CoreFusionTenantID = Guid.Parse("6AB54AA8-BBB5-E411-BF4E-A41F727D4E53");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        public static Guid BasicPackageId = Guid.Parse("B3125B4D-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid UltimatePackageId = Guid.Parse("20419343-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid InvalidPackageId = Guid.Parse("B3125B4D-BBB8-E421-BF4E-A41F727D4E53");

        public static Guid BasicBasePricePlan = Guid.Parse("9797D859-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid Ultimate10PdiscountPricePlan = Guid.Parse("F3235066-BBB5-E411-BF4E-A41F727D4E53");
        public static Guid InvalidPricePlan = Guid.Parse("F3235066-BDD5-E411-BF4E-A41F727D4E53");

        ExceptionHelper exceptionHelper = null;
        public TenantRegistrationUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        #region positive
        /// <summary>
        /// This method is used to test the create tenant.
        /// </summary>
        [TestMethod]
        public void TestCreateTenant()
        {
                using (var client = GetClient())
                {
                    try
                    {
                        var data = client.PostSelfRegistratedTenant(new TenantRegistrationDTO
                        {
                            friendly_code = "Techcello",
                            friendly_name = "Techcello",
                            user_name = "John.techcello",
                            password = "John#123868",
                            first_name = "david",
                            last_name = "john",
                            contact_number = "12346567890",
                            email_id = "john.david@techcello.com",
                            url = "http://www.techcello.com",
                            website = "http://www.techcello.com",
                            approval_status = ApprovalStatus.APPROVED,
                            package_id = BasicPackageId,
                            priceplan_id = BasicBasePricePlan
                        });
                    }
                    catch (Exception Ex)
                    {
                        string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                        Assert.IsNotNull(errorCode);
                        Assert.AreEqual(errorCode, ExceptionHelper.DuplicateTenantNameExceptionErrorCode);
                    }
                }
        }
      
        /// <summary>
        /// This method is used to test the get packages for self registration.
        /// </summary>
        [TestMethod]
        public void GetPackagesForSelfRegistration()
        {
                using (var client = GetClient())
                {
                    var result = client.GetPackagesForSelfRegistration(DefaultPricePlanPickupListId, CelloSaaS.RestApiSDK.BillingFrequency.Monthly);
                }
        }
        /// <summary>
        /// This method is used to test the whether the price plan map code is valid or not.
        /// </summary>
        [TestMethod]
        public void IsPricePlanMapCodeValid()
        {
                using (var client = GetClient())
                {
                    var result = client.IsPricePlanMapCodeValid(planMapCode, planMapPickupListId, packageId, parentTenantId);
                    Assert.IsNotNull(result);
                }           
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the create tenant with duplicate tenant code.
        /// </summary>
        [TestMethod]
        public void TestCreateTenant_WithDuplicateTenantCode()
        {
            try
            {
                using (var client = GetClient())
                {
                    var data = client.PostSelfRegistratedTenant(new TenantRegistrationDTO
                    {
                        friendly_code = "Techcello",
                        friendly_name = "Techcello",
                        user_name = "John.techcello",
                        password = "John#123868",
                        first_name = "david",
                        last_name = "john",
                        contact_number = "12346567890",
                        email_id = "john.david@techcello.com",
                        url = "http://www.techcello.com",
                        website = "http://www.techcello.com",
                        approval_status = ApprovalStatus.APPROVED,
                        package_id = BasicPackageId,
                        priceplan_id = BasicBasePricePlan
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.DuplicateTenantNameExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the create tenant with invalid package identifier.
        /// </summary>
        [TestMethod]
        public void TestCreateTenant_WithInvalidPackageID()
        {
            try
            {
                using (var client = GetClient())
                {
                    var data = client.PostSelfRegistratedTenant(new TenantRegistrationDTO
                    {
                        friendly_code = "OracleUK",
                        friendly_name = "Oracle Corporations",
                        user_name = "administrator",
                        password = "password",
                        first_name = "Jobin",
                        last_name = "Joy",
                        contact_number = "1234567890",
                        email_id = "jobin.joy@oracle.com",
                        url = "http://www.oracle.co.uk",
                        website = "http://www.oracle.co.uk",
                        package_id = InvalidPackageId,
                        priceplan_id = BasicBasePricePlan
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the create tenant with invalid URL.
        /// </summary>
        [TestMethod]
        public void TestCreateTenant_WithInvalidURL()
        {
            try
            {
                using (var client = GetClient())
                {
                    var data = client.PostSelfRegistratedTenant(new TenantRegistrationDTO
                    {
                        friendly_code = "Honeywell",
                        friendly_name = "Honeywell Private Limited",
                        user_name = "Honeywell",
                        password = "Honeywell#123",
                        first_name = "John",
                        last_name = "Joe",
                        contact_number = "1234567890",
                        email_id = "John.Joe@honeywell.com",
                        url = "asdf1234",
                        website = "asdf1234",
                        package_id = BasicPackageId,
                        priceplan_id = BasicBasePricePlan
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.InvalidURLExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the get packages for self registration with Invalid Default Price Plan Pickup List Identifier.
        /// </summary>
        [TestMethod]
        public void GetPackagesForSelfRegistration_WithInvalidId()
        {
            try
            {
                using (var client = GetClient())
                {
                    var result = client.GetPackagesForSelfRegistration(InvalidDefaultPricePlanPickupListId, BillingFrequency.Annual);
                }
            }
            catch (Exception ex)
            {
                Assert.IsTrue(false);
            }
        }
        /// <summary>
        /// This method is used to test the whether the price plan map code with invalid identifiers.
        /// </summary>
        [TestMethod]
        public void IsPricePlanMapCodeValid_WithInvalidId()
        {
            try
            {
                using (var client = GetClient())
                {
                    var result = client.IsPricePlanMapCodeValid(planMapCode, Guid.Empty, Guid.Empty, Guid.Empty);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
            }
        }
        #endregion

        #region client
        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static CelloSaaS.RestApiSDK.TenantRegistrationClient GetClient()
        {
            var client = new TenantRegistrationClient(
                   baseAddress: TenantRegistrationBaseAddress,
                   username: ClientUserName,
                   api_key: APIKey);
            return client;
        }
        #endregion
    }
}
