﻿using System;
using System.Collections.Generic;
using System.Linq;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    /// This class is resposible for unit testing of TenantSettings API.
    /// </summary>
    [TestClass]
    public class TenantSettingsClientUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "TenantSettings/";
        public static string TenantSettingsClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df56-8deb-001ec9dab122");

        public static Guid EmptyUserId = Guid.Empty;
        public static Guid EmptyTenantId = Guid.Empty;
        public static Guid TenantSettings = Guid.Empty;

        public static Guid AttributeId = Guid.Parse("C971B848-2164-4CB7-93C3-3B37DD15535E");
        public static string InvalidAttributeId = "Windows";

        public static string InvalidDateKey = "DateFormats123";
        public static string DateKey="DateFormat";
        public static string DateValue = "dd/MM/yyyy";

        ExceptionHelper exceptionHelper = null;

        public TenantSettingsClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        /// <summary>
        /// This method is used to test the get tenant setting.
        /// </summary>
        [TestMethod]
        public void TestGetTenantSetting()
        {
            using (var client = GetClient())
            {
                var settings = client.GetTenantSetting(TenantId);
                Assert.IsNotNull(settings);
            }
        }
        /// <summary>
        /// This method is used to test the get tenant setting with empty tenant identifer.
        /// </summary>
        [TestMethod]
        public void TestGetTenantSetting_EmptyTenantID()
        {
                using (var client = GetClient())
                {
                    var settings = client.GetTenantSetting(EmptyTenantId);
                    Assert.AreEqual(settings.Count, 0);
                }
        }
        /// <summary>
        /// This method is used to test the get tenant settings with tenant and attribute identifier.
        /// </summary>
        [TestMethod]
        public void TestGetTenantSettings()
        {
            using (var client = GetClient())
            {
                var settings = client.GetTenantSettings(TenantId.ToString(), AttributeId.ToString());
                Assert.IsTrue(settings.Count> 0);
            }
        }
        /// <summary>
        /// This method is used to test the put tenant setting.
        /// </summary>
        [TestMethod]
        public void PutTenantSetting()
        {
            using (var client = GetClient())
            {
                client.PutTenantSetting(new Dictionary<string, string>() { { DateKey, DateValue } });
                var settings = client.GetTenantSetting();
                Assert.IsNotNull(settings);
                Assert.IsTrue(settings.ContainsKey(DateKey));
                Assert.AreEqual(settings[DateKey], DateValue);
            }
        }
        /// <summary>
        /// This method is used to test the delete tenant setting.
        /// </summary>
        [TestMethod]
        public void DeleteTenantSetting()
        {
            using (var client = GetClient())
            {
                client.PutTenantSetting(new Dictionary<string, string>() { { DateKey, DateValue } });

                var settings = client.GetTenantSetting();
                Assert.IsNotNull(settings);
                Assert.IsTrue(settings.ContainsKey(DateKey));
                Assert.AreEqual(settings[DateKey], DateValue);

                client.DeleteTenantSetting(DateKey);

                settings = client.GetTenantSetting();
                Assert.IsNotNull(settings);
                Assert.IsFalse(settings.ContainsKey(DateKey));
            }
        }
        /// <summary>
        /// This method is used to test the delete tenant setting with invalid key.
        /// </summary>
        [TestMethod]
        public void DeleteTenantSetting_InvalidKey()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.PutTenantSetting(new Dictionary<string, string>() { { InvalidDateKey, DateValue } });

                    var settings = client.GetTenantSetting();
                    Assert.IsNotNull(settings);
                    Assert.IsTrue(settings.ContainsKey(DateKey));
                    Assert.AreEqual(settings[DateKey], DateValue);

                    client.DeleteTenantSetting(DateKey);

                    settings = client.GetTenantSetting();
                    Assert.IsNotNull(settings);
                    Assert.IsFalse(settings.ContainsKey(DateKey));
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.TenantSettingsExceptionErrorCode);
                }
            }
           }	

        /// <summary>
        /// Client.
        /// </summary>
        /// <returns>Tenantsettings client</returns>
        private static TenantSettingsClient GetClient()
        {
            var client = new TenantSettingsClient(
                  baseAddress: TenantSettingsClientBaseAddress,
                  username: ClientUserName,
                  api_key: APIKey);
            return client;
        }
    }
}
