﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    ///  This class is resposible for unit testing of Tenants API.
    /// </summary>
    [TestClass]
    public class TenantsClientTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "Tenants/";
        public static string TenantsClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        public static Guid SharedTenantId = Guid.Parse("f1ec1c3c-985e-e411-bb88-000c29361821");

        public static Guid EmptyUserId = Guid.Empty;
        public static Guid EmptyTenantId = Guid.Empty;

        public static string TenantFriendlyCode = "CoreFusion";
        public static string InvalidTenantFriendlyCode = "CyberWing";

        ExceptionHelper exceptionHelper = null;
        public TenantsClientTest()
        {
            exceptionHelper=new ExceptionHelper();
        }
        #region postive
        /// <summary>
        /// This method is used to test the get tenant.
        /// </summary>
        [TestMethod]
        public void TestGetTenant()
        {
            using (var client = GetClient())
            {
                var tenant = client.GetTenant(TenantId);
                Assert.IsNotNull(tenant);
            }
        }
        /// <summary>
        /// This method is used to test the get tenant with shared tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetTenant_WithSharedTenantId()
        {
            using (var client = GetClient())
            {
                try
                {
                    var tenant = client.GetTenant(SharedTenantId);
                    Assert.IsNotNull(tenant);
                }
                catch
                {
                }
            }
        }
        /// <summary>
        /// This method is used to test the get tenant by URL.
        /// </summary>
        [TestMethod]
        public void TestGetTenantByUrl()
        {
            using (var client = GetClient())
            {
                var tenant = client.GetTenantByUrl(new Uri("http://www.microsoft.com"));
                Assert.IsNull(tenant);
            }
        }
        /// <summary>
        /// This method is used to test the get all tenant info details.
        /// </summary>
        [TestMethod]
        public void GetAllTenantInfoDetails()
        {
            using (var client = GetClient())
            {
                var tenant = client.GetTenantInfo();
                Assert.IsNotNull(tenant);
            }
        }
        /// <summary>
        /// This method is used to test the get tenant identifier from tenant friendly code.
        /// </summary>
        [TestMethod]
        public void GetTenantIdfromFriendlyCode()
        {
            using (var client = GetClient())
            {
                var result = client.GetTenantIdFromFriendlyCode(TenantFriendlyCode);
                Assert.IsNotNull(result);
            }
        }
        /// <summary>
        /// This method is used to test the get stratified tenant by user and tenant identifiers.
        /// </summary>
        [TestMethod]
        public void GetStratifiedTenantByUser()
        {
            using (var client = GetClient())
            {
                var result = client.GetStratfiedTenantByUser(UserId, TenantId);
                Assert.IsNotNull(result);
            }
        }
        /// <summary>
        /// This method is used to test the get tenants in trail.
        /// </summary>
        [TestMethod]
        public void GetTenantsInTrail()
        {
            using (var client = GetClient())
            {
               var result = client.GetTenantsInTrail();
            }
        }
        /// <summary>
        /// This method is used to test the get the tenant details by tenant friendly code.
        /// </summary>
        [TestMethod]
        public void GetTenantDetailsbyFriendlyCode()
        {
            using (var client = GetClient())
            {
                var result = client.GetTenantDetailsByFriendlyCode(TenantFriendlyCode);
                Assert.IsNotNull(result);
            }
        }
        /// <summary>
        /// This method is used to test the delete user roles and tenant mapping.
        /// </summary>
        [TestMethod]
        public void DeleteUserRolesandTenantMapping()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.DeleteUserRolesandTenantMapping(UserId, "GR$Member", TenantId);
                }
                catch (Exception ex)
                {
                    Assert.IsTrue(false);
                }
            }
        }
        /// <summary>
        /// This method is used to test the search.
        /// </summary>
        [TestMethod]
        public void TestSearch()
        {
            using (var client = GetClient())
            {
                var result = client.SearchTenants(new TenantSearchConditionDTO
                {
                    page_no = 1,
                    page_size = 25
                });

                Assert.IsNotNull(result);
                Assert.IsNotNull(result.result);
                Assert.AreNotEqual(result.total_count, 0);
            }
        }

        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the get tenant with empty tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetTenant_WithGuidEmpty()
        {
            using (var client = GetClient())
            {
                try
                {
                    var tenant = client.GetTenant(Guid.Empty);
                    Assert.IsNotNull(tenant);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                }
            }
        }
        /// <summary>
        /// This method is used to test the get tenant with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetTenant_WithInValidTenantId()
        {
            using (var client = GetClient())
            {
                    var tenant = client.GetTenant(WrongTenantId);
                    Assert.IsNull(tenant);
            }
        }
        /// <summary>
        /// This method is used to test the search with invalid condition.
        /// </summary>
        [TestMethod]
        public void TestSearch_WithInvalidCondition()
        {
            using (var client = GetClient())
            {
                var result = client.SearchTenants(new TenantSearchConditionDTO
                {
                    page_no = 1,
                    page_size = 25,
                    email = "asdf#.com"

                });
                Assert.IsNull(result);
            }
        }
        /// <summary>
        /// This method is used to test the tenant identifier from tenant friendly code with invalid tenant friendly code.
        /// </summary>
        [TestMethod]
        public void GetTenantIdfromFriendlyCode_WithInvalidFriendlyCode()
        {
            try
            {
                using (var client = GetClient())
                {
                    var result = client.GetTenantIdFromFriendlyCode(InvalidTenantFriendlyCode);
                    Assert.IsNotNull(result);
                }
            }
            catch (Exception Ex)
            {
                Assert.IsNotNull(Ex.Message);
                Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidFriendlyCodeErrorMsg));
            }
        }
        /// <summary>
        /// This method is used to test the get stratfied tenant by user.
        /// </summary>
        [TestMethod]
        public void GetStratfiedTenantByUser_WithInvalidUser()
        {
                using (var client = GetClient())
                {
                    var result = client.GetStratfiedTenantByUser(WrongUserId, TenantId);
                    Assert.IsNull(result);
                }
        }
        /// <summary>
        /// This method is used to test the get tenant details by friendly code with invalid friendly tenant code.
        /// </summary>
        [TestMethod]
        public void GetTenantDetailsbyFriendlyCode_InvalidFriendlyCode()
        {
            try
            {
                using (var client = GetClient())
                {
                    var result = client.GetTenantDetailsByFriendlyCode(InvalidTenantFriendlyCode);
                    Assert.IsNotNull(result);
                }
            }
            catch (Exception ex)
            {
                Assert.IsTrue(true);
            }
        }
        /// <summary>
        /// This method is used to test the get startfied tenant by user with invalid identifiers.
        /// </summary>
        [TestMethod]
        public void GetStratfiedTenantByUser_WithInvalidIds()
        {
                using (var client = GetClient())
                {
                    var result = client.GetStratfiedTenantByUser(WrongUserId, WrongTenantId);
                    Assert.IsNull(result);
                }
        }
        #endregion
        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static TenantsClient GetClient()
        {
            var client = new TenantsClient(
                  baseAddress: TenantsClientBaseAddress,
                  username: ClientUserName,
                  api_key: APIKey);
            return client;
        }
        }    
}
