﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    /// This class is resposible for unit testing of UserDetails API.
    /// </summary>
    [TestClass]
    public class UserDetailsClientTest
    {

        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "Users/";
        public static string UserDetailsClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");        

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        public static Guid SharedTenantId = Guid.Parse("f1ec1c3c-985e-e411-bb88-000c29361821");

        public static Guid AlreadyActive = Guid.Parse("52A7EFA3-0B6F-E411-BF16-A41F727D4E53");
        public static Guid ActiveTenantId = Guid.Parse("B590CD25-3093-DF11-8DEB-001EC9DAB123");

        public static Guid EmptyUserId = Guid.Empty;
        public static Guid EmptyTenantId = Guid.Empty;

        public static string CompanyCode = "company";
        public static string UserName = "admin@company.com";
        public static string InvalidUserName = "administrator123@company.com";
        public static string Password = "company#123";
        public static string InvalidPassword = "QWERTY#123";

        public static string SecurityQuestion = "What is your city of birth?";
        public static string InvalidSecurityQuestion = "How is your role model";
        public static string SecurityAnswer = "NewYork";       
        public static string InvalidSecurityAnswer = "Tokyo";

        public static string OldPassword = Password;
        public static string NewPassword = "company@123";

        public static string UpdatePassword = "copmany%123";
        public static string ResetPassword = "company$123";

        ExceptionHelper exceptionHelper = null;

        public UserDetailsClientTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        #region GetAllUsers

        #region positive
        /// <summary>
        /// This method is used to test the get all users.
        /// </summary>
        [TestMethod]
        public void GetAllUsers()
        {
            using (var client = GetClient())
            {
                var users = client.GetAllUsers();
                Assert.IsNotNull(users);
            }
        }
        /// <summary>
        /// This method is used to test the get all users with tenant identifier.
        /// </summary>
        [TestMethod]
        public void GetAllUsersWithTenantId()
        {
            using (var client = GetClient())
            {
                var users = client.GetAllUsers(TenantId);
                Assert.IsNotNull(users);
            }
        }

        #endregion
        #endregion

        #region GetUserByUserId

        #region positive
        /// <summary>
        /// This method is used to test the get user by user identifier.
        /// </summary>
        [TestMethod]
        public void GetUserByUserId()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);

            using (var client = GetClient())
            {
                var user = client.GetUserByUserId(createUser.id, TenantId);

                UserDetailsEqual(createUser, user);
            }
        }
        /// <summary>
        /// This method is used to get the user by user identifer.
        /// </summary>
        /// <param name="userId">user identifier.</param>
        /// <returns>user details.</returns>
        public static UserDetailsDTO GetUserById(Guid userId)
        {
            UserDetailsDTO users = null;
            using (var client = GetClient())
            {
                users = client.GetUserByUserId(userId);
                Assert.IsNotNull(users);

            }
            return users;
        }

        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the get user with invalid user identifier.
        /// </summary>
        [TestMethod]
        public void GetUserByUserId_WithInvalidId()
        {
            try
            {
                var createUser = CreateUser();

                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);

                using (var client = GetClient())
                {
                    var user = client.GetUserByUserId(createUser.id, WrongUserId);

                    UserDetailsEqual(createUser, user);
                }
            }
            catch (Exception Ex)
            {
                Assert.IsNotNull(Ex.Message);
                Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidUserIDErrorMsg));
            }
        }
        #endregion

        #endregion

        #region GetUserByUsername

        #region positive
        /// <summary>
        /// This method is used to test the get user by user name.
        /// </summary>
        [TestMethod]
        public void GetUserByUsername()
        {
            using (var client = GetClient())
            {
                var users = client.GetUserByUsername(UserName);
                Assert.IsNotNull(users);
                Assert.AreEqual(users.username, UserName);
            }
        }
        /// <summary>
        /// This method is used to test the get user by user name with a newly created user.
        /// </summary>
        [TestMethod]
        public void GetUserByUsernameWithNewlyCreatedUser()
        {
            using (var client = GetClient())
            {
                var createUser = CreateUser();

                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);
                var user = client.GetUserByUsername(createUser.username, createUser.tenant_id);
                UserDetailsEqual(createUser, user);
            }
        }

        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the get user by invalid user name.
        /// </summary>
        [TestMethod]
        public void GetUserByUsername_InvalidUserName()
        {
            try
            {
                using (var client = GetClient())
                {
                    var users = client.GetUserByUsername(InvalidUserName);
                    Assert.IsNotNull(users);
                    Assert.AreEqual(users.username, InvalidUserName);
                }
            }
            catch (Exception Ex)
            {
                Assert.IsNotNull(Ex.Message);
                Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidUserNameErrorMsg));
            }
        }

        #endregion

        #endregion

        #region TestActivateUser

        #region positive
        /// <summary>
        /// This method is used to test the activate user.
        /// </summary>
        [TestMethod]
        public void TestActivateUser()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);

            using (var client = GetClient())
            {
                client.DeleteUser(createUser.id);

                var user = client.GetUserByUserId(createUser.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.status, false);

                client.ActivateUser(user.id);

                user = client.GetUserByUserId(user.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.status, true);
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the activate user with invalid user.
        /// </summary>
        [TestMethod]
        public void TestActivateUser_WithInvalidUser()
        {
            try
            {
                var createUser = CreateUser();

                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);

                using (var client = GetClient())
                {
                    client.DeleteUser(createUser.id);

                    var user = client.GetUserByUserId(createUser.id);
                    Assert.IsNotNull(user);
                    Assert.AreEqual(user.status, false);

                    client.ActivateUser(WrongUserId);

                    user = client.GetUserByUserId(user.id);
                    Assert.IsNotNull(user);
                    Assert.AreEqual(user.status, true);
                }

            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.UserDetailExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the activate user which is already active.
        /// </summary>
        [TestMethod]
        public void TestActivateUser_AlreadyActive()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.ActivateUser(AlreadyActive);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.UserDetailExceptionErrorCode);
            }
        }
        #endregion

        #endregion

        #region UpdateFirstTimeLogOn

        #region positive
        /// <summary>
        /// This method is used to test the updeate first time logon.
        /// </summary>
        [TestMethod]
        public void Test_UpdateFirstTimeLogOn()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);

            using (var client = GetClient())
            {
                var user = client.GetUserByUserId(createUser.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.is_first_time_user, true);
                client.UpdateFirstTimeLogOn(createUser.id, Password,SecurityQuestion, SecurityAnswer, true, createUser.tenant_id);
                user = client.GetUserByUserId(createUser.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.is_first_time_user, false);
            }
        }

        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the update first time logon with invalid user identifier.
        /// </summary>
        [TestMethod]
        public void Test_UpdateFirstTimeLogOn_WithInvalidUserId()
        {
            try
            {
                var createUser = CreateUser();

                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);

                using (var client = GetClient())
                {
                    var user = client.GetUserByUserId(createUser.id);
                    Assert.IsNotNull(user);
                    Assert.AreEqual(user.is_first_time_user, true);
                    client.UpdateFirstTimeLogOn(WrongUserId, Password, SecurityQuestion, SecurityAnswer, true, createUser.tenant_id);
                    user = client.GetUserByUserId(createUser.id);
                    Assert.IsNotNull(user);
                    Assert.AreEqual(user.is_first_time_user, false);
                }
            }
            catch (Exception Ex)
            {
                Assert.IsNotNull(Ex.Message);
                Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidUserIdentifierErrorMsg));
            }
        }
        #endregion

        #endregion

        #region ValidateSecurityAnswer

        #region positive
        /// <summary>
        /// This method is used to test the validate security answer.
        /// </summary>
        [TestMethod]
        public void Test_ValidateSecurityAnswer()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);

            using (var client = GetClient())
            {
                client.UpdateFirstTimeLogOn(createUser.id, Password, SecurityQuestion, SecurityAnswer, true, createUser.tenant_id);
                bool result = client.ValidateSecurityAnswer(CompanyCode, createUser.username, SecurityAnswer);
                Assert.AreEqual(result, true);
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the validate security with invalid security answer.
        /// </summary>
        [TestMethod]        
        public void Test_ValidateSecurityAnswer_WithInvalidSecurityAnswer()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);

            using (var client = GetClient())
            {
                client.UpdateFirstTimeLogOn(createUser.id, Password, SecurityQuestion, SecurityAnswer, true, createUser.tenant_id);
                bool result = client.ValidateSecurityAnswer(CompanyCode, createUser.username, InvalidPassword);
                Assert.AreEqual(result, false);
            }
        }
        #endregion

        #endregion

        #region ChangePassword

        #region positive
        /// <summary>
        /// This method is used to test the change password.
        /// </summary>
        [TestMethod]
        public void Test_ChangePassword()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);

            using (var client = GetClient())
            {
                client.UpdateFirstTimeLogOn(createUser.id, Password, SecurityQuestion, SecurityAnswer, true, createUser.tenant_id);
                client.ChangePassword(createUser.username, OldPassword, NewPassword);

            }
        }

        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the change password with invalid password.
        /// </summary>
        [TestMethod]
        public void Test_ChangePassword_WithInvalidPassword()
        {
            try
            {
                var createUser = CreateUser();

                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);

                using (var client = GetClient())
                {
                    client.UpdateFirstTimeLogOn(createUser.id, Password, SecurityQuestion, SecurityAnswer, true, createUser.tenant_id);
                    client.ChangePassword(createUser.username, InvalidPassword, Password);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.UserDetailExceptionErrorCode);
            }
        }
        #endregion

        #endregion

        #region ForgotPassword

        #region positive
        /// <summary>
        /// This method is used to test the forgot password.
        /// </summary>
        [TestMethod]
        public void ForgotPassword()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);
            using (var client = GetClient())
            {
                client.ForgotPassword(createUser.username, createUser.tenant_id);
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the forgot password with invalid user name.
        /// </summary>
        [TestMethod]
        public void ForgotPassword_WithInvalidUserName()
        {
            try
            {
                var createUser = CreateUser();
                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);
                using (var client = GetClient())
                {
                    client.ForgotPassword(InvalidUserName, createUser.tenant_id);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.InvalidUserNameExceptionErrorCode);
            }
        }
        #endregion

        #endregion

        #region UpdateUserPassword

        #region positive
        /// <summary>
        /// This method is used to test the update user password.
        /// </summary>
        [TestMethod]
        public void UpdateUserPassword()
        {
            using (var client = GetClient())
            {
                var createUser = CreateUser();

                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);
                client.UpdateUserPassword(createUser.id, UpdatePassword, createUser.tenant_id);
            }
        }
        #endregion
        /// <summary>
        /// This method is used to test the update user password with invalid user identifier.
        /// </summary>
        #region negative
        [TestMethod]
        public void UpdateUserPassword_WithInvalidUserId()
        {
            using (var client = GetClient())
            {
                var createUser = CreateUser();

                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);
                try
                {
                    client.UpdateUserPassword(WrongUserId, InvalidPassword, createUser.tenant_id);
                }
                catch (Exception Ex)
                {
                    Assert.IsNotNull(Ex.Message);
                    Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidUserIdentifierErrorMsg));
                }
            }
        }
        /// <summary>
        /// This method is used to test the update user password with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void UpdateUserPassword_WithInvalidTenantId()
        {
            using (var client = GetClient())
            {
                var createUser = CreateUser();

                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);
                try
                {
                    client.UpdateUserPassword(createUser.id, UpdatePassword, WrongTenantId);
                }
                catch (Exception Ex)
                {
                    Assert.IsNotNull(Ex.Message);
                    Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidUserIdentifierErrorMsg));
                }
            }
        }
        #endregion

        #endregion

        #region UpdatePasswordExpiry

        #region positive
        /// <summary>
        /// This method is used to test the update password expiry.
        /// </summary>
        [TestMethod]
        public void UpdatePasswordExpiry()
        {
            using (var client = GetClient())
            {
                client.UpdatePasswordExpiry();
            }
        }
        #endregion
        #endregion

        #region ResetPassword_By_UserName

        #region positive
        /// <summary>
        /// This method is used to test the reset password by user name.
        /// </summary>
        [TestMethod]
        public void Test_ResetPassword_By_UserName()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);
            using (var client = GetClient())
            {
                client.UpdateFirstTimeLogOn(createUser.id, Password, SecurityQuestion, SecurityAnswer, true, createUser.tenant_id);
                var user = client.GetUserByUserId(createUser.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.is_first_time_user, false);
                client.ResetPassword(createUser.username);
                user = client.GetUserByUserId(createUser.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.is_first_time_user, true);
            }
        }

        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the reset password by user name with invalid username.
        /// </summary>
        [TestMethod]
        public void Test_ResetPassword_By_UserName_WithInvalidUsername()
        {
            try
            {
                var createUser = CreateUser();

                Assert.IsNotNull(createUser);
                Assert.AreNotEqual(Guid.Empty, createUser.id);
                using (var client = GetClient())
                {
                    client.UpdateFirstTimeLogOn(createUser.id, Password,SecurityQuestion, SecurityAnswer, true, createUser.tenant_id);
                    var user = client.GetUserByUserId(createUser.id);
                    Assert.IsNotNull(user);
                    Assert.AreEqual(user.is_first_time_user, false);
                    client.ResetPassword(InvalidUserName);
                    user = client.GetUserByUserId(createUser.id);
                    Assert.IsNotNull(user);
                    Assert.AreEqual(user.is_first_time_user, true);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.InvalidUserNameExceptionErrorCode);
            }
        }
        #endregion

        #endregion

        #region ResetPassword_Userids

        #region positive
        /// <summary>
        /// This method is used to test the reset password with user identifiers.
        /// </summary>
        [TestMethod]
        public void ResetPassword_Userids()
        {
            var createUser1 = CreateUser();
            Assert.IsNotNull(createUser1);
            Assert.AreNotEqual(Guid.Empty, createUser1.id);
            using (var client = GetClient())
            {
                    client.UpdateFirstTimeLogOn(createUser1.id,Password,SecurityQuestion, SecurityAnswer, true, createUser1.tenant_id);
                    client.ResetPassword(new Guid[] { createUser1.id },ResetPassword, TenantId);
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the reset password with invalid user and tenant identifiers.
        /// </summary>
        [TestMethod]
        public void ResetPassword_Userids_WithInvalidTenantId()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.ResetPassword(new Guid[] { WrongUserId }, ResetPassword, WrongTenantId);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.UnauthorizedAccessExceptionErrorCode);            
                }
            }
        }
        #endregion

        #endregion

        #region ResetPassword_by_username_with_securityquestion

        #region positive
        /// <summary>
        /// This method is used to test the reset password by username with security question.
        /// </summary>
        [TestMethod]
        public void ResetPassword_by_username_with_securityquestion()
        {
            var createUser1 = CreateUser();

            Assert.IsNotNull(createUser1);
            Assert.AreNotEqual(Guid.Empty, createUser1.id);


            using (var client = GetClient())
            {

                client.UpdateFirstTimeLogOn(createUser1.id, ResetPassword, SecurityQuestion, SecurityAnswer, true, createUser1.tenant_id);
                var user = client.GetUserByUserId(createUser1.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.is_first_time_user, false);
                client.ResetPassword(createUser1.username, SecurityQuestion, SecurityAnswer, createUser1.tenant_id);
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the reset password by username with invalid seciruty question.
        /// </summary>
        [TestMethod]
        public void cResetPassword_by_username_with_securityquestion_WithInvalidSQuestion()
        {
            try
            {
                var createUser1 = CreateUser();
                Assert.IsNotNull(createUser1);
                Assert.AreNotEqual(Guid.Empty, createUser1.id);
                using (var client = GetClient())
                {

                    client.UpdateFirstTimeLogOn(createUser1.id, ResetPassword, SecurityQuestion, SecurityAnswer, true, createUser1.tenant_id);
                    var user = client.GetUserByUserId(createUser1.id);
                    Assert.IsNotNull(user);
                    Assert.AreEqual(user.is_first_time_user, false);
                    client.ResetPassword(createUser1.username, InvalidSecurityQuestion,SecurityAnswer, createUser1.tenant_id);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.DataExceptionErrorCode);  
            }
        }
        /// <summary>
        /// This method is used to test the reset password by username with invalid security answer.
        /// </summary>
        [TestMethod]
        public void ResetPassword_by_username_with_securityquestion_WithInvalidSAnswer()
        {
            try
            {
                var createUser1 = CreateUser();
                Assert.IsNotNull(createUser1);
                Assert.AreNotEqual(Guid.Empty, createUser1.id);
                using (var client = GetClient())
                {

                    client.UpdateFirstTimeLogOn(createUser1.id, ResetPassword, SecurityQuestion, SecurityAnswer, true, createUser1.tenant_id);
                    var user = client.GetUserByUserId(createUser1.id);
                    Assert.IsNotNull(user);
                    Assert.AreEqual(user.is_first_time_user, false);
                    client.ResetPassword(createUser1.username, SecurityQuestion, InvalidSecurityAnswer, createUser1.tenant_id);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.DataExceptionErrorCode);  
            }
        }

        #endregion

        #endregion

        #region UnlockUserAccounts

        #region positive
        /// <summary>
        /// This method is used to test the unlock user accounts.
        /// </summary>
        [TestMethod]
        public void UnlockUserAccounts()
        {
            var createUser1 = CreateUser();

            Assert.IsNotNull(createUser1);
            Assert.AreNotEqual(Guid.Empty, createUser1.id);

            using (var client = GetClient())
            {
                client.lockUserAccounts(new Guid[] { createUser1.id });
                var user = client.GetUserByUserId(createUser1.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.is_locked, true);
                client.UnlockUserAccounts(new Guid[] { createUser1.id });
                user = client.GetUserByUserId(createUser1.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.is_locked, false);
            }
        }
        #endregion


        #region negative
        /// <summary>
        /// This method is used to test the unlcok user accounts with invalid user identifiers.
        /// </summary>
        [TestMethod]
        public void UnlockUserAccounts_WithInvalidUserIds()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.UnlockUserAccounts(new Guid[] { WrongUserId});
                    var user = client.GetUserByUserId(WrongUserId);
                    Assert.IsNotNull(user);
                }
                catch (Exception Ex)
                {
                    Assert.IsNotNull(Ex.Message);
                    Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidUserIDErrorMsg));
                }
            }
        }

        #endregion

        #endregion

        #region lockUserAccounts

        #region positive
        /// <summary>
        /// This method is used to test the lock user accounts.
        /// </summary>
        [TestMethod]
        public void lockUserAccounts()
        {
            var createUser1 = CreateUser();

            Assert.IsNotNull(createUser1);
            Assert.AreNotEqual(Guid.Empty, createUser1.id);
            using (var client = GetClient())
            {
                client.lockUserAccounts(new Guid[] { createUser1.id });
                var user = client.GetUserByUserId(createUser1.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.is_locked, true);
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the lock user accounts with invalid user identifiers.
        /// </summary>
        [TestMethod]
        public void lockUserAccounts_WithInvalidUserIds()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.lockUserAccounts(new Guid[] { WrongUserId });
                    var user = client.GetUserByUserId(WrongUserId);
                    Assert.IsNotNull(user);
                }
                catch (Exception Ex)
                {
                    Assert.IsNotNull(Ex.Message);
                    Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidUserIDErrorMsg));
                }
            }
        }
        #endregion

        #endregion

        #region lockAllUserAccounts

        #region positive
        /// <summary>
        /// This method is used to test the lock all user accounts.
        /// </summary>
        [TestMethod]
        public void lockAllUserAccounts()
        {
            using (var client = GetClient())
            {
                  client.LockAllUserAccounts(TenantId);               
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the lock all user accounts with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void lockAllUserAccounts_WithInvalidTenantId()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.LockAllUserAccounts(WrongTenantId);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.UnauthorizedAccessExceptionErrorCode);  
                }
            }
        }
        #endregion

        #endregion

        #region UnLockAllUserAccounts

        #region positive
        /// <summary>
        /// This method is used to test the unlock all user accounts.
        /// </summary>
        [TestMethod]
        public void UnLockAllUserAccounts()
        {
            using (var client = GetClient())
            {
                    client.UnLockAllUserAccounts(TenantId);
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the unlock all user accounts with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void UnlockAllUserAccounts_WithInvalidTenantId()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.UnLockAllUserAccounts(WrongTenantId);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.UnauthorizedAccessExceptionErrorCode);  
                }
            }
        }
        #endregion

        #endregion

        #region ForceUserPasswordReset

        #region positive
        /// <summary>
        /// This method is used to test the force user password reset.
        /// </summary>
        [TestMethod]
        public void ForceUserPasswordReset()
        {
            var createUser1 = CreateUser();

            Assert.IsNotNull(createUser1);
            Assert.AreNotEqual(Guid.Empty, createUser1.id);

            var createUser2 = CreateUser();

            Assert.IsNotNull(createUser2);
            Assert.AreNotEqual(Guid.Empty, createUser2.id);
            using (var client = GetClient())
            {
                client.ForceUserPasswordReset(new Guid[] { createUser1.id, createUser2.id });
            }
        }

        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the force user password reset with invalid user identifiers.
        /// </summary>
        [TestMethod]
        public void ForceUserPasswordReset_WithInvalidUserIds()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.ForceUserPasswordReset(new Guid[] {WrongUserId,WrongTenantId});
                }
            }
            catch (Exception ex)
            {
                Assert.IsTrue(true);
            }
        }
        #endregion

        #endregion

        #region PostUser

        #region positive
        /// <summary>
        /// This method is used to test the post user.
        /// </summary>
        [TestMethod]
        public void Test_PostUser()
        {
            CreateUser();
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the post usser without first name.
        /// </summary>
        [TestMethod]
        public void PostUser_WithoutFirstName()
        {
            var rand = new Random();
            using (var client = GetClient())
            {
                var uname = "user" + rand.Next();

                try
                {
                    client.CreateUser(new UserDetailsDTO
                    {
                        first_name = "",
                        last_name = "last name",
                        username = uname,
                        email = uname + "@techcello.com",
                        description = "created via web api",
                        status = true,
                        address = new AddressDTO
                        {
                            address1 = "address1",
                            city = "city",
                            state = "state",
                            country = "India",
                            zipcode = "12456",
                            status = true
                        }
                    });
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
            }
        }
        /// <summary>
        /// This method is used to test the post user with invalid email identifier.
        /// </summary>
        [TestMethod]
        public void PostUser_InValidEmailID()
        {
            var rand = new Random();
            using (var client = GetClient())
            {
                var uname = "user" + rand.Next();

                try
                {
                    client.CreateUser(new UserDetailsDTO
                    {
                        first_name = "first name",
                        last_name = "last name",
                        username = uname,
                        email = uname + "@.com",
                        description = "created via web api",
                        status = true,
                        address = new AddressDTO
                        {
                            address1 = "address1",
                            city = "city",
                            state = "state",
                            country = "India",
                            zipcode = "12456",
                            status = true
                        }
                    });
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
            }
        }
        /// <summary>
        /// This method is used to test the post user with empty parameters.
        /// </summary>
        [TestMethod]
        public void Postuser_EmptyParameters()
        {
            var rand = new Random();
            using (var client = GetClient())
            {
                var uname = "user" + rand.Next();
                try
                {
                    client.CreateUser(new UserDetailsDTO
                    {
                        first_name = "",
                        last_name = "",
                        username = "",
                        email = uname + "",
                        description = "",
                        status = true,
                        address = new AddressDTO
                        {
                            address1 = "",
                            city = "",
                            state = "",
                            country = "",
                            zipcode = "",
                            status = true
                        }
                    });
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
                }
            }
        }
        /// <summary>
        /// This method is used to test the post user with mandatory fields.
        /// </summary>
        [TestMethod]
        public void Postuser_MandatoryFields()
        {
            var _checkAfterDelete = new UserDetailsDTO();
            var uname = "admin";
            using (var client = GetClient())
            {
                try
                {
                    var _postedData = client.CreateUser(new UserDetailsDTO
                    {
                        first_name = null,
                        last_name = "last name",
                        username = uname,
                        email = uname + "@techcello.com",
                        description = "created via web api",
                        status = true,
                        address = new AddressDTO
                        {
                            address1 = "address1",
                            city = "city",
                            state = "state",
                            country = "India",
                            zipcode = "12456",
                            status = true
                        }

                    });
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
                }
                try
                {
                    var _postedData = client.CreateUser(new UserDetailsDTO
                    {
                        first_name = uname,
                        last_name = null,
                        username = uname,
                        email = uname + "@techcello.com",
                        description = "created via web api",
                        status = true,
                        address = new AddressDTO
                        {
                            address1 = "address1",
                            city = "city",
                            state = "state",
                            country = "India",
                            zipcode = "12456",
                            status = true
                        }

                    });
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
                }
                try
                {
                    var _postedData = client.CreateUser(new UserDetailsDTO
                    {
                        first_name = uname,
                        last_name = "last name",
                        username = null,
                        email = uname + "@techcello.com",
                        description = "created via web api",
                        status = true,
                        address = new AddressDTO
                        {
                            address1 = "address1",
                            city = "city",
                            state = "state",
                            country = "India",
                            zipcode = "12456",
                            status = true
                        }

                    });
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
                }
                try
                {
                    var _postedData = client.CreateUser(new UserDetailsDTO
                    {
                        first_name = uname,
                        last_name = "last name",
                        username = uname,
                        email = null,
                        description = "created via web api",
                        status = true,
                        address = new AddressDTO
                        {
                            address1 = "address1",
                            city = "city",
                            state = "state",
                            country = "India",
                            zipcode = "12456",
                            status = true
                        }

                    });
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
                }
            }
        }
        #endregion

        #endregion

        #region Test_UpdateUser

        #region positive
        /// <summary>
        /// This method is used to test the update user.
        /// </summary>
        [TestMethod]
        public void Test_UpdateUser()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);

            using (var client = GetClient())
            {
                client.UpdateFirstTimeLogOn(createUser.id, Password, SecurityQuestion, SecurityAnswer, true, createUser.tenant_id);
                createUser = client.GetUserByUserId(createUser.id);
                var rand = new Random();
                var uname = "user" + rand.Next();
                createUser.username = uname;
                client.UpdateUser(createUser);

                var userdetails = client.GetUserByUsername(createUser.username);
                UserDetailsEqual(createUser, userdetails);
            }
        }

        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the update user with null values.
        /// </summary>
        [TestMethod]
        public void UpdateUser_UpdateNullValues()
        {
            var rand = new Random();
            using (var client = GetClient())
            {
                var uname = "user" + rand.Next();
                client.CreateUser(new UserDetailsDTO
                {
                    first_name = uname,
                    last_name = "last name",
                    username = uname,
                    email = uname + "@techcello.com",
                    description = "created via web api",
                    status = true,
                    address = new AddressDTO
                    {
                        address1 = "address1",
                        city = "city",
                        state = "state",
                        country = "India",
                        zipcode = "12456",
                        status = true
                    }
                });
                var user = client.GetUserByUsername(uname);
                Assert.IsNotNull(user);
                try
                {
                    user.first_name = null;
                    user.last_name = null;
                    user.username = null;
                    user.email = null;
                    user.description = null;
                    user.status = false;
                    user.address.address1 = null;
                    user.address.city = null;
                    user.address.state = null;
                    user.address.country = null;
                    user.address.zipcode = null;
                    user.address.status = false;
                    client.UpdateUser(user);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
                }
            }

        }
        /// <summary>
        /// This method is used to test the update user with null object.
        /// </summary>
        [TestMethod]
        public void UpdateUser_UpdateBothNullObjects()
        {
            var rand = new Random();
            using (var client = GetClient())
            {
                var uname = "user" + rand.Next();
                client.CreateUser(new UserDetailsDTO
                {
                    first_name = uname,
                    last_name = "last name",
                    username = uname,
                    email = uname + "@techcello.com",
                    description = "created via web api",
                    status = true,
                    address = new AddressDTO
                    {
                        address1 = "address1",
                        city = "city",
                        state = "state",
                        country = "India",
                        zipcode = "12456",
                        status = true
                    }
                });
                try
                {
                    var user = client.GetUserByUsername(uname);
                    Assert.IsNotNull(user);
                    user = null;
                    client.UpdateUser(user);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentNullErrorCode);
                }
            }

        }
        #endregion

        #endregion

        #region DeleteUser

        #region positive
        /// <summary>
        /// This method is used to test the delete user.
        /// </summary>
        [TestMethod]
        public void Test_DeleteUser()
        {
            var createUser = CreateUser();

            Assert.IsNotNull(createUser);
            Assert.AreNotEqual(Guid.Empty, createUser.id);

            using (var client = GetClient())
            {
                client.DeleteUser(createUser.id);

                var user = client.GetUserByUserId(createUser.id);
                Assert.IsNotNull(user);
                Assert.AreEqual(user.status, false);
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the delete user with invalid user identifier.
        /// </summary>
        [TestMethod]
        public void Test_DeleteUser_WithInvalidUserId()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.DeleteUser(WrongUserId);
                }
            }
            catch (Exception Ex)
            {
                Assert.IsNotNull(Ex.Message);
                Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvalidUserIDErrorMsg));
            }
        }
        #endregion

        #endregion

        #region Test_SearchUser

        #region positive
        /// <summary>
        /// This method is used to test the search user.
        /// </summary>
        [TestMethod]
        public void Test_SearchUser()
        {
            using (var client = GetClient())
            {
                var users = client.Search(new UserDetailsSearchConditionDTO
                {
                    page_no = 1,
                    page_size = 25,
                    search_field = UserDetailsSearchField.user_name,
                    search_value = UserName
                });

                Assert.IsNotNull(users);
                Assert.IsNotNull(users.result);
                Assert.AreNotEqual(users.total_count, 0);
            }
        }
        #endregion

        #region negative
        /// <summary>
        /// This method is used to test the search user with invalid condition.
        /// </summary>
        [TestMethod]
        public void Test_SearchUser_WithInvalidCondition()
        {
                using (var client = GetClient())
                {
                    var users = client.Search(new UserDetailsSearchConditionDTO
                    {
                        page_no = 1562,
                        page_size = 1540,
                        search_field = UserDetailsSearchField.user_name,
                        search_value = InvalidUserName
                    });
                    Assert.AreEqual(users.result, null);
                }           
        }

        #endregion

        #endregion

        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static UserDetailsClient GetClient()
        {
             var client = new UserDetailsClient(
                  baseAddress: UserDetailsClientBaseAddress,
                  username: ClientUserName,
                  api_key: APIKey);
            return client;
        }

        #region private methods
        /// <summary>
        /// This method is used to create a user.
        /// </summary>
        /// <returns>user details</returns>
        public static UserDetailsDTO CreateUser()
        {
            Guid userId = Guid.Empty;
            var rand = new Random();
            var uname = "user" + rand.Next();
            var user = new UserDetailsDTO()
            {
                first_name = uname,
                last_name = "last name",
                username = uname,
                email = uname + "@techcello1.com",
                description = "created via web api",
                status = true,
                address = new AddressDTO
                {
                    address1 = "address1",
                    city = "city",
                    state = "state",
                    country = "India",
                    zipcode = "12456",
                    status = true
                }
            };
            using (var client = GetClient())
            {

                user.id = client.CreateUser(user);
            }
            return user;
        }
        /// <summary>
        /// This method is used to compare 2 userdetails.
        /// </summary>
        /// <param name="source">first user detail</param>
        /// <param name="secondSource">second user detail</param>

        public static void UserDetailsEqual(UserDetailsDTO source, UserDetailsDTO secondSource)
        {
            Assert.IsNotNull(source);
            Assert.IsNotNull(secondSource);

            Assert.AreEqual(source.username, secondSource.username);
            Assert.AreEqual(source.id, secondSource.id);

            Assert.AreEqual(source.first_name, secondSource.first_name);
            Assert.AreEqual(source.last_name, secondSource.last_name);
            Assert.AreEqual(source.description, secondSource.description);
            Assert.AreEqual(source.password, secondSource.password);
            Assert.AreEqual(source.email, secondSource.email);
            AddressEqual(source.address, secondSource.address);

        }

        /// <summary>
        /// This method is used to compare 2 address details.
        /// </summary>
        /// <param name="source">first address detail</param>
        /// <param name="secondSource">second address detail</param>
        public static void AddressEqual(AddressDTO source, AddressDTO secondSource)
        {
            if (source == null)
            {
                Assert.IsNull(secondSource);
            }
            {
                Assert.IsNotNull(source);
                Assert.IsNotNull(secondSource);
                Assert.AreEqual(source.address1, secondSource.address1);
                Assert.AreEqual(source.state, secondSource.state);
                Assert.AreEqual(source.country, secondSource.country);
                Assert.AreEqual(source.zipcode, secondSource.zipcode);
            }

        }

        #endregion

    }
}
