IF NOT EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'StringTable' AND ss.name = N'dbo')
CREATE TYPE [dbo].[StringTable] AS TABLE(
	[StringId] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL
)
IF NOT EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'UniqueTable' AND ss.name = N'dbo')
CREATE TYPE [dbo].[UniqueTable] AS TABLE(
	[UniqueId] [uniqueidentifier] NOT NULL
)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ExceptionLog]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ExceptionLog](
	[LogID] [int] IDENTITY(1,1) NOT NULL,
	[TenantId] [uniqueidentifier] NULL,
	[UserID] [uniqueidentifier] NULL,
	[EventID] [int] NULL,
	[Priority] [int] NOT NULL,
	[Severity] [nvarchar](32) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Title] [nvarchar](256) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Timestamp] [datetime] NOT NULL,
	[MachineName] [nvarchar](32) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[AppDomainName] [nvarchar](512) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ProcessID] [nvarchar](256) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ProcessName] [nvarchar](512) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ThreadName] [nvarchar](512) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Win32ThreadId] [nvarchar](128) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Message] [nvarchar](1500) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FormattedMessage] [ntext] COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_Log] PRIMARY KEY CLUSTERED 
(
	[LogID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ExceptionCategory]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ExceptionCategory](
	[CategoryID] [int] IDENTITY(1,1) NOT NULL,
	[CategoryName] [nvarchar](64) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
 CONSTRAINT [PK_Categories] PRIMARY KEY CLUSTERED 
(
	[CategoryID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ExceptionCategoryLog]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ExceptionCategoryLog](
	[CategoryLogID] [int] IDENTITY(1,1) NOT NULL,
	[CategoryID] [int] NOT NULL,
	[LogID] [int] NOT NULL,
 CONSTRAINT [PK_CategoryLog] PRIMARY KEY CLUSTERED 
(
	[CategoryLogID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ExceptionCategoryLog]') AND name = N'ixCategoryLog')
CREATE NONCLUSTERED INDEX [ixCategoryLog] ON [dbo].[ExceptionCategoryLog] 
(
	[LogID] ASC,
	[CategoryID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[InsertExceptionCategoryLog]') AND type in (N'P', N'PC'))
BEGIN
EXEC dbo.sp_executesql @statement = N'CREATE PROCEDURE InsertExceptionCategoryLog
	@CategoryID INT,
	@LogID INT
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @CatLogID INT
	SELECT @CatLogID FROM ExceptionCategoryLog WHERE CategoryID=@CategoryID and LogID = @LogID
	IF @CatLogID IS NULL
	BEGIN
		INSERT INTO ExceptionCategoryLog (CategoryID, LogID) VALUES(@CategoryID, @LogID)
		RETURN @@IDENTITY
	END
	ELSE RETURN @CatLogID
END
' 
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddExceptionCategory]') AND type in (N'P', N'PC'))
BEGIN
EXEC dbo.sp_executesql @statement = N'

CREATE PROCEDURE [dbo].[AddExceptionCategory]
	-- Add the parameters for the function here
	@CategoryName nvarchar(64),
	@LogID int
AS
BEGIN
	SET NOCOUNT ON;
    DECLARE @CatID INT
	SELECT @CatID = CategoryID FROM ExceptionCategory WHERE CategoryName = @CategoryName
	IF @CatID IS NULL
	BEGIN
		INSERT INTO ExceptionCategory (CategoryName) VALUES(@CategoryName)
		SELECT @CatID = @@IDENTITY
	END

	EXEC InsertExceptionCategoryLog @CatID, @LogID 

	RETURN @CatID
END

' 
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ClearExceptionLogs]') AND type in (N'P', N'PC'))
BEGIN
EXEC dbo.sp_executesql @statement = N'CREATE PROCEDURE ClearExceptionLogs
AS
BEGIN
	SET NOCOUNT ON;

	DELETE FROM ExceptionCategoryLog
	DELETE FROM [ExceptionLog]
    DELETE FROM ExceptionCategory
END
' 
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Country]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Country](
	[Country_ID] [uniqueidentifier] NOT NULL,
	[Country_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Country_CreatedBy] [uniqueidentifier] NOT NULL,
	[Country_CreatedOn] [datetime] NOT NULL,
	[Country_UpdatedBy] [uniqueidentifier] NULL,
	[Country_UpdatedOn] [datetime] NULL,
	[Country_Status] [bit] NOT NULL,
 CONSTRAINT [PK_Country] PRIMARY KEY CLUSTERED 
(
	[Country_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Country_Country_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Country] ADD  CONSTRAINT [DF_Country_Country_ID]  DEFAULT (newid()) FOR [Country_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Address]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Address](
	[Address_ID] [uniqueidentifier] NOT NULL,
	[Address_Address1] [nvarchar](2000) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Address_Address2] [nvarchar](2000) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Address_City] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Address_State] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Address_CountryId] [uniqueidentifier] NOT NULL,
	[Address_PostalCode] [varchar](15) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Address_CreatedBy] [uniqueidentifier] NOT NULL,
	[Address_CreatedOn] [datetime] NOT NULL,
	[Address_UpdatedBy] [uniqueidentifier] NULL,
	[Address_UpdatedOn] [datetime] NULL,
	[Address_Status] [bit] NOT NULL,
	[Address_UniqueId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_Address] PRIMARY KEY CLUSTERED 
(
	[Address_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Address]') AND name = N'Ind_Address_CountryId')
CREATE NONCLUSTERED INDEX [Ind_Address_CountryId] ON [dbo].[Address] 
(
	[Address_CountryId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Address_Address_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Address] ADD  CONSTRAINT [DF_Address_Address_ID]  DEFAULT (newid()) FOR [Address_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_GetAddressIds]') AND type in (N'P', N'PC'))
BEGIN
EXEC dbo.sp_executesql @statement = N'--Procedure to Get Address Ids
CREATE PROCEDURE [dbo].[sp_GetAddressIds]
@dtAddressUniqueId UniqueTable readonly
as
Begin
    Select Address_ID, Address_UniqueID from Address
    where Address_UniqueID in (select UniqueId from @dtAddressUniqueId)
End

' 
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PaymentType]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PaymentType](
	[PaymentType_Code] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PaymentType_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PaymentType_CreatedOn] [datetime] NOT NULL,
	[PaymentType_CreatedBy] [uniqueidentifier] NOT NULL,
	[PaymentType_UpdatedOn] [datetime] NULL,
	[PaymentType_UpdatedBy] [uniqueidentifier] NULL,
	[PaymentType_Status] [bit] NOT NULL,
 CONSTRAINT [PK__PaymentT__0D43BC3B1B0907CE] PRIMARY KEY CLUSTERED 
(
	[PaymentType_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Contact]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Contact](
	[Contact_ID] [uniqueidentifier] NOT NULL,
	[Contact_FirstName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Contact_LastName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Contact_Email] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Contact_Phone] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Contact_Fax] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Contact_CreatedBy] [uniqueidentifier] NOT NULL,
	[Contact_CreatedOn] [datetime] NOT NULL,
	[Contact_UpdatedBy] [uniqueidentifier] NULL,
	[Contact_UpdatedOn] [datetime] NULL,
	[Contact_Status] [bit] NOT NULL,
 CONSTRAINT [PK_Contact] PRIMARY KEY CLUSTERED 
(
	[Contact_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Contact_Contact_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Contact] ADD  CONSTRAINT [DF_Contact_Contact_ID]  DEFAULT (newid()) FOR [Contact_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantDetails](
	[Tenant_Code] [uniqueidentifier] NOT NULL,
	[Tenant_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Tenant_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Tenant_AddressID] [uniqueidentifier] NULL,
	[Tenant_Website] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Tenant_ContactID] [uniqueidentifier] NULL,
	[Tenant_BillingType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Tenant_PaymentTypeCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Tenant_BillOther] [bit] NULL,
	[Tenant_BillingContactID] [uniqueidentifier] NULL,
	[Tenant_BillingAddressID] [uniqueidentifier] NULL,
	[Tenant_IsProductAdmin] [bit] NULL,
	[Tenant_URL] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Tenant_CreatedBy] [uniqueidentifier] NOT NULL,
	[Tenant_CreatedOn] [datetime] NOT NULL,
	[Tenant_UpdatedOn] [datetime] NULL,
	[Tenant_UpdatedBy] [uniqueidentifier] NULL,
	[Tenant_Status] [bit] NOT NULL,
	[Tenant_CodeString] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Tenant_ApprovalStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Tenant_ParentTenantId] [uniqueidentifier] NULL,
	[Tenant_Size] [nvarchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Tenant_EnableAutoDebit] [bit] NOT NULL,
	[Tenant_IsSelfRegistered] [bit] NOT NULL,
	[Tenant_DataPartitionId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_TenantDetails] PRIMARY KEY CLUSTERED 
(
	[Tenant_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TenantDetails]') AND name = N'Ind_Tenant_Name')
CREATE NONCLUSTERED INDEX [Ind_Tenant_Name] ON [dbo].[TenantDetails] 
(
	[Tenant_Name] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TenantDetails]') AND name = N'Ind_Tenant_URL')
CREATE NONCLUSTERED INDEX [Ind_Tenant_URL] ON [dbo].[TenantDetails] 
(
	[Tenant_URL] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantDetails_Tenant_Code]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantDetails] ADD  CONSTRAINT [DF_TenantDetails_Tenant_Code]  DEFAULT (newid()) FOR [Tenant_Code]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__TenantDet__Tenan__78D3EB5B]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantDetails] ADD  DEFAULT ((0)) FOR [Tenant_EnableAutoDebit]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__TenantDet__Tenan__79C80F94]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantDetails] ADD  DEFAULT ((0)) FOR [Tenant_IsSelfRegistered]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Membership]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Membership](
	[Membership_ID] [uniqueidentifier] NOT NULL,
	[Membership_TenantID] [uniqueidentifier] NOT NULL,
	[Membership_UserName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Membership_EmailID] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Membership_Comment] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Membership_Password] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Membership_PasswordQuestion] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Membership_PasswordAnswer] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Membership_IsApproved] [bit] NULL,
	[Membership_LastActivityDate] [datetime] NULL,
	[Membership_LastLoginDate] [datetime] NULL,
	[Membership_LastPasswordChangedDate] [datetime] NULL,
	[Membership_CreationDate] [datetime] NULL,
	[Membership_IsLockedOut] [bit] NULL,
	[Membership_LastLockedOutDate] [datetime] NULL,
	[Membership_FailedPasswordAttemptCount] [int] NULL,
	[Membership_FailedPasswordAttemptWindowStart] [datetime] NULL,
	[Membership_FailedPasswordAnswerAttemptCount] [int] NULL,
	[Membership_FailedPasswordAnswerAttemptWindowStart] [datetime] NULL,
	[Membership_Status] [bit] NOT NULL,
	[Membership_IsFirstTimeUser] [bit] NOT NULL,
	[Membership_IsUserPasswordChangeForced] [bit] NOT NULL,
 CONSTRAINT [PK_Membership] PRIMARY KEY CLUSTERED 
(
	[Membership_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Membership]') AND name = N'Ind_Membership_EmailID')
CREATE NONCLUSTERED INDEX [Ind_Membership_EmailID] ON [dbo].[Membership] 
(
	[Membership_EmailID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Membership]') AND name = N'Ind_Membership_TenantID')
CREATE NONCLUSTERED INDEX [Ind_Membership_TenantID] ON [dbo].[Membership] 
(
	[Membership_TenantID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Membership]') AND name = N'Ind_Membership_UserName')
CREATE NONCLUSTERED INDEX [Ind_Membership_UserName] ON [dbo].[Membership] 
(
	[Membership_UserName] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Membership]') AND name = N'NonClusteredIndex-LastActivityDate')
CREATE NONCLUSTERED INDEX [NonClusteredIndex-LastActivityDate] ON [dbo].[Membership] 
(
	[Membership_LastActivityDate] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Membership]') AND name = N'NonClusteredIndex-Status')
CREATE NONCLUSTERED INDEX [NonClusteredIndex-Status] ON [dbo].[Membership] 
(
	[Membership_Status] ASC
)
INCLUDE ( [Membership_TenantID]) WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Membership_Membership_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Membership] ADD  CONSTRAINT [DF_Membership_Membership_ID]  DEFAULT (newid()) FOR [Membership_ID]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Membershi__Membe__3335971A]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Membership] ADD  DEFAULT ((1)) FOR [Membership_IsFirstTimeUser]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Membership_Membership_IsPasswordForced]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Membership] ADD  CONSTRAINT [DF_Membership_Membership_IsPasswordForced]  DEFAULT ((0)) FOR [Membership_IsUserPasswordChangeForced]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_GetMembershipIds]') AND type in (N'P', N'PC'))
BEGIN
EXEC dbo.sp_executesql @statement = N'--Procedure to Get Membership Ids
CREATE PROCEDURE [dbo].[sp_GetMembershipIds]
@dtUserName StringTable readonly,
@tenantId varchar(50)
as
Begin
    Select Membership_ID, Membership_UserName from Membership 
    where Membership_UserName in (select StringId from @dtUserName) and Membership_TenantID = @tenantId
End

' 
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WriteExceptionLog]') AND type in (N'P', N'PC'))
BEGIN
EXEC dbo.sp_executesql @statement = N'
/****** Object:  Stored Procedure dbo.WriteLog  ******/

CREATE PROCEDURE [dbo].[WriteExceptionLog]
(
	@EventID int, 
	@TenantId uniqueidentifier,
	@UserID uniqueidentifier,
	@Priority int, 
	@Severity nvarchar(32), 
	@Title nvarchar(256), 
	@Timestamp datetime,
	@MachineName nvarchar(32), 
	@AppDomainName nvarchar(512),
	@ProcessID nvarchar(256),
	@ProcessName nvarchar(512),
	@ThreadName nvarchar(512),
	@Win32ThreadId nvarchar(128),
	@Message nvarchar(1500),
	@FormattedMessage ntext,
	@LogId int OUTPUT
)
AS 

	INSERT INTO [ExceptionLog] (
		TenantId,
		UserID,
		EventID,
		Priority,
		Severity,
		Title,
		[Timestamp],
		MachineName,
		AppDomainName,
		ProcessID,
		ProcessName,
		ThreadName,
		Win32ThreadId,
		Message,
		FormattedMessage
	)
	VALUES (
		@TenantId,
		@UserID,
		@EventID, 
		@Priority, 
		@Severity, 
		@Title, 
		@Timestamp,
		@MachineName, 
		@AppDomainName,
		@ProcessID,
		@ProcessName,
		@ThreadName,
		@Win32ThreadId,
		@Message,
		@FormattedMessage)

	SET @LogID = @@IDENTITY
	RETURN @LogID
' 
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Activities]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Activities](
	[ActivityId] [uniqueidentifier] NOT NULL,
	[Name] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Type] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[WorkflowId] [uniqueidentifier] NULL,
	[TypeName] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[AssemblyName] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
	[TenantId] [uniqueidentifier] NULL,
	[Outcomes] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[KeyValues] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_Activities] PRIMARY KEY CLUSTERED 
(
	[ActivityId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Activities_ActivityId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Activities] ADD  CONSTRAINT [DF_Activities_ActivityId]  DEFAULT (newid()) FOR [ActivityId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ActivityContentType]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ActivityContentType](
	[ActivityContentType_ID] [smallint] NOT NULL,
	[ActivityContentType_Description] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
 CONSTRAINT [PK_star_ActivityContentType] PRIMARY KEY CLUSTERED 
(
	[ActivityContentType_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Events]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Events](
	[Event_Id] [uniqueidentifier] NOT NULL,
	[Event_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Event_Category] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Event_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Event_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_Event] PRIMARY KEY CLUSTERED 
(
	[Event_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Events]') AND name = N'EventName')
CREATE NONCLUSTERED INDEX [EventName] ON [dbo].[Events] 
(
	[Event_Name] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Event_Event_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Events] ADD  CONSTRAINT [DF_Event_Event_Id]  DEFAULT (newid()) FOR [Event_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ActivityEvents]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ActivityEvents](
	[ActivityEvent_Id] [uniqueidentifier] NOT NULL,
	[ActivityEvent_EventId] [uniqueidentifier] NOT NULL,
	[ActivityEvent_Description] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ActivityEvent_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ActivityEvents] PRIMARY KEY CLUSTERED 
(
	[ActivityEvent_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF),
 CONSTRAINT [IX_ActivityEvents] UNIQUE NONCLUSTERED 
(
	[ActivityEvent_EventId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ActivityEvents_ActivityEvent_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ActivityEvents] ADD  CONSTRAINT [DF_ActivityEvents_ActivityEvent_Id]  DEFAULT (newid()) FOR [ActivityEvent_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ActivityEvents_ActivityEvent_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ActivityEvents] ADD  CONSTRAINT [DF_ActivityEvents_ActivityEvent_Status]  DEFAULT ((1)) FOR [ActivityEvent_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ActivityLogs]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ActivityLogs](
	[ActivityLog_Id] [uniqueidentifier] NOT NULL,
	[ActivityLog_EventId] [uniqueidentifier] NOT NULL,
	[ActivityLog_LogTime] [datetime] NOT NULL,
	[ActivityLog_Comments] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ActivityLog_UserId] [uniqueidentifier] NULL,
	[ActivityLog_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ActivityLogs] PRIMARY KEY CLUSTERED 
(
	[ActivityLog_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ActivityLogs_ActivityLog_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ActivityLogs] ADD  CONSTRAINT [DF_ActivityLogs_ActivityLog_Id]  DEFAULT (newid()) FOR [ActivityLog_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ActivityLogs_ActivityLog_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ActivityLogs] ADD  CONSTRAINT [DF_ActivityLogs_ActivityLog_Status]  DEFAULT ((1)) FOR [ActivityLog_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ActivityTrail]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ActivityTrail](
	[ActivityTrail_ID] [int] IDENTITY(1,1) NOT NULL,
	[ActivityTrail_ActionTargetUserId] [uniqueidentifier] NULL,
	[ActivityTrail_ActionTakingUserID] [uniqueidentifier] NOT NULL,
	[ActivityTrail_ActionID] [int] NOT NULL,
	[ActivityTrail_ActionDate] [datetime] NOT NULL,
	[ActivityTrail_Notes] [varchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ActivityTrail_ContentTypeID] [smallint] NULL,
	[ActivityTrail_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_ActivityTrial] PRIMARY KEY CLUSTERED 
(
	[ActivityTrail_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Entity]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Entity](
	[Entity_ID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Entity_SchemaTableName] [varchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Entity_ExtendedSchemaTableName] [varchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Entity_PrimaryKeyName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Entity_TypeName] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Entity_AssemblyName] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Entity_CreatedOn] [datetime] NULL,
	[Entity_CreatedBy] [uniqueidentifier] NULL,
	[Entity_UpdatedOn] [datetime] NULL,
	[Entity_UpdatedBy] [uniqueidentifier] NULL,
	[Entity_Status] [bit] NULL,
	[Entity_IsTenantBasedExtn] [bit] NULL,
	[Entity_IsExtendable] [bit] NULL,
	[Entity_ExtnSchemaTableConnectionName] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Entity_SchemaTableConnectionStringName] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Entity_DisplayColumnName] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Entity_TenantIdColumnName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Entity_TenantId] [uniqueidentifier] NULL,
	[Entity_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_Entity] PRIMARY KEY CLUSTERED 
(
	[Entity_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Entity]') AND name = N'Ind_Entity_IsTenantBasedExtn')
CREATE NONCLUSTERED INDEX [Ind_Entity_IsTenantBasedExtn] ON [dbo].[Entity] 
(
	[Entity_IsTenantBasedExtn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddressExtn]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[AddressExtn](
	[AddressExtn_Id] [uniqueidentifier] NOT NULL,
	[AddressExtn_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[AddressExtn_ReferenceId] [uniqueidentifier] NOT NULL,
	[AddressExtn_TenantId] [uniqueidentifier] NOT NULL,
	[AddressExtn_Column1] [int] NULL,
	[AddressExtn_Column2] [int] NULL,
	[AddressExtn_Column3] [int] NULL,
	[AddressExtn_Column4] [int] NULL,
	[AddressExtn_Column5] [int] NULL,
	[AddressExtn_Column6] [datetime] NULL,
	[AddressExtn_Column7] [datetime] NULL,
	[AddressExtn_Column8] [datetime] NULL,
	[AddressExtn_Column9] [datetime] NULL,
	[AddressExtn_Column10] [datetime] NULL,
	[AddressExtn_Column11] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column12] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column13] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column14] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column15] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column16] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column17] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column18] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column19] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column20] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column21] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column22] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column23] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column24] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column25] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column26] [float] NULL,
	[AddressExtn_Column27] [float] NULL,
	[AddressExtn_Column28] [float] NULL,
	[AddressExtn_Column29] [float] NULL,
	[AddressExtn_Column30] [float] NULL,
	[AddressExtn_Column31] [bit] NULL,
	[AddressExtn_Column32] [bit] NULL,
	[AddressExtn_Column33] [bit] NULL,
	[AddressExtn_Column34] [bit] NULL,
	[AddressExtn_Column35] [bit] NULL,
	[AddressExtn_Column36] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column37] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column38] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column39] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_Column40] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AddressExtn_CreatedOn] [datetime] NOT NULL,
	[AddressExtn_CreatedBy] [uniqueidentifier] NOT NULL,
	[AddressExtn_UpdatedOn] [datetime] NULL,
	[AddressExtn_UpdatedBy] [uniqueidentifier] NULL,
	[AddressExtn_Status] [bit] NOT NULL,
 CONSTRAINT [PK_AddressExtn1] PRIMARY KEY CLUSTERED 
(
	[AddressExtn_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_AddressExtn_AddressExtn_Id1]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[AddressExtn] ADD  CONSTRAINT [DF_AddressExtn_AddressExtn_Id1]  DEFAULT (newid()) FOR [AddressExtn_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AttributeSet]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[AttributeSet](
	[AttributeSet_ID] [uniqueidentifier] NOT NULL,
	[AttributeSet_Name] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[AttributeSet_CreatedBy] [uniqueidentifier] NOT NULL,
	[AttributeSet_CreatedOn] [datetime] NOT NULL,
	[AttributeSet_UpdatedBy] [uniqueidentifier] NULL,
	[AttributeSet_UpdatedOn] [datetime] NULL,
	[AttributeSet_Status] [bit] NOT NULL,
 CONSTRAINT [PK_AttributeSet] PRIMARY KEY CLUSTERED 
(
	[AttributeSet_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[BackupRequest]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[BackupRequest](
	[RequestId] [uniqueidentifier] NOT NULL,
	[Source] [text] COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DestinationType] [varchar](10) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DestinationDetails] [text] COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [text] COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[PrimaryTenantId] [uniqueidentifier] NOT NULL,
	[TenantIds] [text] COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[BackupMode] [varchar](10) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RequestDate] [datetime] NOT NULL,
	[RequestBy] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[StartDate] [datetime] NULL,
	[CompletedDate] [datetime] NULL,
	[BackupSize] [float] NULL,
	[BackupStatus] [varchar](15) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ExceptionDetails] [text] COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_BackupRequest] PRIMARY KEY CLUSTERED 
(
	[RequestId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_BackupRequest_RequestId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[BackupRequest] ADD  CONSTRAINT [DF_BackupRequest_RequestId]  DEFAULT (newid()) FOR [RequestId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationBatch]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationBatch](
	[NotificationBatch_Id] [uniqueidentifier] NOT NULL,
	[NotificationBatch_NotificationContentType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[NotificationBatch_NotificationDispatchType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[NotificationBatch_Comments] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[NotificationBatch_Priority] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[NotificationBatch_Combine] [bit] NULL,
	[NotificationBatch_DispatchedStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[NotificationBatch_CreatedBy] [uniqueidentifier] NOT NULL,
	[NotificationBatch_CreatedOn] [datetime] NOT NULL,
	[NotificationBatch_UpdatedBy] [uniqueidentifier] NULL,
	[NotificationBatch_UpdatedOn] [datetime] NULL,
	[NotificationBatch_Status] [bit] NOT NULL,
 CONSTRAINT [PK_NotificationBatch] PRIMARY KEY CLUSTERED 
(
	[NotificationBatch_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_NotificationBatch_NotificationBatch_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationBatch] ADD  CONSTRAINT [DF_NotificationBatch_NotificationBatch_Id]  DEFAULT (newid()) FOR [NotificationBatch_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[BatchFtpDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[BatchFtpDetails](
	[BatchFtpDetails_BatchId] [uniqueidentifier] NOT NULL,
	[BatchFtpDetails_FtpAddress] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[BatchFtpDetails_FtpUserName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[BatchFtpDetails_FtpPassword] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[BatchFtpDetails_FilePath] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[BatchFtpDetails_FileName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[BatchFtpDetails_FtpStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[BatchFtpDetails_StatusMessage] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[BatchFtpDetails_CreatedBy] [uniqueidentifier] NOT NULL,
	[BatchFtpDetails_CreatedOn] [datetime] NOT NULL,
	[BatchFtpDetails_UpdatedBy] [uniqueidentifier] NULL,
	[BatchFtpDetails_UpdatedOn] [datetime] NULL,
	[BatchFtpDetails_Status] [bit] NOT NULL,
	[BatchFtpDetails_IsSecured] [bit] NOT NULL,
 CONSTRAINT [PK_BatchFtpDetails] PRIMARY KEY CLUSTERED 
(
	[BatchFtpDetails_BatchId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[BillingAudits]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[BillingAudits](
	[Id] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NULL,
	[ParentTenantId] [uniqueidentifier] NULL,
	[PackageId] [uniqueidentifier] NULL,
	[PricePlanId] [uniqueidentifier] NULL,
	[InvoiceId] [uniqueidentifier] NULL,
	[InvoiceNo] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[BillingFrequency] [int] NULL,
	[InvoiceDate] [datetime] NOT NULL,
	[Duration] [float] NULL,
	[ExceptionDetails] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TimeStamp] [datetime] NOT NULL,
	[BillingAuditStatus] [int] NOT NULL,
 CONSTRAINT [PK_BillingAudits] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__BillingAudit__Id__753864A1]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[BillingAudits] ADD  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[BillingSettings]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[BillingSettings](
	[Id] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NULL,
	[StartDay] [int] NOT NULL,
	[BillFrequency] [int] NOT NULL,
	[ChargeDay] [int] NOT NULL,
	[Mode] [int] NOT NULL,
 CONSTRAINT [PK_BillingSettings] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__BillingSetti__Id__762C88DA]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[BillingSettings] ADD  DEFAULT (newid()) FOR [Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__BillingSet__Mode__1920BF5C]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[BillingSettings] ADD  DEFAULT ((1)) FOR [Mode]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[CelloVersion]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[CelloVersion](
	[VersionNo] [varchar](10) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[PatchNo] [int] NULL,
	[Description] [text] COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[PatchDate] [datetime] NULL,
	[RunDate] [datetime] NULL,
	[Status] [bit] NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[CelloVersion]') AND name = N'IX_CelloVersion_RunDate')
CREATE CLUSTERED INDEX [IX_CelloVersion_RunDate] ON [dbo].[CelloVersion] 
(
	[RunDate] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ChartDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ChartDetails](
	[ChartDetails_ChartId] [uniqueidentifier] NOT NULL,
	[ChartDetails_DataSourceId] [uniqueidentifier] NOT NULL,
	[ChartDetails_TenantId] [uniqueidentifier] NOT NULL,
	[ChartDetails_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ChartDetails_Type] [nvarchar](150) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ChartDetails_Options] [nvarchar](4000) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ChartDetails_AddedBy] [uniqueidentifier] NOT NULL,
	[ChartDetails_AddedOn] [datetime] NOT NULL,
	[ChartDetails_UpdatedBy] [uniqueidentifier] NULL,
	[ChartDetails_UpdatedOn] [datetime] NULL,
	[ChartDetails_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ChartDetails] PRIMARY KEY CLUSTERED 
(
	[ChartDetails_ChartId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ChartDetails]') AND name = N'IX_ChartDetails_TenantID')
CREATE NONCLUSTERED INDEX [IX_ChartDetails_TenantID] ON [dbo].[ChartDetails] 
(
	[ChartDetails_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ChartDetails_ChartDetails_ChartId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ChartDetails] ADD  CONSTRAINT [DF_ChartDetails_ChartDetails_ChartId]  DEFAULT (newid()) FOR [ChartDetails_ChartId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ChartAxisFields]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ChartAxisFields](
	[ChartAxisFields_Id] [uniqueidentifier] NOT NULL,
	[ChartAxisFields_ChartId] [uniqueidentifier] NOT NULL,
	[ChartAxisFields_AxisName] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ChartAxisFields_AxisFieldName] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ChartAxisFields_AxisTitle] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ChartAxisFields_Interval] [float] NULL,
	[ChartAxisFields_IsDependant] [bit] NULL,
	[ChartAxisFields_AddedBy] [uniqueidentifier] NOT NULL,
	[ChartAxisFields_AddedOn] [datetime] NOT NULL,
	[ChartAxisFields_UpdatedBy] [uniqueidentifier] NULL,
	[ChartAxisFields_UpdatedOn] [datetime] NULL,
	[ChartAxisFields_Status] [bit] NOT NULL,
 CONSTRAINT [PK__ChartAxi__621C04167132C993] PRIMARY KEY NONCLUSTERED 
(
	[ChartAxisFields_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ChartAxisFields]') AND name = N'IX_ChartAxisFields_ChartId')
CREATE CLUSTERED INDEX [IX_ChartAxisFields_ChartId] ON [dbo].[ChartAxisFields] 
(
	[ChartAxisFields_ChartId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ChartAxisFields_ChartAxisFields_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ChartAxisFields] ADD  CONSTRAINT [DF_ChartAxisFields_ChartAxisFields_Id]  DEFAULT (newid()) FOR [ChartAxisFields_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ChartProperty]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ChartProperty](
	[ChartProperty_Id] [uniqueidentifier] NOT NULL,
	[ChartProperty_PropertyName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ChartProperty_AddedBy] [uniqueidentifier] NOT NULL,
	[ChartProperty_AddedOn] [datetime] NOT NULL,
	[ChartProperty_UpdatedBy] [uniqueidentifier] NULL,
	[ChartProperty_UpdatedOn] [datetime] NULL,
	[ChartProperty_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ChartProperty] PRIMARY KEY CLUSTERED 
(
	[ChartProperty_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF),
 CONSTRAINT [U_dbo_ChartProperty_1] UNIQUE NONCLUSTERED 
(
	[ChartProperty_PropertyName] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ChartProperty_ChartProperty_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ChartProperty] ADD  CONSTRAINT [DF_ChartProperty_ChartProperty_Id]  DEFAULT (newid()) FOR [ChartProperty_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ChartSections]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ChartSections](
	[ChartSections_Id] [uniqueidentifier] NOT NULL,
	[ChartSections_SectionName] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ChartSections_AddedBy] [uniqueidentifier] NOT NULL,
	[ChartSections_AddedOn] [datetime] NOT NULL,
	[ChartSections_UpdatedBy] [uniqueidentifier] NULL,
	[ChartSections_UpdatedOn] [datetime] NULL,
	[ChartSections_Status] [bit] NOT NULL,
PRIMARY KEY NONCLUSTERED 
(
	[ChartSections_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF),
 CONSTRAINT [U_dbo_ChartSections_1] UNIQUE CLUSTERED 
(
	[ChartSections_SectionName] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ChartSections_ChartSections_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ChartSections] ADD  CONSTRAINT [DF_ChartSections_ChartSections_Id]  DEFAULT (newid()) FOR [ChartSections_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ChartSectionProperties]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ChartSectionProperties](
	[ChartSectionProperties_Id] [uniqueidentifier] NOT NULL,
	[ChartSectionProperties_PropertyId] [uniqueidentifier] NOT NULL,
	[ChartSectionProperties_SectionId] [uniqueidentifier] NOT NULL,
	[ChartSectionProperties_TenantId] [uniqueidentifier] NULL,
	[ChartSectionProperties_AddedBy] [uniqueidentifier] NOT NULL,
	[ChartSectionProperties_AddedOn] [datetime] NOT NULL,
	[ChartSectionProperties_UpdatedBy] [uniqueidentifier] NULL,
	[ChartSectionProperties_UpdatedOn] [datetime] NULL,
	[ChartSectionProperties_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ChartSectionProperties] PRIMARY KEY CLUSTERED 
(
	[ChartSectionProperties_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ChartSectionProperties]') AND name = N'IX_ChartSectionProperties_PropertyId')
CREATE NONCLUSTERED INDEX [IX_ChartSectionProperties_PropertyId] ON [dbo].[ChartSectionProperties] 
(
	[ChartSectionProperties_PropertyId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ChartSectionProperties]') AND name = N'IX_ChartSectionProperties_SectionId')
CREATE NONCLUSTERED INDEX [IX_ChartSectionProperties_SectionId] ON [dbo].[ChartSectionProperties] 
(
	[ChartSectionProperties_SectionId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ChartSectionProperties]') AND name = N'IX_ChartSectionProperties_TenantId')
CREATE NONCLUSTERED INDEX [IX_ChartSectionProperties_TenantId] ON [dbo].[ChartSectionProperties] 
(
	[ChartSectionProperties_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ChartSectionProperties_ChartSectionProperties_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ChartSectionProperties] ADD  CONSTRAINT [DF_ChartSectionProperties_ChartSectionProperties_Id]  DEFAULT (newid()) FOR [ChartSectionProperties_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ChartPropertyValues]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ChartPropertyValues](
	[ChartPropertyValues_Id] [uniqueidentifier] NULL,
	[ChartPropertyValues_ChartId] [uniqueidentifier] NULL,
	[ChartPropertyValues_SectionPropertyId] [uniqueidentifier] NOT NULL,
	[ChartPropertyValues_Value] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ChartPropertyValues_TenantId] [uniqueidentifier] NULL,
	[ChartPropertyValues_AddedBy] [uniqueidentifier] NOT NULL,
	[ChartPropertyValues_AddedOn] [datetime] NOT NULL,
	[ChartPropertyValues_UpdatedBy] [uniqueidentifier] NULL,
	[ChartPropertyValues_UpdatedOn] [datetime] NULL,
	[ChartPropertyValues_Status] [bit] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ChartPropertyValues]') AND name = N'IX_ChartPropertyValues_ChartId')
CREATE CLUSTERED INDEX [IX_ChartPropertyValues_ChartId] ON [dbo].[ChartPropertyValues] 
(
	[ChartPropertyValues_ChartId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ChartPropertyValues]') AND name = N'IX_ChartPropertyValues_PrimaryKey')
CREATE NONCLUSTERED INDEX [IX_ChartPropertyValues_PrimaryKey] ON [dbo].[ChartPropertyValues] 
(
	[ChartPropertyValues_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ChartPropertyValues]') AND name = N'IX_ChartPropertyValues_SectionPropId')
CREATE NONCLUSTERED INDEX [IX_ChartPropertyValues_SectionPropId] ON [dbo].[ChartPropertyValues] 
(
	[ChartPropertyValues_SectionPropertyId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ChartPropertyValues]') AND name = N'IX_ChartPropertyValues_TenantId')
CREATE NONCLUSTERED INDEX [IX_ChartPropertyValues_TenantId] ON [dbo].[ChartPropertyValues] 
(
	[ChartPropertyValues_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ChartPropertyValues_ChartPropertyValues_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ChartPropertyValues] ADD  CONSTRAINT [DF_ChartPropertyValues_ChartPropertyValues_Id]  DEFAULT (newid()) FOR [ChartPropertyValues_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ContactExtn]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ContactExtn](
	[ContactExtn_Id] [uniqueidentifier] NOT NULL,
	[ContactExtn_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ContactExtn_ReferenceId] [uniqueidentifier] NOT NULL,
	[ContactExtn_TenantId] [uniqueidentifier] NOT NULL,
	[ContactExtn_Column1] [int] NULL,
	[ContactExtn_Column2] [int] NULL,
	[ContactExtn_Column3] [int] NULL,
	[ContactExtn_Column4] [int] NULL,
	[ContactExtn_Column5] [int] NULL,
	[ContactExtn_Column6] [datetime] NULL,
	[ContactExtn_Column7] [datetime] NULL,
	[ContactExtn_Column8] [datetime] NULL,
	[ContactExtn_Column9] [datetime] NULL,
	[ContactExtn_Column10] [datetime] NULL,
	[ContactExtn_Column11] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column12] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column13] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column14] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column15] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column16] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column17] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column18] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column19] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column20] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column21] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column22] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column23] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column24] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column25] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column26] [float] NULL,
	[ContactExtn_Column27] [float] NULL,
	[ContactExtn_Column28] [float] NULL,
	[ContactExtn_Column29] [float] NULL,
	[ContactExtn_Column30] [float] NULL,
	[ContactExtn_Column31] [bit] NULL,
	[ContactExtn_Column32] [bit] NULL,
	[ContactExtn_Column33] [bit] NULL,
	[ContactExtn_Column34] [bit] NULL,
	[ContactExtn_Column35] [bit] NULL,
	[ContactExtn_Column36] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column37] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column38] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column39] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_Column40] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ContactExtn_CreatedOn] [datetime] NOT NULL,
	[ContactExtn_CreatedBy] [uniqueidentifier] NOT NULL,
	[ContactExtn_UpdatedOn] [datetime] NULL,
	[ContactExtn_UpdatedBy] [uniqueidentifier] NULL,
	[ContactExtn_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ContactExtn1] PRIMARY KEY CLUSTERED 
(
	[ContactExtn_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ContactExtn_ContactExtn_Id1]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ContactExtn] ADD  CONSTRAINT [DF_ContactExtn_ContactExtn_Id1]  DEFAULT (newid()) FOR [ContactExtn_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataFieldType]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataFieldType](
	[DataFieldType_ID] [uniqueidentifier] NOT NULL,
	[DataFieldType_Name] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DataFieldType_CreatedOn] [datetime] NOT NULL,
	[DataFieldType_CreatedBy] [uniqueidentifier] NOT NULL,
	[DataFieldType_UpdatedOn] [datetime] NULL,
	[DataFieldType_UpdatedBy] [uniqueidentifier] NULL,
	[DataFieldType_Status] [bit] NOT NULL,
 CONSTRAINT [PK_DataFieldType] PRIMARY KEY CLUSTERED 
(
	[DataFieldType_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataModule]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataModule](
	[DataModuleID] [uniqueidentifier] NOT NULL,
	[DataModuleName] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DataModuleCode] [varchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreateScript] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UpdateScript] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_DataModule] PRIMARY KEY CLUSTERED 
(
	[DataModuleID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_DataModule_DataModuleID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[DataModule] ADD  CONSTRAINT [DF_DataModule_DataModuleID]  DEFAULT (newid()) FOR [DataModuleID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataPartition]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataPartition](
	[DataPartitionID] [uniqueidentifier] NOT NULL,
	[DataPartitionName] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](1000) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[IsDefault] [bit] NOT NULL,
	[Status] [bit] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[Updatedby] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
 CONSTRAINT [PK_DataPartition] PRIMARY KEY CLUSTERED 
(
	[DataPartitionID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataServer]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataServer](
	[DataServerID] [uniqueidentifier] NOT NULL,
	[Name] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ServerAddress] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DataBaseName] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ConnectionString] [nvarchar](1000) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Status] [bit] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
 CONSTRAINT [PK_DataServer] PRIMARY KEY CLUSTERED 
(
	[DataServerID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataPartitionMapping]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataPartitionMapping](
	[DataPartitionMappingID] [uniqueidentifier] NOT NULL,
	[DataModuleID] [uniqueidentifier] NOT NULL,
	[DataServerID] [uniqueidentifier] NOT NULL,
	[DataPartitionID] [uniqueidentifier] NOT NULL,
	[Status] [bit] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
 CONSTRAINT [PK_DataPartitionSetMapping] PRIMARY KEY CLUSTERED 
(
	[DataPartitionMappingID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_DataPartitionMapping_DataPartitionMappingID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[DataPartitionMapping] ADD  CONSTRAINT [DF_DataPartitionMapping_DataPartitionMappingID]  DEFAULT (newid()) FOR [DataPartitionMappingID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataScope]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataScope](
	[DataScope_ID] [uniqueidentifier] NOT NULL,
	[DataScope_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DataScope_SelectQuery] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DataScope_FilteredSubject] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DataScope_Condition] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DataScope_AccessLevel] [smallint] NULL,
	[DataScope_CreatedBy] [uniqueidentifier] NOT NULL,
	[DataScope_CreatedOn] [datetime] NOT NULL,
	[DataScope_UpdatedBy] [uniqueidentifier] NULL,
	[DataScope_UpdatedOn] [datetime] NULL,
	[DataScope_Status] [bit] NOT NULL,
	[DataScope_IsDynamic] [bit] NULL,
 CONSTRAINT [PK_DataScope] PRIMARY KEY CLUSTERED 
(
	[DataScope_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataSource]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataSource](
	[DataSource_Id] [uniqueidentifier] NOT NULL,
	[DataSource_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DataSource_SourceType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DataSource_ContentId] [uniqueidentifier] NOT NULL,
	[DataSource_TenantId] [uniqueidentifier] NOT NULL,
	[DataSource_CreatedBy] [uniqueidentifier] NOT NULL,
	[DataSource_CreatedOn] [datetime] NOT NULL,
	[DataSource_UpdatedBy] [uniqueidentifier] NULL,
	[DataSource_UpdatedOn] [datetime] NULL,
	[DataSource_Status] [bit] NOT NULL,
 CONSTRAINT [PK_DataSource] PRIMARY KEY CLUSTERED 
(
	[DataSource_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataSource]') AND name = N'IX_DataSource')
CREATE NONCLUSTERED INDEX [IX_DataSource] ON [dbo].[DataSource] 
(
	[DataSource_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataSource]') AND name = N'IX_DataSource_TenantId')
CREATE NONCLUSTERED INDEX [IX_DataSource_TenantId] ON [dbo].[DataSource] 
(
	[DataSource_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_DataSource_DataSource_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[DataSource] ADD  CONSTRAINT [DF_DataSource_DataSource_Id]  DEFAULT (newid()) FOR [DataSource_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataView]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataView](
	[DataView_ID] [int] NOT NULL,
	[DataView_DataViewID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DataView_Name] [nvarchar](250) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DataView_Application] [int] NOT NULL,
	[DataView_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DataView_MainEntity] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DataView_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_Form] PRIMARY KEY CLUSTERED 
(
	[DataView_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataView]') AND name = N'Ind_DataView_DataViewID')
CREATE NONCLUSTERED INDEX [Ind_DataView_DataViewID] ON [dbo].[DataView] 
(
	[DataView_DataViewID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataView]') AND name = N'Ind_DataView_MainEntity')
CREATE NONCLUSTERED INDEX [Ind_DataView_MainEntity] ON [dbo].[DataView] 
(
	[DataView_MainEntity] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFeature]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataViewFeature](
	[DataViewFeature_FeatureCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DataViewFeature_DataViewID] [int] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFeature]') AND name = N'Ind_DataViewFeature_DataViewID')
CREATE CLUSTERED INDEX [Ind_DataViewFeature_DataViewID] ON [dbo].[DataViewFeature] 
(
	[DataViewFeature_DataViewID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFeature]') AND name = N'Ind_DataViewFeature_FeatureCode')
CREATE NONCLUSTERED INDEX [Ind_DataViewFeature_FeatureCode] ON [dbo].[DataViewFeature] 
(
	[DataViewFeature_FeatureCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FieldType]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[FieldType](
	[FieldType_ID] [int] NOT NULL,
	[FieldType_Name] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FieldType_CreatedOn] [datetime] NOT NULL,
	[FieldType_CreatedBy] [uniqueidentifier] NOT NULL,
	[FieldType_UpdatedOn] [datetime] NULL,
	[FieldType_UpdatedBy] [uniqueidentifier] NULL,
	[FieldType_Status] [bit] NOT NULL,
 CONSTRAINT [PK_FieldType] PRIMARY KEY CLUSTERED 
(
	[FieldType_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFields]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataViewFields](
	[DataViewField_ID] [int] IDENTITY(1,1) NOT NULL,
	[DataViewField_FieldID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DataViewField_DataViewID] [int] NULL,
	[DataViewField_TypeID] [int] NOT NULL,
	[DataViewField_ExtendedTenantCode] [uniqueidentifier] NULL,
	[DataViewField_GlobalId] [int] NULL,
	[DataViewField_IsGlobalField] [bit] NULL,
	[DataViewField_IsFixed] [bit] NULL,
 CONSTRAINT [PK_FormFields] PRIMARY KEY CLUSTERED 
(
	[DataViewField_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFields]') AND name = N'Ind_DataViewField_DataViewID')
CREATE NONCLUSTERED INDEX [Ind_DataViewField_DataViewID] ON [dbo].[DataViewFields] 
(
	[DataViewField_DataViewID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFields]') AND name = N'Ind_DataViewField_ExtendedTenantCode')
CREATE NONCLUSTERED INDEX [Ind_DataViewField_ExtendedTenantCode] ON [dbo].[DataViewFields] 
(
	[DataViewField_ExtendedTenantCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFields]') AND name = N'Ind_DataViewField_FieldID')
CREATE NONCLUSTERED INDEX [Ind_DataViewField_FieldID] ON [dbo].[DataViewFields] 
(
	[DataViewField_FieldID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFields]') AND name = N'Ind_DataViewField_TypeID')
CREATE NONCLUSTERED INDEX [Ind_DataViewField_TypeID] ON [dbo].[DataViewFields] 
(
	[DataViewField_TypeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PickupList]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PickupList](
	[PickupList_ID] [uniqueidentifier] NOT NULL,
	[PickupList_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PickupList_TenantCode] [uniqueidentifier] NULL,
	[PickupList_CreatedBy] [uniqueidentifier] NOT NULL,
	[PickupList_CreatedOn] [datetime] NOT NULL,
	[PickupList_UpdatedBy] [uniqueidentifier] NULL,
	[PickupList_UpdatedOn] [datetime] NULL,
	[PickupList_Status] [bit] NOT NULL,
	[PickupList_TableName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_PickupList] PRIMARY KEY CLUSTERED 
(
	[PickupList_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PickupList]') AND name = N'Ind_PickupList_TenantCode')
CREATE NONCLUSTERED INDEX [Ind_PickupList_TenantCode] ON [dbo].[PickupList] 
(
	[PickupList_TenantCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_PickupList_PickupList_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PickupList] ADD  CONSTRAINT [DF_PickupList_PickupList_ID]  DEFAULT (newid()) FOR [PickupList_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DataViewFieldProperties](
	[FieldProperties_TenantID] [uniqueidentifier] NULL,
	[FieldProperties_FieldID] [int] NOT NULL,
	[FieldProperties_FieldName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FieldProperties_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FieldProperties_Ordinal] [int] NULL,
	[FieldProperties_IsVisible] [bit] NOT NULL,
	[FieldProperties_IsEditable] [bit] NOT NULL,
	[FieldProperties_IsMandatory] [bit] NULL,
	[FieldProperties_RegExpression] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FieldProperties_MaxLength] [int] NULL,
	[FieldProperties_PickupListID] [uniqueidentifier] NULL,
	[FieldProperties_CreatedOn] [datetime] NULL,
	[FieldProperties_CreatedBy] [uniqueidentifier] NULL,
	[FieldProperties_UpdatedOn] [datetime] NULL,
	[FieldProperties_UpdatedBy] [uniqueidentifier] NULL,
	[FieldProperties_ViewDataName] [nvarchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FieldProperties_ControlFieldID] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FieldProperties_IsMultiLine] [bit] NULL,
	[FieldProperties_IsHidden] bit null,
	[FieldProperties_DefaultValue] nvarchar(500) null,
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]') AND name = N'Ind_FieldProperties_FieldID')
CREATE CLUSTERED INDEX [Ind_FieldProperties_FieldID] ON [dbo].[DataViewFieldProperties] 
(
	[FieldProperties_FieldID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]') AND name = N'Ind_FieldProperties_Ordinal')
CREATE NONCLUSTERED INDEX [Ind_FieldProperties_Ordinal] ON [dbo].[DataViewFieldProperties] 
(
	[FieldProperties_Ordinal] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]') AND name = N'Ind_FieldProperties_PickupListID')
CREATE NONCLUSTERED INDEX [Ind_FieldProperties_PickupListID] ON [dbo].[DataViewFieldProperties] 
(
	[FieldProperties_PickupListID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]') AND name = N'Ind_FieldProperties_TenantID')
CREATE NONCLUSTERED INDEX [Ind_FieldProperties_TenantID] ON [dbo].[DataViewFieldProperties] 
(
	[FieldProperties_TenantID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DefaultRoles]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DefaultRoles](
	[DefaultRole_ID] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DefaultRole_Name] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DefaultRole_Rule] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DefaultRole_CreatedBy] [uniqueidentifier] NOT NULL,
	[DefaultRole_CreatedOn] [datetime] NOT NULL,
	[DefaultRole_UpdatedBy] [uniqueidentifier] NULL,
	[DefaultRole_UpdatedOn] [datetime] NULL,
	[DefaultRole_Status] [bit] NOT NULL,
 CONSTRAINT [PK__DefaultR__F8098907693CA210] PRIMARY KEY CLUSTERED 
(
	[DefaultRole_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Roles]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Roles](
	[Role_ID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Role_Name] [nvarchar](150) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Role_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Role_TenantCode] [uniqueidentifier] NULL,
	[Role_CreatedBy] [uniqueidentifier] NOT NULL,
	[Role_CreatedOn] [datetime] NOT NULL,
	[Role_UpdatedBy] [uniqueidentifier] NULL,
	[Role_UpdatedOn] [datetime] NULL,
	[Role_Status] [bit] NOT NULL,
 CONSTRAINT [PK__Roles__D80AB49B6D0D32F4] PRIMARY KEY CLUSTERED 
(
	[Role_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Roles]') AND name = N'Ind_Role_Name')
CREATE NONCLUSTERED INDEX [Ind_Role_Name] ON [dbo].[Roles] 
(
	[Role_Name] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Roles]') AND name = N'Ind_Role_TenantCode')
CREATE NONCLUSTERED INDEX [Ind_Role_TenantCode] ON [dbo].[Roles] 
(
	[Role_TenantCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RolePrivileges]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RolePrivileges](
	[RolePrivilege_ID] [uniqueidentifier] NOT NULL,
	[RolePrivilege_TenantID] [uniqueidentifier] NULL,
	[RolePrivilege_RoleID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RolePrivilege_PrivilegeID] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RolePrivilege_DataScopeID] [uniqueidentifier] NULL,
	[RolePrivilege_CreatedBy] [uniqueidentifier] NOT NULL,
	[RolePrivilege_CreatedOn] [datetime] NOT NULL,
	[RolePrivilege_UpdatedBy] [uniqueidentifier] NULL,
	[RolePrivilege_UpdatedOn] [datetime] NULL,
	[RolePrivilege_Status] [bit] NOT NULL,
 CONSTRAINT [PK_RolePrivileges] PRIMARY KEY CLUSTERED 
(
	[RolePrivilege_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RolePrivileges]') AND name = N'Ind_RolePrivilege_DataScopeID')
CREATE NONCLUSTERED INDEX [Ind_RolePrivilege_DataScopeID] ON [dbo].[RolePrivileges] 
(
	[RolePrivilege_DataScopeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RolePrivileges]') AND name = N'Ind_RolePrivilege_PrivilegeID')
CREATE NONCLUSTERED INDEX [Ind_RolePrivilege_PrivilegeID] ON [dbo].[RolePrivileges] 
(
	[RolePrivilege_PrivilegeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RolePrivileges]') AND name = N'Ind_RolePrivilege_RoleID')
CREATE NONCLUSTERED INDEX [Ind_RolePrivilege_RoleID] ON [dbo].[RolePrivileges] 
(
	[RolePrivilege_RoleID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RolePrivileges]') AND name = N'Ind_RolePrivilege_TenantID')
CREATE NONCLUSTERED INDEX [Ind_RolePrivilege_TenantID] ON [dbo].[RolePrivileges] 
(
	[RolePrivilege_TenantID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RolePrivileges_RolePrivilege_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RolePrivileges] ADD  CONSTRAINT [DF_RolePrivileges_RolePrivilege_ID]  DEFAULT (newid()) FOR [RolePrivilege_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DynamicVariable]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DynamicVariable](
	[DynamicVariable_Id] [uniqueidentifier] NOT NULL,
	[DynamicVariable_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DynamicVariable_VariableName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DynamicVariable_DataTypeId] [uniqueidentifier] NOT NULL,
	[DynamicVariable_FieldTypeId] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DynamicVariable_DataScopeId] [uniqueidentifier] NOT NULL,
	[DynamicVariable_DynamicValueQuery] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DynamicVariable_CreatedBy] [uniqueidentifier] NOT NULL,
	[DynamicVariable_CreatedOn] [datetime] NOT NULL,
	[DynamicVariable_UpdatedBy] [uniqueidentifier] NULL,
	[DynamicVariable_UpdatedOn] [datetime] NULL,
	[DynamicVariable_Status] [bit] NOT NULL,
 CONSTRAINT [PK_DynamicVariable] PRIMARY KEY CLUSTERED 
(
	[DynamicVariable_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DynamicVariable]') AND name = N'Ind_DynamicVariable_DynamicVariable_DataScopeId')
CREATE NONCLUSTERED INDEX [Ind_DynamicVariable_DynamicVariable_DataScopeId] ON [dbo].[DynamicVariable] 
(
	[DynamicVariable_DataScopeId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DynamicVariable]') AND name = N'Ind_DynamicVariable_DynamicVariable_DataTypeId')
CREATE NONCLUSTERED INDEX [Ind_DynamicVariable_DynamicVariable_DataTypeId] ON [dbo].[DynamicVariable] 
(
	[DynamicVariable_DataTypeId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DynamicVariable]') AND name = N'Ind_DynamicVariable_DynamicVariable_FieldTypeId')
CREATE NONCLUSTERED INDEX [Ind_DynamicVariable_DynamicVariable_FieldTypeId] ON [dbo].[DynamicVariable] 
(
	[DynamicVariable_FieldTypeId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DynamicVariable]') AND name = N'Ind_DynamicVariable_DynamicVariable_Name')
CREATE NONCLUSTERED INDEX [Ind_DynamicVariable_DynamicVariable_Name] ON [dbo].[DynamicVariable] 
(
	[DynamicVariable_Name] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_DynamicVariable_DynamicVariable_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[DynamicVariable] ADD  CONSTRAINT [DF_DynamicVariable_DynamicVariable_Id]  DEFAULT (newid()) FOR [DynamicVariable_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DynamicDataScopeValue]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DynamicDataScopeValue](
	[DynamicDataScopeValue_Id] [uniqueidentifier] NOT NULL,
	[DynamicDataScopeValue_RolePrivilegeId] [uniqueidentifier] NOT NULL,
	[DynamicDataScopeValue_DynamicVariableId] [uniqueidentifier] NOT NULL,
	[DynamicDataScopeValue_Value] [varchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DynamicDataScopeValue_CreatedBy] [uniqueidentifier] NOT NULL,
	[DynamicDataScopeValue_CreatedOn] [datetime] NOT NULL,
	[DynamicDataScopeValue_UpdatedBy] [uniqueidentifier] NULL,
	[DynamicDataScopeValue_UpdatedOn] [datetime] NULL,
	[DynamicDataScopeValue_Status] [bit] NOT NULL,
 CONSTRAINT [PK__DynamicD__39971D05370627FE] PRIMARY KEY NONCLUSTERED 
(
	[DynamicDataScopeValue_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DynamicDataScopeValue]') AND name = N'Ind_DynamicDataScopeValue_DynamicVariable_Id')
CREATE CLUSTERED INDEX [Ind_DynamicDataScopeValue_DynamicVariable_Id] ON [dbo].[DynamicDataScopeValue] 
(
	[DynamicDataScopeValue_DynamicVariableId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[DynamicDataScopeValue]') AND name = N'Ind_DynamicDataScopeValue_Privilege_Id')
CREATE NONCLUSTERED INDEX [Ind_DynamicDataScopeValue_Privilege_Id] ON [dbo].[DynamicDataScopeValue] 
(
	[DynamicDataScopeValue_RolePrivilegeId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__DynamicDa__Dynam__00AA174D]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[DynamicDataScopeValue] ADD  DEFAULT (newid()) FOR [DynamicDataScopeValue_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Template]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Template](
	[Template_Id] [uniqueidentifier] NOT NULL,
	[Template_TenantId] [uniqueidentifier] NULL,
	[Template_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Template_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Template_Category] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Template_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Template_CreatedBy] [uniqueidentifier] NULL,
	[Template_CreatedOn] [datetime] NOT NULL,
	[Template_UpdatedBy] [uniqueidentifier] NULL,
	[Template_UpdatedOn] [datetime] NULL,
	[Template_Status] [bit] NOT NULL,
 CONSTRAINT [PK_Template] PRIMARY KEY CLUSTERED 
(
	[Template_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Template__Templa__75035A77]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Template] ADD  DEFAULT (getdate()) FOR [Template_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Template__Templa__75F77EB0]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Template] ADD  DEFAULT ((1)) FOR [Template_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationContentDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationContentDetails](
	[NotificationContentDetails_Id] [uniqueidentifier] NOT NULL,
	[NotificationContentDetails_Type] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
 CONSTRAINT [PK_NotificationContentDetails] PRIMARY KEY CLUSTERED 
(
	[NotificationContentDetails_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EmailContentDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EmailContentDetails](
	[EmailContentDetails_ContentDetailsId] [uniqueidentifier] NOT NULL,
	[EmailContentDetails_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailContentDetails_AttachmentFileFolderPath] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailContentDetails_AttachmentFiles] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailContentDetails_TemplateId] [uniqueidentifier] NULL,
	[EmailContentDetails_CreatedBy] [uniqueidentifier] NULL,
	[EmailContentDetails_CreatedOn] [datetime] NOT NULL,
	[EmailContentDetails_UpdatedBy] [uniqueidentifier] NULL,
	[EmailContentDetails_UpdatedOn] [datetime] NULL,
	[EmailContentDetails_Status] [bit] NOT NULL,
	[EmailContentDetails_Subject] [nvarchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailContentDetails_RuleSetCode] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_EmailContentDetails] PRIMARY KEY CLUSTERED 
(
	[EmailContentDetails_ContentDetailsId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__EmailCont__Email__02925FBF]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EmailContentDetails] ADD  DEFAULT (getdate()) FOR [EmailContentDetails_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__EmailCont__Email__038683F8]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EmailContentDetails] ADD  DEFAULT ((1)) FOR [EmailContentDetails_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EmailDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EmailDetails](
	[EmailDetails_Id] [uniqueidentifier] NOT NULL,
	[EmailDetails_BatchId] [uniqueidentifier] NULL,
	[EmailDetails_Sender] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDetails_Recipient] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDetails_Subject] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDetails_SecondaryRecipient] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDetails_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDetails_SmtpUserName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDetails_SmtpPassword] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDetails_SmtpAddress] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDetails_EmailStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDetails_StatusMessage] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDetails_AttachmentFileFolderPath] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDetails_AttachmentFiles] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDetails_CreatedBy] [uniqueidentifier] NOT NULL,
	[EmailDetails_CreatedOn] [datetime] NOT NULL,
	[EmailDetails_UpdatedBy] [uniqueidentifier] NULL,
	[EmailDetails_UpdatedOn] [datetime] NULL,
	[EmailDetails_TrackId] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDetails_Bcc] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDetails_PortNumber] [int] NULL,
	[EmailDetails_EnableSSL] [bit] NULL,
 CONSTRAINT [PK_EmailDetails] PRIMARY KEY CLUSTERED 
(
	[EmailDetails_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EmailDetails_EmailDetails_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EmailDetails] ADD  CONSTRAINT [DF_EmailDetails_EmailDetails_Id]  DEFAULT (newid()) FOR [EmailDetails_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationDispatchDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationDispatchDetails](
	[NotificationDispatchDetails_Id] [uniqueidentifier] NOT NULL,
	[NotificationDispatchDetails_Type] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
 CONSTRAINT [PK_NotificationDispatchDetails] PRIMARY KEY CLUSTERED 
(
	[NotificationDispatchDetails_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EmailDispatchDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EmailDispatchDetails](
	[EmailDispatchDetails_DispatchDetailsId] [uniqueidentifier] NOT NULL,
	[EmailDispatchDetails_SenderAddress] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDispatchDetails_RecipientAddress] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDispatchDetails_SecondaryRecipientAddress] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDispatchDetails_Subject] [nvarchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDispatchDetails_SmtpAddress] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDispatchDetails_SmtpUserName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDispatchDetails_SmtpPassword] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailDispatchDetails_CreatedBy] [uniqueidentifier] NULL,
	[EmailDispatchDetails_CreatedOn] [datetime] NOT NULL,
	[EmailDispatchDetails_UpdatedBy] [uniqueidentifier] NULL,
	[EmailDispatchDetails_UpdatedOn] [datetime] NULL,
	[EmailDispatchDetails_Status] [bit] NOT NULL,
	[EmailDispatchDetails_Bcc] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailDispatchDetails_PortNumber] [int] NULL,
	[EmailDispatchDetails_EnableSSL] [bit] NULL,
 CONSTRAINT [PK_EmailDispatchDetails] PRIMARY KEY CLUSTERED 
(
	[EmailDispatchDetails_DispatchDetailsId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__EmailDisp__Email__056ECC6A]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EmailDispatchDetails] ADD  DEFAULT (getdate()) FOR [EmailDispatchDetails_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__EmailDisp__Email__0662F0A3]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EmailDispatchDetails] ADD  DEFAULT ((1)) FOR [EmailDispatchDetails_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationMaster]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationMaster](
	[NotificationMaster_Id] [uniqueidentifier] NOT NULL,
	[NotificationMaster_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[NotificationMaster_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[NotificationMaster_Category] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[NotificationMaster_CreatedBy] [uniqueidentifier] NULL,
	[NotificationMaster_CreatedOn] [datetime] NOT NULL,
	[NotificationMaster_UpdatedBy] [uniqueidentifier] NULL,
	[NotificationMaster_UpdatedOn] [datetime] NULL,
	[NotificationMaster_Status] [bit] NOT NULL,
	[NotificationMaster_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_NotificationMaster] PRIMARY KEY CLUSTERED 
(
	[NotificationMaster_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Notificat__Notif__408F9238]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationMaster] ADD  DEFAULT (getdate()) FOR [NotificationMaster_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Notificat__Notif__4183B671]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationMaster] ADD  DEFAULT ((1)) FOR [NotificationMaster_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationDetails](
	[NotificationDetails_Id] [uniqueidentifier] NOT NULL,
	[NotificationDetails_NotificationId] [uniqueidentifier] NOT NULL,
	[NotificationDetails_ContentDetailsId] [uniqueidentifier] NULL,
	[NotificationDetails_DispatchDetailsId] [uniqueidentifier] NULL,
	[NotificationDetails_TenantId] [uniqueidentifier] NULL,
	[NotificationDetails_CreatedBy] [uniqueidentifier] NULL,
	[NotificationDetails_CreatedOn] [datetime] NOT NULL,
	[NotificationDetails_UpdatedBy] [uniqueidentifier] NULL,
	[NotificationDetails_UpdatedOn] [datetime] NULL,
	[NotificationDetails_Status] [bit] NOT NULL,
 CONSTRAINT [PK_NotificationDetails] PRIMARY KEY CLUSTERED 
(
	[NotificationDetails_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Notificat__Notif__3DB3258D]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationDetails] ADD  DEFAULT (getdate()) FOR [NotificationDetails_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Notificat__Notif__3EA749C6]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationDetails] ADD  DEFAULT ((1)) FOR [NotificationDetails_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationAudit]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationAudit](
	[NotificationAudit_Id] [uniqueidentifier] NOT NULL,
	[NotificationAudit_NotificationContentType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[NotificationAudit_NotificationDispatchType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[NotificationAudit_NotificationStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[NotificationAudit_NotificationId] [uniqueidentifier] NOT NULL,
	[NotificationAudit_NotificationBatchId] [uniqueidentifier] NULL,
	[NotificationAudit_NotificationBatchCount] [int] NULL,
	[NotificationAudit_CreatedBy] [uniqueidentifier] NOT NULL,
	[NotificationAudit_CreatedOn] [datetime] NOT NULL,
	[NotificationAudit_UpdatedBy] [uniqueidentifier] NULL,
	[NotificationAudit_UpdatedOn] [datetime] NULL,
	[NotificationAudit_Status] [bit] NOT NULL,
	[NotificationAudit_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_NotificationAudit] PRIMARY KEY CLUSTERED 
(
	[NotificationAudit_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_NotificationAudit_NotificationAudit_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationAudit] ADD  CONSTRAINT [DF_NotificationAudit_NotificationAudit_Id]  DEFAULT (newid()) FOR [NotificationAudit_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationDispatch]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationDispatch](
	[NotificationDispatch_Id] [uniqueidentifier] NOT NULL,
	[NotificationDispatch_NotificationAuditId] [uniqueidentifier] NOT NULL,
	[NotificationDispatch_CreatedOn] [datetime] NOT NULL,
	[NotificationDispatch_CreatedBy] [uniqueidentifier] NOT NULL,
	[NotificationDispatch_UpdatedOn] [datetime] NULL,
	[NotificationDispatch_UpdatedBy] [uniqueidentifier] NULL,
	[NotificationDispatch_Status] [bit] NOT NULL,
	[NotificationDispatch_TrackId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_NotificationDispatch] PRIMARY KEY CLUSTERED 
(
	[NotificationDispatch_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_NotificationDispatch_NotificationDispatch_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationDispatch] ADD  CONSTRAINT [DF_NotificationDispatch_NotificationDispatch_Id]  DEFAULT (newid()) FOR [NotificationDispatch_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EmailNotificationDispatch]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EmailNotificationDispatch](
	[EmailNotificationDispatch_NotificationDispatchId] [uniqueidentifier] NOT NULL,
	[EmailNotificationDispatch_SenderAddress] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailNotificationDispatch_RecipientAddress] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailNotificationDispatch_SecondaryRecipientAddress] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailNotificationDispatch_SmtpAddress] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailNotificationDispatch_SmtpUserName] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailNotificationDispatch_SmtpPassword] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailNotificationDispatch_CreatedOn] [datetime] NOT NULL,
	[EmailNotificationDispatch_CreatedBy] [uniqueidentifier] NOT NULL,
	[EmailNotificationDispatch_UpdatedOn] [datetime] NULL,
	[EmailNotificationDispatch_UpdatedBy] [uniqueidentifier] NULL,
	[EmailNotificationDispatch_Status] [bit] NOT NULL,
	[EmailNotificationDispatch_Bcc] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailNotificationDispatch_PortNumber] [int] NULL,
	[EmailNotificationDispatch_EnableSSL] [bit] NULL,
 CONSTRAINT [PK_EmailNotificationDispatch] PRIMARY KEY CLUSTERED 
(
	[EmailNotificationDispatch_NotificationDispatchId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EmailQueueLog]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EmailQueueLog](
	[EmailQueue_ID] [uniqueidentifier] NOT NULL,
	[EmailQueue_FromAddress] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailQueue_ToAddress] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailQueue_EmailContent] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailQueue_SenderRole] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EmailQueue_SenderContextID] [int] NOT NULL,
	[EmailQueue_SenderUserID] [uniqueidentifier] NOT NULL,
	[EmailQueue_ReceiverUserID] [uniqueidentifier] NULL,
	[EmailQueue_ErrorDescription] [nvarchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EmailQueue_InviteDate] [datetime] NOT NULL,
	[EmailQueue_Status] [int] NOT NULL,
	[EmailQueue_ReferenceID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_EmailQueueLog] PRIMARY KEY CLUSTERED 
(
	[EmailQueue_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EmailTemplate]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EmailTemplate](
	[Emailtemplate_ID] [int] IDENTITY(1,1) NOT NULL,
	[Emailtemplate_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Emailtemplate_Type] [smallint] NOT NULL,
	[Emailtemplate_TenantID] [uniqueidentifier] NOT NULL,
	[Emailtemplate_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
 CONSTRAINT [PK_EmailTemplate] PRIMARY KEY CLUSTERED 
(
	[Emailtemplate_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EntityBasedTenantScope]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EntityBasedTenantScope](
	[EntityBasedTenantScope_Id] [uniqueidentifier] NOT NULL,
	[EntityBasedTenantScope_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityBasedTenantScope_BridgeCondition] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityBasedTenantScope_CreatedBy] [uniqueidentifier] NOT NULL,
	[EntityBasedTenantScope_CreatedOn] [datetime] NOT NULL,
	[EntityBasedTenantScope_UpdatedBy] [uniqueidentifier] NULL,
	[EntityBasedTenantScope_UpdatedOn] [datetime] NULL,
	[EntityBasedTenantScope_Status] [bit] NOT NULL,
 CONSTRAINT [PK_EntityBasedTenantScope] PRIMARY KEY CLUSTERED 
(
	[EntityBasedTenantScope_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EntityBasedTenantScope_EntityBasedTenantScope_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EntityBasedTenantScope] ADD  CONSTRAINT [DF_EntityBasedTenantScope_EntityBasedTenantScope_Id]  DEFAULT (newid()) FOR [EntityBasedTenantScope_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EntityDataScope]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EntityDataScope](
	[EntityDataScope_ID] [uniqueidentifier] NOT NULL,
	[EntityDataScope_DataScopeID] [uniqueidentifier] NOT NULL,
	[EntityDataScope_EntityID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityDataScope_BridgeCondition] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EntityDataScope_Property] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EntityDataScope_CreatedBy] [uniqueidentifier] NOT NULL,
	[EntityDataScope_CreatedOn] [datetime] NOT NULL,
	[EntityDataScope_UpdatedBy] [uniqueidentifier] NULL,
	[EntityDataScope_UpdatedOn] [datetime] NULL,
	[EntityDataScope_Status] [bit] NOT NULL,
 CONSTRAINT [PK_EntityDataScope] PRIMARY KEY CLUSTERED 
(
	[EntityDataScope_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityDataScope]') AND name = N'Ind_EntityDataScope_DataScopeID')
CREATE NONCLUSTERED INDEX [Ind_EntityDataScope_DataScopeID] ON [dbo].[EntityDataScope] 
(
	[EntityDataScope_DataScopeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityDataScope]') AND name = N'Ind_EntityDataScope_EntityID')
CREATE NONCLUSTERED INDEX [Ind_EntityDataScope_EntityID] ON [dbo].[EntityDataScope] 
(
	[EntityDataScope_EntityID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EntityExtnColumnDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EntityExtnColumnDetails](
	[EntityExtnColumnDetails_Id] [uniqueidentifier] NOT NULL,
	[EntityExtnColumnDetails_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityExtnColumnDetails_ColumnMetadataId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityExtnColumnDetails_ColumnName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityExtnColumnDetails_CreatedBy] [uniqueidentifier] NOT NULL,
	[EntityExtnColumnDetails_CreatedOn] [datetime] NOT NULL,
	[EntityExtnColumnDetails_UpdatedBy] [uniqueidentifier] NULL,
	[EntityExtnColumnDetails_UpdatedOn] [datetime] NULL,
	[EntityExtnColumnDetails_Status] [bit] NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityExtnColumnDetails]') AND name = N'IX_EntityExtnColumnDetails_CreatedOn')
CREATE CLUSTERED INDEX [IX_EntityExtnColumnDetails_CreatedOn] ON [dbo].[EntityExtnColumnDetails] 
(
	[EntityExtnColumnDetails_CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EntityExtnColumnDetails_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EntityExtnColumnDetails] ADD  CONSTRAINT [DF_EntityExtnColumnDetails_ID]  DEFAULT (newid()) FOR [EntityExtnColumnDetails_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EntityExtnColumnMetadata]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EntityExtnColumnMetadata](
	[EntityExtnColumnMetadata_Id] [uniqueidentifier] NOT NULL,
	[EntityExtnColumnMetadata_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DataFieldType_ID] [uniqueidentifier] NOT NULL,
	[EntityExtnColumnMetadata_NoOfColumn] [int] NOT NULL,
	[EntityExtnColumnMetadata_Dimension] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityExtnColumnMetadata_CreatedBy] [uniqueidentifier] NOT NULL,
	[EntityExtnColumnMetadata_CreatedOn] [datetime] NOT NULL,
	[EntityExtnColumnMetadata_UpdatedBy] [uniqueidentifier] NULL,
	[EntityExtnColumnMetadata_UpdatedOn] [datetime] NULL,
	[EntityExtnColumnMetadata_Status] [bit] NOT NULL,
 CONSTRAINT [PK_EntityExtnColumnMetadata] PRIMARY KEY CLUSTERED 
(
	[EntityExtnColumnMetadata_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK__EntityExt__DataF__414EAC47]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityExtnColumnMetadata]'))
ALTER TABLE [dbo].[EntityExtnColumnMetadata]  WITH CHECK ADD FOREIGN KEY([DataFieldType_ID])
REFERENCES [dbo].[DataFieldType] ([DataFieldType_ID])
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EntityExtnColumnMetadata_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EntityExtnColumnMetadata] ADD  CONSTRAINT [DF_EntityExtnColumnMetadata_ID]  DEFAULT (newid()) FOR [EntityExtnColumnMetadata_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EntityFeaturePrivilege]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EntityFeaturePrivilege](
	[EntityFeaturePrivilege_Code] [uniqueidentifier] NOT NULL,
	[EntityFeaturePrivilege_FeatureCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityFeaturePrivilege_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityFeaturePrivilege_CreatedOn] [datetime] NOT NULL,
	[EntityFeaturePrivilege_CreatedBy] [uniqueidentifier] NOT NULL,
	[EntityFeaturePrivilege_UpdatedOn] [datetime] NULL,
	[EntityFeaturePrivilege_UpdatedBy] [uniqueidentifier] NULL,
	[EntityFeaturePrivilege_Status] [bit] NOT NULL,
	[EntityFeaturePrivilege_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_EntityFeaturePrivilege] PRIMARY KEY CLUSTERED 
(
	[EntityFeaturePrivilege_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityFeaturePrivilege]') AND name = N'Ind_EntityFeaturePrivilege_EntityId')
CREATE NONCLUSTERED INDEX [Ind_EntityFeaturePrivilege_EntityId] ON [dbo].[EntityFeaturePrivilege] 
(
	[EntityFeaturePrivilege_EntityId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityFeaturePrivilege]') AND name = N'Ind_EntityFeaturePrivilege_FeatureCode')
CREATE NONCLUSTERED INDEX [Ind_EntityFeaturePrivilege_FeatureCode] ON [dbo].[EntityFeaturePrivilege] 
(
	[EntityFeaturePrivilege_FeatureCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EntityFeaturePrivilege_EntityFeaturePrivilege_Code]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EntityFeaturePrivilege] ADD  CONSTRAINT [DF_EntityFeaturePrivilege_EntityFeaturePrivilege_Code]  DEFAULT (newid()) FOR [EntityFeaturePrivilege_Code]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EntityFields]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EntityFields](
	[EntityFields_TenantId] [uniqueidentifier] NULL,
	[EntityFields_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityFields_DataFieldTypeID] [uniqueidentifier] NOT NULL,
	[EntityFields_Id] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityFields_Name] [nvarchar](1000) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityFields_IsUnique] [bit] NULL,
	[EntityFields_AvailableInMainSchema] [bit] NULL,
	[EntityFields_PickUpListId] [uniqueidentifier] NULL,
	[EntityFields_CreatedOn] [datetime] NULL,
	[EntityFields_CreatedBy] [uniqueidentifier] NULL,
	[EntityFields_UpdatedOn] [datetime] NULL,
	[EntityFields_UpdatedBy] [uniqueidentifier] NULL,
	[EntityFields_Status] [bit] NULL,
	[EntityFields_Length] [int] NULL,
	[EntityFields_validationRegEx] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EntityFields_MappedColumnName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EntityFields_ExtnColumnDetailsId] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EntityFields_ActivatedDate] [datetime] NULL,
	[EntityFields_DeactivatedDate] [datetime] NULL,
	[EntityFields_ReferenceEntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityFields]') AND name = N'Ind_EntityFields_EntityId')
CREATE CLUSTERED INDEX [Ind_EntityFields_EntityId] ON [dbo].[EntityFields] 
(
	[EntityFields_EntityId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityFields]') AND name = N'Ind_EntityFields_DataFieldTypeID')
CREATE NONCLUSTERED INDEX [Ind_EntityFields_DataFieldTypeID] ON [dbo].[EntityFields] 
(
	[EntityFields_DataFieldTypeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityFields]') AND name = N'Ind_EntityFields_PickUpListId')
CREATE NONCLUSTERED INDEX [Ind_EntityFields_PickUpListId] ON [dbo].[EntityFields] 
(
	[EntityFields_PickUpListId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityFields]') AND name = N'Ind_EntityFields_TenantId')
CREATE NONCLUSTERED INDEX [Ind_EntityFields_TenantId] ON [dbo].[EntityFields] 
(
	[EntityFields_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Privileges]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Privileges](
	[Privilege_ID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Privilege_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Privilege_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Privilege_CreatedBy] [uniqueidentifier] NOT NULL,
	[Privilege_CreatedOn] [datetime] NOT NULL,
	[Privilege_UpdatedBy] [uniqueidentifier] NULL,
	[Privilege_UpdatedOn] [datetime] NULL,
	[Privilege_Status] [bit] NOT NULL,
	[Privilege_BasePrivilege] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Privilege_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK__Privileg__645E08121D7B6025] PRIMARY KEY CLUSTERED 
(
	[Privilege_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Privileges]') AND name = N'Ind_Privilege_BasePrivilege')
CREATE NONCLUSTERED INDEX [Ind_Privilege_BasePrivilege] ON [dbo].[Privileges] 
(
	[Privilege_BasePrivilege] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Privileges]') AND name = N'Ind_Privilege_Name')
CREATE NONCLUSTERED INDEX [Ind_Privilege_Name] ON [dbo].[Privileges] 
(
	[Privilege_Name] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Privileges]') AND name = N'IX_Privileges_TenantId')
CREATE NONCLUSTERED INDEX [IX_Privileges_TenantId] ON [dbo].[Privileges] 
(
	[Privilege_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EntityPrivileges]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EntityPrivileges](
	[EntityPrivilegesID] [uniqueidentifier] NOT NULL,
	[EntityID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PrivilegeID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[AddedOn] [datetime] NULL,
	[AddedBy] [uniqueidentifier] NULL,
	[EditedOn] [datetime] NULL,
	[EditedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
	[EntityPrivileges_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_EntityPrivileges] PRIMARY KEY CLUSTERED 
(
	[EntityPrivilegesID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityPrivileges]') AND name = N'Ind_EntityID')
CREATE NONCLUSTERED INDEX [Ind_EntityID] ON [dbo].[EntityPrivileges] 
(
	[EntityID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EntityPrivileges]') AND name = N'Ind_PrivilegeID')
CREATE NONCLUSTERED INDEX [Ind_PrivilegeID] ON [dbo].[EntityPrivileges] 
(
	[PrivilegeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EntityPrivileges_EntityPrivilegesID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EntityPrivileges] ADD  CONSTRAINT [DF_EntityPrivileges_EntityPrivilegesID]  DEFAULT (newid()) FOR [EntityPrivilegesID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EntityTenantScope]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EntityTenantScope](
	[EntityTenantScope_Id] [uniqueidentifier] NOT NULL,
	[EntityTenantScope_TenantId] [uniqueidentifier] NULL,
	[EntityTenantScope_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EntityTenantScope_ViewTenantScopeId] [uniqueidentifier] NULL,
	[EntityTenantScope_EditTenantScopeId] [uniqueidentifier] NULL,
	[EntityTenantScope_DeleteTenantScopeId] [uniqueidentifier] NULL,
	[EntityTenantScope_ViewSelectiveTenant] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EntityTenantScope_DeleteSelectiveTenant] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EntityTenantScope_EditSelectiveTenant] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EntityTenantScope_CreatedBy] [uniqueidentifier] NOT NULL,
	[EntityTenantScope_CreatedOn] [datetime] NOT NULL,
	[EntityTenantScope_UpdatedBy] [uniqueidentifier] NULL,
	[EntityTenantScope_UpdatedOn] [datetime] NULL,
	[EntityTenantScope_Status] [bit] NOT NULL,
 CONSTRAINT [PK_EntityTenantScope] PRIMARY KEY CLUSTERED 
(
	[EntityTenantScope_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EntityTenantScope_EntityTenantScope_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EntityTenantScope] ADD  CONSTRAINT [DF_EntityTenantScope_EntityTenantScope_Id]  DEFAULT (newid()) FOR [EntityTenantScope_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EventAudit]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EventAudit](
	[EventAudit_Id] [uniqueidentifier] NOT NULL,
	[EventAudit_EventId] [uniqueidentifier] NULL,
	[EventAudit_TenantId] [uniqueidentifier] NULL,
	[EventAudit_UserId] [uniqueidentifier] NULL,
	[EventAudit_SubjectId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EventAudit_SubjectType] [nvarchar](150) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EventAudit_SubjectValue] [xml] NULL,
	[EventAudit_SubjectXml] [xml] NULL,
	[EventAudit_TargetId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EventAudit_TargetType] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EventAudit_TargetValue] [xml] NULL,
	[EventAudit_TargetXml] [xml] NULL,
	[EventAudit_ContextId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EventAudit_ContextType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EventAudit_ContextValue] [xml] NULL,
	[EventAudit_ContextXml] [xml] NULL,
	[EventAudit_EventStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EventAudit_Priority] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EventAudit_CreatedOn] [datetime] NOT NULL,
	[EventAudit_CreatedBy] [uniqueidentifier] NOT NULL,
	[EventAudit_UpdatedOn] [datetime] NULL,
	[EventAudit_UpdatedBy] [uniqueidentifier] NULL,
	[EventAudit_Status] [bit] NOT NULL,
	[EventAudit_EventParameterValue] [xml] NULL,
	[EventAudit_EventParameterXml] [xml] NULL,
 CONSTRAINT [PK_ActivityEventLog] PRIMARY KEY NONCLUSTERED 
(
	[EventAudit_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EventAudit]') AND name = N'NonClusteredIndex-CreatedOn')
CREATE CLUSTERED INDEX [NonClusteredIndex-CreatedOn] ON [dbo].[EventAudit] 
(
	[EventAudit_CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EventAudit]') AND name = N'IX_EventAudit_TenantId')
CREATE NONCLUSTERED INDEX [IX_EventAudit_TenantId] ON [dbo].[EventAudit] 
(
	[EventAudit_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ActivityEventLog_ActivityEventLog_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EventAudit] ADD  CONSTRAINT [DF_ActivityEventLog_ActivityEventLog_Id]  DEFAULT (newid()) FOR [EventAudit_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Schedules]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Schedules](
	[Schedule_Id] [uniqueidentifier] NOT NULL,
	[Schedule_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Schedule_TenantId] [uniqueidentifier] NOT NULL,
	[Schedule_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Schedule_CreatedBy] [uniqueidentifier] NULL,
	[Schedule_CreatedOn] [datetime] NULL,
	[Schedule_UpdatedBy] [uniqueidentifier] NULL,
	[Schedule_UpdatedOn] [datetime] NULL,
	[Schedule_Status] [bit] NOT NULL,
 CONSTRAINT [PK_Schedule] PRIMARY KEY CLUSTERED 
(
	[Schedule_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Schedules]') AND name = N'IX_Schedules_TenantId')
CREATE NONCLUSTERED INDEX [IX_Schedules_TenantId] ON [dbo].[Schedules] 
(
	[Schedule_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Schedule_Schedule_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Schedules] ADD  CONSTRAINT [DF_Schedule_Schedule_Id]  DEFAULT (newid()) FOR [Schedule_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Schedule_Schedule_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Schedules] ADD  CONSTRAINT [DF_Schedule_Schedule_CreatedOn]  DEFAULT (getdate()) FOR [Schedule_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Schedule_Schedule_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Schedules] ADD  CONSTRAINT [DF_Schedule_Schedule_Status]  DEFAULT ((1)) FOR [Schedule_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EventSchedules]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EventSchedules](
	[EventSchedule_Id] [uniqueidentifier] NOT NULL,
	[EventSchedule_EventId] [uniqueidentifier] NOT NULL,
	[EventSchedule_ScheduleId] [uniqueidentifier] NOT NULL,
	[EventSchedule_CreatedBy] [uniqueidentifier] NULL,
	[EventSchedule_CreatedOn] [datetime] NULL,
	[EventSchedule_UpdatedBy] [uniqueidentifier] NULL,
	[EventSchedule_UpdatedOn] [datetime] NULL,
	[EventSchedule_Status] [bit] NOT NULL,
 CONSTRAINT [PK_EventSchedule] PRIMARY KEY CLUSTERED 
(
	[EventSchedule_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EventSchedule_EventSchedule_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EventSchedules] ADD  CONSTRAINT [DF_EventSchedule_EventSchedule_Id]  DEFAULT (newid()) FOR [EventSchedule_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EventSchedule_EventSchedule_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EventSchedules] ADD  CONSTRAINT [DF_EventSchedule_EventSchedule_CreatedOn]  DEFAULT (getdate()) FOR [EventSchedule_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EventSchedule_EventSchedule_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EventSchedules] ADD  CONSTRAINT [DF_EventSchedule_EventSchedule_Status]  DEFAULT ((1)) FOR [EventSchedule_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[EventTemplate]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[EventTemplate](
	[EventTemplate_Id] [uniqueidentifier] NOT NULL,
	[EventTemplate_EventId] [uniqueidentifier] NOT NULL,
	[EventTemplate_TenantId] [uniqueidentifier] NOT NULL,
	[EventTemplate_TemplateId] [uniqueidentifier] NOT NULL,
 CONSTRAINT [PK_EventTemplate] PRIMARY KEY CLUSTERED 
(
	[EventTemplate_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[EventTemplate]') AND name = N'IX_EventTemplate_EventId')
CREATE NONCLUSTERED INDEX [IX_EventTemplate_EventId] ON [dbo].[EventTemplate] 
(
	[EventTemplate_EventId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_EventTemplate_EventTemplateId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[EventTemplate] ADD  CONSTRAINT [DF_EventTemplate_EventTemplateId]  DEFAULT (newid()) FOR [EventTemplate_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FeaturePrivileges]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[FeaturePrivileges](
	[FeaturePrivileges_Code] [uniqueidentifier] NOT NULL,
	[FeaturePrivileges_FeatureCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FeaturePrivileges_PrivilegeCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FeaturePrivileges_CreatedOn] [datetime] NOT NULL,
	[FeaturePrivileges_CreatedBy] [uniqueidentifier] NOT NULL,
	[FeaturePrivileges_UpdatedOn] [datetime] NULL,
	[FeaturePrivileges_UpdatedBy] [uniqueidentifier] NULL,
	[FeaturePrivileges_Status] [bit] NOT NULL,
	[FeaturePrivileges_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_FeaturePrivileges] PRIMARY KEY CLUSTERED 
(
	[FeaturePrivileges_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[FeaturePrivileges]') AND name = N'Ind_FeaturePrivileges_FeatureCode')
CREATE NONCLUSTERED INDEX [Ind_FeaturePrivileges_FeatureCode] ON [dbo].[FeaturePrivileges] 
(
	[FeaturePrivileges_FeatureCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[FeaturePrivileges]') AND name = N'Ind_FeaturePrivileges_PrivilegeCode')
CREATE NONCLUSTERED INDEX [Ind_FeaturePrivileges_PrivilegeCode] ON [dbo].[FeaturePrivileges] 
(
	[FeaturePrivileges_PrivilegeCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_FeaturePrivileges_FeaturePrivileges_Code]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[FeaturePrivileges] ADD  CONSTRAINT [DF_FeaturePrivileges_FeaturePrivileges_Code]  DEFAULT (newid()) FOR [FeaturePrivileges_Code]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Services]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Services](
	[Service_Code] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Service_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Service_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Service_CreatedBy] [uniqueidentifier] NOT NULL,
	[Service_CreatedOn] [datetime] NOT NULL,
	[Service_UpdatedBy] [uniqueidentifier] NULL,
	[Service_UpdatedOn] [datetime] NULL,
	[Service_Status] [bit] NOT NULL,
 CONSTRAINT [PK_Services] PRIMARY KEY CLUSTERED 
(
	[Service_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Modules]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Modules](
	[Module_Code] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Module_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Module_ServiceCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Module_CreatedBy] [uniqueidentifier] NOT NULL,
	[Module_CreatedOn] [datetime] NOT NULL,
	[Module_UpdatedBy] [uniqueidentifier] NULL,
	[Module_UpdatedOn] [datetime] NULL,
	[Module_Status] [bit] NOT NULL,
 CONSTRAINT [PK__Modules__EB27D4327F60ED59] PRIMARY KEY CLUSTERED 
(
	[Module_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Features]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Features](
	[Feature_Code] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Feature_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Feature_ModuleCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Feature_CreatedOn] [datetime] NOT NULL,
	[Feature_CreatedBy] [uniqueidentifier] NOT NULL,
	[Feature_UpdatedBy] [uniqueidentifier] NULL,
	[Feature_UpdatedOn] [datetime] NULL,
	[Feature_Status] [bit] NOT NULL,
 CONSTRAINT [PK__Features__75CE315503317E3D] PRIMARY KEY CLUSTERED 
(
	[Feature_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Features]') AND name = N'Ind_Feature_ModuleCode')
CREATE NONCLUSTERED INDEX [Ind_Feature_ModuleCode] ON [dbo].[Features] 
(
	[Feature_ModuleCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationTemplate]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationTemplate](
	[NotificationTemplate_Id] [uniqueidentifier] NOT NULL,
	[NotificationTemplate_Template] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[NotificationTemplate_CreatedBy] [uniqueidentifier] NULL,
	[NotificationTemplate_CreatedOn] [datetime] NOT NULL,
	[NotificationTemplate_UpdatedBy] [uniqueidentifier] NULL,
	[NotificationTemplate_UpdatedOn] [datetime] NULL,
	[NotificationTemplate_Status] [bit] NOT NULL,
	[NotificationTemplate_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[NotificationTemplate_Description] [nvarchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_NotifiactionTemplate] PRIMARY KEY CLUSTERED 
(
	[NotificationTemplate_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Notificat__Notif__4277DAAA]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationTemplate] ADD  DEFAULT (getdate()) FOR [NotificationTemplate_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Notificat__Notif__436BFEE3]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationTemplate] ADD  DEFAULT ((1)) FOR [NotificationTemplate_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FileContentDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[FileContentDetails](
	[FileContentDetails_ContentDetailsId] [uniqueidentifier] NOT NULL,
	[FileContentDetails_FileTemplateId] [uniqueidentifier] NULL,
	[FileContentDetails_FilePath] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FileContentDetails_FileName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FileContentDetails_CreatedBy] [uniqueidentifier] NULL,
	[FileContentDetails_CreatedOn] [datetime] NOT NULL,
	[FileContentDetails_UpdatedBy] [uniqueidentifier] NULL,
	[FileContentDetails_UpdatedOn] [datetime] NULL,
	[FileContentDetails_Status] [bit] NOT NULL,
 CONSTRAINT [PK_FileContentDetails] PRIMARY KEY CLUSTERED 
(
	[FileContentDetails_ContentDetailsId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__FileConte__FileC__13BCEBC1]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[FileContentDetails] ADD  DEFAULT (getdate()) FOR [FileContentDetails_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__FileConte__FileC__14B10FFA]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[FileContentDetails] ADD  DEFAULT ((1)) FOR [FileContentDetails_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationContent]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationContent](
	[NotificationContent_Id] [uniqueidentifier] NOT NULL,
	[NotificationContent_TemplateId] [uniqueidentifier] NULL,
	[NotificationContent_NotificationAuditId] [uniqueidentifier] NULL,
	[NotificationContent_CreatedOn] [datetime] NOT NULL,
	[NotificationContent_CreatedBy] [uniqueidentifier] NOT NULL,
	[NotificationContent_UpdatedOn] [datetime] NULL,
	[NotificationContent_UpdatedBy] [uniqueidentifier] NULL,
	[NotificationContent_Status] [bit] NOT NULL,
 CONSTRAINT [PK_NotificationContent] PRIMARY KEY CLUSTERED 
(
	[NotificationContent_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_NotificationContent_NotificationContent_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationContent] ADD  CONSTRAINT [DF_NotificationContent_NotificationContent_Id]  DEFAULT (newid()) FOR [NotificationContent_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FileNotificationContent]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[FileNotificationContent](
	[FileNotificationContent_NotificationContentId] [uniqueidentifier] NOT NULL,
	[FileNotificationContent_ArchivedFilePath] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FileNotificationContent_FileType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FileNotificationContent_FileName] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FileNotificationContent_CreatedOn] [datetime] NOT NULL,
	[FileNotificationContent_CreatedBy] [uniqueidentifier] NOT NULL,
	[FileNotificationContent_UpdatedOn] [datetime] NULL,
	[FileNotificationContent_UpdatedBy] [uniqueidentifier] NULL,
	[FileNotificationContent_Status] [bit] NOT NULL,
 CONSTRAINT [PK_FileNotificationContent] PRIMARY KEY CLUSTERED 
(
	[FileNotificationContent_NotificationContentId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FtpDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[FtpDetails](
	[FtpDetails_Id] [uniqueidentifier] NOT NULL,
	[FtpDetails_BatchId] [uniqueidentifier] NULL,
	[FtpDetails_FtpAddress] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpDetails_FtpUserName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpDetails_FtpPassword] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpDetails_FilePath] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FtpDetails_FileName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FtpDetails_FtpStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpDetails_StatusMessage] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FtpDetails_CreatedOn] [datetime] NOT NULL,
	[FtpDetails_CreatedBy] [uniqueidentifier] NOT NULL,
	[FtpDetails_UpdatedOn] [datetime] NULL,
	[FtpDetails_UpdatedBy] [uniqueidentifier] NULL,
	[FtpDetails_IsSecured] [bit] NOT NULL,
	[FtpDetails_TrackId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_FtpDetails] PRIMARY KEY CLUSTERED 
(
	[FtpDetails_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_FtpDetails_FtpDetails_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[FtpDetails] ADD  CONSTRAINT [DF_FtpDetails_FtpDetails_Id]  DEFAULT (newid()) FOR [FtpDetails_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FtpDispatchDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[FtpDispatchDetails](
	[FtpDispatchDetails_DispatchDetailsId] [uniqueidentifier] NOT NULL,
	[FtpDispatchDetails_FtpAddress] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpDispatchDetails_UserName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpDispatchDetails_Password] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpDispatchDetails_IsSecured] [bit] NOT NULL,
	[FtpDispatchDetails_CreatedBy] [uniqueidentifier] NULL,
	[FtpDispatchDetails_CreatedOn] [datetime] NOT NULL,
	[FtpDispatchDetails_UpdatedBy] [uniqueidentifier] NULL,
	[FtpDispatchDetails_UpdatedOn] [datetime] NULL,
	[FtpDispatchDetails_Status] [bit] NOT NULL,
 CONSTRAINT [PK_FtpDispatchDetails] PRIMARY KEY CLUSTERED 
(
	[FtpDispatchDetails_DispatchDetailsId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__FtpDispat__FtpDi__1699586C]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[FtpDispatchDetails] ADD  DEFAULT (getdate()) FOR [FtpDispatchDetails_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__FtpDispat__FtpDi__178D7CA5]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[FtpDispatchDetails] ADD  DEFAULT ((1)) FOR [FtpDispatchDetails_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FtpNotificationDispatch]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[FtpNotificationDispatch](
	[FtpNotificationDispatch_NotificationDispatchId] [uniqueidentifier] NOT NULL,
	[FtpNotificationDispatch_FtpAddress] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpNotificationDispatch_UserName] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpNotificationDispatch_Password] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[FtpNotificationDispatch_IsSecured] [bit] NOT NULL,
	[FtpNotificationDispatch_CreatedOn] [datetime] NOT NULL,
	[FtpNotificationDispatch_CreatedBy] [uniqueidentifier] NOT NULL,
	[FtpNotificationDispatch_UpdatedOn] [datetime] NULL,
	[FtpNotificationDispatch_UpdatedBy] [uniqueidentifier] NULL,
	[FtpNotificationDispatch_Status] [bit] NOT NULL,
 CONSTRAINT [PK_FtpNotificationDispatch] PRIMARY KEY CLUSTERED 
(
	[FtpNotificationDispatch_NotificationDispatchId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ImageSource]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ImageSource](
	[ImageSource_SourceId] [uniqueidentifier] NOT NULL,
	[ImageSource_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ImageSource_SourcePath] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ImageSource_TenantId] [uniqueidentifier] NULL,
	[ImageSource_CreatedBy] [uniqueidentifier] NOT NULL,
	[ImageSource_CreatedOn] [datetime] NOT NULL,
	[ImageSource_UpdatedBy] [uniqueidentifier] NULL,
	[ImageSource_UpdatedOn] [datetime] NULL,
	[ImageSource_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ImageSource] PRIMARY KEY CLUSTERED 
(
	[ImageSource_SourceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ImageSource_ImageSource_SourceId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ImageSource] ADD  CONSTRAINT [DF_ImageSource_ImageSource_SourceId]  DEFAULT (newid()) FOR [ImageSource_SourceId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Invoices]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Invoices](
	[Id] [uniqueidentifier] NOT NULL,
	[InvoiceNo] [nvarchar](250) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[PackageId] [uniqueidentifier] NOT NULL,
	[PricePlanId] [uniqueidentifier] NOT NULL,
	[InvoiceDate] [datetime] NOT NULL,
	[StartDate] [datetime] NOT NULL,
	[EndDate] [datetime] NOT NULL,
	[NextInvoiceDate] [datetime] NOT NULL,
	[InvoiceStatus] [int] NOT NULL,
	[InvoiceType] [int] NOT NULL,
	[Memo] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DueDate] [datetime] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[BillFrequency] [int] NOT NULL,
	[PendingBalance] [float] NOT NULL,
	[ParentTenantId] [uniqueidentifier] NOT NULL,
 CONSTRAINT [PK_Invoices] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Invoices]') AND name = N'IX_Invoices_InvoiceDate')
CREATE NONCLUSTERED INDEX [IX_Invoices_InvoiceDate] ON [dbo].[Invoices] 
(
	[InvoiceDate] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Invoices]') AND name = N'IX_Invoices_ParentTenatnId')
CREATE NONCLUSTERED INDEX [IX_Invoices_ParentTenatnId] ON [dbo].[Invoices] 
(
	[ParentTenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Invoices]') AND name = N'IX_Invoices_TenantId')
CREATE NONCLUSTERED INDEX [IX_Invoices_TenantId] ON [dbo].[Invoices] 
(
	[TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Invoices__Id__1B5E0D89]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Invoices] ADD  DEFAULT (newid()) FOR [Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Invoices__Invoic__797309D9]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Invoices] ADD  DEFAULT ((0)) FOR [InvoiceType]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[InvoiceLineItems]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[InvoiceLineItems](
	[Id] [uniqueidentifier] NOT NULL,
	[InvoiceId] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Amount] [float] NOT NULL,
	[Type] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Usage] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[IsUserAdded] [bit] NOT NULL,
	[ParentTenantId] [uniqueidentifier] NOT NULL,
 CONSTRAINT [PK_InvoiceLineItems] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[InvoiceLineItems]') AND name = N'IX_FK_InvoiceInvoiceLineItems')
CREATE NONCLUSTERED INDEX [IX_FK_InvoiceInvoiceLineItems] ON [dbo].[InvoiceLineItems] 
(
	[InvoiceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[InvoiceLineItems]') AND name = N'IX_InvoiceLineItems_ParentTenantId')
CREATE NONCLUSTERED INDEX [IX_InvoiceLineItems_ParentTenantId] ON [dbo].[InvoiceLineItems] 
(
	[ParentTenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[InvoiceLineItems]') AND name = N'IX_InvoiceLineItems_TenantId')
CREATE NONCLUSTERED INDEX [IX_InvoiceLineItems_TenantId] ON [dbo].[InvoiceLineItems] 
(
	[TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__InvoiceLineI__Id__1975C517]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[InvoiceLineItems] ADD  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[InvoicePayments]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[InvoicePayments](
	[Id] [uniqueidentifier] NOT NULL,
	[InvoiceId] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[PaymentMode] [int] NOT NULL,
	[PaidAmount] [float] NOT NULL,
	[PaymentDate] [datetime] NOT NULL,
	[CreatedBy] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Notes] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ParentTenantId] [uniqueidentifier] NOT NULL,
 CONSTRAINT [PK_InvoicePayments] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[InvoicePayments]') AND name = N'IX_FK_InvoicePayments_Invoices')
CREATE NONCLUSTERED INDEX [IX_FK_InvoicePayments_Invoices] ON [dbo].[InvoicePayments] 
(
	[InvoiceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[InvoicePayments]') AND name = N'IX_InvoicePayments_ParentTenantId')
CREATE NONCLUSTERED INDEX [IX_InvoicePayments_ParentTenantId] ON [dbo].[InvoicePayments] 
(
	[ParentTenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[InvoicePayments]') AND name = N'IX_InvoicePayments_TenantId')
CREATE NONCLUSTERED INDEX [IX_InvoicePayments_TenantId] ON [dbo].[InvoicePayments] 
(
	[TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__InvoicePayme__Id__1A69E950]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[InvoicePayments] ADD  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Jobs]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Jobs](
	[Job_Id] [uniqueidentifier] NOT NULL,
	[Job_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Job_Type] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Job_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Job_CreatedBy] [uniqueidentifier] NULL,
	[Job_CreatedOn] [datetime] NULL,
	[Job_UpdatedBy] [uniqueidentifier] NULL,
	[Job_UpdatedOn] [datetime] NULL,
	[Job_Status] [bit] NOT NULL,
 CONSTRAINT [PK_Job] PRIMARY KEY CLUSTERED 
(
	[Job_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Jobs]') AND name = N'jobname')
CREATE NONCLUSTERED INDEX [jobname] ON [dbo].[Jobs] 
(
	[Job_Name] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Jobs_Job_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Jobs] ADD  CONSTRAINT [DF_Jobs_Job_Id]  DEFAULT (newid()) FOR [Job_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[JobAudits]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[JobAudits](
	[JobAudit_Id] [uniqueidentifier] NOT NULL,
	[JobAudit_JobId] [uniqueidentifier] NOT NULL,
	[JobAudit_StartTime] [datetime] NOT NULL,
	[JobAudit_EndTime] [datetime] NULL,
	[JobAudit_ExecutionStatus] [bit] NOT NULL,
	[JobAudit_ExceptionDetails] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[JobAudit_CreatedBy] [uniqueidentifier] NULL,
	[JobAudit_CreatedOn] [datetime] NULL,
	[JobAudit_UpdatedBy] [uniqueidentifier] NULL,
	[JobAudit_UpdatedOn] [datetime] NULL,
	[JobAudit_Status] [bit] NOT NULL,
	[JobAudit_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_JobAudit] PRIMARY KEY CLUSTERED 
(
	[JobAudit_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_JobAudit_JobAudit_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[JobAudits] ADD  CONSTRAINT [DF_JobAudit_JobAudit_Id]  DEFAULT (newid()) FOR [JobAudit_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_JobAudit_JobAudit_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[JobAudits] ADD  CONSTRAINT [DF_JobAudit_JobAudit_CreatedOn]  DEFAULT (getdate()) FOR [JobAudit_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_JobAudit_JobAudit_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[JobAudits] ADD  CONSTRAINT [DF_JobAudit_JobAudit_Status]  DEFAULT ((1)) FOR [JobAudit_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[JobParameter]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[JobParameter](
	[JobParameter_Id] [uniqueidentifier] NOT NULL,
	[JobParameter_EventId] [uniqueidentifier] NULL,
	[JobParameter_TenantId] [uniqueidentifier] NULL,
	[JobParameter_JobId] [uniqueidentifier] NULL,
	[JobParameter_ParameterXML] [xml] NULL,
	[JobParameter_CreatedBy] [uniqueidentifier] NOT NULL,
	[JobParameter_CreatedOn] [datetime] NOT NULL,
	[JobParameter_UpdatedBy] [uniqueidentifier] NULL,
	[JobParameter_UpdatedOn] [datetime] NULL,
	[JobParameter_Status] [bit] NOT NULL,
 CONSTRAINT [PK_JobParameter] PRIMARY KEY CLUSTERED 
(
	[JobParameter_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_JobParameter_JobParameter_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[JobParameter] ADD  CONSTRAINT [DF_JobParameter_JobParameter_Id]  DEFAULT (newid()) FOR [JobParameter_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Language]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Language](
	[Language_Id] [uniqueidentifier] NOT NULL,
	[Language_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Language_Code] [nvarchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Language_CreatedBy] [uniqueidentifier] NOT NULL,
	[Language_CreatedOn] [datetime] NOT NULL,
	[Language_UpdatedBy] [uniqueidentifier] NULL,
	[Language_UpdatedOn] [datetime] NULL,
	[Language_Status] [bit] NOT NULL,
 CONSTRAINT [PK_Language] PRIMARY KEY CLUSTERED 
(
	[Language_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Language__Langua__02084FDA]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Language] ADD  DEFAULT (newid()) FOR [Language_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackage]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LicensePackage](
	[LicensePackage_ID] [uniqueidentifier] NOT NULL,
	[LicensePackage_Name] [nvarchar](250) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LicensePackage_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[LicensePackage_CreatedBy] [uniqueidentifier] NOT NULL,
	[LicensePackage_CreatedOn] [datetime] NOT NULL,
	[LicensePackage_UpdatedBy] [uniqueidentifier] NULL,
	[LicensePackage_UpdatedOn] [datetime] NULL,
	[LicensePackage_Status] [bit] NOT NULL,
	[LicensePackage_TenantId] [uniqueidentifier] NULL,
	[LicensePackage_IntegrateForBilling] [bit] NOT NULL,
	[LicensePackage_IsEvaluation] [bit] NOT NULL,
	[LicensePackage_ExpiryDays] [int] NOT NULL,
 CONSTRAINT [PK_LicensePackage] PRIMARY KEY CLUSTERED 
(
	[LicensePackage_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicensePackage_LicensePackage_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicensePackage] ADD  CONSTRAINT [DF_LicensePackage_LicensePackage_ID]  DEFAULT (newid()) FOR [LicensePackage_ID]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicensePackage_LicensePackage_IntegrateForBilling]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicensePackage] ADD  CONSTRAINT [DF_LicensePackage_LicensePackage_IntegrateForBilling]  DEFAULT ((0)) FOR [LicensePackage_IntegrateForBilling]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicensePackage_LicensePackage_IsEvaluation]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicensePackage] ADD  CONSTRAINT [DF_LicensePackage_LicensePackage_IsEvaluation]  DEFAULT ((0)) FOR [LicensePackage_IsEvaluation]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicensePackage_LicensePackage_ExpiryDays]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicensePackage] ADD  CONSTRAINT [DF_LicensePackage_LicensePackage_ExpiryDays]  DEFAULT ((0)) FOR [LicensePackage_ExpiryDays]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantLicense]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantLicense](
	[TenantLicense_Code] [uniqueidentifier] NOT NULL,
	[TenantLicense_TenantID] [uniqueidentifier] NOT NULL,
	[TenantLicense_ValidityStart] [datetime] NOT NULL,
	[TenantLicense_ValidityEnd] [datetime] NULL,
	[TenantLicense_NumberOfUser] [int] NULL,
	[TenantLicense_LicensePackageId] [uniqueidentifier] NOT NULL,
	[TenantLicense_CreatedOn] [datetime] NOT NULL,
	[TenantLicense_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantLicense_UpdatedOn] [datetime] NULL,
	[TenantLicense_UpdatedBy] [uniqueidentifier] NULL,
	[TenantLicense_Status] [bit] NOT NULL,
	[TenantLicense_PricePlanId] [uniqueidentifier] NULL,
	[TenantLicense_TrialEndDate] [datetime] NULL,
 CONSTRAINT [PK_TenantLicense] PRIMARY KEY CLUSTERED 
(
	[TenantLicense_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TenantLicense]') AND name = N'Ind_TenantLicense_LicensePackageId')
CREATE NONCLUSTERED INDEX [Ind_TenantLicense_LicensePackageId] ON [dbo].[TenantLicense] 
(
	[TenantLicense_LicensePackageId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TenantLicense]') AND name = N'Ind_TenantLicense_TenantID')
CREATE NONCLUSTERED INDEX [Ind_TenantLicense_TenantID] ON [dbo].[TenantLicense] 
(
	[TenantLicense_TenantID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TenantLicense]') AND name = N'Ind_TenantLicense_ValidityEnd')
CREATE NONCLUSTERED INDEX [Ind_TenantLicense_ValidityEnd] ON [dbo].[TenantLicense] 
(
	[TenantLicense_ValidityEnd] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TenantLicense]') AND name = N'Ind_TenantLicense_ValidityStart')
CREATE NONCLUSTERED INDEX [Ind_TenantLicense_ValidityStart] ON [dbo].[TenantLicense] 
(
	[TenantLicense_ValidityStart] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantLicense_TenantLicense_Code]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantLicense] ADD  CONSTRAINT [DF_TenantLicense_TenantLicense_Code]  DEFAULT (newid()) FOR [TenantLicense_Code]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LicenseFeature]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LicenseFeature](
	[LicenseFeature_ID] [uniqueidentifier] NOT NULL,
	[LicenseFeature_TenantLicenseCode] [uniqueidentifier] NOT NULL,
	[LicenseFeature_PurchasedFeatureCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LicenseFeature_ValidityStartDate] [datetime] NOT NULL,
	[LicenseFeature_ValidityEndDate] [datetime] NULL,
	[LicenseFeature_CreatedOn] [datetime] NOT NULL,
	[LicenseFeature_CreatedBy] [uniqueidentifier] NOT NULL,
	[LicenseFeature_UpdatedOn] [datetime] NULL,
	[LicenseFeature_UpdatedBy] [uniqueidentifier] NULL,
	[LicenseFeature_Status] [bit] NOT NULL,
 CONSTRAINT [PK_LicenseFeature] PRIMARY KEY CLUSTERED 
(
	[LicenseFeature_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseFeature]') AND name = N'Ind_LicenseFeature_PurchasedFeatureCode')
CREATE NONCLUSTERED INDEX [Ind_LicenseFeature_PurchasedFeatureCode] ON [dbo].[LicenseFeature] 
(
	[LicenseFeature_PurchasedFeatureCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseFeature]') AND name = N'Ind_LicenseFeature_TenantLicenseCode')
CREATE NONCLUSTERED INDEX [Ind_LicenseFeature_TenantLicenseCode] ON [dbo].[LicenseFeature] 
(
	[LicenseFeature_TenantLicenseCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseFeature]') AND name = N'Ind_LicenseFeature_ValidityEndDate')
CREATE NONCLUSTERED INDEX [Ind_LicenseFeature_ValidityEndDate] ON [dbo].[LicenseFeature] 
(
	[LicenseFeature_ValidityEndDate] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseFeature]') AND name = N'Ind_LicenseFeature_ValidityStartDate')
CREATE NONCLUSTERED INDEX [Ind_LicenseFeature_ValidityStartDate] ON [dbo].[LicenseFeature] 
(
	[LicenseFeature_ValidityStartDate] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicenseFeature_LicenseFeature_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicenseFeature] ADD  CONSTRAINT [DF_LicenseFeature_LicenseFeature_ID]  DEFAULT (newid()) FOR [LicenseFeature_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LicenseModule]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LicenseModule](
	[LicenseModule_TenantLicenseCode] [uniqueidentifier] NOT NULL,
	[LicenseModule_PurchasedModuleCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LicenseModule_NumberOfUsers] [int] NULL,
	[LicenseModule_ValidityStart] [datetime] NOT NULL,
	[LicenseModule_ValidityEnd] [datetime] NULL,
	[LicenseModule_CreatedOn] [datetime] NOT NULL,
	[LicenseModule_CreatedBy] [uniqueidentifier] NOT NULL,
	[LicenseModule_UpdatedOn] [datetime] NULL,
	[LicenseModule_UpdatedBy] [uniqueidentifier] NULL,
	[LicenseModule_Status] [bit] NOT NULL,
	[LicenseModule_ID] [uniqueidentifier] NOT NULL,
 CONSTRAINT [PK_LicenseModule] PRIMARY KEY CLUSTERED 
(
	[LicenseModule_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseModule]') AND name = N'Ind_LicenseModule_PurchasedModuleCode')
CREATE NONCLUSTERED INDEX [Ind_LicenseModule_PurchasedModuleCode] ON [dbo].[LicenseModule] 
(
	[LicenseModule_PurchasedModuleCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseModule]') AND name = N'Ind_LicenseModule_TenantLicenseCode')
CREATE NONCLUSTERED INDEX [Ind_LicenseModule_TenantLicenseCode] ON [dbo].[LicenseModule] 
(
	[LicenseModule_TenantLicenseCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseModule]') AND name = N'Ind_LicenseModule_ValidityEnd')
CREATE NONCLUSTERED INDEX [Ind_LicenseModule_ValidityEnd] ON [dbo].[LicenseModule] 
(
	[LicenseModule_ValidityEnd] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseModule]') AND name = N'Ind_LicenseModule_ValidityStart')
CREATE NONCLUSTERED INDEX [Ind_LicenseModule_ValidityStart] ON [dbo].[LicenseModule] 
(
	[LicenseModule_ValidityStart] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicenseModule_LicenseModule_TenantLicenseCode]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicenseModule] ADD  CONSTRAINT [DF_LicenseModule_LicenseModule_TenantLicenseCode]  DEFAULT (newid()) FOR [LicenseModule_TenantLicenseCode]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackageFeature]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LicensePackageFeature](
	[LicensePackageFeature_ID] [uniqueidentifier] NOT NULL,
	[LicensePackageFeature_LicensePackageID] [uniqueidentifier] NOT NULL,
	[LicensePackageFeature_FeatureID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LicensePackageFeature_CreatedBy] [uniqueidentifier] NOT NULL,
	[LicensePackageFeature_CreatedOn] [datetime] NOT NULL,
	[LicensePackageFeature_UpdatedBy] [uniqueidentifier] NULL,
	[LicensePackageFeature_UpdatedOn] [datetime] NULL,
	[LicensePackageFeature_Status] [bit] NOT NULL,
	[LicensePackageFeature_IsAssignable] [bit] NOT NULL,
	[LicensePackageFeature_IsPossessed] [bit] NOT NULL,
 CONSTRAINT [PK_LicensePackageFeature] PRIMARY KEY CLUSTERED 
(
	[LicensePackageFeature_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackageFeature]') AND name = N'Ind_LicensePackageFeature_FeatureID')
CREATE NONCLUSTERED INDEX [Ind_LicensePackageFeature_FeatureID] ON [dbo].[LicensePackageFeature] 
(
	[LicensePackageFeature_FeatureID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackageFeature]') AND name = N'Ind_LicensePackageFeature_LicensePackageID')
CREATE NONCLUSTERED INDEX [Ind_LicensePackageFeature_LicensePackageID] ON [dbo].[LicensePackageFeature] 
(
	[LicensePackageFeature_LicensePackageID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicensePackageFeature_LicensePackageFeature_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicensePackageFeature] ADD  CONSTRAINT [DF_LicensePackageFeature_LicensePackageFeature_ID]  DEFAULT (newid()) FOR [LicensePackageFeature_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackageModule]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LicensePackageModule](
	[LicensePackageModule_ID] [uniqueidentifier] NOT NULL,
	[LicensePackageModule_ModuleID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LicensePackageModule_LicensepackageID] [uniqueidentifier] NOT NULL,
	[LicensePackageModule_CreatedBy] [uniqueidentifier] NOT NULL,
	[LicensePackageModule_CreatedOn] [datetime] NOT NULL,
	[LicensePackageModule_UpdatedBy] [uniqueidentifier] NULL,
	[LicensePackageModule_UpdatedOn] [datetime] NULL,
	[LicensePackageModule_Status] [bit] NOT NULL,
	[LicensePackageModule_IsAssignable] [bit] NOT NULL,
	[LicensePackageModule_IsPossessed] [bit] NOT NULL,
 CONSTRAINT [PK_LicensePackageModule] PRIMARY KEY CLUSTERED 
(
	[LicensePackageModule_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackageModule]') AND name = N'Ind_LicensePackageModule_LicensepackageID')
CREATE NONCLUSTERED INDEX [Ind_LicensePackageModule_LicensepackageID] ON [dbo].[LicensePackageModule] 
(
	[LicensePackageModule_LicensepackageID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackageModule]') AND name = N'Ind_LicensePackageModule_ModuleID')
CREATE NONCLUSTERED INDEX [Ind_LicensePackageModule_ModuleID] ON [dbo].[LicensePackageModule] 
(
	[LicensePackageModule_ModuleID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicensePackageModule_LicensePackageModule_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicensePackageModule] ADD  CONSTRAINT [DF_LicensePackageModule_LicensePackageModule_ID]  DEFAULT (newid()) FOR [LicensePackageModule_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackageService]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LicensePackageService](
	[LicensePackageService_ID] [uniqueidentifier] NOT NULL,
	[LicensePackageService_ServiceID] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LicensePackageService_LicensepackageID] [uniqueidentifier] NOT NULL,
	[LicensePackageService_CreatedBy] [uniqueidentifier] NOT NULL,
	[LicensePackageService_CreatedOn] [datetime] NOT NULL,
	[LicensePackageService_UpdatedBy] [uniqueidentifier] NULL,
	[LicensePackageService_UpdatedOn] [datetime] NULL,
	[LicensePackageService_Status] [bit] NOT NULL,
	[LicensePackageService_IsAssignable] [bit] NULL,
	[LicensePackageService_IsPossessed] [bit] NULL,
 CONSTRAINT [PK_LicensePackageService] PRIMARY KEY CLUSTERED 
(
	[LicensePackageService_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicensePackageService_LicensePackageService_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicensePackageService] ADD  CONSTRAINT [DF_LicensePackageService_LicensePackageService_ID]  DEFAULT (newid()) FOR [LicensePackageService_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackageUsage]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LicensePackageUsage](
	[LicensePackageUsage_ID] [uniqueidentifier] NOT NULL,
	[LicensePackageUsage_UsageCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LicensePackageUsage_LicensePackageID] [uniqueidentifier] NOT NULL,
	[LicensePackageUsage_CreatedBy] [uniqueidentifier] NOT NULL,
	[LicensePackageUsage_CreatedOn] [datetime] NOT NULL,
	[LicensePackageUsage_UpdatedBy] [uniqueidentifier] NULL,
	[LicensePackageUsage_UpdatedOn] [datetime] NULL,
	[LicensePackageUsage_Status] [bit] NOT NULL,
	[LicensePackageUsage_MinCap] [decimal](18, 0) NULL,
	[LicensePackageUsage_MaxCap] [decimal](18, 0) NULL,
	[LicensePackageUsage_NumberPurchased] [int] NULL,
	[LicensePackageUsage_AllowOverUsage] [bit] NULL,
	[LicensePackageUsage_IsAssignable] [bit] NOT NULL,
	[LicensePackageUsage_IsPossessed] [bit] NOT NULL,
 CONSTRAINT [PK_LicensePackageUsage] PRIMARY KEY CLUSTERED 
(
	[LicensePackageUsage_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicensePackageUsage]') AND name = N'Ind_LicensePackageUsage_LicensePackageID')
CREATE NONCLUSTERED INDEX [Ind_LicensePackageUsage_LicensePackageID] ON [dbo].[LicensePackageUsage] 
(
	[LicensePackageUsage_LicensePackageID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicensePackageUsage_LicensePackageUsage_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicensePackageUsage] ADD  CONSTRAINT [DF_LicensePackageUsage_LicensePackageUsage_ID]  DEFAULT (newid()) FOR [LicensePackageUsage_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UsageType]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UsageType](
	[UsageType_Code] [uniqueidentifier] NOT NULL,
	[UsageType_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[UsageType_CreatedBy] [uniqueidentifier] NOT NULL,
	[UsageType_CreatedOn] [datetime] NOT NULL,
	[UsageType_UpdatedOn] [datetime] NULL,
	[UsageType_UpdatedBy] [uniqueidentifier] NULL,
	[UsageType_Status] [bit] NOT NULL,
 CONSTRAINT [PK_UsageType] PRIMARY KEY CLUSTERED 
(
	[UsageType_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Usage]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Usage](
	[Usage_Code] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Usage_ModuleCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Usage_UsageTypeCode] [uniqueidentifier] NOT NULL,
	[Usage_UnitOfMeasurement] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Usage_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Usage_CreatedBy] [uniqueidentifier] NOT NULL,
	[Usage_CreatedOn] [datetime] NOT NULL,
	[Usage_UpdatedBy] [uniqueidentifier] NULL,
	[Usage_UpdatedOn] [datetime] NULL,
	[Usage_Status] [bit] NOT NULL,
	[Usage_Threshold] [float] NOT NULL,
	[Usage_CanGenerateInvoice] [bit] NOT NULL,
 CONSTRAINT [PK__Usage__F4483462145C0A3F] PRIMARY KEY CLUSTERED 
(
	[Usage_Code] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Usage]') AND name = N'Ind_Usage_ModuleCode')
CREATE NONCLUSTERED INDEX [Ind_Usage_ModuleCode] ON [dbo].[Usage] 
(
	[Usage_ModuleCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Usage]') AND name = N'Ind_Usage_UsageTypeCode')
CREATE NONCLUSTERED INDEX [Ind_Usage_UsageTypeCode] ON [dbo].[Usage] 
(
	[Usage_UsageTypeCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Usage_Usage_Threshold]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Usage] ADD  CONSTRAINT [DF_Usage_Usage_Threshold]  DEFAULT ((10.0)) FOR [Usage_Threshold]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__Usage__Usage_Can__3CF40B7E]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Usage] ADD  DEFAULT ((0)) FOR [Usage_CanGenerateInvoice]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LicenseUsage]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LicenseUsage](
	[LicenseUsage_ID] [uniqueidentifier] NOT NULL,
	[LicenseUsage_TenantLicenseCode] [uniqueidentifier] NOT NULL,
	[LicenseUsage_UsageCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LicenseUsage_MinCap] [decimal](18, 0) NULL,
	[LicenseUsage_MaxCap] [decimal](18, 0) NULL,
	[LicenseUsage_NumberPurchased] [int] NULL,
	[LicenseUsage_PaymentTypeCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[LicenseUsage_AllowOverUsage] [bit] NOT NULL,
	[LicenseUsage_CreatedBy] [uniqueidentifier] NOT NULL,
	[LicenseUsage_CreatedOn] [datetime] NOT NULL,
	[LicenseUsage_UpdatedOn] [datetime] NULL,
	[LicenseUsage_UpdatedBy] [uniqueidentifier] NULL,
	[LicenseUsage_Status] [bit] NOT NULL,
 CONSTRAINT [PK_LicenseUsage] PRIMARY KEY CLUSTERED 
(
	[LicenseUsage_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseUsage]') AND name = N'Ind_LicenseUsage_PaymentTypeCode')
CREATE NONCLUSTERED INDEX [Ind_LicenseUsage_PaymentTypeCode] ON [dbo].[LicenseUsage] 
(
	[LicenseUsage_PaymentTypeCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseUsage]') AND name = N'Ind_LicenseUsage_TenantLicenseCode')
CREATE NONCLUSTERED INDEX [Ind_LicenseUsage_TenantLicenseCode] ON [dbo].[LicenseUsage] 
(
	[LicenseUsage_TenantLicenseCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[LicenseUsage]') AND name = N'Ind_LicenseUsage_UsageCode')
CREATE NONCLUSTERED INDEX [Ind_LicenseUsage_UsageCode] ON [dbo].[LicenseUsage] 
(
	[LicenseUsage_UsageCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LicenseUsage_LicenseUsage_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LicenseUsage] ADD  CONSTRAINT [DF_LicenseUsage_LicenseUsage_ID]  DEFAULT (newid()) FOR [LicenseUsage_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LinkByOtherTenantUsers]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LinkByOtherTenantUsers](
	[LinkByOtherTenantUsers_Id] [uniqueidentifier] NOT NULL,
	[LinkByOtherTenantUsers_TenantId] [uniqueidentifier] NOT NULL,
	[LinkByOtherTenantUsers_UserId] [uniqueidentifier] NOT NULL,
	[LinkByOtherTenantUsers_ReqStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LinkByOtherTenantUsers_Comments] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[LinkByOtherTenantUsers_RequestedBy] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[LinkByOtherTenantUsers_CreatedBy] [uniqueidentifier] NOT NULL,
	[LinkByOtherTenantUsers_CreatedOn] [datetime] NOT NULL,
	[LinkByOtherTenantUsers_UpdatedBy] [uniqueidentifier] NULL,
	[LinkByOtherTenantUsers_UpdatedOn] [datetime] NULL,
	[LinkByOtherTenantUsers_Status] [bit] NOT NULL,
 CONSTRAINT [PK_LinkByOtherTenant] PRIMARY KEY CLUSTERED 
(
	[LinkByOtherTenantUsers_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__LinkByOth__LinkB__2B947552]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LinkByOtherTenantUsers] ADD  DEFAULT (newid()) FOR [LinkByOtherTenantUsers_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LocaleNames]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LocaleNames](
	[LocaleNames_Id] [uniqueidentifier] NOT NULL,
	[LocaleNames_Name] [nvarchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LocaleNames_Description] [nvarchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[LocaleNames_CreatedBy] [uniqueidentifier] NOT NULL,
	[LocaleNames_CreatedOn] [datetime] NOT NULL,
	[LocaleNames_UpdatedBy] [uniqueidentifier] NULL,
	[LocaleNames_UpdatedOn] [datetime] NULL,
	[LocaleNames_Status] [bit] NOT NULL,
 CONSTRAINT [PK_LocaleNames] PRIMARY KEY CLUSTERED 
(
	[LocaleNames_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LocaleTemplate]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LocaleTemplate](
	[LocaleTemplate_TemplateId] [uniqueidentifier] NOT NULL,
	[LocaleTemplate_LocaleId] [uniqueidentifier] NULL,
	[LocaleTemplate_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[LocaleTemplate_Id] [uniqueidentifier] NOT NULL,
	[LocaleTemplate_LocaleName] [varchar](20) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[LocaleTemplate_CreatedBy] [uniqueidentifier] NULL,
	[LocaleTemplate_CreatedOn] [datetime] NULL,
	[LocaleTemplate_UpdatedBy] [uniqueidentifier] NULL,
	[LocaleTemplate_UpdatedOn] [datetime] NULL,
	[LocaleTemplate_Status] [bit] NULL,
 CONSTRAINT [PK_LocaleTemplate] PRIMARY KEY CLUSTERED 
(
	[LocaleTemplate_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__LocaleTem__Local__2C88998B]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LocaleTemplate] ADD  DEFAULT (newid()) FOR [LocaleTemplate_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__LocaleTem__Local__2D7CBDC4]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LocaleTemplate] ADD  DEFAULT ('3398f837-b988-4708-999d-d3dfe11875b3') FOR [LocaleTemplate_CreatedBy]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__LocaleTem__Local__2E70E1FD]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LocaleTemplate] ADD  DEFAULT (getdate()) FOR [LocaleTemplate_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__LocaleTem__Local__2F650636]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LocaleTemplate] ADD  DEFAULT ((1)) FOR [LocaleTemplate_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LookupRuleSetGroup]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LookupRuleSetGroup](
	[RuleSetGroupId] [uniqueidentifier] NOT NULL,
	[RuleSetGroupName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RuleSetGroupCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RuleSetGroupDescription] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedDate] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedDate] [datetime] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_LookupRulesSetGroup] PRIMARY KEY CLUSTERED 
(
	[RuleSetGroupId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_LookupRulesSetGroup_RuleSetGroupId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LookupRuleSetGroup] ADD  CONSTRAINT [DF_LookupRulesSetGroup_RuleSetGroupId]  DEFAULT (newid()) FOR [RuleSetGroupId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LookupRuleSetType]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[LookupRuleSetType](
	[RuleSetTypeId] [uniqueidentifier] NOT NULL,
	[RuleSetTypeName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RuleSetTypeCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RuleSetTypeDescription] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedDate] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedDate] [datetime] NULL,
	[Status] [bit] NOT NULL,
	[ScreenType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_LookupRuleSetType] PRIMARY KEY CLUSTERED 
(
	[RuleSetTypeId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RuleType_RuleTypeId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[LookupRuleSetType] ADD  CONSTRAINT [DF_RuleType_RuleTypeId]  DEFAULT (newid()) FOR [RuleSetTypeId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[MembershipHistory]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[MembershipHistory](
	[Membership_ID] [uniqueidentifier] NOT NULL,
	[Membership_TenantID] [uniqueidentifier] NOT NULL,
	[Membership_UserName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Membership_EmailID] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Membership_Comment] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Membership_Password] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Membership_PasswordQuestion] [varchar](225) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Membership_PasswordAnswer] [varchar](225) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Membership_IsApproved] [bit] NULL,
	[Membership_LastActivityDate] [datetime] NULL,
	[Membership_LastLoginDate] [datetime] NULL,
	[Membership_LastPasswordChangedDate] [datetime] NULL,
	[Membership_CreationDate] [datetime] NULL,
	[Membership_IsLockedOut] [bit] NULL,
	[Membership_LastLockedOutDate] [datetime] NULL,
	[Membership_FailedPasswordAttemptCount] [int] NULL,
	[Membership_FailedPasswordAttemptWindowStart] [datetime] NULL,
	[Membership_FailedPasswordAnswerAttemptCount] [int] NULL,
	[Membership_FailedPasswordAnswerAttemptWindowStart] [datetime] NULL,
	[Membership_Status] [bit] NOT NULL,
	[Membership_IsFirstTimeUser] [bit] NOT NULL,
	[Membership_IsUserPasswordChangeForced] [bit] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[MembershipHistory]') AND name = N'Ind_MembershipHistory_MembershipId')
CREATE CLUSTERED INDEX [Ind_MembershipHistory_MembershipId] ON [dbo].[MembershipHistory] 
(
	[Membership_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Metering]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Metering](
	[Metering_ID] [uniqueidentifier] NOT NULL,
	[Metering_TenantID] [uniqueidentifier] NOT NULL,
	[Metering_UsageCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Metering_UsageAmount] [decimal](18, 0) NOT NULL,
	[Metering_CreatedBy] [uniqueidentifier] NOT NULL,
	[Metering_CreatedOn] [datetime] NOT NULL,
	[Metering_UpdatedBy] [uniqueidentifier] NULL,
	[Metering_UpdatedOn] [datetime] NULL,
	[Metering_Status] [bit] NOT NULL,
 CONSTRAINT [PK_Metering] PRIMARY KEY CLUSTERED 
(
	[Metering_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Metering]') AND name = N'Ind_Metering_TenantID')
CREATE NONCLUSTERED INDEX [Ind_Metering_TenantID] ON [dbo].[Metering] 
(
	[Metering_TenantID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Metering]') AND name = N'Ind_Metering_UsageCode')
CREATE NONCLUSTERED INDEX [Ind_Metering_UsageCode] ON [dbo].[Metering] 
(
	[Metering_UsageCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Metering_Metering_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Metering] ADD  CONSTRAINT [DF_Metering_Metering_ID]  DEFAULT (newid()) FOR [Metering_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[MeteringLog]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[MeteringLog](
	[MeteringLog_ID] [uniqueidentifier] NOT NULL,
	[MeteringLog_UsageCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[MeteringLog_Amount] [decimal](18, 0) NOT NULL,
	[MeteringLog_TenantID] [uniqueidentifier] NULL,
	[MeteringLog_LoggedBy] [uniqueidentifier] NOT NULL,
	[MeteringLog_LoggedDate] [datetime] NOT NULL,
 CONSTRAINT [PK_MeteringLog] PRIMARY KEY CLUSTERED 
(
	[MeteringLog_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[MeteringLog]') AND name = N'Ind_MeteringLog_TenantID')
CREATE NONCLUSTERED INDEX [Ind_MeteringLog_TenantID] ON [dbo].[MeteringLog] 
(
	[MeteringLog_TenantID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[MeteringLog]') AND name = N'Ind_MeteringLog_UsageCode')
CREATE NONCLUSTERED INDEX [Ind_MeteringLog_UsageCode] ON [dbo].[MeteringLog] 
(
	[MeteringLog_UsageCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[MeteringLog]') AND name = N'IX_MeteringLog_Date')
CREATE NONCLUSTERED INDEX [IX_MeteringLog_Date] ON [dbo].[MeteringLog] 
(
	[MeteringLog_LoggedDate] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_MeteringLog_MeteringLog_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[MeteringLog] ADD  CONSTRAINT [DF_MeteringLog_MeteringLog_ID]  DEFAULT (newid()) FOR [MeteringLog_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ModuleConfiguration]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ModuleConfiguration](
	[ModuleConfiguration_Id] [uniqueidentifier] NOT NULL,
	[ModuleConfiguration_Name] [nvarchar](150) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ModuleConfiguration_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ModuleConfiguration_CreatedOn] [datetime] NOT NULL,
	[ModuleConfiguration_CreatedBy] [uniqueidentifier] NOT NULL,
	[ModuleConfiguration_UpdatedOn] [datetime] NULL,
	[ModuleConfiguration_UpdatedBy] [uniqueidentifier] NULL,
	[ModuleConfiguration_Status] [bit] NOT NULL,
 CONSTRAINT [PK_SettingConfiguration] PRIMARY KEY CLUSTERED 
(
	[ModuleConfiguration_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_SettingConfiguration_SettingConfiguration_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ModuleConfiguration] ADD  CONSTRAINT [DF_SettingConfiguration_SettingConfiguration_Id]  DEFAULT (newid()) FOR [ModuleConfiguration_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ModuleConfigurationSetting]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ModuleConfigurationSetting](
	[ModuleConfigurationSetting_Id] [uniqueidentifier] NOT NULL,
	[ModuleConfigurationSetting_TenantId] [uniqueidentifier] NULL,
	[ModuleConfigurationSetting_ConfigurationId] [uniqueidentifier] NOT NULL,
	[ModuleConfigurationSetting_Value] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ModuleConfigurationSetting_CreatedOn] [datetime] NOT NULL,
	[ModuleConfigurationSetting_CreatedBy] [uniqueidentifier] NOT NULL,
	[ModuleConfigurationSetting_UpdatedOn] [datetime] NULL,
	[ModuleConfigurationSetting_UpdatedBy] [uniqueidentifier] NULL,
	[ModuleConfigurationSetting_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ModuleConfigurationSetting] PRIMARY KEY CLUSTERED 
(
	[ModuleConfigurationSetting_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ModuleConfigurationSetting_ModuleConfigurationSetting_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ModuleConfigurationSetting] ADD  CONSTRAINT [DF_ModuleConfigurationSetting_ModuleConfigurationSetting_Id]  DEFAULT (newid()) FOR [ModuleConfigurationSetting_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ModuleConfigurationSetting_ModuleConfigurationSetting_ConfigurationId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ModuleConfigurationSetting] ADD  CONSTRAINT [DF_ModuleConfigurationSetting_ModuleConfigurationSetting_ConfigurationId]  DEFAULT (newid()) FOR [ModuleConfigurationSetting_ConfigurationId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NotificationAuditFailure]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[NotificationAuditFailure](
	[NotificationAuditFailure_Id] [uniqueidentifier] NOT NULL,
	[NotificationAuditFailure_NotificationAuditId] [uniqueidentifier] NOT NULL,
	[NotificationAuditFailure_NotificationContent] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[NotificationAuditFailure_NotificationDispatch] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[NotificationAuditFailure_CreatedBy] [uniqueidentifier] NOT NULL,
	[NotificationAuditFailure_CreatedOn] [datetime] NOT NULL,
	[NotificationAuditFailure_UpdatedBy] [uniqueidentifier] NOT NULL,
	[NotificationAuditFailure_UpdatedOn] [datetime] NOT NULL,
	[NotificationAuditFailure_Status] [bit] NOT NULL,
	[NotificationAuditFailure_TrackId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_NotificationAuditFailure] PRIMARY KEY CLUSTERED 
(
	[NotificationAuditFailure_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_NotificationAuditFailure_NotificationAuditFailure_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[NotificationAuditFailure] ADD  CONSTRAINT [DF_NotificationAuditFailure_NotificationAuditFailure_Id]  DEFAULT (newid()) FOR [NotificationAuditFailure_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SettingsAttribute](
	[SettingsAttribute_ID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SettingsAttribute_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SettingsAttribute_DataFieldTypeId] [uniqueidentifier] NOT NULL,
	[SettingsAttribute_ModuleCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[SettingsAttribute_FeatureCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[SettingsAttribute_AttributeSetId] [uniqueidentifier] NULL,
	[SettingsAttribute_CreatedBy] [uniqueidentifier] NOT NULL,
	[SettingsAttribute_CreatedOn] [datetime] NOT NULL,
	[SettingsAttribute_UpdatedBy] [uniqueidentifier] NULL,
	[SettingsAttribute_UpdatedOn] [datetime] NULL,
	[SettingsAttribute_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantSettingsAttribute] PRIMARY KEY CLUSTERED 
(
	[SettingsAttribute_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]') AND name = N'Ind_SettingsAttribute_AttributeSetId')
CREATE NONCLUSTERED INDEX [Ind_SettingsAttribute_AttributeSetId] ON [dbo].[SettingsAttribute] 
(
	[SettingsAttribute_AttributeSetId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]') AND name = N'Ind_SettingsAttribute_DataFieldTypeId')
CREATE NONCLUSTERED INDEX [Ind_SettingsAttribute_DataFieldTypeId] ON [dbo].[SettingsAttribute] 
(
	[SettingsAttribute_DataFieldTypeId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]') AND name = N'Ind_SettingsAttribute_FeatureCode')
CREATE NONCLUSTERED INDEX [Ind_SettingsAttribute_FeatureCode] ON [dbo].[SettingsAttribute] 
(
	[SettingsAttribute_FeatureCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]') AND name = N'Ind_SettingsAttribute_ModuleCode')
CREATE NONCLUSTERED INDEX [Ind_SettingsAttribute_ModuleCode] ON [dbo].[SettingsAttribute] 
(
	[SettingsAttribute_ModuleCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PackageSetting]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PackageSetting](
	[PackageSetting_PackageID] [uniqueidentifier] NOT NULL,
	[PackageSetting_AttributeID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PackageSetting_AttributeValue] [varchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[PackageSetting_CreatedBy] [uniqueidentifier] NOT NULL,
	[PackageSetting_CreatedOn] [datetime] NOT NULL,
	[PackageSetting_UpdatedBy] [uniqueidentifier] NULL,
	[PackageSetting_UpdatedOn] [datetime] NULL,
	[PackageSetting_Status] [bit] NOT NULL,
 CONSTRAINT [PK_PackageSetting] PRIMARY KEY CLUSTERED 
(
	[PackageSetting_PackageID] ASC,
	[PackageSetting_AttributeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PaymentAccountSettings]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PaymentAccountSettings](
	[TenantId] [uniqueidentifier] NOT NULL,
	[AccountType] [int] NOT NULL,
	[StoreKey] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Email] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CurrencyCode] [varchar](10) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CreatedBy] [nvarchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [nvarchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UpdatedOn] [datetime] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_PaymentAccountSettings] PRIMARY KEY CLUSTERED 
(
	[TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__PaymentAc__Tenan__4460231C]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PaymentAccountSettings] ADD  DEFAULT (newid()) FOR [TenantId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PaymentApiAccounts]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PaymentApiAccounts](
	[Id] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[AccountType] [int] NOT NULL,
	[UserName] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Password] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Signature] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Email] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[AppId] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Mode] [bit] NOT NULL,
 CONSTRAINT [PK_PaymentApiAccounts] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__PaymentApiAc__Id__45544755]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PaymentApiAccounts] ADD  DEFAULT (newid()) FOR [Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__PaymentApi__Mode__46486B8E]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PaymentApiAccounts] ADD  DEFAULT ((0)) FOR [Mode]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PaymentAudits]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PaymentAudits](
	[Id] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[AccountType] [int] NOT NULL,
	[InvoiceId] [uniqueidentifier] NOT NULL,
	[InvoiceNo] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TransactionId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TransactionDate] [datetime] NOT NULL,
	[Details] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Amount] [float] NOT NULL,
	[Attempt] [int] NOT NULL,
	[Status] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
 CONSTRAINT [PK_PaymentAudits] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__PaymentAudit__Id__473C8FC7]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PaymentAudits] ADD  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PickupList_Country]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PickupList_Country](
	[PickupListValue_ID] [uniqueidentifier] NOT NULL,
	[PickupListValue_Value] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PickupListValue_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[PickupListValue_PickupListID] [uniqueidentifier] NOT NULL,
	[PickupListValue_TenantCode] [uniqueidentifier] NULL,
	[PickupListValue_CreatedBy] [uniqueidentifier] NOT NULL,
	[PickupListValue_CreatedOn] [datetime] NOT NULL,
	[PickupListValue_UpdatedBy] [uniqueidentifier] NULL,
	[PickupListValue_UpdatedOn] [datetime] NULL,
	[PickupListValue_Status] [bit] NOT NULL,
 CONSTRAINT [PK_PickupList_Country] PRIMARY KEY CLUSTERED 
(
	[PickupListValue_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PickupList_Country]') AND name = N'Ind_PickupListValue_TenantCode')
CREATE NONCLUSTERED INDEX [Ind_PickupListValue_TenantCode] ON [dbo].[PickupList_Country] 
(
	[PickupListValue_TenantCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PickupListValue]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PickupListValue](
	[PickupListValue_ID] [uniqueidentifier] NOT NULL,
	[PickupListValue_ItemID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PickupListValue_ItemName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PickupListValue_PickupListID] [uniqueidentifier] NOT NULL,
	[PickupListValue_TenantCode] [uniqueidentifier] NULL,
	[PickupListValue_CreatedBy] [uniqueidentifier] NOT NULL,
	[PickupListValue_CreatedOn] [datetime] NOT NULL,
	[PickupListValue_UpdatedBy] [uniqueidentifier] NULL,
	[PickupListValue_UpdatedOn] [datetime] NULL,
	[PickupListValue_Status] [bit] NOT NULL,
	[PickupListValue_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_PickupListValue] PRIMARY KEY CLUSTERED 
(
	[PickupListValue_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PickupListValue]') AND name = N'Ind_PickupListValue_ItemID')
CREATE NONCLUSTERED INDEX [Ind_PickupListValue_ItemID] ON [dbo].[PickupListValue] 
(
	[PickupListValue_ItemID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PickupListValue]') AND name = N'Ind_PickupListValue_PickupListID')
CREATE NONCLUSTERED INDEX [Ind_PickupListValue_PickupListID] ON [dbo].[PickupListValue] 
(
	[PickupListValue_PickupListID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PickupListValue]') AND name = N'Ind_PickupListValue_TenantCode')
CREATE NONCLUSTERED INDEX [Ind_PickupListValue_TenantCode] ON [dbo].[PickupListValue] 
(
	[PickupListValue_TenantCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_PickupListValue_PickupListValue_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PickupListValue] ADD  CONSTRAINT [DF_PickupListValue_PickupListValue_ID]  DEFAULT (newid()) FOR [PickupListValue_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RelationshipMetaData]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RelationshipMetaData](
	[RelationshipMetaData_ID] [uniqueidentifier] NOT NULL,
	[RelationshipMetaData_SourceID] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RelationshipMetaData_TargetID] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RelationshipMetaData_TargetKeyName] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[RelationshipMetaData_TargetDisplayName] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[RelationshipMetaData_RelationType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RelationshipMetaData_TargetType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RelationshipMetaData_TenantID] [uniqueidentifier] NULL,
	[RelationshipMetaData_CreatedOn] [datetime] NOT NULL,
	[RelationshipMetaData_CreatedBy] [uniqueidentifier] NOT NULL,
	[RelationshipMetaData_UpdatedOn] [datetime] NULL,
	[RelationshipMetaData_UpdatedBy] [uniqueidentifier] NULL,
	[RelationshipMetaData_Status] [bit] NOT NULL,
 CONSTRAINT [PK_RelationshipMetaData] PRIMARY KEY CLUSTERED 
(
	[RelationshipMetaData_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RelationshipMetaData]') AND name = N'Ind_RelationshipMetaData_SourceID')
CREATE NONCLUSTERED INDEX [Ind_RelationshipMetaData_SourceID] ON [dbo].[RelationshipMetaData] 
(
	[RelationshipMetaData_SourceID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RelationshipMetaData]') AND name = N'Ind_RelationshipMetaData_TargetID')
CREATE NONCLUSTERED INDEX [Ind_RelationshipMetaData_TargetID] ON [dbo].[RelationshipMetaData] 
(
	[RelationshipMetaData_TargetID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RelationshipMetaData_RelationshipMetaData_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RelationshipMetaData] ADD  CONSTRAINT [DF_RelationshipMetaData_RelationshipMetaData_ID]  DEFAULT (newid()) FOR [RelationshipMetaData_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PickupListValueMapping]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PickupListValueMapping](
	[PickupListValueMapping_ID] [uniqueidentifier] NOT NULL,
	[PickupListValueMapping_SourceValueID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PickupListValueMapping_TargetValueID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PickupListValueMapping_RelationshipMetaDataID] [uniqueidentifier] NOT NULL,
	[PickupListValueMapping_TenantID] [uniqueidentifier] NULL,
	[PickupListValueMapping_CreatedOn] [datetime] NOT NULL,
	[PickupListValueMapping_CreatedBy] [uniqueidentifier] NOT NULL,
	[PickupListValueMapping_UpdatedOn] [datetime] NULL,
	[PickupListValueMapping_UpdatedBy] [uniqueidentifier] NULL,
	[PickupListValueMapping_Status] [bit] NOT NULL,
 CONSTRAINT [PK_PickupListValueMapping] PRIMARY KEY CLUSTERED 
(
	[PickupListValueMapping_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PickupListValueMapping]') AND name = N'Ind_PickupListValueMapping_RelationshipMetaDataID')
CREATE NONCLUSTERED INDEX [Ind_PickupListValueMapping_RelationshipMetaDataID] ON [dbo].[PickupListValueMapping] 
(
	[PickupListValueMapping_RelationshipMetaDataID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PickupListValueMapping]') AND name = N'Ind_PickupListValueMapping_SourceValueID')
CREATE NONCLUSTERED INDEX [Ind_PickupListValueMapping_SourceValueID] ON [dbo].[PickupListValueMapping] 
(
	[PickupListValueMapping_SourceValueID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PickupListValueMapping]') AND name = N'Ind_PickupListValueMapping_TargetValueID')
CREATE NONCLUSTERED INDEX [Ind_PickupListValueMapping_TargetValueID] ON [dbo].[PickupListValueMapping] 
(
	[PickupListValueMapping_TargetValueID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PickupListValueMapping]') AND name = N'Ind_PickupListValueMapping_TenantID')
CREATE NONCLUSTERED INDEX [Ind_PickupListValueMapping_TenantID] ON [dbo].[PickupListValueMapping] 
(
	[PickupListValueMapping_TenantID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_PickupListValueMapping_PickupListValueMapping_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PickupListValueMapping] ADD  CONSTRAINT [DF_PickupListValueMapping_PickupListValueMapping_ID]  DEFAULT (newid()) FOR [PickupListValueMapping_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PlaceHolderMapping]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PlaceHolderMapping](
	[PlaceHolderMapping_ID] [int] NOT NULL,
	[PlaceHolderMapping_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PlaceHolderMapping_EntityName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[PlaceHolderMapping_Propertyname] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
 CONSTRAINT [PK_PlaceHolderMapping] PRIMARY KEY CLUSTERED 
(
	[PlaceHolderMapping_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PricePlans]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PricePlans](
	[Id] [uniqueidentifier] NOT NULL,
	[PackageId] [uniqueidentifier] NOT NULL,
	[SkipCalculation] [bit] NOT NULL,
	[Price] [float] NOT NULL,
	[BillFrequency] [int] NOT NULL,
	[CreatedBy] [nvarchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [nvarchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UpdatedOn] [datetime] NULL,
	[Status] [bit] NOT NULL,
	[Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
 CONSTRAINT [PK_PricePlans] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__PricePlans__Id__4C0144E4]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PricePlans] ADD  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PriceTables]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PriceTables](
	[Id] [uniqueidentifier] NOT NULL,
	[SlabMeterVariable] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ActedOnMeterVariable] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FactorType] [int] NOT NULL,
	[CalculationType] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
 CONSTRAINT [PK_PriceTables] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__PriceTables__Id__4DE98D56]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PriceTables] ADD  DEFAULT (newid()) FOR [Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__PriceTabl__Facto__4EDDB18F]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PriceTables] ADD  DEFAULT ((0)) FOR [FactorType]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PricePlanLineItems]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PricePlanLineItems](
	[Id] [uniqueidentifier] NOT NULL,
	[Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[PriceTableId] [uniqueidentifier] NOT NULL,
	[PricePlanId] [uniqueidentifier] NOT NULL,
 CONSTRAINT [PK_PricePlanLineItems] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PricePlanLineItems]') AND name = N'IX_FK_PricePlanLineItemPriceTable')
CREATE NONCLUSTERED INDEX [IX_FK_PricePlanLineItemPriceTable] ON [dbo].[PricePlanLineItems] 
(
	[PriceTableId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PricePlanLineItems]') AND name = N'IX_FK_PricePlanPricePlanLineItem')
CREATE NONCLUSTERED INDEX [IX_FK_PricePlanPricePlanLineItem] ON [dbo].[PricePlanLineItems] 
(
	[PricePlanId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__PricePlanLin__Id__4B0D20AB]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PricePlanLineItems] ADD  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PriceSlabs]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[PriceSlabs](
	[Id] [uniqueidentifier] NOT NULL,
	[PriceTableId] [uniqueidentifier] NOT NULL,
	[StartValue] [float] NOT NULL,
	[EndValue] [float] NULL,
	[Factor] [float] NOT NULL,
 CONSTRAINT [PK_PriceSlabs] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PriceSlabs]') AND name = N'IX_FK_PriceTablePriceSlab')
CREATE NONCLUSTERED INDEX [IX_FK_PriceTablePriceSlab] ON [dbo].[PriceSlabs] 
(
	[PriceTableId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__PriceSlabs__Id__4CF5691D]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[PriceSlabs] ADD  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ProductAnalytics]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ProductAnalytics](
	[ProductAnalytics_Id] [uniqueidentifier] NOT NULL,
	[ProductAnalytics_UserId] [uniqueidentifier] NOT NULL,
	[ProductAnalytics_TenantId] [uniqueidentifier] NOT NULL,
	[ProductAnalytics_Controller] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ProductAnalytics_Action] [varchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ProductAnalytics_TimeStamp] [datetime] NOT NULL,
	[ProductAnalytics_BrowserData] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ProductAnalytics_MachineName] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ProductAnalytics_RequestMethod] [varchar](10) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ProductAnalytics_RequestMode] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ProductAnalytics_Exception] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ProductAnalytics_Url] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ProductAnalytics_ResponseCode] [int] NULL,
	[ProductAnalytics_ResponseDescription] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ProductAnalytics_CreatedOn] [datetime] NOT NULL,
	[ProductAnalytics_CreatedBy] [uniqueidentifier] NOT NULL,
	[ProductAnalytics_Status] [bit] NOT NULL,
	[ProductAnalytics_UserIP] [varchar](30) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ProductAnalytics_SessionID] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ProductAnalytics_ResponseLength] [bigint] NOT NULL,
 CONSTRAINT [PK__ProductA__C969771745544755] PRIMARY KEY NONCLUSTERED 
(
	[ProductAnalytics_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ProductAnalytics]') AND name = N'Ind_ProductAnalytics_CreatedOn')
CREATE CLUSTERED INDEX [Ind_ProductAnalytics_CreatedOn] ON [dbo].[ProductAnalytics] 
(
	[ProductAnalytics_CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ProductAnalytics]') AND name = N'Ind_ProductAnalytics_TenantId')
CREATE NONCLUSTERED INDEX [Ind_ProductAnalytics_TenantId] ON [dbo].[ProductAnalytics] 
(
	[ProductAnalytics_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ProductAnalytics]') AND name = N'Ind_ProductAnalytics_UserId')
CREATE NONCLUSTERED INDEX [Ind_ProductAnalytics_UserId] ON [dbo].[ProductAnalytics] 
(
	[ProductAnalytics_UserId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__ProductAn__Produ__4FD1D5C8]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ProductAnalytics] ADD  DEFAULT (newid()) FOR [ProductAnalytics_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ProductAnalytics_ProductAnalytics_ResponseLength]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ProductAnalytics] ADD  CONSTRAINT [DF_ProductAnalytics_ProductAnalytics_ResponseLength]  DEFAULT ((0)) FOR [ProductAnalytics_ResponseLength]
END
GO
SET ANSI_NULLS OFF
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ProductSettings]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ProductSettings](
	[Key] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Value] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_ProductSettings] PRIMARY KEY CLUSTERED 
(
	[Key] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Query]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Query](
	[Query_Id] [uniqueidentifier] NOT NULL,
	[Query_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Query_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Query_XmlQuery] [xml] NULL,
	[Query_TenantId] [uniqueidentifier] NULL,
	[Query_CreatedBy] [uniqueidentifier] NOT NULL,
	[Query_CreatedOn] [datetime] NOT NULL,
	[Query_UpdatedBy] [uniqueidentifier] NULL,
	[Query_UpdatedOn] [datetime] NULL,
	[Query_Status] [bit] NOT NULL,
	[Query_Type] [int] NULL,
	[Query_RenderMode] [varchar](10) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_Query] PRIMARY KEY CLUSTERED 
(
	[Query_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Query]') AND name = N'FK_Query_QueryId')
CREATE NONCLUSTERED INDEX [FK_Query_QueryId] ON [dbo].[Query] 
(
	[Query_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Query]') AND name = N'FK_Query_QueryName')
CREATE NONCLUSTERED INDEX [FK_Query_QueryName] ON [dbo].[Query] 
(
	[Query_Name] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Query_QueryId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Query] ADD  CONSTRAINT [DF_Query_QueryId]  DEFAULT (newid()) FOR [Query_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[QueryRelations]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[QueryRelations](
	[Id] [uniqueidentifier] NOT NULL,
	[Name] [nvarchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[ParentQueryId] [uniqueidentifier] NOT NULL,
	[RelatedQueryId] [uniqueidentifier] NOT NULL,
	[VariableMappings] [xml] NULL,
	[RelationType] [varchar](10) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RenderMode] [varchar](10) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RenderArea] [varchar](10) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
 CONSTRAINT [PK_QueryRelations] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_QueryRelations_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[QueryRelations] ADD  CONSTRAINT [DF_QueryRelations_Id]  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Report]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[Report](
	[Report_Id] [uniqueidentifier] NOT NULL,
	[Report_TenantId] [uniqueidentifier] NOT NULL,
	[Report_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Report_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Report_CreatedBy] [uniqueidentifier] NOT NULL,
	[Report_CreatedOn] [datetime] NOT NULL,
	[Report_UpdatedBy] [uniqueidentifier] NULL,
	[Report_UpdatedOn] [datetime] NULL,
	[Report_Status] [bit] NOT NULL,
 CONSTRAINT [PK_Report] PRIMARY KEY CLUSTERED 
(
	[Report_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Report_Report_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[Report] ADD  CONSTRAINT [DF_Report_Report_Id]  DEFAULT (newid()) FOR [Report_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ReportNotificationContent]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ReportNotificationContent](
	[ReportNotificationContent_NotificationContentId] [uniqueidentifier] NOT NULL,
	[ReportNotificationContent_ReportId] [uniqueidentifier] NULL,
	[ReportNotificationContent_StoredReportPath] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ReportNotificationContent_CreatedOn] [datetime] NOT NULL,
	[ReportNotificationContent_CreatedBy] [uniqueidentifier] NOT NULL,
	[ReportNotificationContent_UpdatedOn] [datetime] NULL,
	[ReportNotificationContent_UpdatedBy] [uniqueidentifier] NULL,
	[ReportNotificationContent_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ReportNotificationContent] PRIMARY KEY CLUSTERED 
(
	[ReportNotificationContent_NotificationContentId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ReportObjectType]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ReportObjectType](
	[ReportObjectType_Id] [uniqueidentifier] NOT NULL,
	[ReportObjectType_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ReportObjectType_CreatedBy] [uniqueidentifier] NOT NULL,
	[ReportObjectType_CreatedOn] [datetime] NOT NULL,
	[ReportObjectType_UpdatedBy] [uniqueidentifier] NULL,
	[ReportObjectType_UpdatedOn] [datetime] NULL,
	[ReportObjectType_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ReportObjectType] PRIMARY KEY CLUSTERED 
(
	[ReportObjectType_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ReportObjectType_ReportObjectType_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ReportObjectType] ADD  CONSTRAINT [DF_ReportObjectType_ReportObjectType_Id]  DEFAULT (newid()) FOR [ReportObjectType_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ReportObject]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ReportObject](
	[ReportObject_Id] [uniqueidentifier] NOT NULL,
	[ReportObject_TenantId] [uniqueidentifier] NOT NULL,
	[ReportObject_ReportId] [uniqueidentifier] NOT NULL,
	[ReportObject_SourceId] [uniqueidentifier] NULL,
	[ReportObject_Ordinal] [int] NOT NULL,
	[ReportObject_TypeId] [uniqueidentifier] NOT NULL,
	[ReportObject_CreatedBy] [uniqueidentifier] NOT NULL,
	[ReportObject_CreatedOn] [datetime] NOT NULL,
	[ReportObject_UpdatedBy] [uniqueidentifier] NULL,
	[ReportObject_UpdatedOn] [datetime] NULL,
	[ReportObject_Status] [bit] NOT NULL,
	[ReportObject_RowsPerPage] [int] NULL,
	[ReportObject_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_ReportObject] PRIMARY KEY CLUSTERED 
(
	[ReportObject_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ReportObject]') AND name = N'IX_ReportObject_ReportId')
CREATE NONCLUSTERED INDEX [IX_ReportObject_ReportId] ON [dbo].[ReportObject] 
(
	[ReportObject_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ReportObject]') AND name = N'IX_ReportObject_Sources')
CREATE NONCLUSTERED INDEX [IX_ReportObject_Sources] ON [dbo].[ReportObject] 
(
	[ReportObject_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ReportObject]') AND name = N'IX_ReportObject_TenantId')
CREATE NONCLUSTERED INDEX [IX_ReportObject_TenantId] ON [dbo].[ReportObject] 
(
	[ReportObject_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ReportObject]') AND name = N'IX_ReportObject_Types')
CREATE NONCLUSTERED INDEX [IX_ReportObject_Types] ON [dbo].[ReportObject] 
(
	[ReportObject_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ReportObject_ReportObject_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ReportObject] ADD  CONSTRAINT [DF_ReportObject_ReportObject_Id]  DEFAULT (newid()) FOR [ReportObject_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RoleAssignablePrivilege]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RoleAssignablePrivilege](
	[RoleAssignablePrivilege_Id] [uniqueidentifier] NOT NULL,
	[RoleAssignablePrivilege_RoleId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RoleAssignablePrivilege_PrivilegeId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RoleAssignablePrivilege_CreatedBy] [uniqueidentifier] NOT NULL,
	[RoleAssignablePrivilege_CreatedOn] [datetime] NOT NULL,
	[RoleAssignablePrivilege_UpdatedBy] [uniqueidentifier] NULL,
	[RoleAssignablePrivilege_UpdatedOn] [datetime] NULL,
	[RoleAssignablePrivilege_Status] [bit] NOT NULL,
 CONSTRAINT [PK_RoleAssignablePrivilege] PRIMARY KEY CLUSTERED 
(
	[RoleAssignablePrivilege_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RoleFeature]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RoleFeature](
	[RoleFeature_ID] [uniqueidentifier] NOT NULL,
	[RoleFeature_TenantID] [uniqueidentifier] NULL,
	[RoleFeature_RoleID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RoleFeature_FeatureID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RoleFeature_DataScopeID] [uniqueidentifier] NULL,
	[RoleFeature_CreatedBy] [uniqueidentifier] NOT NULL,
	[RoleFeature_CreatedOn] [datetime] NOT NULL,
	[RoleFeature_UpdatedBy] [uniqueidentifier] NULL,
	[RoleFeature_UpdatedOn] [datetime] NULL,
	[RoleFeature_Status] [bit] NOT NULL,
 CONSTRAINT [PK_RoleFeature] PRIMARY KEY CLUSTERED 
(
	[RoleFeature_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RoleFeature]') AND name = N'Ind_RoleFeature_DataScopeID')
CREATE NONCLUSTERED INDEX [Ind_RoleFeature_DataScopeID] ON [dbo].[RoleFeature] 
(
	[RoleFeature_DataScopeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RoleFeature]') AND name = N'Ind_RoleFeature_FeatureID')
CREATE NONCLUSTERED INDEX [Ind_RoleFeature_FeatureID] ON [dbo].[RoleFeature] 
(
	[RoleFeature_FeatureID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RoleFeature]') AND name = N'Ind_RoleFeature_RoleID')
CREATE NONCLUSTERED INDEX [Ind_RoleFeature_RoleID] ON [dbo].[RoleFeature] 
(
	[RoleFeature_RoleID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[RoleFeature]') AND name = N'Ind_RoleFeature_TenantID')
CREATE NONCLUSTERED INDEX [Ind_RoleFeature_TenantID] ON [dbo].[RoleFeature] 
(
	[RoleFeature_TenantID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RoleFeature_RoleFeature_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RoleFeature] ADD  CONSTRAINT [DF_RoleFeature_RoleFeature_ID]  DEFAULT (newid()) FOR [RoleFeature_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RoleService]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RoleService](
	[RoleService_Id] [uniqueidentifier] NOT NULL,
	[RoleService_RoleId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RoleService_ServiceCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RoleService_CreatedBy] [uniqueidentifier] NOT NULL,
	[RoleService_CreatedOn] [datetime] NOT NULL,
	[RoleService_UpdatedBy] [uniqueidentifier] NULL,
	[RoleService_UpdatedOn] [datetime] NULL,
	[RoleService_Status] [bit] NOT NULL,
 CONSTRAINT [PK_RoleService] PRIMARY KEY CLUSTERED 
(
	[RoleService_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RoleService_RoleService_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RoleService] ADD  CONSTRAINT [DF_RoleService_RoleService_Id]  DEFAULT (newid()) FOR [RoleService_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RoleSetting]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RoleSetting](
	[RoleSetting_RoleID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RoleSetting_AttributeID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RoleSetting_AttributeValue] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[RoleSetting_CreatedBy] [uniqueidentifier] NOT NULL,
	[RoleSetting_CreatedOn] [datetime] NOT NULL,
	[RoleSetting_UpdatedBy] [uniqueidentifier] NULL,
	[RoleSetting_UpdatedOn] [datetime] NULL,
	[RoleSetting_Status] [bit] NOT NULL,
 CONSTRAINT [PK_RoleSetting] PRIMARY KEY CLUSTERED 
(
	[RoleSetting_RoleID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RoleTenant]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RoleTenant](
	[RoleTenant_Id] [uniqueidentifier] NOT NULL,
	[RoleTenant_RoleId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[RoleTenant_TenantCode] [uniqueidentifier] NOT NULL,
	[RoleTenant_CreatedBy] [uniqueidentifier] NOT NULL,
	[RoleTenant_CreatedOn] [datetime] NOT NULL,
	[RoleTenant_UpdatedBy] [uniqueidentifier] NULL,
	[RoleTenant_UpdatedOn] [datetime] NULL,
	[RoleTenant_Status] [bit] NOT NULL,
 CONSTRAINT [PK_RoleTenant] PRIMARY KEY CLUSTERED 
(
	[RoleTenant_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RoleTenant_RoleTenant_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RoleTenant] ADD  CONSTRAINT [DF_RoleTenant_RoleTenant_Id]  DEFAULT (newid()) FOR [RoleTenant_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RuleActivities]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RuleActivities](
	[Id] [uniqueidentifier] NOT NULL,
	[RulesetCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantId] [uniqueidentifier] NULL,
	[ActivityType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ServiceEndpointId] [uniqueidentifier] NULL,
	[AssemblyTypeName] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[Status] [bit] NOT NULL,
	[ServiceOutputXML] [xml] NULL,
 CONSTRAINT [PK_RuleActivities] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RuleActivities_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RuleActivities] ADD  CONSTRAINT [DF_RuleActivities_Id]  DEFAULT (newid()) FOR [Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RuleActivities_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RuleActivities] ADD  CONSTRAINT [DF_RuleActivities_Status]  DEFAULT ((1)) FOR [Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RuleSet]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RuleSet](
	[RuleSetId] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[RuleSetTypeId] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[RulesSetGroupId] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[RuleContent] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedDate] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedDate] [datetime] NULL,
 CONSTRAINT [PK_Rule] PRIMARY KEY CLUSTERED 
(
	[RuleSetId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RuleSet_RuleSetId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RuleSet] ADD  CONSTRAINT [DF_RuleSet_RuleSetId]  DEFAULT (newid()) FOR [RuleSetId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[RuleSetMetadata]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[RuleSetMetadata](
	[Id] [uniqueidentifier] NOT NULL,
	[RulesetCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantId] [uniqueidentifier] NULL,
	[Category] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ScreenType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_RuleMetadata] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RuleMetadata_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RuleSetMetadata] ADD  CONSTRAINT [DF_RuleMetadata_Id]  DEFAULT (newid()) FOR [Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_RuleMetadata_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[RuleSetMetadata] ADD  CONSTRAINT [DF_RuleMetadata_Status]  DEFAULT ((1)) FOR [Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ScheduleJobs]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ScheduleJobs](
	[ScheduleJobs_Id] [uniqueidentifier] NOT NULL,
	[ScheduleJobs_ScheduleId] [uniqueidentifier] NOT NULL,
	[ScheduleJobs_JobId] [uniqueidentifier] NOT NULL,
	[ScheduleJobs_CreatedBy] [uniqueidentifier] NULL,
	[ScheduleJobs_CreatedOn] [datetime] NULL,
	[ScheduleJobs_UpdatedBy] [uniqueidentifier] NULL,
	[ScheduleJobs_UpdatedOn] [datetime] NULL,
	[ScheduleJobs_Status] [bit] NOT NULL,
 CONSTRAINT [PK_ScheduleJobs] PRIMARY KEY CLUSTERED 
(
	[ScheduleJobs_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ScheduleJobs_ScheduleJobs_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ScheduleJobs] ADD  CONSTRAINT [DF_ScheduleJobs_ScheduleJobs_Id]  DEFAULT (newid()) FOR [ScheduleJobs_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ScheduleJobs_ScheduleJobs_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ScheduleJobs] ADD  CONSTRAINT [DF_ScheduleJobs_ScheduleJobs_CreatedOn]  DEFAULT (getdate()) FOR [ScheduleJobs_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ScheduleJobs_ScheduleJobs_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ScheduleJobs] ADD  CONSTRAINT [DF_ScheduleJobs_ScheduleJobs_Status]  DEFAULT ((1)) FOR [ScheduleJobs_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ServiceEndpoints]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ServiceEndpoints](
	[Id] [uniqueidentifier] NOT NULL,
	[Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[EndpointURL] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SOAPAction] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantId] [uniqueidentifier] NULL,
	[STSEndpointURL] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[RPSTSEndpointURL] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[MessageModifierEndpointURL] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ServiceCertificate] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ClientCertificate] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CredentialType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[SecurityKeyType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Username] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Password] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[InputXML] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[EstablishSecurityContext] [bit] NULL,
	[NegotiateServiceCredential] [bit] NULL,
	[ContentType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Encoding] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[RequestTimeout] [int] NULL,
	[RequestMethod] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[RequestParameters] [xml] NULL,
	[HeaderParameters] [xml] NULL,
	[OutputXML] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_ServiceEndpoints] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ServiceEndpoints_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ServiceEndpoints] ADD  CONSTRAINT [DF_ServiceEndpoints_Id]  DEFAULT (newid()) FOR [Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ServiceEndpoints_SecurityKeyType]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ServiceEndpoints] ADD  CONSTRAINT [DF_ServiceEndpoints_SecurityKeyType]  DEFAULT ('SymmetricKey') FOR [SecurityKeyType]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ServiceEndpoints_EstablishSecurityContext]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ServiceEndpoints] ADD  CONSTRAINT [DF_ServiceEndpoints_EstablishSecurityContext]  DEFAULT ((0)) FOR [EstablishSecurityContext]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ServiceEndpoints_NegotiateServiceCredential]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ServiceEndpoints] ADD  CONSTRAINT [DF_ServiceEndpoints_NegotiateServiceCredential]  DEFAULT ((0)) FOR [NegotiateServiceCredential]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ServiceEndpoints_RequestTimeout]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ServiceEndpoints] ADD  CONSTRAINT [DF_ServiceEndpoints_RequestTimeout]  DEFAULT ((0)) FOR [RequestTimeout]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_ServiceEndpoints_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[ServiceEndpoints] ADD  CONSTRAINT [DF_ServiceEndpoints_Status]  DEFAULT ((1)) FOR [Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SettingTemplate]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SettingTemplate](
	[SettingTemplate_Id] [uniqueidentifier] NOT NULL,
	[SettingTemplate_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SettingTemplate_Type] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SettingTemplate_TenantId] [uniqueidentifier] NULL,
	[SettingTemplate_CreatedOn] [datetime] NOT NULL,
	[SettingTemplate_CreatedBy] [uniqueidentifier] NOT NULL,
	[SettingTemplate_UpdatedOn] [datetime] NULL,
	[SettingTemplate_UpdatedBy] [uniqueidentifier] NULL,
	[SettingTemplate_Status] [bit] NOT NULL,
 CONSTRAINT [PK_SettingTemplate] PRIMARY KEY CLUSTERED 
(
	[SettingTemplate_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_SettingTemplate_SettingTemplate_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[SettingTemplate] ADD  CONSTRAINT [DF_SettingTemplate_SettingTemplate_Id]  DEFAULT (newid()) FOR [SettingTemplate_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SettingTemplateValue]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SettingTemplateValue](
	[SettingTemplateValue_Id] [uniqueidentifier] NOT NULL,
	[SettingTemplateValue_SettingTemplateId] [uniqueidentifier] NOT NULL,
	[SettingTemplateValue_AttributeId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SettingTemplateValue_AttributeValue] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[SettingTemplateValue_CreatedOn] [datetime] NOT NULL,
	[SettingTemplateValue_CreatedBy] [uniqueidentifier] NOT NULL,
	[SettingTemplateValue_UpdatedOn] [datetime] NULL,
	[SettingTemplateValue_UpdatedBy] [uniqueidentifier] NULL,
	[SettingTemplateValue_Status] [bit] NOT NULL,
 CONSTRAINT [PK_SettingTemplateValue] PRIMARY KEY CLUSTERED 
(
	[SettingTemplateValue_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_SettingTemplateValue_SettingTemplateValue_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[SettingTemplateValue] ADD  CONSTRAINT [DF_SettingTemplateValue_SettingTemplateValue_Id]  DEFAULT (newid()) FOR [SettingTemplateValue_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SourceContent]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SourceContent](
	[SourceContent_Id] [uniqueidentifier] NOT NULL,
	[SourceContent_Content] [xml] NOT NULL,
	[SourceContent_CreatedBy] [uniqueidentifier] NOT NULL,
	[SourceContent_CreatedOn] [datetime] NOT NULL,
	[SourceContent_UpdatedBy] [uniqueidentifier] NULL,
	[SourceContent_UpdatedOn] [datetime] NULL,
	[SourceContent_Status] [bit] NOT NULL,
 CONSTRAINT [PK_SourceContent] PRIMARY KEY CLUSTERED 
(
	[SourceContent_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_SourceContent_SourceContent_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[SourceContent] ADD  CONSTRAINT [DF_SourceContent_SourceContent_Id]  DEFAULT (newid()) FOR [SourceContent_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SystemNotificationContent]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SystemNotificationContent](
	[SystemNotificationContent_NotificationContentId] [uniqueidentifier] NOT NULL,
	[SystemNotificationContent_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[SystemNotificationContent_ProcessedContent] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SystemNotificationContent_CreatedOn] [datetime] NOT NULL,
	[SystemNotificationContent_CreatedBy] [uniqueidentifier] NOT NULL,
	[SystemNotificationContent_UpdatedOn] [datetime] NULL,
	[SystemNotificationContent_UpdatedBy] [uniqueidentifier] NULL,
	[SystemNotificationContent_Status] [bit] NOT NULL,
 CONSTRAINT [PK_SystemNotificationContent] PRIMARY KEY CLUSTERED 
(
	[SystemNotificationContent_NotificationContentId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SystemNotificationContentDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SystemNotificationContentDetails](
	[SystemNotificationContentDetails_ContentDetailsId] [uniqueidentifier] NOT NULL,
	[SystemNotificationContentDetails_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[SystemNotificationContentDetails_TemplateId] [uniqueidentifier] NULL,
	[SystemNotificationContentDetails_CreatedBy] [uniqueidentifier] NULL,
	[SystemNotificationContentDetails_CreatedOn] [datetime] NOT NULL,
	[SystemNotificationContentDetails_UpdatedBy] [uniqueidentifier] NULL,
	[SystemNotificationContentDetails_UpdatedOn] [datetime] NULL,
	[SystemNotificationContentDetails_Status] [bit] NOT NULL,
	[SystemNotificationContentDetails_RuleSetCode] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_SystemNotificationContentDetails] PRIMARY KEY CLUSTERED 
(
	[SystemNotificationContentDetails_ContentDetailsId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Table_1_EmailContentDetails_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[SystemNotificationContentDetails] ADD  CONSTRAINT [DF_Table_1_EmailContentDetails_CreatedOn]  DEFAULT (getdate()) FOR [SystemNotificationContentDetails_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Table_1_EmailContentDetails_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[SystemNotificationContentDetails] ADD  CONSTRAINT [DF_Table_1_EmailContentDetails_Status]  DEFAULT ((1)) FOR [SystemNotificationContentDetails_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SystemNotificationDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SystemNotificationDetails](
	[SystemNotificationDetails_Id] [uniqueidentifier] NOT NULL,
	[SystemNotificationDetails_BatchId] [uniqueidentifier] NULL,
	[SystemNotificationDetails_MapId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SystemNotificationDetails_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SystemNotificationDetails_SystemNotificationStatus] [varchar](15) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SystemNotificationDetails_StatusMessage] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[SystemNotificationDetails_CreatedBy] [uniqueidentifier] NOT NULL,
	[SystemNotificationDetails_CreatedOn] [datetime] NOT NULL,
	[SystemNotificationDetails_UpdatedBy] [uniqueidentifier] NULL,
	[SystemNotificationDetails_UpdatedOn] [datetime] NULL,
	[SystemNotificationDetails_TrackId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_SystemNotificationDetails] PRIMARY KEY CLUSTERED 
(
	[SystemNotificationDetails_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_SystemNotificationDetails_SystemNotificationDetails_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[SystemNotificationDetails] ADD  CONSTRAINT [DF_SystemNotificationDetails_SystemNotificationDetails_Id]  DEFAULT (newid()) FOR [SystemNotificationDetails_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SystemNotificationDispatch]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SystemNotificationDispatch](
	[SystemNotificationDispatch_NotificationDispatchId] [uniqueidentifier] NOT NULL,
	[SystemNotificationDispatch_MapId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SystemNotificationDispatch_CreatedOn] [datetime] NOT NULL,
	[SystemNotificationDispatch_CreatedBy] [uniqueidentifier] NOT NULL,
	[SystemNotificationDispatch_UpdatedOn] [datetime] NULL,
	[SystemNotificationDispatch_UpdatedBy] [uniqueidentifier] NULL,
	[SystemNotificationDispatch_Status] [bit] NOT NULL,
 CONSTRAINT [PK_SystemNotificationDispatch] PRIMARY KEY CLUSTERED 
(
	[SystemNotificationDispatch_NotificationDispatchId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SystemNotificationDispatchDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SystemNotificationDispatchDetails](
	[SystemNotificationDispatchDetails_DispatchDetailsId] [uniqueidentifier] NOT NULL,
	[SystemNotificationDispatchDetails_MapId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[SystemNotificationDispatchDetails_CreatedBy] [uniqueidentifier] NULL,
	[SystemNotificationDispatchDetails_CreatedOn] [datetime] NOT NULL,
	[SystemNotificationDispatchDetails_UpdatedBy] [uniqueidentifier] NULL,
	[SystemNotificationDispatchDetails_UpdatedOn] [datetime] NULL,
	[SystemNotificationDispatchDetails_Status] [bit] NOT NULL,
 CONSTRAINT [PK_SystemNotificationDispatchDetails] PRIMARY KEY CLUSTERED 
(
	[SystemNotificationDispatchDetails_DispatchDetailsId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Table_1_EmailDispatchDetails_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[SystemNotificationDispatchDetails] ADD  CONSTRAINT [DF_Table_1_EmailDispatchDetails_CreatedOn]  DEFAULT (getdate()) FOR [SystemNotificationDispatchDetails_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_Table_1_EmailDispatchDetails_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[SystemNotificationDispatchDetails] ADD  CONSTRAINT [DF_Table_1_EmailDispatchDetails_Status]  DEFAULT ((1)) FOR [SystemNotificationDispatchDetails_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TableSource]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TableSource](
	[TableSource_Id] [uniqueidentifier] NOT NULL,
	[TableSource_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TableSource_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TableSource_DataSourceId] [uniqueidentifier] NULL,
	[TableSource_TenantId] [uniqueidentifier] NOT NULL,
	[TableSource_CreatedBy] [uniqueidentifier] NOT NULL,
	[TableSource_CreatedOn] [datetime] NOT NULL,
	[TableSource_UpdatedBy] [uniqueidentifier] NULL,
	[TableSource_UpdatedOn] [datetime] NULL,
	[TableSource_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TableSource] PRIMARY KEY CLUSTERED 
(
	[TableSource_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TableSource]') AND name = N'IX_TableSource')
CREATE NONCLUSTERED INDEX [IX_TableSource] ON [dbo].[TableSource] 
(
	[TableSource_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TableSource]') AND name = N'IX_TableSource_DataSource')
CREATE NONCLUSTERED INDEX [IX_TableSource_DataSource] ON [dbo].[TableSource] 
(
	[TableSource_DataSourceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TableSource]') AND name = N'IX_TableSource_TenantId')
CREATE NONCLUSTERED INDEX [IX_TableSource_TenantId] ON [dbo].[TableSource] 
(
	[TableSource_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TableSource_TableSource_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TableSource] ADD  CONSTRAINT [DF_TableSource_TableSource_Id]  DEFAULT (newid()) FOR [TableSource_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TableResultColumn]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TableResultColumn](
	[TableResultColumn_Id] [uniqueidentifier] NOT NULL,
	[TableResultColumn_TenantId] [uniqueidentifier] NOT NULL,
	[TableResultColumn_TableSourceId] [uniqueidentifier] NOT NULL,
	[TableResultColumn_ReportObjectId] [uniqueidentifier] NOT NULL,
	[TableResultColumn_Column] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TableResultColumn_DisplayName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TableResultColumn_Ordinal] [int] NULL,
	[TableResultColumn_CreatedBy] [uniqueidentifier] NOT NULL,
	[TableResultColumn_CreatedOn] [datetime] NOT NULL,
	[TableResultColumn_UpdatedBy] [uniqueidentifier] NULL,
	[TableResultColumn_UpdatedOn] [datetime] NULL,
	[TableResultColumn_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TableResultColumn] PRIMARY KEY CLUSTERED 
(
	[TableResultColumn_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TableResultColumn]') AND name = N'IX_TableResultColumn')
CREATE NONCLUSTERED INDEX [IX_TableResultColumn] ON [dbo].[TableResultColumn] 
(
	[TableResultColumn_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TableResultColumn]') AND name = N'IX_TableResultColumn_ReportObjectId')
CREATE NONCLUSTERED INDEX [IX_TableResultColumn_ReportObjectId] ON [dbo].[TableResultColumn] 
(
	[TableResultColumn_ReportObjectId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TableResultColumn]') AND name = N'IX_TableResultColumn_TableSourceId')
CREATE NONCLUSTERED INDEX [IX_TableResultColumn_TableSourceId] ON [dbo].[TableResultColumn] 
(
	[TableResultColumn_TableSourceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TableResultColumn]') AND name = N'IX_TableResultColumn_TenantId')
CREATE NONCLUSTERED INDEX [IX_TableResultColumn_TenantId] ON [dbo].[TableResultColumn] 
(
	[TableResultColumn_TableSourceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TableResultColumn_TableResultColumn_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TableResultColumn] ADD  CONSTRAINT [DF_TableResultColumn_TableResultColumn_Id]  DEFAULT (newid()) FOR [TableResultColumn_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TaxRate]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TaxRate](
	[TaxRateId] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_TaxRate] PRIMARY KEY CLUSTERED 
(
	[TaxRateId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantActivationDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantActivationDetails](
	[TenantId] [uniqueidentifier] NOT NULL,
	[ActivationKey] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ExpirationDate] [datetime] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantActivationDetails] PRIMARY KEY CLUSTERED 
(
	[TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantCryptKey]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantCryptKey](
	[TenantCryptKey_Id] [uniqueidentifier] NOT NULL,
	[TenantCryptKey_TenantId] [uniqueidentifier] NULL,
	[TenantCryptKey_Key] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantCryptKey_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantCryptKey_CreatedOn] [datetime] NOT NULL,
	[TenantCryptKey_UpdatedBy] [uniqueidentifier] NULL,
	[TenantCryptKey_UpdatedOn] [datetime] NULL,
	[TenantCryptKey_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantCryptKey] PRIMARY KEY CLUSTERED 
(
	[TenantCryptKey_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TenantCryptKey]') AND name = N'tenantid')
CREATE NONCLUSTERED INDEX [tenantid] ON [dbo].[TenantCryptKey] 
(
	[TenantCryptKey_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantCryptKey_TenantCryptKey_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantCryptKey] ADD  CONSTRAINT [DF_TenantCryptKey_TenantCryptKey_Id]  DEFAULT (newid()) FOR [TenantCryptKey_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantDetailsExtn]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantDetailsExtn](
	[TenantDetailsExtn_Id] [uniqueidentifier] NOT NULL,
	[TenantDetailsExtn_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantDetailsExtn_ReferenceId] [uniqueidentifier] NOT NULL,
	[TenantDetailsExtn_TenantId] [uniqueidentifier] NOT NULL,
	[TenantDetailsExtn_Column1] [int] NULL,
	[TenantDetailsExtn_Column2] [int] NULL,
	[TenantDetailsExtn_Column3] [int] NULL,
	[TenantDetailsExtn_Column4] [int] NULL,
	[TenantDetailsExtn_Column5] [int] NULL,
	[TenantDetailsExtn_Column6] [datetime] NULL,
	[TenantDetailsExtn_Column7] [datetime] NULL,
	[TenantDetailsExtn_Column8] [datetime] NULL,
	[TenantDetailsExtn_Column9] [datetime] NULL,
	[TenantDetailsExtn_Column10] [datetime] NULL,
	[TenantDetailsExtn_Column11] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column12] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column13] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column14] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column15] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column16] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column17] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column18] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column19] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column20] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column21] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column22] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column23] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column24] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column25] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column26] [float] NULL,
	[TenantDetailsExtn_Column27] [float] NULL,
	[TenantDetailsExtn_Column28] [float] NULL,
	[TenantDetailsExtn_Column29] [float] NULL,
	[TenantDetailsExtn_Column30] [float] NULL,
	[TenantDetailsExtn_Column31] [bit] NULL,
	[TenantDetailsExtn_Column32] [bit] NULL,
	[TenantDetailsExtn_Column33] [bit] NULL,
	[TenantDetailsExtn_Column34] [bit] NULL,
	[TenantDetailsExtn_Column35] [bit] NULL,
	[TenantDetailsExtn_Column36] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column37] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column38] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column39] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_Column40] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantDetailsExtn_CreatedOn] [datetime] NOT NULL,
	[TenantDetailsExtn_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantDetailsExtn_UpdatedOn] [datetime] NULL,
	[TenantDetailsExtn_UpdatedBy] [uniqueidentifier] NULL,
	[TenantDetailsExtn_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantDetailsExtn1] PRIMARY KEY CLUSTERED 
(
	[TenantDetailsExtn_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantDetailsExtn_TenantDetailsExtn_Id1]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantDetailsExtn] ADD  CONSTRAINT [DF_TenantDetailsExtn_TenantDetailsExtn_Id1]  DEFAULT (newid()) FOR [TenantDetailsExtn_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantDetailsTenantType]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantDetailsTenantType](
	[TenantDetailsTenantType_ID] [uniqueidentifier] NOT NULL,
	[TenantDetailsTenantType_Tenant_Code] [uniqueidentifier] NOT NULL,
	[TenantDetailsTenantType_TenantType_ID] [uniqueidentifier] NOT NULL,
	[TenantDetailsTenantType_Priority] [smallint] NULL,
	[TenantDetailsTenantType_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantDetailsTenantType_CreatedOn] [date] NOT NULL,
	[TenantDetailsTenantType_UpdatedBy] [uniqueidentifier] NULL,
	[TenantDetailsTenantType_UpdatedOn] [date] NULL,
	[TenantDetailsTenantType_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantDetailsTenantType] PRIMARY KEY CLUSTERED 
(
	[TenantDetailsTenantType_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantDetailsTenantType_TenantDetailsTenantType_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantDetailsTenantType] ADD  CONSTRAINT [DF_TenantDetailsTenantType_TenantDetailsTenantType_ID]  DEFAULT (newid()) FOR [TenantDetailsTenantType_ID]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantDetailsTenantType_TenantDetailsTenantType_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantDetailsTenantType] ADD  CONSTRAINT [DF_TenantDetailsTenantType_TenantDetailsTenantType_CreatedOn]  DEFAULT (getdate()) FOR [TenantDetailsTenantType_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantDetailsTenantType_TenantDetailsTenantType_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantDetailsTenantType] ADD  CONSTRAINT [DF_TenantDetailsTenantType_TenantDetailsTenantType_Status]  DEFAULT ((1)) FOR [TenantDetailsTenantType_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantRelationExpanded]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantRelationExpanded](
	[TenantRelationExpanded_Id] [uniqueidentifier] NOT NULL,
	[TenantRelationExpanded_MappingId] [uniqueidentifier] NOT NULL,
	[TenantRelationExpanded_ChildTenantId] [uniqueidentifier] NOT NULL,
	[TenantRelationExpanded_ParentTenantId] [uniqueidentifier] NOT NULL,
	[TenantRelationExpanded_Level] [int] NOT NULL,
	[TenantRelationExpanded_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantRelationExpanded_CreatedOn] [date] NOT NULL,
	[TenantRelationExpanded_UpdatedBy] [uniqueidentifier] NULL,
	[TenantRelationExpanded_UpdatedOn] [date] NULL,
	[TenantRelationExpanded_Status] [bit] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TenantRelationExpanded]') AND name = N'Ind_TenantRelationExpanded_CreatedOn')
CREATE CLUSTERED INDEX [Ind_TenantRelationExpanded_CreatedOn] ON [dbo].[TenantRelationExpanded] 
(
	[TenantRelationExpanded_CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantRelationExpanded_TenantRelationExpanded_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantRelationExpanded] ADD  CONSTRAINT [DF_TenantRelationExpanded_TenantRelationExpanded_Id]  DEFAULT (newid()) FOR [TenantRelationExpanded_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantRelationMappings]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantRelationMappings](
	[TenantRelationMapping_ID] [uniqueidentifier] NOT NULL,
	[TenantRelationMapping_ParentTenantID] [uniqueidentifier] NULL,
	[TenantRelationMapping_ChildTenantID] [uniqueidentifier] NOT NULL,
	[TenantRelationMapping_TenantRelationTypeID] [uniqueidentifier] NOT NULL,
	[TenantRelationMapping_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantRelationMapping_CreatedOn] [date] NOT NULL,
	[TenantRelationMapping_UpdatedBy] [uniqueidentifier] NULL,
	[TenantRelationMapping_UpdatedOn] [date] NULL,
	[TenantRelationMapping_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantRealationMappings] PRIMARY KEY CLUSTERED 
(
	[TenantRelationMapping_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantRealationMappings_TenantRealationMapping_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantRelationMappings] ADD  CONSTRAINT [DF_TenantRealationMappings_TenantRealationMapping_ID]  DEFAULT (newid()) FOR [TenantRelationMapping_ID]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantRelationMappings_TenantRelationMapping_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantRelationMappings] ADD  CONSTRAINT [DF_TenantRelationMappings_TenantRelationMapping_CreatedOn]  DEFAULT (getdate()) FOR [TenantRelationMapping_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantRealationMappings_TenantRealationMapping_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantRelationMappings] ADD  CONSTRAINT [DF_TenantRealationMappings_TenantRealationMapping_Status]  DEFAULT ((1)) FOR [TenantRelationMapping_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantRelationTypes]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantRelationTypes](
	[TenantRelationType_ID] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantRelationType_Name] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantRelationType_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantRelationType_CreatedOn] [date] NOT NULL,
	[TenantRelationType_UpdateBy] [uniqueidentifier] NULL,
	[TenantRelationType_UpdateOn] [date] NULL,
	[TenantRelationType_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantRelationType] PRIMARY KEY CLUSTERED 
(
	[TenantRelationType_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantRelationTypes_TenantRelationType_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantRelationTypes] ADD  CONSTRAINT [DF_TenantRelationTypes_TenantRelationType_ID]  DEFAULT (newid()) FOR [TenantRelationType_ID]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantRelationTypes_TenantRelationType_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantRelationTypes] ADD  CONSTRAINT [DF_TenantRelationTypes_TenantRelationType_CreatedOn]  DEFAULT (getdate()) FOR [TenantRelationType_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantRelationType_TenantRelationType_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantRelationTypes] ADD  CONSTRAINT [DF_TenantRelationType_TenantRelationType_Status]  DEFAULT ((1)) FOR [TenantRelationType_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantScope]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantScope](
	[TenantScope_Id] [uniqueidentifier] NOT NULL,
	[TenantScope_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantScope_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantScope_FilterQuery] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantScope_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantScope_CreatedOn] [datetime] NOT NULL,
	[TenantScope_UpdatedBy] [uniqueidentifier] NULL,
	[TenantScope_UpdatedOn] [datetime] NULL,
	[TenantScope_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantScope] PRIMARY KEY CLUSTERED 
(
	[TenantScope_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantScope_TenantScope_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantScope] ADD  CONSTRAINT [DF_TenantScope_TenantScope_Id]  DEFAULT (newid()) FOR [TenantScope_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantServiceMapping]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantServiceMapping](
	[TenantServiceMapping_Id] [uniqueidentifier] NOT NULL,
	[TenantServiceMapping_TenantId] [uniqueidentifier] NOT NULL,
	[TenantServiceMapping_ServiceId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantServiceMapping_StartDate] [datetime] NULL,
	[TenantServiceMapping_EndDate] [datetime] NULL,
	[TenantServiceMapping_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantServiceMapping_CreatedOn] [datetime] NOT NULL,
	[TenantServiceMapping_UpdatedBy] [uniqueidentifier] NULL,
	[TenantServiceMapping_UpdatedOn] [datetime] NULL,
	[TenantServiceMapping_Status] [bit] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TenantServiceMapping]') AND name = N'Ind_TenantServiceMapping_CreatedOn')
CREATE CLUSTERED INDEX [Ind_TenantServiceMapping_CreatedOn] ON [dbo].[TenantServiceMapping] 
(
	[TenantServiceMapping_CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantServiceMapping_TenantServiceMapping_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantServiceMapping] ADD  CONSTRAINT [DF_TenantServiceMapping_TenantServiceMapping_Id]  DEFAULT (newid()) FOR [TenantServiceMapping_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantSetting]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantSetting](
	[TenantSetting_TenantID] [uniqueidentifier] NOT NULL,
	[TenantSetting_AttributeID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantSetting_AttributeValue] [nvarchar](4000) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantSetting_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantSetting_CreatedOn] [datetime] NOT NULL,
	[TenantSetting_UpdatedBy] [uniqueidentifier] NULL,
	[TenantSetting_UpdatedOn] [datetime] NULL,
	[TenantSetting_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantSetting] PRIMARY KEY CLUSTERED 
(
	[TenantSetting_TenantID] ASC,
	[TenantSetting_AttributeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantSettingTemplate]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantSettingTemplate](
	[TenantSettingTemplate_Id] [uniqueidentifier] NOT NULL,
	[TenantSettingTemplate_SettingTemplateId] [uniqueidentifier] NOT NULL,
	[TenantSettingTemplate_TenantId] [uniqueidentifier] NOT NULL,
	[TenantSettingTemplate_CreatedOn] [datetime] NOT NULL,
	[TenantSettingTemplate_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantSettingTemplate_UpdatedOn] [datetime] NULL,
	[TenantSettingTemplate_UpdatedBy] [uniqueidentifier] NULL,
	[TenantSettingTemplate_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantSettingTemplate] PRIMARY KEY CLUSTERED 
(
	[TenantSettingTemplate_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__TenantSet__Tenan__08162EEB]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantSettingTemplate] ADD  DEFAULT (newid()) FOR [TenantSettingTemplate_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantTaxRate]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantTaxRate](
	[TenantTaxRateId] [uniqueidentifier] NOT NULL,
	[TaxRateId] [uniqueidentifier] NOT NULL,
	[Name] [nvarchar](150) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](300) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[Percentage] [float] NOT NULL,
	[TaxOrder] [smallint] NOT NULL,
 CONSTRAINT [PK_TenantTaxRate] PRIMARY KEY CLUSTERED 
(
	[TenantTaxRateId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__TenantTax__Perce__090A5324]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantTaxRate] ADD  DEFAULT ((0)) FOR [Percentage]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF__TenantTax__TaxOr__09FE775D]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantTaxRate] ADD  DEFAULT ((0)) FOR [TaxOrder]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TenantTypes]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TenantTypes](
	[TenantType_ID] [uniqueidentifier] NOT NULL,
	[TenantType_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TenantType_AccessLevel] [smallint] NOT NULL,
	[TenantType_CreatedBy] [uniqueidentifier] NOT NULL,
	[TenantType_CreatedOn] [date] NOT NULL,
	[TenantType_UpdatedBy] [uniqueidentifier] NULL,
	[TenantType_UpdatedOn] [date] NULL,
	[TenantType_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TenantType] PRIMARY KEY CLUSTERED 
(
	[TenantType_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantType_TenantType_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantTypes] ADD  CONSTRAINT [DF_TenantType_TenantType_ID]  DEFAULT (newid()) FOR [TenantType_ID]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantType_TenantType_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantTypes] ADD  CONSTRAINT [DF_TenantType_TenantType_CreatedOn]  DEFAULT (getdate()) FOR [TenantType_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TenantType_TenantType_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TenantTypes] ADD  CONSTRAINT [DF_TenantType_TenantType_Status]  DEFAULT ((1)) FOR [TenantType_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TextNotificationContent]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TextNotificationContent](
	[TextNotificationContent_NotificationContentId] [uniqueidentifier] NOT NULL,
	[TextNotificationContent_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TextNotificationContent_ProcessedContent] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TextNotificationContent_AttachmentFileFolderPath] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TextNotificationContent_AttachmentFiles] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TextNotificationContent_CreatedOn] [datetime] NOT NULL,
	[TextNotificationContent_CreatedBy] [uniqueidentifier] NOT NULL,
	[TextNotificationContent_UpdatedOn] [datetime] NULL,
	[TextNotificationContent_UpdatedBy] [uniqueidentifier] NULL,
	[TextNotificationContent_Status] [bit] NOT NULL,
	[TextNotificationContent_Subject] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_TextNotificationContent] PRIMARY KEY CLUSTERED 
(
	[TextNotificationContent_NotificationContentId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TextSource]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TextSource](
	[TextSource_Id] [uniqueidentifier] NOT NULL,
	[TextSource_Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TextSource_Content] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TextSource_TenantId] [uniqueidentifier] NOT NULL,
	[TextSource_CreatedBy] [uniqueidentifier] NOT NULL,
	[TextSource_CreatedOn] [datetime] NOT NULL,
	[TextSource_UpdatedBy] [uniqueidentifier] NULL,
	[TextSource_UpdatedOn] [datetime] NULL,
	[TextSource_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TextSourceId] PRIMARY KEY CLUSTERED 
(
	[TextSource_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TextSourceId_TextSource_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TextSource] ADD  CONSTRAINT [DF_TextSourceId_TextSource_Id]  DEFAULT (newid()) FOR [TextSource_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TimeSchedules]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[TimeSchedules](
	[TimeSchedule_Id] [uniqueidentifier] NOT NULL,
	[TimeSchedule_ScheduleId] [uniqueidentifier] NOT NULL,
	[TimeSchedule_JobId] [uniqueidentifier] NOT NULL,
	[TimeSchedule_JobReferenceName] [varchar](500) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TimeSchedule_StartDateTime] [datetime] NULL,
	[TimeSchedule_EndDateTime] [datetime] NULL,
	[TimeSchedule_CreatedBy] [uniqueidentifier] NULL,
	[TimeSchedule_CreatedOn] [datetime] NULL,
	[TimeSchedule_UpdatedBy] [uniqueidentifier] NULL,
	[TimeSchedule_UpdatedOn] [datetime] NULL,
	[TimeSchedule_Status] [bit] NOT NULL,
 CONSTRAINT [PK_TimeSchedules] PRIMARY KEY CLUSTERED 
(
	[TimeSchedule_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TimeSchedules_TimeSchedule_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TimeSchedules] ADD  CONSTRAINT [DF_TimeSchedules_TimeSchedule_Id]  DEFAULT (newid()) FOR [TimeSchedule_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TimeSchedules_TimeSchedule_CreatedOn]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TimeSchedules] ADD  CONSTRAINT [DF_TimeSchedules_TimeSchedule_CreatedOn]  DEFAULT (getdate()) FOR [TimeSchedule_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_TimeSchedules_TimeSchedule_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[TimeSchedules] ADD  CONSTRAINT [DF_TimeSchedules_TimeSchedule_Status]  DEFAULT ((1)) FOR [TimeSchedule_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserActivity]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserActivity](
	[UserActivity_Id] [uniqueidentifier] NOT NULL,
	[UserActivity_UserId] [uniqueidentifier] NOT NULL,
	[UserActivity_LastActivityTimeStamp] [datetime] NOT NULL,
 CONSTRAINT [PK_UserActivity] PRIMARY KEY CLUSTERED 
(
	[UserActivity_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserActivity]') AND name = N'IX_UserActivity')
CREATE NONCLUSTERED INDEX [IX_UserActivity] ON [dbo].[UserActivity] 
(
	[UserActivity_UserId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_UserActivity_UserActivity_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[UserActivity] ADD  CONSTRAINT [DF_UserActivity_UserActivity_Id]  DEFAULT (newid()) FOR [UserActivity_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserDetails](
	[User_UserID] [uniqueidentifier] NOT NULL,
	[User_MembershipID] [uniqueidentifier] NOT NULL,
	[User_FirstName] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[User_LastName] [nvarchar](200) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[User_AddressID] [uniqueidentifier] NULL,
	[User_CreatedBy] [uniqueidentifier] NOT NULL,
	[User_CreatedOn] [datetime] NOT NULL,
	[User_UpdatedOn] [datetime] NULL,
	[User_UpdatedBY] [uniqueidentifier] NULL,
	[User_Status] [bit] NOT NULL,
	[User_Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[User_TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_UserDetails] PRIMARY KEY CLUSTERED 
(
	[User_UserID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserDetails]') AND name = N'Ind_User_FirstName')
CREATE NONCLUSTERED INDEX [Ind_User_FirstName] ON [dbo].[UserDetails] 
(
	[User_FirstName] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserDetails]') AND name = N'Ind_User_LastName')
CREATE NONCLUSTERED INDEX [Ind_User_LastName] ON [dbo].[UserDetails] 
(
	[User_LastName] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserDetails]') AND name = N'Ind_User_MembershipID')
CREATE NONCLUSTERED INDEX [Ind_User_MembershipID] ON [dbo].[UserDetails] 
(
	[User_MembershipID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_UserDetails_User_UserID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[UserDetails] ADD  CONSTRAINT [DF_UserDetails_User_UserID]  DEFAULT (newid()) FOR [User_UserID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserDetailExtn]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserDetailExtn](
	[UserDetailExtn_Id] [uniqueidentifier] NOT NULL,
	[UserDetailExtn_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[UserDetailExtn_ReferenceId] [uniqueidentifier] NOT NULL,
	[UserDetailExtn_TenantId] [uniqueidentifier] NOT NULL,
	[UserDetailExtn_Column1] [int] NULL,
	[UserDetailExtn_Column2] [int] NULL,
	[UserDetailExtn_Column3] [int] NULL,
	[UserDetailExtn_Column4] [int] NULL,
	[UserDetailExtn_Column5] [int] NULL,
	[UserDetailExtn_Column6] [datetime] NULL,
	[UserDetailExtn_Column7] [datetime] NULL,
	[UserDetailExtn_Column8] [datetime] NULL,
	[UserDetailExtn_Column9] [datetime] NULL,
	[UserDetailExtn_Column10] [datetime] NULL,
	[UserDetailExtn_Column11] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column12] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column13] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column14] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column15] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column16] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column17] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column18] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column19] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column20] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column21] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column22] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column23] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column24] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column25] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column26] [float] NULL,
	[UserDetailExtn_Column27] [float] NULL,
	[UserDetailExtn_Column28] [float] NULL,
	[UserDetailExtn_Column29] [float] NULL,
	[UserDetailExtn_Column30] [float] NULL,
	[UserDetailExtn_Column31] [bit] NULL,
	[UserDetailExtn_Column32] [bit] NULL,
	[UserDetailExtn_Column33] [bit] NULL,
	[UserDetailExtn_Column34] [bit] NULL,
	[UserDetailExtn_Column35] [bit] NULL,
	[UserDetailExtn_Column36] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column37] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column38] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column39] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_Column40] [varchar](36) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserDetailExtn_CreatedOn] [datetime] NOT NULL,
	[UserDetailExtn_CreatedBy] [uniqueidentifier] NOT NULL,
	[UserDetailExtn_UpdatedOn] [datetime] NULL,
	[UserDetailExtn_UpdatedBy] [uniqueidentifier] NULL,
	[UserDetailExtn_Status] [bit] NOT NULL,
 CONSTRAINT [PK_UserDetailExtn1] PRIMARY KEY CLUSTERED 
(
	[UserDetailExtn_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_UserDetailExtn_UserDetailExtn_Id1]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[UserDetailExtn] ADD  CONSTRAINT [DF_UserDetailExtn_UserDetailExtn_Id1]  DEFAULT (newid()) FOR [UserDetailExtn_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityPermission]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserEntityPermission](
	[Id] [uniqueidentifier] NOT NULL,
	[EntityId] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ReferenceId] [uniqueidentifier] NULL,
	[ReferenceName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[FieldIdentifier] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[UserId] [uniqueidentifier] NULL,
	[RoleId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[ViewPermission] [bit] NULL,
	[AddPermission] [bit] NULL,
	[EditPermission] [bit] NULL,
	[DeletePermission] [bit] NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_UserEntityPermission] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityPermission]') AND name = N'IX_UserEntityPermission_EntityId')
CREATE NONCLUSTERED INDEX [IX_UserEntityPermission_EntityId] ON [dbo].[UserEntityPermission] 
(
	[EntityId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityPermission]') AND name = N'IX_UserEntityPermission_FieldIdentifier')
CREATE NONCLUSTERED INDEX [IX_UserEntityPermission_FieldIdentifier] ON [dbo].[UserEntityPermission] 
(
	[FieldIdentifier] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityPermission]') AND name = N'IX_UserEntityPermission_ReferenceId')
CREATE NONCLUSTERED INDEX [IX_UserEntityPermission_ReferenceId] ON [dbo].[UserEntityPermission] 
(
	[ReferenceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityPermission]') AND name = N'IX_UserEntityPermission_RoleId')
CREATE NONCLUSTERED INDEX [IX_UserEntityPermission_RoleId] ON [dbo].[UserEntityPermission] 
(
	[RoleId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityPermission]') AND name = N'IX_UserEntityPermission_TenantId')
CREATE NONCLUSTERED INDEX [IX_UserEntityPermission_TenantId] ON [dbo].[UserEntityPermission] 
(
	[TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityPermission]') AND name = N'IX_UserEntityPermission_UserId')
CREATE NONCLUSTERED INDEX [IX_UserEntityPermission_UserId] ON [dbo].[UserEntityPermission] 
(
	[UserId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_UserEntityPermission_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[UserEntityPermission] ADD  CONSTRAINT [DF_UserEntityPermission_Id]  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityOtherPermission]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserEntityOtherPermission](
	[OtherPermissionId] [uniqueidentifier] NOT NULL,
	[UserEntityPermissionId] [uniqueidentifier] NOT NULL,
	[Privilege] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedBy] [uniqueidentifier] NULL,
	[CreatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_UserEntityOtherPermission] PRIMARY KEY CLUSTERED 
(
	[OtherPermissionId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityOtherPermission]') AND name = N'IX_UserEntityOtherPermission_Privilege')
CREATE NONCLUSTERED INDEX [IX_UserEntityOtherPermission_Privilege] ON [dbo].[UserEntityOtherPermission] 
(
	[Privilege] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserEntityOtherPermission]') AND name = N'IX_UserEntityOtherPermission_UserEntityPermissionId')
CREATE NONCLUSTERED INDEX [IX_UserEntityOtherPermission_UserEntityPermissionId] ON [dbo].[UserEntityOtherPermission] 
(
	[UserEntityPermissionId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_UserEntityOtherPermission_OtherPermissionId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[UserEntityOtherPermission] ADD  CONSTRAINT [DF_UserEntityOtherPermission_OtherPermissionId]  DEFAULT (newid()) FOR [OtherPermissionId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserPasswordExpiration]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserPasswordExpiration](
	[Expiration_Id] [uniqueidentifier] NOT NULL,
	[Expiration_Membership_Id] [uniqueidentifier] NOT NULL,
	[Expiration_ExpirationDate] [date] NOT NULL,
	[Expiration_PasswordChangedDate] [datetime] NULL,
	[Expiration_IsExpired] [bit] NOT NULL,
	[Expiration_CreatedBy] [uniqueidentifier] NOT NULL,
	[Expiration_CreatedOn] [datetime] NOT NULL,
	[Expiration_UpdatedBy] [uniqueidentifier] NULL,
	[Expiration_UpdatedOn] [datetime] NULL,
	[Expiration_Status] [bit] NOT NULL,
 CONSTRAINT [PK_UserPwdExpiration] PRIMARY KEY CLUSTERED 
(
	[Expiration_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_UserPasswordExpiration_Expiration_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[UserPasswordExpiration] ADD  CONSTRAINT [DF_UserPasswordExpiration_Expiration_Id]  DEFAULT (newid()) FOR [Expiration_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserRoles]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserRoles](
	[UserRole_ID] [uniqueidentifier] NOT NULL,
	[UserRole_UserID] [uniqueidentifier] NOT NULL,
	[UserRole_RoleID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[UserRole_CreatedBy] [uniqueidentifier] NOT NULL,
	[UserRole_CreatedOn] [datetime] NOT NULL,
	[UserRole_UpdatedBy] [uniqueidentifier] NULL,
	[UserRole_UpdatedOn] [datetime] NULL,
	[UserRole_Status] [bit] NOT NULL,
 CONSTRAINT [PK_UserRoles] PRIMARY KEY CLUSTERED 
(
	[UserRole_ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserRoles]') AND name = N'Ind_UserRole_RoleID')
CREATE NONCLUSTERED INDEX [Ind_UserRole_RoleID] ON [dbo].[UserRoles] 
(
	[UserRole_RoleID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserRoles]') AND name = N'Ind_UserRole_UserID')
CREATE NONCLUSTERED INDEX [Ind_UserRole_UserID] ON [dbo].[UserRoles] 
(
	[UserRole_UserID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_UserRoles_UserRole_ID]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[UserRoles] ADD  CONSTRAINT [DF_UserRoles_UserRole_ID]  DEFAULT (newid()) FOR [UserRole_ID]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserRoleTenant]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserRoleTenant](
	[UserRoleTenant_Id] [uniqueidentifier] NOT NULL,
	[UserRoleTenant_UserRoleId] [uniqueidentifier] NOT NULL,
	[UserRoleTenant_StratifiedTenantId] [uniqueidentifier] NOT NULL,
	[UserRoleTenant_CreatedBy] [uniqueidentifier] NOT NULL,
	[UserRoleTenant_CreatedOn] [datetime] NOT NULL,
	[UserRoleTenant_UpdatedBy] [uniqueidentifier] NULL,
	[UserRoleTenant_UpdatedOn] [datetime] NULL,
	[UserRoleTenant_Status] [bit] NOT NULL,
	[UserRoleTenant_RoleAssigningTenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_UserRoleTenant] PRIMARY KEY CLUSTERED 
(
	[UserRoleTenant_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserRoleTenant]') AND name = N'IX_AssignedTenantId')
CREATE NONCLUSTERED INDEX [IX_AssignedTenantId] ON [dbo].[UserRoleTenant] 
(
	[UserRoleTenant_RoleAssigningTenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserRoleTenant]') AND name = N'IX_StratifiedTenantIndex')
CREATE NONCLUSTERED INDEX [IX_StratifiedTenantIndex] ON [dbo].[UserRoleTenant] 
(
	[UserRoleTenant_StratifiedTenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserRoleTenant]') AND name = N'IX_UserRoleIdStatus')
CREATE NONCLUSTERED INDEX [IX_UserRoleIdStatus] ON [dbo].[UserRoleTenant] 
(
	[UserRoleTenant_UserRoleId] ASC,
	[UserRoleTenant_Status] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_UserRoleTenant_UserRoleTenant_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[UserRoleTenant] ADD  CONSTRAINT [DF_UserRoleTenant_UserRoleTenant_Id]  DEFAULT (newid()) FOR [UserRoleTenant_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserService]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserService](
	[UserService_Id] [uniqueidentifier] NOT NULL,
	[UserService_UserId] [uniqueidentifier] NOT NULL,
	[UserService_ServiceCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[UserService_CreatedBy] [uniqueidentifier] NOT NULL,
	[UserService_CreatedOn] [datetime] NOT NULL,
	[UserService_UpdatedBy] [uniqueidentifier] NULL,
	[UserService_UpdatedOn] [datetime] NULL,
	[UserService_Status] [bit] NOT NULL,
 CONSTRAINT [PK_UserService] PRIMARY KEY CLUSTERED 
(
	[UserService_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_UserService_UserService_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[UserService] ADD  CONSTRAINT [DF_UserService_UserService_Id]  DEFAULT (newid()) FOR [UserService_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserSetting]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserSetting](
	[UserSetting_UserID] [uniqueidentifier] NOT NULL,
	[UserSetting_AttributeID] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[UserSetting_AttributeValue] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[UserSetting_CreatedBy] [uniqueidentifier] NOT NULL,
	[UserSetting_CreatedOn] [datetime] NOT NULL,
	[UserSetting_UpdatedBy] [uniqueidentifier] NULL,
	[UserSetting_UpdatedOn] [datetime] NULL,
	[UserSetting_Status] [bit] NOT NULL,
 CONSTRAINT [PK_UserSetting] PRIMARY KEY CLUSTERED 
(
	[UserSetting_UserID] ASC,
	[UserSetting_AttributeID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[VirtualEntity]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[VirtualEntity](
	[VirtualEntity_Id] [uniqueidentifier] NOT NULL,
	[VirtualEntity_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[VirtualEntity_TenantId] [uniqueidentifier] NOT NULL,
	[VirtualEntity_CreatedBy] [uniqueidentifier] NOT NULL,
	[VirtualEntity_CreatedOn] [datetime] NOT NULL,
	[VirtualEntity_UpdatedBy] [uniqueidentifier] NULL,
	[VirtualEntity_UpdatedOn] [datetime] NULL,
	[VirtualEntity_Status] [bit] NULL,
 CONSTRAINT [PK_VirtualEntity] PRIMARY KEY CLUSTERED 
(
	[VirtualEntity_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[VirtualEntity]') AND name = N'EntityIdIndex')
CREATE NONCLUSTERED INDEX [EntityIdIndex] ON [dbo].[VirtualEntity] 
(
	[VirtualEntity_EntityId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[VirtualEntity]') AND name = N'TenantIdIndex')
CREATE NONCLUSTERED INDEX [TenantIdIndex] ON [dbo].[VirtualEntity] 
(
	[VirtualEntity_TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_VirtualEntity_Reference_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[VirtualEntity] ADD  CONSTRAINT [DF_VirtualEntity_Reference_Id]  DEFAULT (newid()) FOR [VirtualEntity_Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_VirtualEntity_Created_On]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[VirtualEntity] ADD  CONSTRAINT [DF_VirtualEntity_Created_On]  DEFAULT (getdate()) FOR [VirtualEntity_CreatedOn]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_VirtualEntity_Status]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[VirtualEntity] ADD  CONSTRAINT [DF_VirtualEntity_Status]  DEFAULT ((1)) FOR [VirtualEntity_Status]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[VirtualEntityExtn]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[VirtualEntityExtn](
	[VirtualEntityExtn_Id] [uniqueidentifier] NOT NULL,
	[VirtualEntityExtn_EntityId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[VirtualEntityExtn_ReferenceId] [uniqueidentifier] NOT NULL,
	[VirtualEntityExtn_TenantId] [uniqueidentifier] NOT NULL,
	[VirtualEntityExtn_Column1] [int] NULL,
	[VirtualEntityExtn_Column2] [int] NULL,
	[VirtualEntityExtn_Column3] [int] NULL,
	[VirtualEntityExtn_Column4] [int] NULL,
	[VirtualEntityExtn_Column5] [int] NULL,
	[VirtualEntityExtn_Column6] [datetime] NULL,
	[VirtualEntityExtn_Column7] [datetime] NULL,
	[VirtualEntityExtn_Column8] [datetime] NULL,
	[VirtualEntityExtn_Column9] [datetime] NULL,
	[VirtualEntityExtn_Column10] [datetime] NULL,
	[VirtualEntityExtn_Column11] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column12] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column13] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column14] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column15] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column16] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column17] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column18] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column19] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column20] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column21] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column22] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column23] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column24] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column25] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column26] [float] NULL,
	[VirtualEntityExtn_Column27] [float] NULL,
	[VirtualEntityExtn_Column28] [float] NULL,
	[VirtualEntityExtn_Column29] [float] NULL,
	[VirtualEntityExtn_Column30] [float] NULL,
	[VirtualEntityExtn_Column31] [bit] NULL,
	[VirtualEntityExtn_Column32] [bit] NULL,
	[VirtualEntityExtn_Column33] [bit] NULL,
	[VirtualEntityExtn_Column34] [bit] NULL,
	[VirtualEntityExtn_Column35] [bit] NULL,
	[VirtualEntityExtn_Column36] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column37] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column38] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column39] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_Column40] [varchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[VirtualEntityExtn_CreatedOn] [datetime] NOT NULL,
	[VirtualEntityExtn_CreatedBy] [uniqueidentifier] NOT NULL,
	[VirtualEntityExtn_UpdatedOn] [datetime] NULL,
	[VirtualEntityExtn_UpdatedBy] [uniqueidentifier] NULL,
	[VirtualEntityExtn_Status] [bit] NOT NULL,
 CONSTRAINT [PK_VirtualEntityExtn_ID] PRIMARY KEY CLUSTERED 
(
	[VirtualEntityExtn_Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_VirtualEntityExtn_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[VirtualEntityExtn] ADD  CONSTRAINT [DF_VirtualEntityExtn_Id]  DEFAULT (newid()) FOR [VirtualEntityExtn_Id]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WorkFlow]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WorkFlow](
	[WfId] [uniqueidentifier] NOT NULL,
	[Name] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Description] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
	[WFInputFinderType] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[TenantId] [uniqueidentifier] NULL,
	[CategoryId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[WFXmlTransform] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_WorkFlow] PRIMARY KEY CLUSTERED 
(
	[WfId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WorkFlow]') AND name = N'IX_WFName')
CREATE NONCLUSTERED INDEX [IX_WFName] ON [dbo].[WorkFlow] 
(
	[Name] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_WorkFlow_WfId]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[WorkFlow] ADD  CONSTRAINT [DF_WorkFlow_WfId]  DEFAULT (newid()) FOR [WfId]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFDefinition]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFDefinition](
	[WfDefinitionId] [uniqueidentifier] NOT NULL,
	[WfId] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NULL,
	[DefinitionXml] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Version] [int] NULL,
	[WfDefinitionStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
	[FilterId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_WFDefinition] PRIMARY KEY CLUSTERED 
(
	[WfDefinitionId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFDefinitionUI]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFDefinitionUI](
	[WFDefinitionId] [uniqueidentifier] NULL,
	[UIDefinitionXml] [xml] NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFDefinitionUI]') AND name = N'IX_WFDefinitionUI')
CREATE CLUSTERED INDEX [IX_WFDefinitionUI] ON [dbo].[WFDefinitionUI] 
(
	[WFDefinitionId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFInstance]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFInstance](
	[WfInstanceId] [uniqueidentifier] NOT NULL,
	[WfDefinitionId] [uniqueidentifier] NOT NULL,
	[InstanceStatus] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[WfId] [uniqueidentifier] NULL,
	[WfName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[MapId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
	[TenantId] [uniqueidentifier] NULL,
 CONSTRAINT [PK_WFInstance] PRIMARY KEY CLUSTERED 
(
	[WfInstanceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstance]') AND name = N'IX_WFInstance')
CREATE NONCLUSTERED INDEX [IX_WFInstance] ON [dbo].[WFInstance] 
(
	[TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstance]') AND name = N'IX_WFInstance_WfName')
CREATE NONCLUSTERED INDEX [IX_WFInstance_WfName] ON [dbo].[WFInstance] 
(
	[WfName] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstance]') AND name = N'IX_WFinstanceMapId')
CREATE NONCLUSTERED INDEX [IX_WFinstanceMapId] ON [dbo].[WFInstance] 
(
	[MapId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstance]') AND name = N'IX_WFInstanceStatus')
CREATE NONCLUSTERED INDEX [IX_WFInstanceStatus] ON [dbo].[WFInstance] 
(
	[InstanceStatus] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFDynamicTaskActor]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFDynamicTaskActor](
	[WFInstanceId] [uniqueidentifier] NOT NULL,
	[TaskCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ActorId] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
	[Status] [bit] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFDynamicTaskActor]') AND name = N'IX_WFDynamicActor_CreatedOn')
CREATE CLUSTERED INDEX [IX_WFDynamicActor_CreatedOn] ON [dbo].[WFDynamicTaskActor] 
(
	[CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFDynamicTaskActor]') AND name = N'IX_WFDynamicTaskActor_ACtorId')
CREATE NONCLUSTERED INDEX [IX_WFDynamicTaskActor_ACtorId] ON [dbo].[WFDynamicTaskActor] 
(
	[ActorId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFDynamicTaskActor]') AND name = N'IX_WFDynamicTaskActor_TaskCode')
CREATE NONCLUSTERED INDEX [IX_WFDynamicTaskActor_TaskCode] ON [dbo].[WFDynamicTaskActor] 
(
	[TaskCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFDynamicTaskActor]') AND name = N'IX_WFDynamicTaskActor_wfinstanceid')
CREATE NONCLUSTERED INDEX [IX_WFDynamicTaskActor_wfinstanceid] ON [dbo].[WFDynamicTaskActor] 
(
	[WFInstanceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFExceptionDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFExceptionDetails](
	[WFExceptionId] [uniqueidentifier] NOT NULL,
	[WFInstanceId] [uniqueidentifier] NOT NULL,
	[WFTaskInstanceId] [uniqueidentifier] NOT NULL,
	[ExceptionMessage] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ExceptionDetails] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DateTime] [datetime] NULL,
 CONSTRAINT [PK_WFExceptionDetails] PRIMARY KEY CLUSTERED 
(
	[WFExceptionId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFInstanceDetails](
	[WFInstanceId] [uniqueidentifier] NOT NULL,
	[Input] [nvarchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[WFState] [varchar](max) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
 CONSTRAINT [PK_WFInstanceDetails] PRIMARY KEY CLUSTERED 
(
	[WFInstanceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRun]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFInstanceTaskRun](
	[WfInstanceTaskRunId] [uniqueidentifier] NOT NULL,
	[WfInstanceId] [uniqueidentifier] NOT NULL,
	[WfTaskDefinitionId] [uniqueidentifier] NOT NULL,
	[TaskCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CurrentExecutionStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[ActorId] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[DateTimeUpdated] [datetime] NULL,
	[DateTimeStarted] [datetime] NULL,
	[DateTimeCompleted] [datetime] NULL,
	[DateTimeExpired] [datetime] NULL,
	[InvokedTaskDefinitionRouterId] [uniqueidentifier] NULL,
	[TaskType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[ExecutionOrder] [int] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_WFInstanceTaskRun] PRIMARY KEY CLUSTERED 
(
	[WfInstanceTaskRunId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRun]') AND name = N'IX_WFInstanceId')
CREATE NONCLUSTERED INDEX [IX_WFInstanceId] ON [dbo].[WFInstanceTaskRun] 
(
	[WfInstanceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRun]') AND name = N'IX_WFTaskCode')
CREATE NONCLUSTERED INDEX [IX_WFTaskCode] ON [dbo].[WFInstanceTaskRun] 
(
	[TaskCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRun]') AND name = N'IX_WFTaskCurrentStatus')
CREATE NONCLUSTERED INDEX [IX_WFTaskCurrentStatus] ON [dbo].[WFInstanceTaskRun] 
(
	[CurrentExecutionStatus] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRun]') AND name = N'IX_WFTaskDefId')
CREATE NONCLUSTERED INDEX [IX_WFTaskDefId] ON [dbo].[WFInstanceTaskRun] 
(
	[WfTaskDefinitionId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_WFInstanceTaskRun_ExecutionOrder]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[WFInstanceTaskRun] ADD  CONSTRAINT [DF_WFInstanceTaskRun_ExecutionOrder]  DEFAULT ((0)) FOR [ExecutionOrder]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunExecutionAudit]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFInstanceTaskRunExecutionAudit](
	[WfInstanceTaskRunId] [uniqueidentifier] NOT NULL,
	[ExecutionStatus] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DateTime] [datetime] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunExecutionAudit]') AND name = N'IX_WFInstanceTaskRunExecutionAudit_CreatedOn')
CREATE CLUSTERED INDEX [IX_WFInstanceTaskRunExecutionAudit_CreatedOn] ON [dbo].[WFInstanceTaskRunExecutionAudit] 
(
	[CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunExecutionAudit]') AND name = N'IX_WFInstanceTaskRunExecutionAudit')
CREATE NONCLUSTERED INDEX [IX_WFInstanceTaskRunExecutionAudit] ON [dbo].[WFInstanceTaskRunExecutionAudit] 
(
	[WfInstanceTaskRunId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunOutcomes]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFInstanceTaskRunOutcomes](
	[WfInstanceTaskRunId] [uniqueidentifier] NOT NULL,
	[Outcome] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DateTime] [datetime] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunOutcomes]') AND name = N'IX_WFInstanceTaskRunOutcomes_CreatedOn')
CREATE CLUSTERED INDEX [IX_WFInstanceTaskRunOutcomes_CreatedOn] ON [dbo].[WFInstanceTaskRunOutcomes] 
(
	[CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunOutcomes]') AND name = N'IX_WFInstanceTaskRunOutcomes')
CREATE NONCLUSTERED INDEX [IX_WFInstanceTaskRunOutcomes] ON [dbo].[WFInstanceTaskRunOutcomes] 
(
	[WfInstanceTaskRunId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFRouterDefinition]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFRouterDefinition](
	[WFRouterDefinitionId] [uniqueidentifier] NOT NULL,
	[WfDefinitionId] [uniqueidentifier] NOT NULL,
	[RouterName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFRouterDefinition]') AND name = N'IX_WFRouterDefinition_CreatedOn')
CREATE CLUSTERED INDEX [IX_WFRouterDefinition_CreatedOn] ON [dbo].[WFRouterDefinition] 
(
	[CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFTaskActor]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFTaskActor](
	[WfInstanceTaskRunId] [uniqueidentifier] NOT NULL,
	[ActorId] [varchar](100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
	[Status] [bit] NOT NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFTaskActor]') AND name = N'IX_WFTaskActor_ActorId')
CREATE CLUSTERED INDEX [IX_WFTaskActor_ActorId] ON [dbo].[WFTaskActor] 
(
	[ActorId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFTaskActor]') AND name = N'IX_WFTaskActor')
CREATE NONCLUSTERED INDEX [IX_WFTaskActor] ON [dbo].[WFTaskActor] 
(
	[WfInstanceTaskRunId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFTaskDefinition]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFTaskDefinition](
	[WFTaskDefinitionId] [uniqueidentifier] NOT NULL,
	[WfDefinitionId] [uniqueidentifier] NOT NULL,
	[TaskName] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TaskCode] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
	[TaskType] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL
)
END

SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AuthenticationProviders]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[AuthenticationProviders](
	[Id] [uniqueidentifier] NOT NULL,
	[Name] [varchar](50) NOT NULL,
	[Description] [varchar](255) NULL,
	[CreatedOn] [datetime] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_AuthenticationProviders] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF ) 
)
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_AuthenticationProviders_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[AuthenticationProviders] ADD  CONSTRAINT [DF_AuthenticationProviders_Id]  DEFAULT (newid()) FOR [Id]
END
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AuthenticationTypes]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[AuthenticationTypes](
	[Id] [uniqueidentifier] NOT NULL,
	[Name] [varchar](255) NOT NULL,
	[Description] [varchar](255) NULL,
	[Links] [varchar](512) NULL,
	[Mode] [varchar](255) NOT NULL,
	[ProviderId] [uniqueidentifier] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_AuthorizationTypes] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF) ) 
END

GO
SET ANSI_PADDING OFF
GO

/****** Object:  Table [dbo].[ClientDetails] ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ClientDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[ClientDetails](
	[Id] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[Name] [varchar](255) NOT NULL,
	[Description] [varchar](512) NULL,
	[Secret] [varchar](255) NOT NULL,
	[Uri] [varchar](255) NULL,
	[LogoUri] [varchar](255) NULL,
	[ApplicationTypes] [varchar](512) NULL,
	[RequireConsent] [bit] NOT NULL,
	[AllowRememberConsent] [bit] NOT NULL,
	[Flows] [varchar](50) NULL,
	[RedirectUris] [varchar](1024) NULL,
	[SigningKeyType] [varchar](50) NOT NULL,
	[IdentityTokenLifetime] [bigint] NOT NULL,
	[AccessTokenLifetime] [bigint] NOT NULL,
	[RefreshTokenLifetime] [bigint] NULL,
	[AuthorizationCodeLifetime] [bigint] NULL,
	[ScopeRestrictions] [varchar](max) NULL,
	[AccessTokenType] [varchar](50) NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
	[PublicAccessAllowed] [bit] NOT NULL,
	[ClientType] [varchar](50) NOT NULL,
 CONSTRAINT [PK_ClientDetails] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF)
)
End

/****** Object:  Table [dbo].[SubjectDetails]  ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SubjectDetails]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[SubjectDetails](
	[Id] [uniqueidentifier] NOT NULL,
	[UserId] [uniqueidentifier] NOT NULL,
	[AuthenticationType] [uniqueidentifier] NULL,
	[ClientId] [uniqueidentifier] NULL,
	[Name] [varchar](50) NOT NULL,
	[MembershipId] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[ProviderId] [varchar](512) NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
 CONSTRAINT [PK_SubjectDetails] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF) 
)
End

/****** Object:  Table [dbo].[UserConsent]  ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserConsent]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[UserConsent](
	[Id] [uniqueidentifier] NOT NULL,
	[ClientId] [uniqueidentifier] NOT NULL,
	[SubjectId] [uniqueidentifier] NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[IsConsentStorageAllowed] [bit] NOT NULL,
	[ConsentedScopes] [nvarchar](max) NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
	[RememberConsent] [bit] NOT NULL,
	[WasConsentGranted] [bit] NOT NULL,
 CONSTRAINT [PK_UserConsent] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF) 
) 
End

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFTaskDefinition]') AND name = N'IX_WFTaskDefinition_CreatedOn')
CREATE CLUSTERED INDEX [IX_WFTaskDefinition_CreatedOn] ON [dbo].[WFTaskDefinition] 
(
	[CreatedOn] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFTaskDefinition]') AND name = N'IX_WFDefinitionId')
CREATE NONCLUSTERED INDEX [IX_WFDefinitionId] ON [dbo].[WFTaskDefinition] 
(
	[WfDefinitionId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFTaskDefinition]') AND name = N'IX_WFTaskCode')
CREATE NONCLUSTERED INDEX [IX_WFTaskCode] ON [dbo].[WFTaskDefinition] 
(
	[TaskCode] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFTaskDefinition]') AND name = N'IX_WFTaskDefinition')
CREATE NONCLUSTERED INDEX [IX_WFTaskDefinition] ON [dbo].[WFTaskDefinition] 
(
	[WFTaskDefinitionId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFTrackAudit]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFTrackAudit](
	[Id] [uniqueidentifier] NOT NULL,
	[WFTaskInstanceRunId] [uniqueidentifier] NOT NULL,
	[PreviousStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CurrentStatus] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DateTime] [datetime] NOT NULL,
 CONSTRAINT [PK_WFTrackAudit] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_WFTrackAudit_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[WFTrackAudit] ADD  CONSTRAINT [DF_WFTrackAudit_Id]  DEFAULT (newid()) FOR [Id]
END
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_WFTrackAudit_DateTime]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[WFTrackAudit] ADD  CONSTRAINT [DF_WFTrackAudit_DateTime]  DEFAULT (getdate()) FOR [DateTime]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WFTracking]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WFTracking](
	[WFTaskInstanceRunId] [uniqueidentifier] NOT NULL,
	[WFInstanceId] [uniqueidentifier] NOT NULL,
	[Status] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[DateTime] [datetime] NOT NULL,
	[MapId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[TrackingId] [uniqueidentifier] NULL
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFTracking]') AND name = N'IX_WFTracking_Status')
CREATE CLUSTERED INDEX [IX_WFTracking_Status] ON [dbo].[WFTracking] 
(
	[Status] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFTracking]') AND name = N'IX_WFTracking_WFInstanceId')
CREATE NONCLUSTERED INDEX [IX_WFTracking_WFInstanceId] ON [dbo].[WFTracking] 
(
	[WFInstanceId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WFTracking]') AND name = N'IX_WFTracking_WFTaskInstanceRunId')
CREATE NONCLUSTERED INDEX [IX_WFTracking_WFTaskInstanceRunId] ON [dbo].[WFTracking] 
(
	[WFTaskInstanceRunId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)
GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_WFTracking_DateTime]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[WFTracking] ADD  CONSTRAINT [DF_WFTracking_DateTime]  DEFAULT (getdate()) FOR [DateTime]
END
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[WorkflowRequest]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[WorkflowRequest](
	[Id] [uniqueidentifier] NOT NULL,
	[TenantId] [uniqueidentifier] NOT NULL,
	[WfId] [uniqueidentifier] NOT NULL,
	[MapId] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Status] [varchar](50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[UpdatedOn] [datetime] NULL,
 CONSTRAINT [PK_WRId] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WorkflowRequest]') AND name = N'IX_WRMapId')
CREATE NONCLUSTERED INDEX [IX_WRMapId] ON [dbo].[WorkflowRequest] 
(
	[MapId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WorkflowRequest]') AND name = N'IX_WRTenantId')
CREATE NONCLUSTERED INDEX [IX_WRTenantId] ON [dbo].[WorkflowRequest] 
(
	[TenantId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[WorkflowRequest]') AND name = N'IX_WRWFId')
CREATE NONCLUSTERED INDEX [IX_WRWFId] ON [dbo].[WorkflowRequest] 
(
	[WfId] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF)

GO
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[DF_WorkflowRequest_Id]') AND type = 'D')
BEGIN
ALTER TABLE [dbo].[WorkflowRequest] ADD  CONSTRAINT [DF_WorkflowRequest_Id]  DEFAULT (newid()) FOR [Id]
END
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_CategoryLog_Category]') AND parent_object_id = OBJECT_ID(N'[dbo].[ExceptionCategoryLog]'))
ALTER TABLE [dbo].[ExceptionCategoryLog]  WITH CHECK ADD  CONSTRAINT [FK_CategoryLog_Category] FOREIGN KEY([CategoryID])
REFERENCES [dbo].[ExceptionCategory] ([CategoryID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_CategoryLog_Category]') AND parent_object_id = OBJECT_ID(N'[dbo].[ExceptionCategoryLog]'))
ALTER TABLE [dbo].[ExceptionCategoryLog] CHECK CONSTRAINT [FK_CategoryLog_Category]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_CategoryLog_Log]') AND parent_object_id = OBJECT_ID(N'[dbo].[ExceptionCategoryLog]'))
ALTER TABLE [dbo].[ExceptionCategoryLog]  WITH CHECK ADD  CONSTRAINT [FK_CategoryLog_Log] FOREIGN KEY([LogID])
REFERENCES [dbo].[ExceptionLog] ([LogID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_CategoryLog_Log]') AND parent_object_id = OBJECT_ID(N'[dbo].[ExceptionCategoryLog]'))
ALTER TABLE [dbo].[ExceptionCategoryLog] CHECK CONSTRAINT [FK_CategoryLog_Log]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Address_Country]') AND parent_object_id = OBJECT_ID(N'[dbo].[Address]'))
ALTER TABLE [dbo].[Address]  WITH CHECK ADD  CONSTRAINT [FK_Address_Country] FOREIGN KEY([Address_CountryId])
REFERENCES [dbo].[Country] ([Country_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Address_Country]') AND parent_object_id = OBJECT_ID(N'[dbo].[Address]'))
ALTER TABLE [dbo].[Address] CHECK CONSTRAINT [FK_Address_Country]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetails_Address]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetails]'))
ALTER TABLE [dbo].[TenantDetails]  WITH CHECK ADD  CONSTRAINT [FK_TenantDetails_Address] FOREIGN KEY([Tenant_AddressID])
REFERENCES [dbo].[Address] ([Address_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetails_Address]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetails]'))
ALTER TABLE [dbo].[TenantDetails] CHECK CONSTRAINT [FK_TenantDetails_Address]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetails_Contact]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetails]'))
ALTER TABLE [dbo].[TenantDetails]  WITH CHECK ADD  CONSTRAINT [FK_TenantDetails_Contact] FOREIGN KEY([Tenant_ContactID])
REFERENCES [dbo].[Contact] ([Contact_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetails_Contact]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetails]'))
ALTER TABLE [dbo].[TenantDetails] CHECK CONSTRAINT [FK_TenantDetails_Contact]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetails_PaymentType]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetails]'))
ALTER TABLE [dbo].[TenantDetails]  WITH CHECK ADD  CONSTRAINT [FK_TenantDetails_PaymentType] FOREIGN KEY([Tenant_PaymentTypeCode])
REFERENCES [dbo].[PaymentType] ([PaymentType_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetails_PaymentType]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetails]'))
ALTER TABLE [dbo].[TenantDetails] CHECK CONSTRAINT [FK_TenantDetails_PaymentType]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Membership_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[Membership]'))
ALTER TABLE [dbo].[Membership]  WITH CHECK ADD  CONSTRAINT [FK_Membership_TenantDetails] FOREIGN KEY([Membership_TenantID])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Membership_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[Membership]'))
ALTER TABLE [dbo].[Membership] CHECK CONSTRAINT [FK_Membership_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ActivityEvents_Events]') AND parent_object_id = OBJECT_ID(N'[dbo].[ActivityEvents]'))
ALTER TABLE [dbo].[ActivityEvents]  WITH CHECK ADD  CONSTRAINT [FK_ActivityEvents_Events] FOREIGN KEY([ActivityEvent_EventId])
REFERENCES [dbo].[Events] ([Event_Id])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ActivityEvents_Events]') AND parent_object_id = OBJECT_ID(N'[dbo].[ActivityEvents]'))
ALTER TABLE [dbo].[ActivityEvents] CHECK CONSTRAINT [FK_ActivityEvents_Events]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ActivityLogs_ActivityEvents]') AND parent_object_id = OBJECT_ID(N'[dbo].[ActivityLogs]'))
ALTER TABLE [dbo].[ActivityLogs]  WITH CHECK ADD  CONSTRAINT [FK_ActivityLogs_ActivityEvents] FOREIGN KEY([ActivityLog_EventId])
REFERENCES [dbo].[ActivityEvents] ([ActivityEvent_EventId])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ActivityLogs_ActivityEvents]') AND parent_object_id = OBJECT_ID(N'[dbo].[ActivityLogs]'))
ALTER TABLE [dbo].[ActivityLogs] CHECK CONSTRAINT [FK_ActivityLogs_ActivityEvents]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AddressExtn_Address1]') AND parent_object_id = OBJECT_ID(N'[dbo].[AddressExtn]'))
ALTER TABLE [dbo].[AddressExtn]  WITH CHECK ADD  CONSTRAINT [FK_AddressExtn_Address1] FOREIGN KEY([AddressExtn_ReferenceId])
REFERENCES [dbo].[Address] ([Address_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AddressExtn_Address1]') AND parent_object_id = OBJECT_ID(N'[dbo].[AddressExtn]'))
ALTER TABLE [dbo].[AddressExtn] CHECK CONSTRAINT [FK_AddressExtn_Address1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AddressExtn_Entity2]') AND parent_object_id = OBJECT_ID(N'[dbo].[AddressExtn]'))
ALTER TABLE [dbo].[AddressExtn]  WITH CHECK ADD  CONSTRAINT [FK_AddressExtn_Entity2] FOREIGN KEY([AddressExtn_EntityId])
REFERENCES [dbo].[Entity] ([Entity_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AddressExtn_Entity2]') AND parent_object_id = OBJECT_ID(N'[dbo].[AddressExtn]'))
ALTER TABLE [dbo].[AddressExtn] CHECK CONSTRAINT [FK_AddressExtn_Entity2]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AddressExtn_TenantDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[AddressExtn]'))
ALTER TABLE [dbo].[AddressExtn]  WITH CHECK ADD  CONSTRAINT [FK_AddressExtn_TenantDetails1] FOREIGN KEY([AddressExtn_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AddressExtn_TenantDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[AddressExtn]'))
ALTER TABLE [dbo].[AddressExtn] CHECK CONSTRAINT [FK_AddressExtn_TenantDetails1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_BatchFtpDetails_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[BatchFtpDetails]'))
ALTER TABLE [dbo].[BatchFtpDetails]  WITH CHECK ADD  CONSTRAINT [FK_BatchFtpDetails_NotificationBatch] FOREIGN KEY([BatchFtpDetails_BatchId])
REFERENCES [dbo].[NotificationBatch] ([NotificationBatch_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_BatchFtpDetails_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[BatchFtpDetails]'))
ALTER TABLE [dbo].[BatchFtpDetails] CHECK CONSTRAINT [FK_BatchFtpDetails_NotificationBatch]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_dbo_ChartAxisFields_ChartDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartAxisFields]'))
ALTER TABLE [dbo].[ChartAxisFields]  WITH CHECK ADD  CONSTRAINT [FK_dbo_ChartAxisFields_ChartDetails] FOREIGN KEY([ChartAxisFields_ChartId])
REFERENCES [dbo].[ChartDetails] ([ChartDetails_ChartId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_dbo_ChartAxisFields_ChartDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartAxisFields]'))
ALTER TABLE [dbo].[ChartAxisFields] CHECK CONSTRAINT [FK_dbo_ChartAxisFields_ChartDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ChartSectionProperties_Property]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartSectionProperties]'))
ALTER TABLE [dbo].[ChartSectionProperties]  WITH CHECK ADD  CONSTRAINT [FK_ChartSectionProperties_Property] FOREIGN KEY([ChartSectionProperties_PropertyId])
REFERENCES [dbo].[ChartProperty] ([ChartProperty_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ChartSectionProperties_Property]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartSectionProperties]'))
ALTER TABLE [dbo].[ChartSectionProperties] CHECK CONSTRAINT [FK_ChartSectionProperties_Property]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ChartSectionProperties_Sections]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartSectionProperties]'))
ALTER TABLE [dbo].[ChartSectionProperties]  WITH CHECK ADD  CONSTRAINT [FK_ChartSectionProperties_Sections] FOREIGN KEY([ChartSectionProperties_SectionId])
REFERENCES [dbo].[ChartSections] ([ChartSections_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ChartSectionProperties_Sections]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartSectionProperties]'))
ALTER TABLE [dbo].[ChartSectionProperties] CHECK CONSTRAINT [FK_ChartSectionProperties_Sections]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ChartPropertyValues_ChartDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartPropertyValues]'))
ALTER TABLE [dbo].[ChartPropertyValues]  WITH CHECK ADD  CONSTRAINT [FK_ChartPropertyValues_ChartDetails] FOREIGN KEY([ChartPropertyValues_ChartId])
REFERENCES [dbo].[ChartDetails] ([ChartDetails_ChartId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ChartPropertyValues_ChartDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartPropertyValues]'))
ALTER TABLE [dbo].[ChartPropertyValues] CHECK CONSTRAINT [FK_ChartPropertyValues_ChartDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ChartPropertyValues_SectionProperties]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartPropertyValues]'))
ALTER TABLE [dbo].[ChartPropertyValues]  WITH CHECK ADD  CONSTRAINT [FK_ChartPropertyValues_SectionProperties] FOREIGN KEY([ChartPropertyValues_SectionPropertyId])
REFERENCES [dbo].[ChartSectionProperties] ([ChartSectionProperties_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ChartPropertyValues_SectionProperties]') AND parent_object_id = OBJECT_ID(N'[dbo].[ChartPropertyValues]'))
ALTER TABLE [dbo].[ChartPropertyValues] CHECK CONSTRAINT [FK_ChartPropertyValues_SectionProperties]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ContactExtn_Contact1]') AND parent_object_id = OBJECT_ID(N'[dbo].[ContactExtn]'))
ALTER TABLE [dbo].[ContactExtn]  WITH CHECK ADD  CONSTRAINT [FK_ContactExtn_Contact1] FOREIGN KEY([ContactExtn_ReferenceId])
REFERENCES [dbo].[Contact] ([Contact_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ContactExtn_Contact1]') AND parent_object_id = OBJECT_ID(N'[dbo].[ContactExtn]'))
ALTER TABLE [dbo].[ContactExtn] CHECK CONSTRAINT [FK_ContactExtn_Contact1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ContactExtn_Entity1]') AND parent_object_id = OBJECT_ID(N'[dbo].[ContactExtn]'))
ALTER TABLE [dbo].[ContactExtn]  WITH CHECK ADD  CONSTRAINT [FK_ContactExtn_Entity1] FOREIGN KEY([ContactExtn_EntityId])
REFERENCES [dbo].[Entity] ([Entity_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ContactExtn_Entity1]') AND parent_object_id = OBJECT_ID(N'[dbo].[ContactExtn]'))
ALTER TABLE [dbo].[ContactExtn] CHECK CONSTRAINT [FK_ContactExtn_Entity1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ContactExtn_TenantDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[ContactExtn]'))
ALTER TABLE [dbo].[ContactExtn]  WITH CHECK ADD  CONSTRAINT [FK_ContactExtn_TenantDetails1] FOREIGN KEY([ContactExtn_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ContactExtn_TenantDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[ContactExtn]'))
ALTER TABLE [dbo].[ContactExtn] CHECK CONSTRAINT [FK_ContactExtn_TenantDetails1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataPartitionMapping_DataModule]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataPartitionMapping]'))
ALTER TABLE [dbo].[DataPartitionMapping]  WITH CHECK ADD  CONSTRAINT [FK_DataPartitionMapping_DataModule] FOREIGN KEY([DataModuleID])
REFERENCES [dbo].[DataModule] ([DataModuleID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataPartitionMapping_DataModule]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataPartitionMapping]'))
ALTER TABLE [dbo].[DataPartitionMapping] CHECK CONSTRAINT [FK_DataPartitionMapping_DataModule]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataPartitionMapping_DataPartition]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataPartitionMapping]'))
ALTER TABLE [dbo].[DataPartitionMapping]  WITH CHECK ADD  CONSTRAINT [FK_DataPartitionMapping_DataPartition] FOREIGN KEY([DataPartitionID])
REFERENCES [dbo].[DataPartition] ([DataPartitionID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataPartitionMapping_DataPartition]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataPartitionMapping]'))
ALTER TABLE [dbo].[DataPartitionMapping] CHECK CONSTRAINT [FK_DataPartitionMapping_DataPartition]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataPartitionMapping_DataServer]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataPartitionMapping]'))
ALTER TABLE [dbo].[DataPartitionMapping]  WITH CHECK ADD  CONSTRAINT [FK_DataPartitionMapping_DataServer] FOREIGN KEY([DataServerID])
REFERENCES [dbo].[DataServer] ([DataServerID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataPartitionMapping_DataServer]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataPartitionMapping]'))
ALTER TABLE [dbo].[DataPartitionMapping] CHECK CONSTRAINT [FK_DataPartitionMapping_DataServer]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataSource_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataSource]'))
ALTER TABLE [dbo].[DataSource]  WITH CHECK ADD  CONSTRAINT [FK_DataSource_TenantDetails] FOREIGN KEY([DataSource_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataSource_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataSource]'))
ALTER TABLE [dbo].[DataSource] CHECK CONSTRAINT [FK_DataSource_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFeature_DataView]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFeature]'))
ALTER TABLE [dbo].[DataViewFeature]  WITH CHECK ADD  CONSTRAINT [FK_DataViewFeature_DataView] FOREIGN KEY([DataViewFeature_DataViewID])
REFERENCES [dbo].[DataView] ([DataView_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFeature_DataView]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFeature]'))
ALTER TABLE [dbo].[DataViewFeature] CHECK CONSTRAINT [FK_DataViewFeature_DataView]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFields_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFields]'))
ALTER TABLE [dbo].[DataViewFields]  WITH CHECK ADD  CONSTRAINT [FK_DataViewFields_TenantDetails] FOREIGN KEY([DataViewField_ExtendedTenantCode])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFields_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFields]'))
ALTER TABLE [dbo].[DataViewFields] CHECK CONSTRAINT [FK_DataViewFields_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FormFields_FieldType]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFields]'))
ALTER TABLE [dbo].[DataViewFields]  WITH CHECK ADD  CONSTRAINT [FK_FormFields_FieldType] FOREIGN KEY([DataViewField_TypeID])
REFERENCES [dbo].[FieldType] ([FieldType_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FormFields_FieldType]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFields]'))
ALTER TABLE [dbo].[DataViewFields] CHECK CONSTRAINT [FK_FormFields_FieldType]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PickupList_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupList]'))
ALTER TABLE [dbo].[PickupList]  WITH CHECK ADD  CONSTRAINT [FK_PickupList_TenantDetails] FOREIGN KEY([PickupList_TenantCode])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PickupList_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupList]'))
ALTER TABLE [dbo].[PickupList] CHECK CONSTRAINT [FK_PickupList_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFieldProperties_DataViewFields]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]'))
ALTER TABLE [dbo].[DataViewFieldProperties]  WITH CHECK ADD  CONSTRAINT [FK_DataViewFieldProperties_DataViewFields] FOREIGN KEY([FieldProperties_FieldID])
REFERENCES [dbo].[DataViewFields] ([DataViewField_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFieldProperties_DataViewFields]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]'))
ALTER TABLE [dbo].[DataViewFieldProperties] CHECK CONSTRAINT [FK_DataViewFieldProperties_DataViewFields]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFieldProperties_PickupList]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]'))
ALTER TABLE [dbo].[DataViewFieldProperties]  WITH CHECK ADD  CONSTRAINT [FK_DataViewFieldProperties_PickupList] FOREIGN KEY([FieldProperties_PickupListID])
REFERENCES [dbo].[PickupList] ([PickupList_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFieldProperties_PickupList]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]'))
ALTER TABLE [dbo].[DataViewFieldProperties] CHECK CONSTRAINT [FK_DataViewFieldProperties_PickupList]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFieldProperties_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]'))
ALTER TABLE [dbo].[DataViewFieldProperties]  WITH CHECK ADD  CONSTRAINT [FK_DataViewFieldProperties_TenantDetails] FOREIGN KEY([FieldProperties_TenantID])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DataViewFieldProperties_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[DataViewFieldProperties]'))
ALTER TABLE [dbo].[DataViewFieldProperties] CHECK CONSTRAINT [FK_DataViewFieldProperties_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Roles_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[Roles]'))
ALTER TABLE [dbo].[Roles]  WITH CHECK ADD  CONSTRAINT [FK_Roles_TenantDetails] FOREIGN KEY([Role_TenantCode])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Roles_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[Roles]'))
ALTER TABLE [dbo].[Roles] CHECK CONSTRAINT [FK_Roles_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RolePrivileges_DataScope]') AND parent_object_id = OBJECT_ID(N'[dbo].[RolePrivileges]'))
ALTER TABLE [dbo].[RolePrivileges]  WITH CHECK ADD  CONSTRAINT [FK_RolePrivileges_DataScope] FOREIGN KEY([RolePrivilege_DataScopeID])
REFERENCES [dbo].[DataScope] ([DataScope_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RolePrivileges_DataScope]') AND parent_object_id = OBJECT_ID(N'[dbo].[RolePrivileges]'))
ALTER TABLE [dbo].[RolePrivileges] CHECK CONSTRAINT [FK_RolePrivileges_DataScope]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RolePrivileges_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RolePrivileges]'))
ALTER TABLE [dbo].[RolePrivileges]  WITH CHECK ADD  CONSTRAINT [FK_RolePrivileges_Roles] FOREIGN KEY([RolePrivilege_RoleID])
REFERENCES [dbo].[Roles] ([Role_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RolePrivileges_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RolePrivileges]'))
ALTER TABLE [dbo].[RolePrivileges] CHECK CONSTRAINT [FK_RolePrivileges_Roles]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RolePrivileges_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[RolePrivileges]'))
ALTER TABLE [dbo].[RolePrivileges]  WITH CHECK ADD  CONSTRAINT [FK_RolePrivileges_TenantDetails] FOREIGN KEY([RolePrivilege_TenantID])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RolePrivileges_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[RolePrivileges]'))
ALTER TABLE [dbo].[RolePrivileges] CHECK CONSTRAINT [FK_RolePrivileges_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DynamicDataScopeValue_DynamicVariable]') AND parent_object_id = OBJECT_ID(N'[dbo].[DynamicDataScopeValue]'))
ALTER TABLE [dbo].[DynamicDataScopeValue]  WITH CHECK ADD  CONSTRAINT [FK_DynamicDataScopeValue_DynamicVariable] FOREIGN KEY([DynamicDataScopeValue_Id])
REFERENCES [dbo].[DynamicVariable] ([DynamicVariable_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DynamicDataScopeValue_DynamicVariable]') AND parent_object_id = OBJECT_ID(N'[dbo].[DynamicDataScopeValue]'))
ALTER TABLE [dbo].[DynamicDataScopeValue] CHECK CONSTRAINT [FK_DynamicDataScopeValue_DynamicVariable]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DynamicDataScopeValue_RolePrivileges]') AND parent_object_id = OBJECT_ID(N'[dbo].[DynamicDataScopeValue]'))
ALTER TABLE [dbo].[DynamicDataScopeValue]  WITH CHECK ADD  CONSTRAINT [FK_DynamicDataScopeValue_RolePrivileges] FOREIGN KEY([DynamicDataScopeValue_RolePrivilegeId])
REFERENCES [dbo].[RolePrivileges] ([RolePrivilege_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DynamicDataScopeValue_RolePrivileges]') AND parent_object_id = OBJECT_ID(N'[dbo].[DynamicDataScopeValue]'))
ALTER TABLE [dbo].[DynamicDataScopeValue] CHECK CONSTRAINT [FK_DynamicDataScopeValue_RolePrivileges]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailContentDetails_NotificationContentDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailContentDetails]'))
ALTER TABLE [dbo].[EmailContentDetails]  WITH CHECK ADD  CONSTRAINT [FK_EmailContentDetails_NotificationContentDetails] FOREIGN KEY([EmailContentDetails_ContentDetailsId])
REFERENCES [dbo].[NotificationContentDetails] ([NotificationContentDetails_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailContentDetails_NotificationContentDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailContentDetails]'))
ALTER TABLE [dbo].[EmailContentDetails] CHECK CONSTRAINT [FK_EmailContentDetails_NotificationContentDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailContentDetails_Template]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailContentDetails]'))
ALTER TABLE [dbo].[EmailContentDetails]  WITH CHECK ADD  CONSTRAINT [FK_EmailContentDetails_Template] FOREIGN KEY([EmailContentDetails_TemplateId])
REFERENCES [dbo].[Template] ([Template_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailContentDetails_Template]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailContentDetails]'))
ALTER TABLE [dbo].[EmailContentDetails] CHECK CONSTRAINT [FK_EmailContentDetails_Template]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailDetails_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailDetails]'))
ALTER TABLE [dbo].[EmailDetails]  WITH NOCHECK ADD  CONSTRAINT [FK_EmailDetails_NotificationBatch] FOREIGN KEY([EmailDetails_BatchId])
REFERENCES [dbo].[NotificationBatch] ([NotificationBatch_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailDetails_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailDetails]'))
ALTER TABLE [dbo].[EmailDetails] CHECK CONSTRAINT [FK_EmailDetails_NotificationBatch]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailDispatchDetails_NotificationDispatchDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailDispatchDetails]'))
ALTER TABLE [dbo].[EmailDispatchDetails]  WITH CHECK ADD  CONSTRAINT [FK_EmailDispatchDetails_NotificationDispatchDetails] FOREIGN KEY([EmailDispatchDetails_DispatchDetailsId])
REFERENCES [dbo].[NotificationDispatchDetails] ([NotificationDispatchDetails_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailDispatchDetails_NotificationDispatchDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailDispatchDetails]'))
ALTER TABLE [dbo].[EmailDispatchDetails] CHECK CONSTRAINT [FK_EmailDispatchDetails_NotificationDispatchDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationDetails_NotificationContentDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationDetails]'))
ALTER TABLE [dbo].[NotificationDetails]  WITH CHECK ADD  CONSTRAINT [FK_NotificationDetails_NotificationContentDetails] FOREIGN KEY([NotificationDetails_ContentDetailsId])
REFERENCES [dbo].[NotificationContentDetails] ([NotificationContentDetails_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationDetails_NotificationContentDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationDetails]'))
ALTER TABLE [dbo].[NotificationDetails] CHECK CONSTRAINT [FK_NotificationDetails_NotificationContentDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationDetails_NotificationDispatchDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationDetails]'))
ALTER TABLE [dbo].[NotificationDetails]  WITH CHECK ADD  CONSTRAINT [FK_NotificationDetails_NotificationDispatchDetails] FOREIGN KEY([NotificationDetails_DispatchDetailsId])
REFERENCES [dbo].[NotificationDispatchDetails] ([NotificationDispatchDetails_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationDetails_NotificationDispatchDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationDetails]'))
ALTER TABLE [dbo].[NotificationDetails] CHECK CONSTRAINT [FK_NotificationDetails_NotificationDispatchDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationDetails_NotificationMaster]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationDetails]'))
ALTER TABLE [dbo].[NotificationDetails]  WITH CHECK ADD  CONSTRAINT [FK_NotificationDetails_NotificationMaster] FOREIGN KEY([NotificationDetails_NotificationId])
REFERENCES [dbo].[NotificationMaster] ([NotificationMaster_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationDetails_NotificationMaster]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationDetails]'))
ALTER TABLE [dbo].[NotificationDetails] CHECK CONSTRAINT [FK_NotificationDetails_NotificationMaster]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationAudit_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationAudit]'))
ALTER TABLE [dbo].[NotificationAudit]  WITH CHECK ADD  CONSTRAINT [FK_NotificationAudit_NotificationBatch] FOREIGN KEY([NotificationAudit_NotificationBatchId])
REFERENCES [dbo].[NotificationBatch] ([NotificationBatch_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationAudit_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationAudit]'))
ALTER TABLE [dbo].[NotificationAudit] CHECK CONSTRAINT [FK_NotificationAudit_NotificationBatch]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationAudit_NotificationDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationAudit]'))
ALTER TABLE [dbo].[NotificationAudit]  WITH CHECK ADD  CONSTRAINT [FK_NotificationAudit_NotificationDetails] FOREIGN KEY([NotificationAudit_NotificationId])
REFERENCES [dbo].[NotificationDetails] ([NotificationDetails_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationAudit_NotificationDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationAudit]'))
ALTER TABLE [dbo].[NotificationAudit] CHECK CONSTRAINT [FK_NotificationAudit_NotificationDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationDispatch_NotificationAudit]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationDispatch]'))
ALTER TABLE [dbo].[NotificationDispatch]  WITH NOCHECK ADD  CONSTRAINT [FK_NotificationDispatch_NotificationAudit] FOREIGN KEY([NotificationDispatch_NotificationAuditId])
REFERENCES [dbo].[NotificationAudit] ([NotificationAudit_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationDispatch_NotificationAudit]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationDispatch]'))
ALTER TABLE [dbo].[NotificationDispatch] CHECK CONSTRAINT [FK_NotificationDispatch_NotificationAudit]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailNotificationDispatch_NotificationDispatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailNotificationDispatch]'))
ALTER TABLE [dbo].[EmailNotificationDispatch]  WITH NOCHECK ADD  CONSTRAINT [FK_EmailNotificationDispatch_NotificationDispatch] FOREIGN KEY([EmailNotificationDispatch_NotificationDispatchId])
REFERENCES [dbo].[NotificationDispatch] ([NotificationDispatch_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EmailNotificationDispatch_NotificationDispatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[EmailNotificationDispatch]'))
ALTER TABLE [dbo].[EmailNotificationDispatch] CHECK CONSTRAINT [FK_EmailNotificationDispatch_NotificationDispatch]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityDataScope_DataScope]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityDataScope]'))
ALTER TABLE [dbo].[EntityDataScope]  WITH CHECK ADD  CONSTRAINT [FK_EntityDataScope_DataScope] FOREIGN KEY([EntityDataScope_DataScopeID])
REFERENCES [dbo].[DataScope] ([DataScope_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityDataScope_DataScope]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityDataScope]'))
ALTER TABLE [dbo].[EntityDataScope] CHECK CONSTRAINT [FK_EntityDataScope_DataScope]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FeatureEntity_Entity]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityFeaturePrivilege]'))
ALTER TABLE [dbo].[EntityFeaturePrivilege]  WITH CHECK ADD  CONSTRAINT [FK_FeatureEntity_Entity] FOREIGN KEY([EntityFeaturePrivilege_EntityId])
REFERENCES [dbo].[Entity] ([Entity_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FeatureEntity_Entity]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityFeaturePrivilege]'))
ALTER TABLE [dbo].[EntityFeaturePrivilege] CHECK CONSTRAINT [FK_FeatureEntity_Entity]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityFields_DataFieldType]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityFields]'))
ALTER TABLE [dbo].[EntityFields]  WITH CHECK ADD  CONSTRAINT [FK_EntityFields_DataFieldType] FOREIGN KEY([EntityFields_DataFieldTypeID])
REFERENCES [dbo].[DataFieldType] ([DataFieldType_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityFields_DataFieldType]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityFields]'))
ALTER TABLE [dbo].[EntityFields] CHECK CONSTRAINT [FK_EntityFields_DataFieldType]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityFields_PickupList]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityFields]'))
ALTER TABLE [dbo].[EntityFields]  WITH CHECK ADD  CONSTRAINT [FK_EntityFields_PickupList] FOREIGN KEY([EntityFields_PickUpListId])
REFERENCES [dbo].[PickupList] ([PickupList_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityFields_PickupList]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityFields]'))
ALTER TABLE [dbo].[EntityFields] CHECK CONSTRAINT [FK_EntityFields_PickupList]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityFields_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityFields]'))
ALTER TABLE [dbo].[EntityFields]  WITH CHECK ADD  CONSTRAINT [FK_EntityFields_TenantDetails] FOREIGN KEY([EntityFields_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityFields_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityFields]'))
ALTER TABLE [dbo].[EntityFields] CHECK CONSTRAINT [FK_EntityFields_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Privileges_Privileges]') AND parent_object_id = OBJECT_ID(N'[dbo].[Privileges]'))
ALTER TABLE [dbo].[Privileges]  WITH CHECK ADD  CONSTRAINT [FK_Privileges_Privileges] FOREIGN KEY([Privilege_BasePrivilege])
REFERENCES [dbo].[Privileges] ([Privilege_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Privileges_Privileges]') AND parent_object_id = OBJECT_ID(N'[dbo].[Privileges]'))
ALTER TABLE [dbo].[Privileges] CHECK CONSTRAINT [FK_Privileges_Privileges]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityID]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityPrivileges]'))
ALTER TABLE [dbo].[EntityPrivileges]  WITH CHECK ADD  CONSTRAINT [FK_EntityID] FOREIGN KEY([EntityID])
REFERENCES [dbo].[Entity] ([Entity_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EntityID]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityPrivileges]'))
ALTER TABLE [dbo].[EntityPrivileges] CHECK CONSTRAINT [FK_EntityID]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PrivilegeID]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityPrivileges]'))
ALTER TABLE [dbo].[EntityPrivileges]  WITH CHECK ADD  CONSTRAINT [FK_PrivilegeID] FOREIGN KEY([PrivilegeID])
REFERENCES [dbo].[Privileges] ([Privilege_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PrivilegeID]') AND parent_object_id = OBJECT_ID(N'[dbo].[EntityPrivileges]'))
ALTER TABLE [dbo].[EntityPrivileges] CHECK CONSTRAINT [FK_PrivilegeID]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EventSchedules_Events]') AND parent_object_id = OBJECT_ID(N'[dbo].[EventSchedules]'))
ALTER TABLE [dbo].[EventSchedules]  WITH CHECK ADD  CONSTRAINT [FK_EventSchedules_Events] FOREIGN KEY([EventSchedule_EventId])
REFERENCES [dbo].[Events] ([Event_Id])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EventSchedules_Events]') AND parent_object_id = OBJECT_ID(N'[dbo].[EventSchedules]'))
ALTER TABLE [dbo].[EventSchedules] CHECK CONSTRAINT [FK_EventSchedules_Events]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EventSchedules_Schedules]') AND parent_object_id = OBJECT_ID(N'[dbo].[EventSchedules]'))
ALTER TABLE [dbo].[EventSchedules]  WITH CHECK ADD  CONSTRAINT [FK_EventSchedules_Schedules] FOREIGN KEY([EventSchedule_ScheduleId])
REFERENCES [dbo].[Schedules] ([Schedule_Id])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_EventSchedules_Schedules]') AND parent_object_id = OBJECT_ID(N'[dbo].[EventSchedules]'))
ALTER TABLE [dbo].[EventSchedules] CHECK CONSTRAINT [FK_EventSchedules_Schedules]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Services_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[Services]'))
ALTER TABLE [dbo].[Services]  WITH CHECK ADD  CONSTRAINT [FK_Services_Services] FOREIGN KEY([Service_Code])
REFERENCES [dbo].[Services] ([Service_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Services_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[Services]'))
ALTER TABLE [dbo].[Services] CHECK CONSTRAINT [FK_Services_Services]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Modules_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[Modules]'))
ALTER TABLE [dbo].[Modules]  WITH CHECK ADD  CONSTRAINT [FK_Modules_Services] FOREIGN KEY([Module_ServiceCode])
REFERENCES [dbo].[Services] ([Service_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Modules_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[Modules]'))
ALTER TABLE [dbo].[Modules] CHECK CONSTRAINT [FK_Modules_Services]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Features_Modules]') AND parent_object_id = OBJECT_ID(N'[dbo].[Features]'))
ALTER TABLE [dbo].[Features]  WITH CHECK ADD  CONSTRAINT [FK_Features_Modules] FOREIGN KEY([Feature_ModuleCode])
REFERENCES [dbo].[Modules] ([Module_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Features_Modules]') AND parent_object_id = OBJECT_ID(N'[dbo].[Features]'))
ALTER TABLE [dbo].[Features] CHECK CONSTRAINT [FK_Features_Modules]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FileContentDetails_NotifiactionTemplate]') AND parent_object_id = OBJECT_ID(N'[dbo].[FileContentDetails]'))
ALTER TABLE [dbo].[FileContentDetails]  WITH CHECK ADD  CONSTRAINT [FK_FileContentDetails_NotifiactionTemplate] FOREIGN KEY([FileContentDetails_FileTemplateId])
REFERENCES [dbo].[NotificationTemplate] ([NotificationTemplate_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FileContentDetails_NotifiactionTemplate]') AND parent_object_id = OBJECT_ID(N'[dbo].[FileContentDetails]'))
ALTER TABLE [dbo].[FileContentDetails] CHECK CONSTRAINT [FK_FileContentDetails_NotifiactionTemplate]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FileContentDetails_NotificationContentDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[FileContentDetails]'))
ALTER TABLE [dbo].[FileContentDetails]  WITH CHECK ADD  CONSTRAINT [FK_FileContentDetails_NotificationContentDetails1] FOREIGN KEY([FileContentDetails_ContentDetailsId])
REFERENCES [dbo].[NotificationContentDetails] ([NotificationContentDetails_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FileContentDetails_NotificationContentDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[FileContentDetails]'))
ALTER TABLE [dbo].[FileContentDetails] CHECK CONSTRAINT [FK_FileContentDetails_NotificationContentDetails1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationContent_NotificationAudit]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationContent]'))
ALTER TABLE [dbo].[NotificationContent]  WITH NOCHECK ADD  CONSTRAINT [FK_NotificationContent_NotificationAudit] FOREIGN KEY([NotificationContent_NotificationAuditId])
REFERENCES [dbo].[NotificationAudit] ([NotificationAudit_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationContent_NotificationAudit]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationContent]'))
ALTER TABLE [dbo].[NotificationContent] CHECK CONSTRAINT [FK_NotificationContent_NotificationAudit]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FileNotificationContent_NotificationContent]') AND parent_object_id = OBJECT_ID(N'[dbo].[FileNotificationContent]'))
ALTER TABLE [dbo].[FileNotificationContent]  WITH NOCHECK ADD  CONSTRAINT [FK_FileNotificationContent_NotificationContent] FOREIGN KEY([FileNotificationContent_NotificationContentId])
REFERENCES [dbo].[NotificationContent] ([NotificationContent_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FileNotificationContent_NotificationContent]') AND parent_object_id = OBJECT_ID(N'[dbo].[FileNotificationContent]'))
ALTER TABLE [dbo].[FileNotificationContent] CHECK CONSTRAINT [FK_FileNotificationContent_NotificationContent]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FtpDetails_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[FtpDetails]'))
ALTER TABLE [dbo].[FtpDetails]  WITH NOCHECK ADD  CONSTRAINT [FK_FtpDetails_NotificationBatch] FOREIGN KEY([FtpDetails_BatchId])
REFERENCES [dbo].[NotificationBatch] ([NotificationBatch_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FtpDetails_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[FtpDetails]'))
ALTER TABLE [dbo].[FtpDetails] CHECK CONSTRAINT [FK_FtpDetails_NotificationBatch]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FtpDispatchDetails_NotificationDispatchDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[FtpDispatchDetails]'))
ALTER TABLE [dbo].[FtpDispatchDetails]  WITH CHECK ADD  CONSTRAINT [FK_FtpDispatchDetails_NotificationDispatchDetails] FOREIGN KEY([FtpDispatchDetails_DispatchDetailsId])
REFERENCES [dbo].[NotificationDispatchDetails] ([NotificationDispatchDetails_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FtpDispatchDetails_NotificationDispatchDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[FtpDispatchDetails]'))
ALTER TABLE [dbo].[FtpDispatchDetails] CHECK CONSTRAINT [FK_FtpDispatchDetails_NotificationDispatchDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FtpNotificationDispatch_NotificationDispatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[FtpNotificationDispatch]'))
ALTER TABLE [dbo].[FtpNotificationDispatch]  WITH NOCHECK ADD  CONSTRAINT [FK_FtpNotificationDispatch_NotificationDispatch] FOREIGN KEY([FtpNotificationDispatch_NotificationDispatchId])
REFERENCES [dbo].[NotificationDispatch] ([NotificationDispatch_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FtpNotificationDispatch_NotificationDispatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[FtpNotificationDispatch]'))
ALTER TABLE [dbo].[FtpNotificationDispatch] CHECK CONSTRAINT [FK_FtpNotificationDispatch_NotificationDispatch]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ImageSource_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ImageSource]'))
ALTER TABLE [dbo].[ImageSource]  WITH CHECK ADD  CONSTRAINT [FK_ImageSource_TenantDetails] FOREIGN KEY([ImageSource_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ImageSource_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ImageSource]'))
ALTER TABLE [dbo].[ImageSource] CHECK CONSTRAINT [FK_ImageSource_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_InvoiceInvoiceLineItems]') AND parent_object_id = OBJECT_ID(N'[dbo].[InvoiceLineItems]'))
ALTER TABLE [dbo].[InvoiceLineItems]  WITH CHECK ADD  CONSTRAINT [FK_InvoiceInvoiceLineItems] FOREIGN KEY([InvoiceId])
REFERENCES [dbo].[Invoices] ([Id])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_InvoiceInvoiceLineItems]') AND parent_object_id = OBJECT_ID(N'[dbo].[InvoiceLineItems]'))
ALTER TABLE [dbo].[InvoiceLineItems] CHECK CONSTRAINT [FK_InvoiceInvoiceLineItems]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_InvoicePayments_Invoices]') AND parent_object_id = OBJECT_ID(N'[dbo].[InvoicePayments]'))
ALTER TABLE [dbo].[InvoicePayments]  WITH CHECK ADD  CONSTRAINT [FK_InvoicePayments_Invoices] FOREIGN KEY([InvoiceId])
REFERENCES [dbo].[Invoices] ([Id])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_InvoicePayments_Invoices]') AND parent_object_id = OBJECT_ID(N'[dbo].[InvoicePayments]'))
ALTER TABLE [dbo].[InvoicePayments] CHECK CONSTRAINT [FK_InvoicePayments_Invoices]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_JobAudits_Jobs]') AND parent_object_id = OBJECT_ID(N'[dbo].[JobAudits]'))
ALTER TABLE [dbo].[JobAudits]  WITH CHECK ADD  CONSTRAINT [FK_JobAudits_Jobs] FOREIGN KEY([JobAudit_JobId])
REFERENCES [dbo].[Jobs] ([Job_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_JobAudits_Jobs]') AND parent_object_id = OBJECT_ID(N'[dbo].[JobAudits]'))
ALTER TABLE [dbo].[JobAudits] CHECK CONSTRAINT [FK_JobAudits_Jobs]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Tenant_License_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantLicense]'))
ALTER TABLE [dbo].[TenantLicense]  WITH CHECK ADD  CONSTRAINT [FK_Tenant_License_TenantDetails] FOREIGN KEY([TenantLicense_TenantID])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Tenant_License_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantLicense]'))
ALTER TABLE [dbo].[TenantLicense] CHECK CONSTRAINT [FK_Tenant_License_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantLicense_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantLicense]'))
ALTER TABLE [dbo].[TenantLicense]  WITH CHECK ADD  CONSTRAINT [FK_TenantLicense_LicensePackage] FOREIGN KEY([TenantLicense_LicensePackageId])
REFERENCES [dbo].[LicensePackage] ([LicensePackage_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantLicense_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantLicense]'))
ALTER TABLE [dbo].[TenantLicense] CHECK CONSTRAINT [FK_TenantLicense_LicensePackage]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicenseFeature_Features]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseFeature]'))
ALTER TABLE [dbo].[LicenseFeature]  WITH CHECK ADD  CONSTRAINT [FK_LicenseFeature_Features] FOREIGN KEY([LicenseFeature_PurchasedFeatureCode])
REFERENCES [dbo].[Features] ([Feature_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicenseFeature_Features]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseFeature]'))
ALTER TABLE [dbo].[LicenseFeature] CHECK CONSTRAINT [FK_LicenseFeature_Features]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicenseFeature_TenantLicense]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseFeature]'))
ALTER TABLE [dbo].[LicenseFeature]  WITH CHECK ADD  CONSTRAINT [FK_LicenseFeature_TenantLicense] FOREIGN KEY([LicenseFeature_TenantLicenseCode])
REFERENCES [dbo].[TenantLicense] ([TenantLicense_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicenseFeature_TenantLicense]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseFeature]'))
ALTER TABLE [dbo].[LicenseFeature] CHECK CONSTRAINT [FK_LicenseFeature_TenantLicense]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_License_Module_Modules]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseModule]'))
ALTER TABLE [dbo].[LicenseModule]  WITH CHECK ADD  CONSTRAINT [FK_License_Module_Modules] FOREIGN KEY([LicenseModule_PurchasedModuleCode])
REFERENCES [dbo].[Modules] ([Module_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_License_Module_Modules]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseModule]'))
ALTER TABLE [dbo].[LicenseModule] CHECK CONSTRAINT [FK_License_Module_Modules]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicenseModule_TenantLicense]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseModule]'))
ALTER TABLE [dbo].[LicenseModule]  WITH CHECK ADD  CONSTRAINT [FK_LicenseModule_TenantLicense] FOREIGN KEY([LicenseModule_TenantLicenseCode])
REFERENCES [dbo].[TenantLicense] ([TenantLicense_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicenseModule_TenantLicense]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseModule]'))
ALTER TABLE [dbo].[LicenseModule] CHECK CONSTRAINT [FK_LicenseModule_TenantLicense]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicensePackageFeature_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicensePackageFeature]'))
ALTER TABLE [dbo].[LicensePackageFeature]  WITH CHECK ADD  CONSTRAINT [FK_LicensePackageFeature_LicensePackage] FOREIGN KEY([LicensePackageFeature_LicensePackageID])
REFERENCES [dbo].[LicensePackage] ([LicensePackage_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicensePackageFeature_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicensePackageFeature]'))
ALTER TABLE [dbo].[LicensePackageFeature] CHECK CONSTRAINT [FK_LicensePackageFeature_LicensePackage]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicensePackageModule_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicensePackageModule]'))
ALTER TABLE [dbo].[LicensePackageModule]  WITH CHECK ADD  CONSTRAINT [FK_LicensePackageModule_LicensePackage] FOREIGN KEY([LicensePackageModule_LicensepackageID])
REFERENCES [dbo].[LicensePackage] ([LicensePackage_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicensePackageModule_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicensePackageModule]'))
ALTER TABLE [dbo].[LicensePackageModule] CHECK CONSTRAINT [FK_LicensePackageModule_LicensePackage]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicensePackageService_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicensePackageService]'))
ALTER TABLE [dbo].[LicensePackageService]  WITH CHECK ADD  CONSTRAINT [FK_LicensePackageService_LicensePackage] FOREIGN KEY([LicensePackageService_LicensepackageID])
REFERENCES [dbo].[LicensePackage] ([LicensePackage_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicensePackageService_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicensePackageService]'))
ALTER TABLE [dbo].[LicensePackageService] CHECK CONSTRAINT [FK_LicensePackageService_LicensePackage]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicensePackageUsage_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicensePackageUsage]'))
ALTER TABLE [dbo].[LicensePackageUsage]  WITH CHECK ADD  CONSTRAINT [FK_LicensePackageUsage_LicensePackage] FOREIGN KEY([LicensePackageUsage_LicensePackageID])
REFERENCES [dbo].[LicensePackage] ([LicensePackage_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicensePackageUsage_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicensePackageUsage]'))
ALTER TABLE [dbo].[LicensePackageUsage] CHECK CONSTRAINT [FK_LicensePackageUsage_LicensePackage]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Usage_UsageType]') AND parent_object_id = OBJECT_ID(N'[dbo].[Usage]'))
ALTER TABLE [dbo].[Usage]  WITH CHECK ADD  CONSTRAINT [FK_Usage_UsageType] FOREIGN KEY([Usage_UsageTypeCode])
REFERENCES [dbo].[UsageType] ([UsageType_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Usage_UsageType]') AND parent_object_id = OBJECT_ID(N'[dbo].[Usage]'))
ALTER TABLE [dbo].[Usage] CHECK CONSTRAINT [FK_Usage_UsageType]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Licnese_Usage_PaymentType]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseUsage]'))
ALTER TABLE [dbo].[LicenseUsage]  WITH CHECK ADD  CONSTRAINT [FK_Licnese_Usage_PaymentType] FOREIGN KEY([LicenseUsage_PaymentTypeCode])
REFERENCES [dbo].[PaymentType] ([PaymentType_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Licnese_Usage_PaymentType]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseUsage]'))
ALTER TABLE [dbo].[LicenseUsage] CHECK CONSTRAINT [FK_Licnese_Usage_PaymentType]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Licnese_Usage_Usage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseUsage]'))
ALTER TABLE [dbo].[LicenseUsage]  WITH CHECK ADD  CONSTRAINT [FK_Licnese_Usage_Usage] FOREIGN KEY([LicenseUsage_UsageCode])
REFERENCES [dbo].[Usage] ([Usage_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Licnese_Usage_Usage]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseUsage]'))
ALTER TABLE [dbo].[LicenseUsage] CHECK CONSTRAINT [FK_Licnese_Usage_Usage]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicneseUsage_TenantLicense]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseUsage]'))
ALTER TABLE [dbo].[LicenseUsage]  WITH CHECK ADD  CONSTRAINT [FK_LicneseUsage_TenantLicense] FOREIGN KEY([LicenseUsage_TenantLicenseCode])
REFERENCES [dbo].[TenantLicense] ([TenantLicense_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LicneseUsage_TenantLicense]') AND parent_object_id = OBJECT_ID(N'[dbo].[LicenseUsage]'))
ALTER TABLE [dbo].[LicenseUsage] CHECK CONSTRAINT [FK_LicneseUsage_TenantLicense]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LocaleTemplate_LocaleId]') AND parent_object_id = OBJECT_ID(N'[dbo].[LocaleTemplate]'))
ALTER TABLE [dbo].[LocaleTemplate]  WITH NOCHECK ADD  CONSTRAINT [FK_LocaleTemplate_LocaleId] FOREIGN KEY([LocaleTemplate_LocaleId])
REFERENCES [dbo].[LocaleNames] ([LocaleNames_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_LocaleTemplate_LocaleId]') AND parent_object_id = OBJECT_ID(N'[dbo].[LocaleTemplate]'))
ALTER TABLE [dbo].[LocaleTemplate] CHECK CONSTRAINT [FK_LocaleTemplate_LocaleId]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Metering_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[Metering]'))
ALTER TABLE [dbo].[Metering]  WITH CHECK ADD  CONSTRAINT [FK_Metering_TenantDetails] FOREIGN KEY([Metering_TenantID])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Metering_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[Metering]'))
ALTER TABLE [dbo].[Metering] CHECK CONSTRAINT [FK_Metering_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_MeteringLog_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[MeteringLog]'))
ALTER TABLE [dbo].[MeteringLog]  WITH CHECK ADD  CONSTRAINT [FK_MeteringLog_TenantDetails] FOREIGN KEY([MeteringLog_TenantID])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_MeteringLog_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[MeteringLog]'))
ALTER TABLE [dbo].[MeteringLog] CHECK CONSTRAINT [FK_MeteringLog_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ModuleConfigurationSetting_ModuleConfigurationSetting]') AND parent_object_id = OBJECT_ID(N'[dbo].[ModuleConfigurationSetting]'))
ALTER TABLE [dbo].[ModuleConfigurationSetting]  WITH CHECK ADD  CONSTRAINT [FK_ModuleConfigurationSetting_ModuleConfigurationSetting] FOREIGN KEY([ModuleConfigurationSetting_Id])
REFERENCES [dbo].[ModuleConfigurationSetting] ([ModuleConfigurationSetting_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ModuleConfigurationSetting_ModuleConfigurationSetting]') AND parent_object_id = OBJECT_ID(N'[dbo].[ModuleConfigurationSetting]'))
ALTER TABLE [dbo].[ModuleConfigurationSetting] CHECK CONSTRAINT [FK_ModuleConfigurationSetting_ModuleConfigurationSetting]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationAuditFailure_NotificationAudit]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationAuditFailure]'))
ALTER TABLE [dbo].[NotificationAuditFailure]  WITH CHECK ADD  CONSTRAINT [FK_NotificationAuditFailure_NotificationAudit] FOREIGN KEY([NotificationAuditFailure_NotificationAuditId])
REFERENCES [dbo].[NotificationAudit] ([NotificationAudit_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_NotificationAuditFailure_NotificationAudit]') AND parent_object_id = OBJECT_ID(N'[dbo].[NotificationAuditFailure]'))
ALTER TABLE [dbo].[NotificationAuditFailure] CHECK CONSTRAINT [FK_NotificationAuditFailure_NotificationAudit]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SettingsAttribute_AttributeSet]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]'))
ALTER TABLE [dbo].[SettingsAttribute]  WITH CHECK ADD  CONSTRAINT [FK_SettingsAttribute_AttributeSet] FOREIGN KEY([SettingsAttribute_AttributeSetId])
REFERENCES [dbo].[AttributeSet] ([AttributeSet_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SettingsAttribute_AttributeSet]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]'))
ALTER TABLE [dbo].[SettingsAttribute] CHECK CONSTRAINT [FK_SettingsAttribute_AttributeSet]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SettingsAttribute_DataFieldType]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]'))
ALTER TABLE [dbo].[SettingsAttribute]  WITH CHECK ADD  CONSTRAINT [FK_SettingsAttribute_DataFieldType] FOREIGN KEY([SettingsAttribute_DataFieldTypeId])
REFERENCES [dbo].[DataFieldType] ([DataFieldType_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SettingsAttribute_DataFieldType]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]'))
ALTER TABLE [dbo].[SettingsAttribute] CHECK CONSTRAINT [FK_SettingsAttribute_DataFieldType]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSettingsAttribute_Features]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]'))
ALTER TABLE [dbo].[SettingsAttribute]  WITH CHECK ADD  CONSTRAINT [FK_TenantSettingsAttribute_Features] FOREIGN KEY([SettingsAttribute_FeatureCode])
REFERENCES [dbo].[Features] ([Feature_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSettingsAttribute_Features]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]'))
ALTER TABLE [dbo].[SettingsAttribute] CHECK CONSTRAINT [FK_TenantSettingsAttribute_Features]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSettingsAttribute_Modules]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]'))
ALTER TABLE [dbo].[SettingsAttribute]  WITH CHECK ADD  CONSTRAINT [FK_TenantSettingsAttribute_Modules] FOREIGN KEY([SettingsAttribute_ModuleCode])
REFERENCES [dbo].[Modules] ([Module_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSettingsAttribute_Modules]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingsAttribute]'))
ALTER TABLE [dbo].[SettingsAttribute] CHECK CONSTRAINT [FK_TenantSettingsAttribute_Modules]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PackageSetting_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[PackageSetting]'))
ALTER TABLE [dbo].[PackageSetting]  WITH CHECK ADD  CONSTRAINT [FK_PackageSetting_LicensePackage] FOREIGN KEY([PackageSetting_PackageID])
REFERENCES [dbo].[LicensePackage] ([LicensePackage_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PackageSetting_LicensePackage]') AND parent_object_id = OBJECT_ID(N'[dbo].[PackageSetting]'))
ALTER TABLE [dbo].[PackageSetting] CHECK CONSTRAINT [FK_PackageSetting_LicensePackage]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PackageSetting_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[PackageSetting]'))
ALTER TABLE [dbo].[PackageSetting]  WITH CHECK ADD  CONSTRAINT [FK_PackageSetting_SettingsAttribute] FOREIGN KEY([PackageSetting_AttributeID])
REFERENCES [dbo].[SettingsAttribute] ([SettingsAttribute_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PackageSetting_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[PackageSetting]'))
ALTER TABLE [dbo].[PackageSetting] CHECK CONSTRAINT [FK_PackageSetting_SettingsAttribute]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_CountryList_PickupList]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupList_Country]'))
ALTER TABLE [dbo].[PickupList_Country]  WITH CHECK ADD  CONSTRAINT [FK_CountryList_PickupList] FOREIGN KEY([PickupListValue_PickupListID])
REFERENCES [dbo].[PickupList] ([PickupList_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_CountryList_PickupList]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupList_Country]'))
ALTER TABLE [dbo].[PickupList_Country] CHECK CONSTRAINT [FK_CountryList_PickupList]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_CountryList_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupList_Country]'))
ALTER TABLE [dbo].[PickupList_Country]  WITH CHECK ADD  CONSTRAINT [FK_CountryList_TenantDetails] FOREIGN KEY([PickupListValue_TenantCode])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_CountryList_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupList_Country]'))
ALTER TABLE [dbo].[PickupList_Country] CHECK CONSTRAINT [FK_CountryList_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PickupListValue_PickupList]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupListValue]'))
ALTER TABLE [dbo].[PickupListValue]  WITH CHECK ADD  CONSTRAINT [FK_PickupListValue_PickupList] FOREIGN KEY([PickupListValue_PickupListID])
REFERENCES [dbo].[PickupList] ([PickupList_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PickupListValue_PickupList]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupListValue]'))
ALTER TABLE [dbo].[PickupListValue] CHECK CONSTRAINT [FK_PickupListValue_PickupList]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PickupListValue_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupListValue]'))
ALTER TABLE [dbo].[PickupListValue]  WITH CHECK ADD  CONSTRAINT [FK_PickupListValue_TenantDetails] FOREIGN KEY([PickupListValue_TenantCode])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PickupListValue_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupListValue]'))
ALTER TABLE [dbo].[PickupListValue] CHECK CONSTRAINT [FK_PickupListValue_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RelationshipMetaData_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[RelationshipMetaData]'))
ALTER TABLE [dbo].[RelationshipMetaData]  WITH CHECK ADD  CONSTRAINT [FK_RelationshipMetaData_TenantDetails] FOREIGN KEY([RelationshipMetaData_TenantID])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RelationshipMetaData_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[RelationshipMetaData]'))
ALTER TABLE [dbo].[RelationshipMetaData] CHECK CONSTRAINT [FK_RelationshipMetaData_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PickupListValueMapping_RelationshipMetaData]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupListValueMapping]'))
ALTER TABLE [dbo].[PickupListValueMapping]  WITH CHECK ADD  CONSTRAINT [FK_PickupListValueMapping_RelationshipMetaData] FOREIGN KEY([PickupListValueMapping_RelationshipMetaDataID])
REFERENCES [dbo].[RelationshipMetaData] ([RelationshipMetaData_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PickupListValueMapping_RelationshipMetaData]') AND parent_object_id = OBJECT_ID(N'[dbo].[PickupListValueMapping]'))
ALTER TABLE [dbo].[PickupListValueMapping] CHECK CONSTRAINT [FK_PickupListValueMapping_RelationshipMetaData]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PricePlanLineItemPriceTable]') AND parent_object_id = OBJECT_ID(N'[dbo].[PricePlanLineItems]'))
ALTER TABLE [dbo].[PricePlanLineItems]  WITH CHECK ADD  CONSTRAINT [FK_PricePlanLineItemPriceTable] FOREIGN KEY([PriceTableId])
REFERENCES [dbo].[PriceTables] ([Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PricePlanLineItemPriceTable]') AND parent_object_id = OBJECT_ID(N'[dbo].[PricePlanLineItems]'))
ALTER TABLE [dbo].[PricePlanLineItems] CHECK CONSTRAINT [FK_PricePlanLineItemPriceTable]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PricePlanPricePlanLineItem]') AND parent_object_id = OBJECT_ID(N'[dbo].[PricePlanLineItems]'))
ALTER TABLE [dbo].[PricePlanLineItems]  WITH CHECK ADD  CONSTRAINT [FK_PricePlanPricePlanLineItem] FOREIGN KEY([PricePlanId])
REFERENCES [dbo].[PricePlans] ([Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PricePlanPricePlanLineItem]') AND parent_object_id = OBJECT_ID(N'[dbo].[PricePlanLineItems]'))
ALTER TABLE [dbo].[PricePlanLineItems] CHECK CONSTRAINT [FK_PricePlanPricePlanLineItem]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PriceTablePriceSlab]') AND parent_object_id = OBJECT_ID(N'[dbo].[PriceSlabs]'))
ALTER TABLE [dbo].[PriceSlabs]  WITH CHECK ADD  CONSTRAINT [FK_PriceTablePriceSlab] FOREIGN KEY([PriceTableId])
REFERENCES [dbo].[PriceTables] ([Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PriceTablePriceSlab]') AND parent_object_id = OBJECT_ID(N'[dbo].[PriceSlabs]'))
ALTER TABLE [dbo].[PriceSlabs] CHECK CONSTRAINT [FK_PriceTablePriceSlab]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Report_Report]') AND parent_object_id = OBJECT_ID(N'[dbo].[Report]'))
ALTER TABLE [dbo].[Report]  WITH CHECK ADD  CONSTRAINT [FK_Report_Report] FOREIGN KEY([Report_Id])
REFERENCES [dbo].[Report] ([Report_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Report_Report]') AND parent_object_id = OBJECT_ID(N'[dbo].[Report]'))
ALTER TABLE [dbo].[Report] CHECK CONSTRAINT [FK_Report_Report]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Report_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[Report]'))
ALTER TABLE [dbo].[Report]  WITH CHECK ADD  CONSTRAINT [FK_Report_TenantDetails] FOREIGN KEY([Report_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Report_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[Report]'))
ALTER TABLE [dbo].[Report] CHECK CONSTRAINT [FK_Report_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ReportNotificationContent_NotificationContent]') AND parent_object_id = OBJECT_ID(N'[dbo].[ReportNotificationContent]'))
ALTER TABLE [dbo].[ReportNotificationContent]  WITH CHECK ADD  CONSTRAINT [FK_ReportNotificationContent_NotificationContent] FOREIGN KEY([ReportNotificationContent_NotificationContentId])
REFERENCES [dbo].[NotificationContent] ([NotificationContent_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ReportNotificationContent_NotificationContent]') AND parent_object_id = OBJECT_ID(N'[dbo].[ReportNotificationContent]'))
ALTER TABLE [dbo].[ReportNotificationContent] CHECK CONSTRAINT [FK_ReportNotificationContent_NotificationContent]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ReportObject_Report]') AND parent_object_id = OBJECT_ID(N'[dbo].[ReportObject]'))
ALTER TABLE [dbo].[ReportObject]  WITH CHECK ADD  CONSTRAINT [FK_ReportObject_Report] FOREIGN KEY([ReportObject_TypeId])
REFERENCES [dbo].[ReportObjectType] ([ReportObjectType_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ReportObject_Report]') AND parent_object_id = OBJECT_ID(N'[dbo].[ReportObject]'))
ALTER TABLE [dbo].[ReportObject] CHECK CONSTRAINT [FK_ReportObject_Report]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ReportObject_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ReportObject]'))
ALTER TABLE [dbo].[ReportObject]  WITH CHECK ADD  CONSTRAINT [FK_ReportObject_TenantDetails] FOREIGN KEY([ReportObject_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ReportObject_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ReportObject]'))
ALTER TABLE [dbo].[ReportObject] CHECK CONSTRAINT [FK_ReportObject_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleAssignablePrivilege_Privileges]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleAssignablePrivilege]'))
ALTER TABLE [dbo].[RoleAssignablePrivilege]  WITH CHECK ADD  CONSTRAINT [FK_RoleAssignablePrivilege_Privileges] FOREIGN KEY([RoleAssignablePrivilege_PrivilegeId])
REFERENCES [dbo].[Privileges] ([Privilege_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleAssignablePrivilege_Privileges]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleAssignablePrivilege]'))
ALTER TABLE [dbo].[RoleAssignablePrivilege] CHECK CONSTRAINT [FK_RoleAssignablePrivilege_Privileges]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleAssignablePrivilege_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleAssignablePrivilege]'))
ALTER TABLE [dbo].[RoleAssignablePrivilege]  WITH CHECK ADD  CONSTRAINT [FK_RoleAssignablePrivilege_Roles] FOREIGN KEY([RoleAssignablePrivilege_RoleId])
REFERENCES [dbo].[Roles] ([Role_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleAssignablePrivilege_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleAssignablePrivilege]'))
ALTER TABLE [dbo].[RoleAssignablePrivilege] CHECK CONSTRAINT [FK_RoleAssignablePrivilege_Roles]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleFeature_DataScope]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleFeature]'))
ALTER TABLE [dbo].[RoleFeature]  WITH CHECK ADD  CONSTRAINT [FK_RoleFeature_DataScope] FOREIGN KEY([RoleFeature_DataScopeID])
REFERENCES [dbo].[DataScope] ([DataScope_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleFeature_DataScope]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleFeature]'))
ALTER TABLE [dbo].[RoleFeature] CHECK CONSTRAINT [FK_RoleFeature_DataScope]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleFeature_Features]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleFeature]'))
ALTER TABLE [dbo].[RoleFeature]  WITH CHECK ADD  CONSTRAINT [FK_RoleFeature_Features] FOREIGN KEY([RoleFeature_FeatureID])
REFERENCES [dbo].[Features] ([Feature_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleFeature_Features]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleFeature]'))
ALTER TABLE [dbo].[RoleFeature] CHECK CONSTRAINT [FK_RoleFeature_Features]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleFeature_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleFeature]'))
ALTER TABLE [dbo].[RoleFeature]  WITH CHECK ADD  CONSTRAINT [FK_RoleFeature_Roles] FOREIGN KEY([RoleFeature_RoleID])
REFERENCES [dbo].[Roles] ([Role_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleFeature_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleFeature]'))
ALTER TABLE [dbo].[RoleFeature] CHECK CONSTRAINT [FK_RoleFeature_Roles]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleService_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleService]'))
ALTER TABLE [dbo].[RoleService]  WITH CHECK ADD  CONSTRAINT [FK_RoleService_Roles] FOREIGN KEY([RoleService_RoleId])
REFERENCES [dbo].[Roles] ([Role_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleService_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleService]'))
ALTER TABLE [dbo].[RoleService] CHECK CONSTRAINT [FK_RoleService_Roles]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleService_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleService]'))
ALTER TABLE [dbo].[RoleService]  WITH CHECK ADD  CONSTRAINT [FK_RoleService_Services] FOREIGN KEY([RoleService_ServiceCode])
REFERENCES [dbo].[Services] ([Service_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleService_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleService]'))
ALTER TABLE [dbo].[RoleService] CHECK CONSTRAINT [FK_RoleService_Services]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleSetting_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleSetting]'))
ALTER TABLE [dbo].[RoleSetting]  WITH CHECK ADD  CONSTRAINT [FK_RoleSetting_Roles] FOREIGN KEY([RoleSetting_RoleID])
REFERENCES [dbo].[Roles] ([Role_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleSetting_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleSetting]'))
ALTER TABLE [dbo].[RoleSetting] CHECK CONSTRAINT [FK_RoleSetting_Roles]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleSetting_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleSetting]'))
ALTER TABLE [dbo].[RoleSetting]  WITH CHECK ADD  CONSTRAINT [FK_RoleSetting_SettingsAttribute] FOREIGN KEY([RoleSetting_AttributeID])
REFERENCES [dbo].[SettingsAttribute] ([SettingsAttribute_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleSetting_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleSetting]'))
ALTER TABLE [dbo].[RoleSetting] CHECK CONSTRAINT [FK_RoleSetting_SettingsAttribute]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleTenant_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleTenant]'))
ALTER TABLE [dbo].[RoleTenant]  WITH CHECK ADD  CONSTRAINT [FK_RoleTenant_Roles] FOREIGN KEY([RoleTenant_RoleId])
REFERENCES [dbo].[Roles] ([Role_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleTenant_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleTenant]'))
ALTER TABLE [dbo].[RoleTenant] CHECK CONSTRAINT [FK_RoleTenant_Roles]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleTenant_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleTenant]'))
ALTER TABLE [dbo].[RoleTenant]  WITH CHECK ADD  CONSTRAINT [FK_RoleTenant_TenantDetails] FOREIGN KEY([RoleTenant_TenantCode])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_RoleTenant_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[RoleTenant]'))
ALTER TABLE [dbo].[RoleTenant] CHECK CONSTRAINT [FK_RoleTenant_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ScheduleJobs_Jobs]') AND parent_object_id = OBJECT_ID(N'[dbo].[ScheduleJobs]'))
ALTER TABLE [dbo].[ScheduleJobs]  WITH CHECK ADD  CONSTRAINT [FK_ScheduleJobs_Jobs] FOREIGN KEY([ScheduleJobs_JobId])
REFERENCES [dbo].[Jobs] ([Job_Id])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ScheduleJobs_Jobs]') AND parent_object_id = OBJECT_ID(N'[dbo].[ScheduleJobs]'))
ALTER TABLE [dbo].[ScheduleJobs] CHECK CONSTRAINT [FK_ScheduleJobs_Jobs]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ScheduleJobs_Schedules]') AND parent_object_id = OBJECT_ID(N'[dbo].[ScheduleJobs]'))
ALTER TABLE [dbo].[ScheduleJobs]  WITH CHECK ADD  CONSTRAINT [FK_ScheduleJobs_Schedules] FOREIGN KEY([ScheduleJobs_ScheduleId])
REFERENCES [dbo].[Schedules] ([Schedule_Id])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ScheduleJobs_Schedules]') AND parent_object_id = OBJECT_ID(N'[dbo].[ScheduleJobs]'))
ALTER TABLE [dbo].[ScheduleJobs] CHECK CONSTRAINT [FK_ScheduleJobs_Schedules]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SettingTemplateValue_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingTemplateValue]'))
ALTER TABLE [dbo].[SettingTemplateValue]  WITH NOCHECK ADD  CONSTRAINT [FK_SettingTemplateValue_SettingsAttribute] FOREIGN KEY([SettingTemplateValue_AttributeId])
REFERENCES [dbo].[SettingsAttribute] ([SettingsAttribute_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SettingTemplateValue_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingTemplateValue]'))
ALTER TABLE [dbo].[SettingTemplateValue] CHECK CONSTRAINT [FK_SettingTemplateValue_SettingsAttribute]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SettingTemplateValue_SettingTemplate1]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingTemplateValue]'))
ALTER TABLE [dbo].[SettingTemplateValue]  WITH NOCHECK ADD  CONSTRAINT [FK_SettingTemplateValue_SettingTemplate1] FOREIGN KEY([SettingTemplateValue_SettingTemplateId])
REFERENCES [dbo].[SettingTemplate] ([SettingTemplate_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SettingTemplateValue_SettingTemplate1]') AND parent_object_id = OBJECT_ID(N'[dbo].[SettingTemplateValue]'))
ALTER TABLE [dbo].[SettingTemplateValue] CHECK CONSTRAINT [FK_SettingTemplateValue_SettingTemplate1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationContent_NotificationContent]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationContent]'))
ALTER TABLE [dbo].[SystemNotificationContent]  WITH CHECK ADD  CONSTRAINT [FK_SystemNotificationContent_NotificationContent] FOREIGN KEY([SystemNotificationContent_NotificationContentId])
REFERENCES [dbo].[NotificationContent] ([NotificationContent_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationContent_NotificationContent]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationContent]'))
ALTER TABLE [dbo].[SystemNotificationContent] CHECK CONSTRAINT [FK_SystemNotificationContent_NotificationContent]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationContentDetails_NotificationContentDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationContentDetails]'))
ALTER TABLE [dbo].[SystemNotificationContentDetails]  WITH CHECK ADD  CONSTRAINT [FK_SystemNotificationContentDetails_NotificationContentDetails] FOREIGN KEY([SystemNotificationContentDetails_ContentDetailsId])
REFERENCES [dbo].[NotificationContentDetails] ([NotificationContentDetails_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationContentDetails_NotificationContentDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationContentDetails]'))
ALTER TABLE [dbo].[SystemNotificationContentDetails] CHECK CONSTRAINT [FK_SystemNotificationContentDetails_NotificationContentDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationContentDetails_Template]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationContentDetails]'))
ALTER TABLE [dbo].[SystemNotificationContentDetails]  WITH CHECK ADD  CONSTRAINT [FK_SystemNotificationContentDetails_Template] FOREIGN KEY([SystemNotificationContentDetails_TemplateId])
REFERENCES [dbo].[Template] ([Template_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationContentDetails_Template]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationContentDetails]'))
ALTER TABLE [dbo].[SystemNotificationContentDetails] CHECK CONSTRAINT [FK_SystemNotificationContentDetails_Template]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationDetails_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationDetails]'))
ALTER TABLE [dbo].[SystemNotificationDetails]  WITH NOCHECK ADD  CONSTRAINT [FK_SystemNotificationDetails_NotificationBatch] FOREIGN KEY([SystemNotificationDetails_BatchId])
REFERENCES [dbo].[NotificationBatch] ([NotificationBatch_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationDetails_NotificationBatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationDetails]'))
ALTER TABLE [dbo].[SystemNotificationDetails] CHECK CONSTRAINT [FK_SystemNotificationDetails_NotificationBatch]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationDispatch_NotificationDispatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationDispatch]'))
ALTER TABLE [dbo].[SystemNotificationDispatch]  WITH NOCHECK ADD  CONSTRAINT [FK_SystemNotificationDispatch_NotificationDispatch] FOREIGN KEY([SystemNotificationDispatch_NotificationDispatchId])
REFERENCES [dbo].[NotificationDispatch] ([NotificationDispatch_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationDispatch_NotificationDispatch]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationDispatch]'))
ALTER TABLE [dbo].[SystemNotificationDispatch] CHECK CONSTRAINT [FK_SystemNotificationDispatch_NotificationDispatch]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationDispatchDetails_NotificationDispatchDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationDispatchDetails]'))
ALTER TABLE [dbo].[SystemNotificationDispatchDetails]  WITH NOCHECK ADD  CONSTRAINT [FK_SystemNotificationDispatchDetails_NotificationDispatchDetails] FOREIGN KEY([SystemNotificationDispatchDetails_DispatchDetailsId])
REFERENCES [dbo].[NotificationDispatchDetails] ([NotificationDispatchDetails_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SystemNotificationDispatchDetails_NotificationDispatchDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[SystemNotificationDispatchDetails]'))
ALTER TABLE [dbo].[SystemNotificationDispatchDetails] CHECK CONSTRAINT [FK_SystemNotificationDispatchDetails_NotificationDispatchDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableSource_DataSource]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableSource]'))
ALTER TABLE [dbo].[TableSource]  WITH CHECK ADD  CONSTRAINT [FK_TableSource_DataSource] FOREIGN KEY([TableSource_DataSourceId])
REFERENCES [dbo].[DataSource] ([DataSource_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableSource_DataSource]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableSource]'))
ALTER TABLE [dbo].[TableSource] CHECK CONSTRAINT [FK_TableSource_DataSource]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableSource_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableSource]'))
ALTER TABLE [dbo].[TableSource]  WITH CHECK ADD  CONSTRAINT [FK_TableSource_TenantDetails] FOREIGN KEY([TableSource_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableSource_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableSource]'))
ALTER TABLE [dbo].[TableSource] CHECK CONSTRAINT [FK_TableSource_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableResultColumn_ReportObjects]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableResultColumn]'))
ALTER TABLE [dbo].[TableResultColumn]  WITH CHECK ADD  CONSTRAINT [FK_TableResultColumn_ReportObjects] FOREIGN KEY([TableResultColumn_ReportObjectId])
REFERENCES [dbo].[ReportObject] ([ReportObject_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableResultColumn_ReportObjects]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableResultColumn]'))
ALTER TABLE [dbo].[TableResultColumn] CHECK CONSTRAINT [FK_TableResultColumn_ReportObjects]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableResultColumn_TableSource]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableResultColumn]'))
ALTER TABLE [dbo].[TableResultColumn]  WITH CHECK ADD  CONSTRAINT [FK_TableResultColumn_TableSource] FOREIGN KEY([TableResultColumn_TableSourceId])
REFERENCES [dbo].[TableSource] ([TableSource_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableResultColumn_TableSource]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableResultColumn]'))
ALTER TABLE [dbo].[TableResultColumn] CHECK CONSTRAINT [FK_TableResultColumn_TableSource]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableResultColumn_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableResultColumn]'))
ALTER TABLE [dbo].[TableResultColumn]  WITH CHECK ADD  CONSTRAINT [FK_TableResultColumn_TenantDetails] FOREIGN KEY([TableResultColumn_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TableResultColumn_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TableResultColumn]'))
ALTER TABLE [dbo].[TableResultColumn] CHECK CONSTRAINT [FK_TableResultColumn_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantActivationDetails_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantActivationDetails]'))
ALTER TABLE [dbo].[TenantActivationDetails]  WITH CHECK ADD  CONSTRAINT [FK_TenantActivationDetails_TenantDetails] FOREIGN KEY([TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantActivationDetails_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantActivationDetails]'))
ALTER TABLE [dbo].[TenantActivationDetails] CHECK CONSTRAINT [FK_TenantActivationDetails_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantCryptKey_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantCryptKey]'))
ALTER TABLE [dbo].[TenantCryptKey]  WITH CHECK ADD  CONSTRAINT [FK_TenantCryptKey_TenantDetails] FOREIGN KEY([TenantCryptKey_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantCryptKey_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantCryptKey]'))
ALTER TABLE [dbo].[TenantCryptKey] CHECK CONSTRAINT [FK_TenantCryptKey_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetailsExtn_Entity1]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetailsExtn]'))
ALTER TABLE [dbo].[TenantDetailsExtn]  WITH CHECK ADD  CONSTRAINT [FK_TenantDetailsExtn_Entity1] FOREIGN KEY([TenantDetailsExtn_EntityId])
REFERENCES [dbo].[Entity] ([Entity_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetailsExtn_Entity1]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetailsExtn]'))
ALTER TABLE [dbo].[TenantDetailsExtn] CHECK CONSTRAINT [FK_TenantDetailsExtn_Entity1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetailsExtn_TenantDetails2]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetailsExtn]'))
ALTER TABLE [dbo].[TenantDetailsExtn]  WITH CHECK ADD  CONSTRAINT [FK_TenantDetailsExtn_TenantDetails2] FOREIGN KEY([TenantDetailsExtn_ReferenceId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetailsExtn_TenantDetails2]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetailsExtn]'))
ALTER TABLE [dbo].[TenantDetailsExtn] CHECK CONSTRAINT [FK_TenantDetailsExtn_TenantDetails2]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetailsExtn_TenantDetails3]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetailsExtn]'))
ALTER TABLE [dbo].[TenantDetailsExtn]  WITH CHECK ADD  CONSTRAINT [FK_TenantDetailsExtn_TenantDetails3] FOREIGN KEY([TenantDetailsExtn_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantDetailsExtn_TenantDetails3]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantDetailsExtn]'))
ALTER TABLE [dbo].[TenantDetailsExtn] CHECK CONSTRAINT [FK_TenantDetailsExtn_TenantDetails3]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantServiceMapping_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantServiceMapping]'))
ALTER TABLE [dbo].[TenantServiceMapping]  WITH CHECK ADD  CONSTRAINT [FK_TenantServiceMapping_Services] FOREIGN KEY([TenantServiceMapping_ServiceId])
REFERENCES [dbo].[Services] ([Service_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantServiceMapping_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantServiceMapping]'))
ALTER TABLE [dbo].[TenantServiceMapping] CHECK CONSTRAINT [FK_TenantServiceMapping_Services]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantServiceMapping_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantServiceMapping]'))
ALTER TABLE [dbo].[TenantServiceMapping]  WITH CHECK ADD  CONSTRAINT [FK_TenantServiceMapping_TenantDetails] FOREIGN KEY([TenantServiceMapping_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantServiceMapping_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantServiceMapping]'))
ALTER TABLE [dbo].[TenantServiceMapping] CHECK CONSTRAINT [FK_TenantServiceMapping_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSetting_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantSetting]'))
ALTER TABLE [dbo].[TenantSetting]  WITH CHECK ADD  CONSTRAINT [FK_TenantSetting_SettingsAttribute] FOREIGN KEY([TenantSetting_AttributeID])
REFERENCES [dbo].[SettingsAttribute] ([SettingsAttribute_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSetting_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantSetting]'))
ALTER TABLE [dbo].[TenantSetting] CHECK CONSTRAINT [FK_TenantSetting_SettingsAttribute]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSetting_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantSetting]'))
ALTER TABLE [dbo].[TenantSetting]  WITH CHECK ADD  CONSTRAINT [FK_TenantSetting_TenantDetails] FOREIGN KEY([TenantSetting_TenantID])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSetting_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantSetting]'))
ALTER TABLE [dbo].[TenantSetting] CHECK CONSTRAINT [FK_TenantSetting_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSettingTemplate_SettingTemplate]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantSettingTemplate]'))
ALTER TABLE [dbo].[TenantSettingTemplate]  WITH CHECK ADD  CONSTRAINT [FK_TenantSettingTemplate_SettingTemplate] FOREIGN KEY([TenantSettingTemplate_SettingTemplateId])
REFERENCES [dbo].[SettingTemplate] ([SettingTemplate_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSettingTemplate_SettingTemplate]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantSettingTemplate]'))
ALTER TABLE [dbo].[TenantSettingTemplate] CHECK CONSTRAINT [FK_TenantSettingTemplate_SettingTemplate]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSettingTemplate_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantSettingTemplate]'))
ALTER TABLE [dbo].[TenantSettingTemplate]  WITH CHECK ADD  CONSTRAINT [FK_TenantSettingTemplate_TenantDetails] FOREIGN KEY([TenantSettingTemplate_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TenantSettingTemplate_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TenantSettingTemplate]'))
ALTER TABLE [dbo].[TenantSettingTemplate] CHECK CONSTRAINT [FK_TenantSettingTemplate_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TextNotificationContent_NotificationContent]') AND parent_object_id = OBJECT_ID(N'[dbo].[TextNotificationContent]'))
ALTER TABLE [dbo].[TextNotificationContent]  WITH NOCHECK ADD  CONSTRAINT [FK_TextNotificationContent_NotificationContent] FOREIGN KEY([TextNotificationContent_NotificationContentId])
REFERENCES [dbo].[NotificationContent] ([NotificationContent_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TextNotificationContent_NotificationContent]') AND parent_object_id = OBJECT_ID(N'[dbo].[TextNotificationContent]'))
ALTER TABLE [dbo].[TextNotificationContent] CHECK CONSTRAINT [FK_TextNotificationContent_NotificationContent]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TextSource_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TextSource]'))
ALTER TABLE [dbo].[TextSource]  WITH CHECK ADD  CONSTRAINT [FK_TextSource_TenantDetails] FOREIGN KEY([TextSource_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_TextSource_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[TextSource]'))
ALTER TABLE [dbo].[TextSource] CHECK CONSTRAINT [FK_TextSource_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetails_Address]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetails]'))
ALTER TABLE [dbo].[UserDetails]  WITH CHECK ADD  CONSTRAINT [FK_UserDetails_Address] FOREIGN KEY([User_AddressID])
REFERENCES [dbo].[Address] ([Address_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetails_Address]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetails]'))
ALTER TABLE [dbo].[UserDetails] CHECK CONSTRAINT [FK_UserDetails_Address]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetails_Membership]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetails]'))
ALTER TABLE [dbo].[UserDetails]  WITH CHECK ADD  CONSTRAINT [FK_UserDetails_Membership] FOREIGN KEY([User_MembershipID])
REFERENCES [dbo].[Membership] ([Membership_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetails_Membership]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetails]'))
ALTER TABLE [dbo].[UserDetails] CHECK CONSTRAINT [FK_UserDetails_Membership]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetailExtn_Entity1]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetailExtn]'))
ALTER TABLE [dbo].[UserDetailExtn]  WITH CHECK ADD  CONSTRAINT [FK_UserDetailExtn_Entity1] FOREIGN KEY([UserDetailExtn_EntityId])
REFERENCES [dbo].[Entity] ([Entity_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetailExtn_Entity1]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetailExtn]'))
ALTER TABLE [dbo].[UserDetailExtn] CHECK CONSTRAINT [FK_UserDetailExtn_Entity1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetailExtn_TenantDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetailExtn]'))
ALTER TABLE [dbo].[UserDetailExtn]  WITH CHECK ADD  CONSTRAINT [FK_UserDetailExtn_TenantDetails1] FOREIGN KEY([UserDetailExtn_TenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetailExtn_TenantDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetailExtn]'))
ALTER TABLE [dbo].[UserDetailExtn] CHECK CONSTRAINT [FK_UserDetailExtn_TenantDetails1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetailExtn_UserDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetailExtn]'))
ALTER TABLE [dbo].[UserDetailExtn]  WITH CHECK ADD  CONSTRAINT [FK_UserDetailExtn_UserDetails1] FOREIGN KEY([UserDetailExtn_ReferenceId])
REFERENCES [dbo].[UserDetails] ([User_UserID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserDetailExtn_UserDetails1]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserDetailExtn]'))
ALTER TABLE [dbo].[UserDetailExtn] CHECK CONSTRAINT [FK_UserDetailExtn_UserDetails1]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserEntityOtherPermission_UserEntityPermission]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserEntityOtherPermission]'))
ALTER TABLE [dbo].[UserEntityOtherPermission]  WITH CHECK ADD  CONSTRAINT [FK_UserEntityOtherPermission_UserEntityPermission] FOREIGN KEY([UserEntityPermissionId])
REFERENCES [dbo].[UserEntityPermission] ([Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserEntityOtherPermission_UserEntityPermission]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserEntityOtherPermission]'))
ALTER TABLE [dbo].[UserEntityOtherPermission] CHECK CONSTRAINT [FK_UserEntityOtherPermission_UserEntityPermission]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserPasswordExpiration_Membership]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserPasswordExpiration]'))
ALTER TABLE [dbo].[UserPasswordExpiration]  WITH CHECK ADD  CONSTRAINT [FK_UserPasswordExpiration_Membership] FOREIGN KEY([Expiration_Membership_Id])
REFERENCES [dbo].[Membership] ([Membership_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserPasswordExpiration_Membership]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserPasswordExpiration]'))
ALTER TABLE [dbo].[UserPasswordExpiration] CHECK CONSTRAINT [FK_UserPasswordExpiration_Membership]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserRoles_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserRoles]'))
ALTER TABLE [dbo].[UserRoles]  WITH CHECK ADD  CONSTRAINT [FK_UserRoles_Roles] FOREIGN KEY([UserRole_RoleID])
REFERENCES [dbo].[Roles] ([Role_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserRoles_Roles]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserRoles]'))
ALTER TABLE [dbo].[UserRoles] CHECK CONSTRAINT [FK_UserRoles_Roles]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserRoles_UserDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserRoles]'))
ALTER TABLE [dbo].[UserRoles]  WITH CHECK ADD  CONSTRAINT [FK_UserRoles_UserDetails] FOREIGN KEY([UserRole_UserID])
REFERENCES [dbo].[UserDetails] ([User_UserID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserRoles_UserDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserRoles]'))
ALTER TABLE [dbo].[UserRoles] CHECK CONSTRAINT [FK_UserRoles_UserDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserRoleTenant_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserRoleTenant]'))
ALTER TABLE [dbo].[UserRoleTenant]  WITH CHECK ADD  CONSTRAINT [FK_UserRoleTenant_TenantDetails] FOREIGN KEY([UserRoleTenant_StratifiedTenantId])
REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserRoleTenant_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserRoleTenant]'))
ALTER TABLE [dbo].[UserRoleTenant] CHECK CONSTRAINT [FK_UserRoleTenant_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserRoleTenant_UserRoles]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserRoleTenant]'))
ALTER TABLE [dbo].[UserRoleTenant]  WITH CHECK ADD  CONSTRAINT [FK_UserRoleTenant_UserRoles] FOREIGN KEY([UserRoleTenant_UserRoleId])
REFERENCES [dbo].[UserRoles] ([UserRole_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserRoleTenant_UserRoles]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserRoleTenant]'))
ALTER TABLE [dbo].[UserRoleTenant] CHECK CONSTRAINT [FK_UserRoleTenant_UserRoles]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserService_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserService]'))
ALTER TABLE [dbo].[UserService]  WITH CHECK ADD  CONSTRAINT [FK_UserService_Services] FOREIGN KEY([UserService_ServiceCode])
REFERENCES [dbo].[Services] ([Service_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserService_Services]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserService]'))
ALTER TABLE [dbo].[UserService] CHECK CONSTRAINT [FK_UserService_Services]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserService_UserDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserService]'))
ALTER TABLE [dbo].[UserService]  WITH CHECK ADD  CONSTRAINT [FK_UserService_UserDetails] FOREIGN KEY([UserService_UserId])
REFERENCES [dbo].[UserDetails] ([User_UserID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserService_UserDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserService]'))
ALTER TABLE [dbo].[UserService] CHECK CONSTRAINT [FK_UserService_UserDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserSetting_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserSetting]'))
ALTER TABLE [dbo].[UserSetting]  WITH CHECK ADD  CONSTRAINT [FK_UserSetting_SettingsAttribute] FOREIGN KEY([UserSetting_AttributeID])
REFERENCES [dbo].[SettingsAttribute] ([SettingsAttribute_ID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserSetting_SettingsAttribute]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserSetting]'))
ALTER TABLE [dbo].[UserSetting] CHECK CONSTRAINT [FK_UserSetting_SettingsAttribute]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserSetting_UserDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserSetting]'))
ALTER TABLE [dbo].[UserSetting]  WITH CHECK ADD  CONSTRAINT [FK_UserSetting_UserDetails] FOREIGN KEY([UserSetting_UserID])
REFERENCES [dbo].[UserDetails] ([User_UserID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserSetting_UserDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserSetting]'))
ALTER TABLE [dbo].[UserSetting] CHECK CONSTRAINT [FK_UserSetting_UserDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_VirtualEntityExtn]') AND parent_object_id = OBJECT_ID(N'[dbo].[VirtualEntityExtn]'))
ALTER TABLE [dbo].[VirtualEntityExtn]  WITH CHECK ADD  CONSTRAINT [FK_VirtualEntityExtn] FOREIGN KEY([VirtualEntityExtn_ReferenceId])
REFERENCES [dbo].[VirtualEntity] ([VirtualEntity_Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_VirtualEntityExtn]') AND parent_object_id = OBJECT_ID(N'[dbo].[VirtualEntityExtn]'))
ALTER TABLE [dbo].[VirtualEntityExtn] CHECK CONSTRAINT [FK_VirtualEntityExtn]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDefinition_WorkFlow]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFDefinition]'))
ALTER TABLE [dbo].[WFDefinition]  WITH CHECK ADD  CONSTRAINT [FK_WFDefinition_WorkFlow] FOREIGN KEY([WfId])
REFERENCES [dbo].[WorkFlow] ([WfId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDefinition_WorkFlow]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFDefinition]'))
ALTER TABLE [dbo].[WFDefinition] CHECK CONSTRAINT [FK_WFDefinition_WorkFlow]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDefinitionUI_WFDefinition]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFDefinitionUI]'))
ALTER TABLE [dbo].[WFDefinitionUI]  WITH CHECK ADD  CONSTRAINT [FK_WFDefinitionUI_WFDefinition] FOREIGN KEY([WFDefinitionId])
REFERENCES [dbo].[WFDefinition] ([WfDefinitionId])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDefinitionUI_WFDefinition]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFDefinitionUI]'))
ALTER TABLE [dbo].[WFDefinitionUI] CHECK CONSTRAINT [FK_WFDefinitionUI_WFDefinition]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstance_WFDefinition]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstance]'))
ALTER TABLE [dbo].[WFInstance]  WITH NOCHECK ADD  CONSTRAINT [FK_WFInstance_WFDefinition] FOREIGN KEY([WfDefinitionId])
REFERENCES [dbo].[WFDefinition] ([WfDefinitionId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstance_WFDefinition]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstance]'))
ALTER TABLE [dbo].[WFInstance] CHECK CONSTRAINT [FK_WFInstance_WFDefinition]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstance_WorkFlow]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstance]'))
ALTER TABLE [dbo].[WFInstance]  WITH NOCHECK ADD  CONSTRAINT [FK_WFInstance_WorkFlow] FOREIGN KEY([WfId])
REFERENCES [dbo].[WorkFlow] ([WfId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstance_WorkFlow]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstance]'))
ALTER TABLE [dbo].[WFInstance] CHECK CONSTRAINT [FK_WFInstance_WorkFlow]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDynamicTaskActor_WFInstance]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFDynamicTaskActor]'))
ALTER TABLE [dbo].[WFDynamicTaskActor]  WITH NOCHECK ADD  CONSTRAINT [FK_WFDynamicTaskActor_WFInstance] FOREIGN KEY([WFInstanceId])
REFERENCES [dbo].[WFInstance] ([WfInstanceId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDynamicTaskActor_WFInstance]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFDynamicTaskActor]'))
ALTER TABLE [dbo].[WFDynamicTaskActor] CHECK CONSTRAINT [FK_WFDynamicTaskActor_WFInstance]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceDetails_WFInstance]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceDetails]'))
ALTER TABLE [dbo].[WFInstanceDetails]  WITH CHECK ADD  CONSTRAINT [FK_WFInstanceDetails_WFInstance] FOREIGN KEY([WFInstanceId])
REFERENCES [dbo].[WFInstance] ([WfInstanceId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceDetails_WFInstance]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceDetails]'))
ALTER TABLE [dbo].[WFInstanceDetails] CHECK CONSTRAINT [FK_WFInstanceDetails_WFInstance]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceTaskRun_WFInstance]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRun]'))
ALTER TABLE [dbo].[WFInstanceTaskRun]  WITH NOCHECK ADD  CONSTRAINT [FK_WFInstanceTaskRun_WFInstance] FOREIGN KEY([WfInstanceId])
REFERENCES [dbo].[WFInstance] ([WfInstanceId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceTaskRun_WFInstance]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRun]'))
ALTER TABLE [dbo].[WFInstanceTaskRun] CHECK CONSTRAINT [FK_WFInstanceTaskRun_WFInstance]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceTaskRun_WFInstanceTaskRun]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRun]'))
ALTER TABLE [dbo].[WFInstanceTaskRun]  WITH NOCHECK ADD  CONSTRAINT [FK_WFInstanceTaskRun_WFInstanceTaskRun] FOREIGN KEY([WfInstanceTaskRunId])
REFERENCES [dbo].[WFInstanceTaskRun] ([WfInstanceTaskRunId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceTaskRun_WFInstanceTaskRun]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRun]'))
ALTER TABLE [dbo].[WFInstanceTaskRun] CHECK CONSTRAINT [FK_WFInstanceTaskRun_WFInstanceTaskRun]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceTaskRunExecutionAudit_WFInstanceTaskRun]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunExecutionAudit]'))
ALTER TABLE [dbo].[WFInstanceTaskRunExecutionAudit]  WITH NOCHECK ADD  CONSTRAINT [FK_WFInstanceTaskRunExecutionAudit_WFInstanceTaskRun] FOREIGN KEY([WfInstanceTaskRunId])
REFERENCES [dbo].[WFInstanceTaskRun] ([WfInstanceTaskRunId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceTaskRunExecutionAudit_WFInstanceTaskRun]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunExecutionAudit]'))
ALTER TABLE [dbo].[WFInstanceTaskRunExecutionAudit] CHECK CONSTRAINT [FK_WFInstanceTaskRunExecutionAudit_WFInstanceTaskRun]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceTaskRunOutcomes_WFInstanceTaskRun]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunOutcomes]'))
ALTER TABLE [dbo].[WFInstanceTaskRunOutcomes]  WITH NOCHECK ADD  CONSTRAINT [FK_WFInstanceTaskRunOutcomes_WFInstanceTaskRun] FOREIGN KEY([WfInstanceTaskRunId])
REFERENCES [dbo].[WFInstanceTaskRun] ([WfInstanceTaskRunId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFInstanceTaskRunOutcomes_WFInstanceTaskRun]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFInstanceTaskRunOutcomes]'))
ALTER TABLE [dbo].[WFInstanceTaskRunOutcomes] CHECK CONSTRAINT [FK_WFInstanceTaskRunOutcomes_WFInstanceTaskRun]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDefinitionRouters_WFDefinition]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFRouterDefinition]'))
ALTER TABLE [dbo].[WFRouterDefinition]  WITH CHECK ADD  CONSTRAINT [FK_WFDefinitionRouters_WFDefinition] FOREIGN KEY([WfDefinitionId])
REFERENCES [dbo].[WFDefinition] ([WfDefinitionId])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDefinitionRouters_WFDefinition]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFRouterDefinition]'))
ALTER TABLE [dbo].[WFRouterDefinition] CHECK CONSTRAINT [FK_WFDefinitionRouters_WFDefinition]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFTaskActor_WFInstanceTaskRun]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFTaskActor]'))
ALTER TABLE [dbo].[WFTaskActor]  WITH NOCHECK ADD  CONSTRAINT [FK_WFTaskActor_WFInstanceTaskRun] FOREIGN KEY([WfInstanceTaskRunId])
REFERENCES [dbo].[WFInstanceTaskRun] ([WfInstanceTaskRunId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFTaskActor_WFInstanceTaskRun]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFTaskActor]'))
ALTER TABLE [dbo].[WFTaskActor] CHECK CONSTRAINT [FK_WFTaskActor_WFInstanceTaskRun]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDefinitionTasks_WFDefinition]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFTaskDefinition]'))
ALTER TABLE [dbo].[WFTaskDefinition]  WITH CHECK ADD  CONSTRAINT [FK_WFDefinitionTasks_WFDefinition] FOREIGN KEY([WfDefinitionId])
REFERENCES [dbo].[WFDefinition] ([WfDefinitionId])
ON DELETE CASCADE
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFDefinitionTasks_WFDefinition]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFTaskDefinition]'))
ALTER TABLE [dbo].[WFTaskDefinition] CHECK CONSTRAINT [FK_WFDefinitionTasks_WFDefinition]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFTracking_WFInstance]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFTracking]'))
ALTER TABLE [dbo].[WFTracking]  WITH CHECK ADD  CONSTRAINT [FK_WFTracking_WFInstance] FOREIGN KEY([WFInstanceId])
REFERENCES [dbo].[WFInstance] ([WfInstanceId])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_WFTracking_WFInstance]') AND parent_object_id = OBJECT_ID(N'[dbo].[WFTracking]'))
ALTER TABLE [dbo].[WFTracking] CHECK CONSTRAINT [FK_WFTracking_WFInstance]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AuthenticationTypes_AuthenticationProviders]') AND parent_object_id = OBJECT_ID(N'[dbo].[AuthenticationTypes]'))
	ALTER TABLE [dbo].[AuthenticationTypes]  WITH CHECK ADD  CONSTRAINT [FK_AuthenticationTypes_AuthenticationProviders] FOREIGN KEY([ProviderId])
	REFERENCES [dbo].[AuthenticationProviders] ([Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AuthenticationTypes_AuthenticationProviders]') AND parent_object_id = OBJECT_ID(N'[dbo].[AuthenticationTypes]'))
ALTER TABLE [dbo].[AuthenticationTypes] CHECK CONSTRAINT [FK_AuthenticationTypes_AuthenticationProviders]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ClientDetails_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ClientDetails]'))
	ALTER TABLE [dbo].[ClientDetails]  WITH CHECK ADD  CONSTRAINT [FK_ClientDetails_TenantDetails] FOREIGN KEY([TenantId])
	REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_ClientDetails_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[ClientDetails]'))
ALTER TABLE [dbo].[ClientDetails] CHECK CONSTRAINT [FK_ClientDetails_TenantDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SubjectDetails_ClientDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[SubjectDetails]'))
	ALTER TABLE [dbo].[SubjectDetails]  WITH CHECK ADD  CONSTRAINT [FK_SubjectDetails_ClientDetails] FOREIGN KEY([ClientId])
	REFERENCES [dbo].[ClientDetails] ([Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SubjectDetails_ClientDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[SubjectDetails]'))
ALTER TABLE [dbo].[SubjectDetails] CHECK CONSTRAINT [FK_SubjectDetails_ClientDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SubjectDetails_UserDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[SubjectDetails]'))
	ALTER TABLE [dbo].[SubjectDetails]  WITH CHECK ADD  CONSTRAINT [FK_SubjectDetails_UserDetails] FOREIGN KEY([UserId])
	REFERENCES [dbo].[UserDetails] ([User_UserID])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_SubjectDetails_UserDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[SubjectDetails]'))
ALTER TABLE [dbo].[SubjectDetails] CHECK CONSTRAINT [FK_SubjectDetails_UserDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserConsent_ClientDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserConsent]'))
	ALTER TABLE [dbo].[UserConsent]  WITH CHECK ADD  CONSTRAINT [FK_UserConsent_ClientDetails] FOREIGN KEY([ClientId])
	REFERENCES [dbo].[ClientDetails] ([Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserConsent_ClientDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserConsent]'))
ALTER TABLE [dbo].[UserConsent] CHECK CONSTRAINT [FK_UserConsent_ClientDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserConsent_SubjectDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserConsent]'))
	ALTER TABLE [dbo].[UserConsent]  WITH CHECK ADD  CONSTRAINT [FK_UserConsent_SubjectDetails] FOREIGN KEY([SubjectId])
	REFERENCES [dbo].[SubjectDetails] ([Id])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserConsent_SubjectDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserConsent]'))
ALTER TABLE [dbo].[UserConsent] CHECK CONSTRAINT [FK_UserConsent_SubjectDetails]
GO
IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserConsent_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserConsent]'))
	ALTER TABLE [dbo].[UserConsent]  WITH CHECK ADD  CONSTRAINT [FK_UserConsent_TenantDetails] FOREIGN KEY([TenantId])
	REFERENCES [dbo].[TenantDetails] ([Tenant_Code])
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_UserConsent_TenantDetails]') AND parent_object_id = OBJECT_ID(N'[dbo].[UserConsent]'))
ALTER TABLE [dbo].[UserConsent] CHECK CONSTRAINT [FK_UserConsent_TenantDetails]
GO
/****** Object:  UserDefinedFunction [dbo].[fnGetTenant]    Script Date: 9/10/2014 1:09:30 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE FUNCTION [dbo].[fnGetTenant]()
RETURNS varchar(36) AS 
BEGIN
    DECLARE @retval varchar(36) 
    DECLARE @inputstring varchar(255) = APP_NAME()
    DECLARE @end INT = CHARINDEX('_', @inputstring) 
    DECLARE @ctxTenantId varchar(36) = null;
    SELECT @ctxTenantId =  CONTEXT_INFO();
    
	   IF @inputstring = NULL
		  IF @ctxTenantId IS NOT NULL 
			 BEGIN
       			RETURN @ctxTenantId
			 END
	   ELSE
		  return  SUBSTRING(@inputstring, @end+1, LEN(@inputstring))
	   RETURN NULL
END
GO

/****** Object:  Table [dbo].[AuthorizationCodes]    Script Date: 2/11/2015 2:00:06 PM ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

SET ANSI_PADDING ON
GO
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AuthorizationCodes]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[AuthorizationCodes](
	[Id] [uniqueidentifier] NOT NULL,
	[CodeString] [varchar](max) NOT NULL,
	[CompanyCode] [varchar](255) NOT NULL,
	[SubjectId] [uniqueidentifier] NOT NULL,
	[ClientId] [uniqueidentifier] NOT NULL,
	[CreatedBy] [uniqueidentifier] NOT NULL,
	[CreatedOn] [datetime] NOT NULL,
	[UpdatedOn] [datetime] NULL,
	[UpdatedBy] [uniqueidentifier] NULL,
	[Status] [bit] NOT NULL,
	[ScopeRestrictions] [varchar](max) NULL,
	[AccessTokenType] [varchar](25) NULL,
 CONSTRAINT [PK_AuthorizationCodes] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
END
GO

ALTER TABLE [dbo].[Privileges] DROP CONSTRAINT [FK_Privileges_Privileges]